Package: SchoolDataIT
Type: Package
Title: Retrieve, Harmonise and Map Open Data Regarding the Italian
        School System
Version: 0.2.13
Authors@R: c(person(given = "Leonardo", 
             family = "Cefalo", 
             role = c("aut", "cre"),
             email = "leonardo.cefalo@uniba.it",
             comment = c(ORCID = "0009-0001-7776-723X")),
             person(given = "Alessio",
             family = "Pollice", 
             role = c("ctb","ths"),
             email="alessio.pollice@uniba.it",
             comment = c(ORCID = "0000-0002-2818-9373")),
             person(given = "Paolo",
             family = "Maranzano",
             role = c("ctb"),
             email = "pmaranzano.ricercastatistica@gmail.com",
             comment = c(ORCID = "0000-0002-9228-2759")))
Author: Leonardo Cefalo [aut, cre] (ORCID:
    <https://orcid.org/0009-0001-7776-723X>),
  Alessio Pollice [ctb, ths] (ORCID:
    <https://orcid.org/0000-0002-2818-9373>),
  Paolo Maranzano [ctb] (ORCID: <https://orcid.org/0000-0002-9228-2759>)
Maintainer: Leonardo Cefalo <leonardo.cefalo@uniba.it>
Description: Compiles and displays the available data sets regarding the Italian school system, with a focus on the infrastructural aspects.
    Input datasets are downloaded from the web, with the aim of updating everything to real time.  
    The functions are divided in four main modules, namely 
    'Get', to scrape raw data from the web
    'Util', various utilities needed to process raw data
    'Group', to aggregate data at the municipality or province level
    'Map', to visualize the output datasets.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://github.com/lcef97/SchoolDataIT
LazyData: true
RoxygenNote: 7.3.2
Imports: curl, dplyr, ggplot2, grDevices, httr, leafpop, magrittr,
        readr, rlang, rvest, sf, stringr, tidyr, utils, xml2
Suggests: knitr, mapview, readxl, rmarkdown, testthat (>= 3.0.0),
        tidyverse
Config/testthat/edition: 3
Depends: R (>= 2.10)
NeedsCompilation: no
Packaged: 2026-05-14 14:29:04 UTC; Leonardo
Repository: CRAN
Date/Publication: 2026-05-14 14:50:20 UTC
Built: R 4.6.0; ; 2026-05-14 19:26:50 UTC; unix

# Generated by roxygen2: do not edit by hand

export(Get_AdmUnNames)
export(Get_BroadBand)
export(Get_DB_MIUR)
export(Get_InnerAreas)
export(Get_Invalsi_IS)
export(Get_Registry)
export(Get_School2mun)
export(Get_Shapefile)
export(Get_nstud)
export(Get_nteachers_prov)
export(Group_DB_MIUR)
export(Group_nstud)
export(Group_teachers4stud)
export(Map_DB)
export(Map_Invalsi)
export(Map_School_Buildings)
export(Set_DB)
export(Util_Check_nstud_availability)
export(Util_DB_MIUR_num)
export(Util_Invalsi_filter)
export(Util_nstud_wide)
import(sf)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(rlang,.data)

# SchoolDataIT Version 0.2.13

## Package overview

Compiles and displays the available data sets regarding the Italian school system, with a focus on the infrastructural aspects.
Input datasets are downloaded from the web, with the aim of updating everything to real time.  
The functions are divided in four main modules, namely:
    'Get', to scrape raw data from the web
    'Util', various utilities needed to process raw data
    'Group', to aggregate data at the municipality or province level
    'Map', to visualize the output datasets.

More information can be found at [CRAN](https://CRAN.R-project.org/package=SchoolDataIT).

Additionally, the use of this package is described extensively in [this paper](https://link.springer.com/article/10.1007/s10260-025-00805-0) 
Even though the package does not include vignettes at the moment, the supplementary material of this paper serves similar purposes.

## Installation from GitHub

Experimental package versions are stacked in GitHub before being submitted to CRAN
in order to comply with the CRAN policy of waiting about one month from a submission to another. 
Here, you will always find the latest developments; 
[here](https://github.com/lcef97/SchoolDataIT/blob/main/NEWS.md) you can see the whole version history. 
To install the latest package version, run the code
``` r
devtools::install_github("lcef97/SchoolDataIT")
```
If you want to force R not to upgrade the dependencies, which may take some time, use instead:
``` r
devtools::install_github("lcef97/SchoolDataIT", upgrade = "never")
```

#' @keywords internal
#'
Group_Count <- function(data, groupcol, startgroup, count = TRUE, countname = NULL, FUN){

  options(dplyr.summarise.inform = FALSE)

  startcol <- startgroup - length(groupcol)
  endcol <- ncol(data) - length(groupcol)
  res <- data %>%
    dplyr::group_by_at(groupcol) %>%
    dplyr::summarise(dplyr::across(c(startcol:endcol), list(FUN)), Count = dplyr::n()) %>% dplyr::ungroup()
  res <- res %>% dplyr::relocate(.data$Count, .after = names(res)[length(groupcol)])
  if(count == FALSE){
    res <- res %>% dplyr::select(-.data$Count)
  }
  if(!is.null(countname)){
    names(res)[which(names(res)=="Count")] = countname
  }

  names(res) <- sub("_1$", "", names(res))
  return(res)
}

#' Associate a Municipality (LAU) code to each school
#'
#' @description  This function associates the relevant municipality codes to all the schools listed in the two main registries provided by the Italian Ministry of Education, University and Research, namely:
#'  \itemize{
#'    \item The registry of school buildings, here referred to as \code{Registry_from_buildings} (\code{\link{Get_DB_MIUR}})
#'    \item The official schools registry, here referred to as \code{Registry_from_registry} (see \code{\link{Get_Registry}})
#'  }
#'
#'
#'
#' @param Year Numeric or character value (last available is 2023).
#' Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023.} \code{2023} by default.
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param show_col_types Logical. If \code{TRUE}, if the \code{verbose} argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.
#' @param input_AdmUnNames Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_AdmUnNames}}
#' The ISTAT file including all the administrative units codes for the year in scope.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.
#' @param input_Registry Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_Registry}}
#' The school registry corresonding to the year in scope.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default
#' @param autoAbort Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#'
#' @source
#'  \itemize{
#'    \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0101EDIANAGRAFESTA2021> For Buildings registry (2021 onwards);
#'    \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0200EDIANAGRAFESTA> For Buindings registry(until 2019);
#'    \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Scuole> For Schools registry
#'   }
#'
#' @return An object of class \code{list}, including 4 elements:
#' \itemize{
#'   \item \code{$Registry_from_buildings}: Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}: the schools listed in the buildings registry
#'   \item \code{$Registry_from_registry}: Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}: the schools listed in the schools registry
#'   \item \code{$Any}: Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}: schools listed anywhere
#'   \item \code{$Both}: Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}: schools listed in both the sections
#'
#' }
#'
#'
#'
#' @examples
#'
#' \donttest{
#' Get_School2mun(Year = 2023, autoAbort = TRUE)
#' }
#'
#'
#' @export


Get_School2mun <- function(Year = 2023, show_col_types = FALSE, verbose = TRUE,
                           input_AdmUnNames = NULL, input_Registry = NULL, autoAbort = FALSE) {

  start.zero <- Sys.time()

  pattern <- year.patternB(Year)
  YearMinus1 <- as.numeric(substr(year.patternA(Year),1,4))
  if(is.null(input_AdmUnNames)){
    if(verbose) cat("Mapping cadastral codes to municipality (LAU) codes: \n")
    AdmUnYear <- ifelse(any(pattern %in% c(year.patternB(2016), year.patternB(2018))), YearMinus1+1, YearMinus1)
    AdmUnDate <- ifelse(any(pattern %in% c(year.patternB(2016), year.patternB(2018))), "01-01", "09-01")
    input_AdmUnNames <- Get_AdmUnNames( Date = paste0(AdmUnYear, "-", AdmUnDate), autoAbort = autoAbort)
  }
  # Case: failure
  if(is.null(input_AdmUnNames)) return(NULL)

  temp.R1 <- input_AdmUnNames %>% dplyr::select(.data$Cadastral_code, .data$Municipality_code)
  temp.R2 <- input_AdmUnNames %>% dplyr::select(.data$Cadastral_code, .data$Province_initials, .data$Municipality_code)

  if(verbose){cat("Retrieving registry from the buildings section ... \n")}

  starttime <- Sys.time()

  if(!Check_connection(autoAbort)) return(NULL)

  pattern <- year.patternB(Year)
  home.url <-"https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Edilizia%20Scolastica"
  homepage <- NULL
  attempt <- 0
  while(is.null(homepage)){
    homepage <- tryCatch({
      httr::content(httr::GET(home.url))
    }, error = function(e){
      message("Cannot read the html. If the problem persists, please contact the maintainer.\n")
      return(NULL)
    })

    if(is.null(homepage)){
      attempt <- attempt + 1
      message(10 - attempt, " attempts left)")
    }
    if(attempt >= 10) {
      message("Maximum attempts reached. Abort. We apologise for the inconvenience")
      return(NULL)
    }
  }
  #if(is.null(homepage)) return(NULL)
  name_pattern <- "([0-9]+)\\.(csv)$"
  links <- homepage %>% rvest::html_nodes("a") %>% rvest::html_attr("href") %>%
    unique()
  links <- grep("EDIANAGRAFESTA", links, value = TRUE)
  links <- grep(".csv", links, value = TRUE)

  file_to_download <- NULL
  for (string in links) {
    num_numeric_digits <- sum(unlist(gregexpr("[0-9]", string) ) > 0)
    nchar_min <- min(nchar(pattern))
    nchar_max <- max(nchar(pattern))
    if (num_numeric_digits >= nchar_min ){
      first_nchar_min <- stringr::str_extract(string, paste0("[0-9]{", nchar_min, "}"))
      first_nchar_max <- stringr::str_extract(string, paste0("[0-9]{", nchar_max, "}"))
      if (!is.na(first_nchar_min) & !is.na(first_nchar_max) &
          any(pattern %in% c(first_nchar_min, first_nchar_max))) {
        file_to_download <-  string
      }
    }
  }

  if(!is.null(file_to_download)){
    base.url <- dirname(home.url)
    file.url <- file.path(base.url, file_to_download)

    status <- 0
    attempt <- 0
    while(status != 200){
      response <- tryCatch({
        httr::GET(file.url)
      }, error = function(e) {
        message("Error occurred during scraping, attempt repeated ... \n")
        NULL
      })
      status <- response$status_code
      if(is.null(response)){
        status <- 0
      }
      status <- response$status_code
      if(is.null(response)){
        status <- 0
      }
      if(status != 200){
        attempt <- attempt + 1
        message("Operation exited with status: ", status, "; operation repeated (",
                10 - attempt, " attempts left)")
      }
      if(attempt >= 10) {
        message("Maximum attempts reached. Abort. We apologise for the inconvenience")
        return(NULL)
      }
    }

    if (httr::http_type(response) %in% c("application/csv", "text/csv", "application/octet-stream")) {
      content <- rawToChar(response$content)
      if(nchar(content)==0){
        message("Empty file. Operation aborted.
        There seems to be something wrong with the website.
        Please contact the maintainer, maybe it could help. \n")
        return(NULL)
      } else {
        input_Registry1 <- readr::read_csv(content, show_col_types = FALSE)
      }
    } else {
      message(paste("Wrong file type:", httr::http_type(response)) )
      message("Failed to download and process:", file_to_download, "\n")
      return(NULL)
    }

    # This is for the province of Naples whose abbreviation is `NA`
    input_Registry1$SIGLAPROVINCIA <- stringr::str_replace_na(input_Registry1$SIGLAPROVINCIA, "NA")

    # This is for the municipality of Bladen/Plodn/Sappada which changed it province in 2018
    if(!any(pattern %in% year.patternB(2016))){
      input_Registry1 <-input_Registry1  %>%
        dplyr::mutate(dplyr::across(.data$SIGLAPROVINCIA, ~ dplyr::case_when(
          toupper(.data$DESCRIZIONECOMUNE) == "SAPPADA" ~ "UD",
          TRUE ~ .data$SIGLAPROVINCIA
        )))
    }

    tabrename <- tabrename.manual()
    for (j in (1:ncol(input_Registry1))){
      if (names(input_Registry1)[j] %in% tabrename$Input){
        names(input_Registry1)[j] <- tabrename[which(tabrename$Input == names(input_Registry1)[j]),4]
      }
    }

    endtime <- Sys.time()
    if(verbose) {
      cat(round(difftime(endtime, starttime, units="secs") ,2), "seconds needed for the download \n" )
    }

    if(any(pattern %in% c("201516", "2016", "201718", "201819"))){
      Registry1 <- input_Registry1 %>%
        dplyr::rename(Cadastral_code = .data$Municipality_code) %>%
        dplyr::select(.data$School_code, .data$Province_initials,
                      .data$Cadastral_code, .data$Municipality_description) %>%
        dplyr::left_join(temp.R1, by = "Cadastral_code") %>%
        dplyr::select(-.data$Cadastral_code) %>%
        fixMun.manual(Year)
    } else {
      Registry1 <- input_Registry1 %>%
        dplyr::select(.data$School_code, .data$Province_initials, .data$Municipality_code, .data$Municipality_description)
    }
    Registry1 <- Registry1 %>% dplyr::mutate(Province_code = as.numeric(substr(.data$Municipality_code, 1, 3))) %>%
      dplyr::select(.data$School_code, .data$Province_code, .data$Province_initials,
                    .data$Municipality_code, .data$Municipality_description) %>%
      dplyr::mutate(Municipality_description = stringr::str_to_title(.data$Municipality_description)) %>% unique()
  } else {
    message("Unable to find buildings registry for year ",
            Year, "; we apologise for the inconvenience")
    Registry1 <- NULL
  }

  if(is.null(input_Registry)){
    if(verbose) cat("Retrieving registry from registry section ... \n ")
    starttime <- Sys.time()
    input_Registry <- Get_Registry(Year = Year, autoAbort = autoAbort)
    endtime <- Sys.time()
    if(verbose){
      cat(round(difftime(endtime, starttime, units="secs") ,2), "seconds needed for the download \n"  )
    }
  }
  if(is.null(input_Registry)) return(NULL)

  Registry2 <- input_Registry %>% dplyr::select(
    .data$School_code,.data$Cadastral_code, .data$Municipality_description)%>%
    dplyr::left_join(temp.R2, by = "Cadastral_code") %>%
    dplyr::select(-.data$Cadastral_code) %>%
    dplyr::mutate(Province_code = as.numeric(substr(.data$Municipality_code, 1, 3))) %>%
    dplyr::select(.data$School_code, .data$Province_code, .data$Province_initials,
                  .data$Municipality_code, .data$Municipality_description) %>%
    dplyr::mutate(Municipality_description = stringr::str_to_title(.data$Municipality_description)) %>%
    unique() %>% fixMun.manual(Year)

  res <- list()
  res[["Registry_from_buildings"]] <- Registry1
  res[["Registry_from_registry"]] <- Registry2
  if(!is.null(Registry1)){
    res[["Any"]] <- rbind(Registry2, Registry1) %>% unique()
    res[["Both"]] <- Registry2 %>% dplyr::filter(.data$School_code %in% Registry1$School_code)
  }

  return(res)
}

#' Map schools included in the ultra-broadband plan to their LAU codes.
#'
#' @description
#' Helper function to provide the ultra-broadband dataset obtained with \code{\link{Get_BroadBand}}
#' with the statistical codes of the relevant municipalities, obtained with \code{\link{Get_School2mun}},
#' in case the ultra-broadband dataset has been downloaded with argument \code{include_municipality_code = FALSE}.
#'
#'
#' @param data Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}
#' obtained with the function \code{\link{Get_BroadBand}}
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param autoAbort Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param input_School2mun Object of class \code{list} obtained with \code{\link{Get_School2mun}},
#' including the mapping from school codes to municipality (and province) codes.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.
#' @param input_Registry If \code{input_School2mun} is required, an object of class \code{tbl_df}, \code{tbl} and \code{data.frame}
#' corresponding to the national school registry (preferably of the last year, i.e. 2024/2025) obtained with \code{\link{Get_Registry}}. NULL by default.
#' @param input_AdmUnNames If \code{input_School2mun} is required, an object of class \code{tbl_df}, \code{tbl} and \code{data.frame}
#' corresponding to the statistical codes of administrative units (preferably referring to the period corresponding to school year 2024/2025)
#'  obtained with \code{\link{Get_AdmUnNames}}. NULL by default.
#'
#' @return An object of class \code{tbl_df}, \code{tbl} and \code{data.frame},
#' identical to the output of \code{\link{Get_BroadBand}} with an additional column for LAU codes
#'
#' @source  Broadband dashboard: <https://bandaultralarga.italia.it/scuole-voucher/dashboard-scuole/> .
#' ISTAT LAU codes: <https://situas.istat.it/web/#/territorio>
#'
#'
#'
#' @details see \code{\link{Get_BroadBand}}
#'

#' @keywords internal

Util_BroadBand2mun <- function(data, input_School2mun = NULL, input_Registry = NULL,
                          input_AdmUnNames = NULL, verbose = FALSE,
                          autoAbort = FALSE){
  if(is.null(input_School2mun)){
    input_School2mun <- Get_School2mun (Year = 2025, verbose = verbose,
                               input_AdmUnNames = input_AdmUnNames,
                               input_Registry = input_Registry,
                               autoAbort = autoAbort)
  }
  broadband <- data
  if(is.null(broadband) || is.null(input_School2mun)) {
    message("Something wrong occurred. Operation aborted")
    res <- NULL
  }else{
    df.R <- input_School2mun$Registry_from_registry %>%
      dplyr::select(.data$School_code, .data$Municipality_code)
    res <- broadband %>% dplyr::left_join(df.R, by = "School_code") %>%
      dplyr::filter(!is.na(.data$Municipality_code)) %>%
      dplyr::relocate(.data$Municipality_code, .before = .data$Municipality_description)
  }

  return(res)
}

#' Download the shapefiles of Italian NUTS-3 and LAU administrative units
#'
#' @description Downloads either the boundaries or the centroids of the relevant administrative units,
#' either provinces or municipalities, from the ISTAT website. Geometries are expressed in meters.
#'
#'
#'
#' @param Year Numeric. Reference year for the administrative units.
#' @param level Character. Either  \code{"LAU"/"Municipality"}, \code{"NUTS-3"/"Province"},  \code{"NUTS-2"/"Region"}, . \code{"LAU"} by default
#' @param lightShp Logical. If \code{TRUE}, the function downloads a generalised, i.e.less detailed, and lighter version of the shapefiles.
#' \code{TRUE} by default.
#' @param centroids Logical. Whether to switch from polygon geometry to point geometry. In the latter case, the point is located at the centroid of the relevant area. \code{FALSE} by default.
#' @param autoAbort Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#'
#'
#' @return A spatial data frame of class \code{data.frame} and \code{sf}.
#'
#' @source <https://www.istat.it/it/archivio/222527>
#'
#' @examples
#'
#' \donttest{
#'   library(magrittr)
#'
#'
#'   Prov23_shp <- Get_Shapefile(2023, lightShp = TRUE, level = "NUTS-3", autoAbort = TRUE)
#'   ggplot2::ggplot() + ggplot2::geom_sf(data = Prov23_shp) +
#'     ggplot2::ggtitle("Italian provinces in 2023/01/01")
#'  }
#'
#'
#' @export


Get_Shapefile <- function(Year, level = "LAU", lightShp = TRUE,
                          autoAbort = FALSE, centroids = FALSE){

  while(Year < 2001 & Year != 1991){
    message(paste("Year", Year, "not available. Please choose another year between 2001 and the current year"))
    Year <- readline(prompt= "   ")
  }

  if(!Check_connection(autoAbort)) return(NULL)

  home.ISTAT.Shp <- "https://www.istat.it/it/archivio/222527"
  homepage <- NULL
  attempt <- 0
  while(is.null(homepage) && attempt <= 10){
    homepage <- tryCatch({
      httr::content(httr::GET(home.ISTAT.Shp))
    }, error = function(e){
      message("Cannot read the html; ", 10 - attempt,
              " attempts left. If the problem persists, please contact the maintainer.\n")
      return(NULL)
    })
    attempt <- attempt + 1
  }
  if(is.null(homepage)) return(NULL)

  links <- homepage %>% rvest::html_nodes("a") %>% rvest::html_attr("href") %>% unique()
  links <- grep(paste0("confini_amministrativi.*", Year), links, value = TRUE)
  links.NR <- grep("ED50", links, value = TRUE)
  links <- links[which(!links %in% links.NR)]

  if(lightShp == FALSE){
    link <- grep(paste0(Year, ".zip"),links, value=TRUE)
  } else {
    link <- grep(paste0(Year, "_g.zip"),links, value=TRUE)
  }

  if(length(link) == 0L){
    message("Shapefiles not found for year: ", Year)
    return(NULL)
  }

  base.url <- dirname(home.ISTAT.Shp)
  file.url <- xml2::url_absolute(link, base.url)
  temp1 <- tempfile()
  temp2 <- tempfile()
  timeout.default <- options("timeout")$timeout
  on.exit(options(timeout = timeout.default))
  options(timeout = max(timeout.default, 120))
  x <- tryCatch({
    utils::download.file(url = file.url, destfile = temp1)},
    error = function(e) {
      NULL
    })

  if(is.null(x)){
    unlink(temp1, recursive = TRUE, force = TRUE)
    unlink(temp2, recursive = TRUE, force = TRUE)
    return(NULL)
  }
  utils::unzip(zipfile = temp1, exdir = temp2)

  files.int <- list.files(list.files(temp2, full.names = TRUE), full.names = TRUE)
  if(!any(grepl(".shp$", files.int))){
    files.int <- list.files(files.int, full.names=TRUE)
  }

  pattern <- dplyr::case_when(
    toupper(level) %in% c("LAU", "NUTS-4", "MUNICIPALITY", "MUN") ~ "Com",
    toupper(level) %in% c("NUTS-3", "PROVINCE", "PROV") ~ "Prov",
    toupper(level) %in% c("NUTS-2", "REGION", "REG") ~ "Reg")


  filename.int <- grep(pattern, files.int, value = TRUE)
  filename.int <- filename.int[which(substr(filename.int, nchar(filename.int)-2, nchar(filename.int)) == "shp")]

  res <- sf::read_sf( filename.int)
  if(centroids) {
    sf::st_agr(res) <- "constant"
    res <- sf::st_point_on_surface(res)
  }

  unlink(temp1, recursive = TRUE, force = TRUE)
  unlink(temp2, recursive = TRUE, force = TRUE)

  return(res)
}

#' Aggregate the students number data by class at the municipality and province level
#'
#' @description  This function creates two dataframes with the number of students, classes and students by class, aggregated at the province and municipality level
#'
#'
#'
#'
#' @param data Either an object of class \code{list}, obtained as output of the \code{\link{Get_nstud}} function,
#' or an object of class class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the \code{\link{Util_nstud_wide}} function,
#' if \code{NULL}, the function will download it automatically but it will not be saved in the global environment. \code{NULL} by default.
#' @param Year Numeric or character value. The reference school year, if either of the \code{input_} arguments must be retrieved.
#' Available in the formats: \code{2022}, \code{"2022/2023"}, \code{"202223"}, \code{"20222023"}. \code{2023} by default
#' @param check Logical. If \code{TRUE}, the function runs the test of the students number availability across all school included in the school registries (see \code{\link{Util_Check_nstud_availability}}). \code{TRUE} by default
#' @param check_ggplot Logical. If \code{check == TRUE}, whether to display or not a static map of the availability of the students number by province; see also \code{\link{Util_Check_nstud_availability}}. \code{TRUE} by default.
#' @param check_registry Character. If \code{check == TRUE}, the school registries included in the \code{input_School2mun} object (see \code{\link{Get_School2mun}}) whose availability has to be checked. Either \code{"Registry_from_buildings"} (buildings section), \code{"Registry_from_registry"} (registry section), \code{"Any"} or \code{"Both"}. \code{"Any"} by default.
#' @param InnerAreas Logical. If \code{check == TRUE}, Whether it must be checked if municipalities belong to Inner areas or not. \code{TRUE} by default.
#' @param ord_InnerAreas Logical. If \code{check == TRUE} and \code{InnerAreas == TRUE}, whether the Inner areas classification should be treated as an ordinal variable rather than as a categorical one (see \code{\link{Get_InnerAreas}} for the classification).
#' \code{FALSE} by default.
#' @param missing_to_1 Logical. Only needed if \code{data} is not provided in wide format. Whether the number of classes should be imputed to 1 when it is missing; see \code{\link{Util_nstud_wide}}. \code{FALSE} by default.
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param input_Registry Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_Registry}}
#' If \code{check == TRUE}, the school registry (the properly said one, from the registry section).
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default
#' @param input_InnerAreas Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#' The classification of peripheral municipalities, obtained as output of the \code{\link{Get_InnerAreas}} function.
#' Needed only if \code{check == TRUE} and \code{InnerAreas == TRUE}.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default
#' @param input_Prov_shp Object of class \code{sf}, \code{tbl_df}, \code{tbl}, \code{data.frame}. The relevant shapefile of Italian municipalities, if both the \code{check} and \code{check_ggplot} options are chosen.
#' If \code{NULL} it is downloaded automatically but not saved in the global environment. \code{NULL} by default.
#' @param input_School2mun Object of class \code{list} with elements of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_School2mun}}.
#' The mapping from school codes to municipality (and province) codes. Needed only if `check == TRUE`.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.
#' @param input_AdmUnNames Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_AdmUnNames}}
#' The ISTAT file including all the codes and the names of the administrative units for the year in scope.
#' Only needed if \code{check == TRUE} and the argument \code{input_School2mun} is \code{NULL}.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.
#' @param autoAbort Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param ... Additional arguments to the function \code{\link{Util_nstud_wide}} if \code{data} is not provided.
#'
#'
#'
#'
#' @return An object of class \code{list} including:
#'
#' \itemize{
#'   \item \code{$Municipality_data}:
#'    object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, the output dataframe detailed at the municipality level
#'
#'   \item \code{$Province_data}: object of class 'tbl_df', 'tbl' and 'data.frame', the output dataframe detailad at the province level
#'  }
#'
#' @details Numerical variables are summarised by the mean; Boolean variables are summarised by the mean as well, thus they become frequency indicators.
#' Qualitative values, if included, are summarised by the mode. Summary measures do not include NAs.
#'
#'
#' @examples
#
#' Year <- 2023
#'
#' nstud23_aggr <- Group_nstud(data = example_input_nstud23, Year = Year,
#'                            input_Registry = example_input_Registry23,
#'                            InnerAreas = FALSE,
#'                            input_School2mun = example_School2mun23)
#'
#' summary(nstud23_aggr$Municipality_data[,c(46,47,48)])
#'
#' summary(nstud23_aggr$Province_data[,c(44,45,46)])
#'
#'
#' @export


Group_nstud <- function(data = NULL, Year = 2023,
                        check = TRUE, verbose = TRUE, check_registry = "Any",
                        InnerAreas = TRUE,
                        ord_InnerAreas = FALSE, check_ggplot = FALSE,
                        missing_to_1 = FALSE, input_Registry = NULL,
                        input_InnerAreas = NULL, input_Prov_shp = NULL,
                        input_School2mun = NULL, input_AdmUnNames = NULL,
                        autoAbort = FALSE, ...) {

  options(dplyr.summarise.inform = FALSE)
  . <- NULL

  start.zero <- Sys.time()

  while(is.null(data)){
    data <- Get_nstud(Year = Year, verbose = verbose, autoAbort = autoAbort)
    if(is.null(data)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during students counts retrieving. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  if(is.data.frame(data)){
    if(any(grepl("per.*class", names(data), ignore.case = TRUE)) ||
       any(grepl("full_time", names(data), ignore.case = TRUE))){
      nstud.wide <- data
    } else {
      nstud.wide <- Util_nstud_wide(data = data, missing_to_1 = missing_to_1, ... )
    }
  } else nstud.wide <- Util_nstud_wide(data = data, missing_to_1 = missing_to_1, ... )

  if(verbose) cat("Linking schools to reference municipalities \n")
  while(is.null(input_School2mun)){
    input_School2mun <- Get_School2mun(
      Year = Year,verbose = verbose,
      input_Registry = input_Registry, input_AdmUnNames = input_AdmUnNames,
      autoAbort = autoAbort)
    if(is.null(input_School2mun)){
      if(!autoAbort){
        holdOn <- ""
        message("Error occurred in mapping school to municipalities. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }
  if(!is.data.frame(input_School2mun)) School2mun.R <- input_School2mun[[check_registry]]

  if(verbose) cat("Aggregating schools data \n")

  suppressWarnings(
    nstud.wide_Mun <- dplyr::left_join(
      nstud.wide, School2mun.R ,by = "School_code") %>%
      dplyr::filter(!is.na(.data$Municipality_code)) %>% dplyr::relocate(
        c(.data$Municipality_code, .data$Municipality_description,
          .data$Province_code, .data$Province_initials), .after ="School_code") %>%
      dplyr::select(-grep("Students_per_class", names(.))) %>% Group_Count(
        groupcol = c("Municipality_code", "Municipality_description",
                     "Province_code", "Province_initials", "Order"),
        startgroup = 7, FUN = sum, countname = "nschools")
  )

  suppressWarnings(
    nstud.wide_Prov <- dplyr::left_join(
      nstud.wide, School2mun.R, by = "School_code") %>%
      dplyr::filter(!is.na(.data$Municipality_code)) %>% dplyr::relocate(
        c(.data$Municipality_code, .data$Municipality_description,
          .data$Province_code, .data$Province_initials), .after ="School_code") %>%
      dplyr::select(-grep("Students_per_class", names(.))) %>% Group_Count(
        groupcol = c("Province_code", "Province_initials", "Order"),
        startgroup = 7, FUN = sum, countname = "nschools")
  )

  if(any(grepl("Students_per_class", names(nstud.wide)))){
    for (i in (1:14)){
      j <- 2*i + 5
      nstud.wide_Mun <- nstud.wide_Mun %>%
        dplyr::mutate(xx = as.numeric(unlist(dplyr::select(.,j)/dplyr::select(.,j+1) ) ) )
      names(nstud.wide_Mun)[ncol(nstud.wide_Mun)] <- paste("Students_per_class_", ifelse(i<14, i,"Tot"), sep = "")
    }
    nstud.wide_Mun[, c(6:ncol(nstud.wide_Mun))][is.na(nstud.wide_Mun[ ,c(6:ncol(nstud.wide_Mun))])] <- 0

    for (i in c(1:13)){
      j <- i + 34
      k <- 3*i + 5
      nstud.wide_Mun <- nstud.wide_Mun %>% dplyr::relocate(j, .after = k)
    }

    for (i in (1:14)){
      j <- 2*i + 3
      nstud.wide_Prov <- nstud.wide_Prov %>%
        dplyr::mutate(xx = as.numeric(unlist(dplyr::select(.,j)/dplyr::select(.,j+1) ) ) )
      names(nstud.wide_Prov)[ncol(nstud.wide_Prov)] <- paste("Students_per_class_", ifelse(i<14, i,"Tot"), sep = "")
    }
    nstud.wide_Prov[, c(4:ncol(nstud.wide_Prov))][is.na(nstud.wide_Prov[ ,c(4:ncol(nstud.wide_Prov))])] <- 0

    for (i in c(1:13)){
      j <- i + 32
      k <- 3*i + 3
      nstud.wide_Prov <- nstud.wide_Prov %>%
        dplyr::relocate(j, .after = k)
    }
  }

  nstud.check <- NULL
  if(check){
    if(verbose) cat("Checking whether registered schools are included \n")
    nstud.check <-
        Util_Check_nstud_availability(nstud.wide, Year = Year,cutout = c("IC", "IS", "NR"),
                                      ggplot = check_ggplot,
                                      verbose = verbose, InnerAreas = InnerAreas,
                                      ord_InnerAreas = ord_InnerAreas,
                                      input_Registry = input_Registry, input_AdmUnNames = NULL,
                                      input_InnerAreas = input_InnerAreas,
                                      input_Prov_shp = input_Prov_shp, input_School2mun = input_School2mun,
                                      autoAbort = autoAbort)
    if(is.null(nstud.check)){
      message("Error occurred during the students count availability check")
    } else {
      check_mun <- nstud.check$Municipality_data[[check_registry]]
      if(InnerAreas){
        if(ord_InnerAreas){
          check_mun <- check_mun %>% dplyr::select(.data$Order, .data$Municipality_code, .data$Availability, .data$Inner_area,
                                                   .data$A_mun, .data$B_mun, .data$C_mun, .data$D_mun, .data$E_mun, .data$F_mun)
        } else {
          check_mun <- check_mun %>% dplyr::select(.data$Order, .data$Municipality_code, .data$Availability, .data$Inner_area)
        }
      } else {
        check_mun <- check_mun %>% dplyr::select(.data$Order, .data$Municipality_code, .data$Availability)
      }
      check_mun <- check_mun %>%
        tidyr::unite("ID", c(.data$Municipality_code, .data$Order), sep = "___") %>%
        dplyr::mutate(ID = gsub(" ", "_", .data$ID))

      nstud.wide_Mun <- nstud.wide_Mun %>%
        tidyr::unite("ID", c(.data$Municipality_code, .data$Order), sep = "___") %>%
        dplyr::mutate(ID = gsub(" ", "_", .data$ID)) %>%
        dplyr::left_join(check_mun, by = "ID") %>%
        tidyr::separate(.data$ID, into = c("Municipality_code", "Order"), sep = "___") %>%
        dplyr::mutate(Order = gsub("_", " ", .data$Order))

      check_prov <- nstud.check$Province_data[[check_registry]]
      if(InnerAreas){
        if(ord_InnerAreas){
          check_prov <- check_prov %>%
            dplyr::select(.data$Order, .data$Province_code, .data$Availability, .data$Inner_area,
                          .data$A_mun, .data$B_mun, .data$C_mun, .data$D_mun, .data$E_mun, .data$F_mun)
        } else {
          check_prov <- check_prov %>%
            dplyr::select(.data$Order, .data$Province_code, .data$Availability, .data$Inner_area)
        }
      } else {
        check_prov <- check_prov %>%
          dplyr::select(.data$Order, .data$Province_code, .data$Availability)
      }
      check_prov  <- check_prov %>%
        tidyr::unite("ID", c(.data$Province_code, .data$Order), sep = "___") %>%
        dplyr::mutate(ID = gsub(" ", "_", .data$ID))

      nstud.wide_Prov <- nstud.wide_Prov %>%
        tidyr::unite("ID", c(.data$Province_code, .data$Order), sep = "___") %>%
        dplyr::mutate(ID = gsub(" ", "_", .data$ID)) %>%
        dplyr::left_join(check_prov, by = "ID") %>%
        tidyr::separate(.data$ID, into = c("Province_code", "Order"), sep = "___") %>%
        dplyr::mutate(Order = gsub("_", " ", .data$Order)) %>%
        dplyr::mutate(Province_code = as.numeric(.data$Province_code))
    }
  }

  endtime <- Sys.time()

  if(verbose){
    cat(paste("Total time needed to aggregate students number data",
              ifelse(check, "and check for data availability in school registries", ""),
              ifelse(InnerAreas, "and for schools belonging to inner areas", ""),
              ":"), difftime(endtime, start.zero, units="secs"), "seconds \n"  )
  }

  return(list(Municipality_data = nstud.wide_Mun, Province_data = nstud.wide_Prov))
}

#' @keywords internal
#'

fixMun.manual <- function(data, Year){
  pattern <- year.patternB(Year)
  if("Municipality_description" %in% names(data)){
    data$Municipality_description <-
      stringr::str_replace(data$Municipality_description, "\\s*\\(.*$", "")
  }
  if("Province_initials" %in% names(data)){
    data <- data %>% dplyr::mutate(dplyr::across(.data$Province_initials, ~ dplyr::case_when(
      toupper(.data$Municipality_description) == "S. OMOBONO TERME" ~ "BG",
      toupper(.data$Municipality_description) == "PORRETTA TERME" ~ "BO",
      toupper(.data$Municipality_description) == "CORTEOLONA" ~ "PV",
      toupper(.data$Municipality_description) == "ZIBELLO" ~ "PR",
      toupper(.data$Municipality_description) == "MACCAGNO" ~ "VA",
      toupper(.data$Municipality_description) %in% c("VIRGILIO", "BORGOFORTE") ~ "MN",
      toupper(.data$Municipality_description) == "BREMBILLA" ~ "BG",
      toupper(.data$Municipality_description) == "RAMISETO" ~ "RE",
      toupper(.data$Municipality_description) == "BUSANA" ~ "RE",
      toupper(.data$Municipality_description) == "OSSUCCIO" ~ "CO",
      toupper(.data$Municipality_description) == "TREMEZZO" ~ "CO",
      toupper(.data$Municipality_description) == "BELLAGIO" ~ "CO",
      toupper(.data$Municipality_description) == "LESSONA" ~ "BI",
      toupper(.data$Municipality_description) == "MONTESCUDO" ~ "RN",
      toupper(.data$Municipality_description) == "CAMPIGLIA CERVO" ~ "BI",
      toupper(.data$Municipality_description) == "CAVACURTA" ~ "LO",
      toupper(.data$Municipality_description) == "CASTIGLIONE D'INTELVI" ~ "CO",
      toupper(.data$Municipality_description) == "PERGINE VALDARNO" ~ "AR",
      toupper(.data$Municipality_description) == "VILLA POMA" ~ "MN",
      toupper(.data$Municipality_description) == "CASSANO SPINOLA" ~ "AL",
      toupper(.data$Municipality_description) == "ALLUVIONI CAMBIO'" ~ "AL",
      toupper(.data$Municipality_description) == "LATERINA" ~ "AR",
      toupper(.data$Municipality_description) == "REVERE" ~ "MN",
      toupper(.data$Municipality_description) == "VESTRENO" ~ "LC",
      toupper(.data$Municipality_description) == "RIO NELL'ELBA" ~ "LI",
      toupper(.data$Municipality_description) == "SAN FEDELE INTELVI" ~ "CO",
      toupper(.data$Municipality_description) == "RIO MARINA" ~ "LI",
      toupper(.data$Municipality_description) == "PIEVE DI CORIANO" ~ "MN",
      toupper(.data$Municipality_description) == "PECORARA" ~ "PC",
      toupper(.data$Municipality_description) %in% c("CELLIO", "CELLIO (FINO AL 01/01") ~ "VC",
      toupper(.data$Municipality_description) == "NIBBIANO" ~ "PC",
      toupper(.data$Municipality_description) %in% c("CORIGLIANO CALABRO", "ROSSANO") &
        any(pattern == year.patternB(2019)) ~ "CS",
      toupper(.data$Municipality_description) == "FIUMICELLO VILLA VICENTINA" ~ "UD",
      toupper(.data$Municipality_description) %in% c("FIUMICELLO", "VILLA VICENTINA") &
        any(pattern == year.patternB(2019))~ "UD",
      toupper(.data$Municipality_description) %in% c("BARBARANO VICENTINO", "MOSSANO") &
        any(pattern == year.patternB(2019))~ "VI",
      toupper(.data$Municipality_description) %in% c("MEGLIADINO SAN FIDENZIO", "SANTA MARGHERITA D'ADIGE", "SALETTO") &
        any(pattern == year.patternB(2019))~ "PD",
      toupper(.data$Municipality_description) == "TREPPO LIGOSULLO" ~ "UD",
      toupper(.data$Municipality_description) == "TREPPO CARNICO" &
        any(pattern == year.patternB(2019))~ "UD",
      toupper(.data$Municipality_description) == "CASSANO SPINOLA" ~ "AL",
      TRUE ~ .data$Province_initials
    )))
  }
  if("Province_code" %in% names(data)){
    data <- data %>% dplyr::mutate(dplyr::across(.data$Province_code, ~ dplyr::case_when(
      toupper(.data$Municipality_description) == "S. OMOBONO TERME" ~ 16,
      toupper(.data$Municipality_description) == "PORRETTA TERME" ~ 37,
      toupper(.data$Municipality_description) == "CORTEOLONA" ~ 18,
      toupper(.data$Municipality_description) == "ZIBELLO" ~ 34,
      toupper(.data$Municipality_description) == "MACCAGNO" ~ 12,
      toupper(.data$Municipality_description) %in% c("VIRGILIO", "BORGOFORTE") ~ 20,
      toupper(.data$Municipality_description) == "BREMBILLA" ~ 16,
      toupper(.data$Municipality_description) == "RAMISETO" ~ 35,
      toupper(.data$Municipality_description) == "BUSANA" ~ 35,
      toupper(.data$Municipality_description) == "OSSUCCIO" ~ 13,
      toupper(.data$Municipality_description) == "TREMEZZO" ~ 13,
      toupper(.data$Municipality_description) == "BELLAGIO" ~ 13,
      toupper(.data$Municipality_description) == "LESSONA" ~ 96,
      toupper(.data$Municipality_description) == "MONTESCUDO" ~ 99,
      toupper(.data$Municipality_description) == "CAMPIGLIA CERVO" ~ 96,
      toupper(.data$Municipality_description) == "CAVACURTA" ~ 98,
      toupper(.data$Municipality_description) == "CASTIGLIONE D'INTELVI" ~ 13,
      toupper(.data$Municipality_description) == "PERGINE VALDARNO" ~ 51,
      toupper(.data$Municipality_description) == "VILLA POMA" ~ 20,
      toupper(.data$Municipality_description) == "CASSANO SPINOLA" ~ 6,
      toupper(.data$Municipality_description) == "ALLUVIONI CAMBIO'" ~ 6,
      toupper(.data$Municipality_description) == "LATERINA" ~ 51,
      toupper(.data$Municipality_description) == "REVERE" ~ 20,
      toupper(.data$Municipality_description) == "VESTRENO" ~ 97,
      toupper(.data$Municipality_description) == "RIO NELL'ELBA" ~ 49 ,
      toupper(.data$Municipality_description) == "SAN FEDELE INTELVI" ~ 13,
      toupper(.data$Municipality_description) == "RIO MARINA" ~ 49,
      toupper(.data$Municipality_description) == "PIEVE DI CORIANO" ~ 20,
      toupper(.data$Municipality_description) == "PECORARA" ~ 33 ,
      toupper(.data$Municipality_description) %in% c("CELLIO", "CELLIO (FINO AL 01/01") ~ 2,
      toupper(.data$Municipality_description) == "NIBBIANO" ~ 33,
      toupper(.data$Municipality_description) %in% c("CORIGLIANO CALABRO", "ROSSANO") &
        any(pattern == year.patternB(2019)) ~ 78,
      toupper(.data$Municipality_description) == "FIUMICELLO VILLA VICENTINA" ~ 30,
      toupper(.data$Municipality_description) %in% c("FIUMICELLO", "VILLA VICENTINA") &
        any(pattern == year.patternB(2019))~ 30,
      toupper(.data$Municipality_description) %in% c("BARBARANO VICENTINO", "MOSSANO") &
        any(pattern == year.patternB(2019))~ 24,
      toupper(.data$Municipality_description) %in% c("MEGLIADINO SAN FIDENZIO", "SANTA MARGHERITA D'ADIGE", "SALETTO") &
        any(pattern == year.patternB(2019))~ 28,
      toupper(.data$Municipality_description) == "TREPPO LIGOSULLO" ~ 30,
      toupper(.data$Municipality_description) == "TREPPO CARNICO" &
        any(pattern == year.patternB(2019))~ 30,
      toupper(.data$Municipality_description) == "CASSANO SPINOLA" ~ 6,
      TRUE ~ .data$Province_code
    )))
  }
  if("Municipality_code" %in% names(data)){
    data <- data %>% dplyr::mutate(dplyr::across(.data$Municipality_code, ~ dplyr::case_when(
      toupper(.data$Municipality_description) == "S. OMOBONO TERME" ~ "016252",
      toupper(.data$Municipality_description) == "PORRETTA TERME" ~ "037062",
      toupper(.data$Municipality_description) == "CORTEOLONA" ~ "018046",
      toupper(.data$Municipality_description) == "ZIBELLO" ~ "034048",
      toupper(.data$Municipality_description) == "MACCAGNO" ~ "012042",
      toupper(.data$Municipality_description) %in% c("VIRGILIO", "BORGOFORTE") ~ "020071",
      toupper(.data$Municipality_description) == "BREMBILLA" ~ "016253",
      toupper(.data$Municipality_description) == "RAMISETO" ~ "035031",
      toupper(.data$Municipality_description) == "BUSANA" ~ "035007",
      toupper(.data$Municipality_description) == "OSSUCCIO" ~ "013257",
      toupper(.data$Municipality_description) == "TREMEZZO" ~ "013252",
      toupper(.data$Municipality_description) == "BELLAGIO" ~ "013250",
      toupper(.data$Municipality_description) == "LESSONA" ~ "096085",
      toupper(.data$Municipality_description) == "MONTESCUDO" ~ "099029",
      toupper(.data$Municipality_description) == "CAMPIGLIA CERVO" ~ "096086",
      toupper(.data$Municipality_description) == "CAVACURTA" ~ "098016",
      toupper(.data$Municipality_description) == "CASTIGLIONE D'INTELVI" ~ "013060",
      toupper(.data$Municipality_description) %in% c("LATERINA", "PERGINE VALDARNO") &
        !any(pattern %in% year.patternB(2016)) ~ "051042",
      toupper(.data$Municipality_description) == "VILLA POMA" ~ "020067",
      toupper(.data$Municipality_description) == "CASSANO SPINOLA" ~ "006191",
      toupper(.data$Municipality_description) == "ALLUVIONI CAMBIO'" ~ "006006",
      toupper(.data$Municipality_description) == "REVERE" ~ "020049",
      toupper(.data$Municipality_description) == "VESTRENO" ~ "097089",
      toupper(.data$Municipality_description) == "RIO NELL'ELBA" ~ "04916",
      toupper(.data$Municipality_description) == "SAN FEDELE INTELVI" ~ "013205",
      toupper(.data$Municipality_description) == "RIO MARINA" ~ "049015",
      toupper(.data$Municipality_description) == "PIEVE DI CORIANO" ~ "020040",
      toupper(.data$Municipality_description) == "PECORARA" ~ "033031",
      toupper(.data$Municipality_description) %in% c("CELLIO", "CELLIO (FINO AL 01/01") ~ "002038",
      toupper(.data$Municipality_description) == "NIBBIANO" ~ "033029",
      toupper(.data$Municipality_description) %in% c("CORIGLIANO CALABRO", "ROSSANO") &
        any(pattern == year.patternB(2019)) ~ "078157",
      toupper(.data$Municipality_description) == "FIUMICELLO VILLA VICENTINA" ~ "030190",
      toupper(.data$Municipality_description) %in% c("FIUMICELLO", "VILLA VICENTINA") &
        any(pattern == year.patternB(2019))~ "030191",
      toupper(.data$Municipality_description) %in% c("BARBARANO VICENTINO", "MOSSANO") &
        any(pattern == year.patternB(2019))~ "024124",
      toupper(.data$Municipality_description) %in% c("MEGLIADINO SAN FIDENZIO", "SANTA MARGHERITA D'ADIGE", "SALETTO") &
        any(pattern == year.patternB(2019))~ "028107",
      toupper(.data$Municipality_description) == "TREPPO CARNICO" &
        any(pattern == year.patternB(2019))~ "030191",
      toupper(.data$Municipality_description) == "TREPPO LIGOSULLO" ~ "030191",
      TRUE ~ .data$Municipality_code
    )))
  }
  return(data)
}

#' Build up a comprehensive database regarding the school system
#'
#' @description This function generates a unique dataframe of the school system data including a customary choice of available datasets. This function allows the user to aggregate the desired datasets, when available, among these:
#' \itemize{
#'   \item Invalsi census survey
#'   \item School buildings
#'   \item Number of students and school classes
#'   \item Number of teachers
#'   \item Broadband connection availability
#' }
#'
#'
#' To save as much time as possible it is possible to plug in ready-made input data; otherwise they will be downloaded automatically but not saved in the global environment
#' When a new dataset is joined to the existing ones, it is possible that some observations in this datasets are missing. In this case, by default, the choice of keeping as much observational units as possible, or to remove units with missing variables is left to the user.
#'
#'
#' @param Year Numeric or Character. The relevant school year. Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023}.
#' Important: if input datasets are plugged in, please select the same \code{Year} argument used to download the input data. \code{2023} by default.
#' @param level Character. The administrative level of detail at which data must be aggregated.
#' Either \code{"LAU"}/\code{"Municipality"} or \code{"NUTS-3"}/\code{"Province"}. \code{"LAU"} by default.
#' @param conservative Logical. If \code{FALSE}, only the schools included in all the datasets are taken as input. \code{TRUE} by default.
#' @param Invalsi Logical. Whether the Invalsi census data must be included (see \code{\link{Get_Invalsi_IS}}. \code{TRUE} by default.
#' @param SchoolBuildings Logical. Whether the school buildings dataset must be included (see \code{link{Get_DB_MIUR}}, \code{\link{Util_DB_MIUR_num}}. \code{TRUE} by default.
#' @param nstud Logical. Whether the students number per class must be included (see \code{\link{Get_nstud}}. \code{TRUE} by default.
#' @param nteachers Logical. Whether the number of teachers by province must be included (see \code{link{Get_nteachers_prov}}). \code{TRUE} by default.
#' @param BroadBand Logical. Whether the broadband availability in schools must be included (see \code{\link{Get_BroadBand}}). \code{TRUE} by default
#' @param InnerAreas Logical. Whether the percentage of schools belonging to inner/internal areas must be included (see \code{\link{Get_InnerAreas}}). TRUE by default.
#' @param ord_InnerAreas Logical. If \code{check == TRUE} and \code{InnerAreas == TRUE}, whether the Inner areas classification should be treated as an ordinal variable rather than as a categorical one (see \code{\link{Get_InnerAreas}} for the classification). \code{FALSE} by default.
#' @param Invalsi_subj Character. If \code{Invalsi == TRUE}, the school subject(s) to include, among \code{"Englis_listening"}/\code{"ELI"}, \code{"English_reading"}/\code{"ERE"}, \code{"Italian"}/\code{"Ita"} and \code{"Mathematics"}/\code{"MAT"}. All four by default.
#' @param Invalsi_grade Numeric. If \code{Invalsi == TRUE}, the educational grade to choose. Either \code{2} (2nd year of primary school), \code{5} (last year of primary school), \code{8} (last year of middle shcool), \code{10} (2nd year of high school) or \code{13} (last year of school). All by default.
#' @param Invalsi_WLE Logical. Whether to express Invalsi scores as averagev WLE score rather that the percentage of sufficient tests, if both are Invalsi_grade is either or \code{2} \code{5}. \code{FALSE} by default
#' @param SchoolBuildings_certifications Logical. If the school buldings database has to be downloaded, whether to include safety certifications.  Only relevant from schol year 2020/21 onwards (see \code{\link{Get_DB_MIUR}}). \code{FALSE} by default
#' @param SchoolBuildings_include_numerics Logical. Whether to include strictly numeric variables alongside with Boolean ones in the school buildings database (see \code{\link{Util_DB_MIUR_num}}). \code{TRUE} by default.
#' @param SchoolBuildings_include_qualitatives Logical. Whether to include qualitative variables alongside with Boolean ones in the school buildings database (see \code{\link{Util_DB_MIUR_num}}). \code{FALSE} by default.
#' @param SchoolBuildings_row_cutout Logical. Whether to filter out rows including missing fields in the school buildings database (see \code{\link{Util_DB_MIUR_num}}). \code{FALSE} by default.
#' @param SchoolBuildings_col_cut_thresh Numeric. The threshold of missing values allowed for each variable in the school buildings database (see \code{\link{Util_DB_MIUR_num}}).
#' If a variable as a higher number of missing observations, then it is cut out. \code{20.000} by default.
#' Warning: if the option \code{SchoolBuildings_row_cutout} is active, please select a lower threshold (e.g. \code{1000})
#' @param SchoolBuildings_unique_buildings Logical. If school buildings DB is included at the building level,
#' whether to remove records in which the building code is duplicated and all other fields are as well.
#' As rows are combinations of building ID and school ID, if a school is hosted by several buildings, and each field other than
#' \code{School_code} are duplicated, then only one row is retained. \code{TRUE} by default. See \code{\link{Util_DB_MIUR_num}}.
#' @param SchoolBuildings_flag_outliers Logical. Whether to assign NA to outliers in numeric variables; see \code{\link{Util_DB_MIUR_num}} for more details. \code{TRUE} by default.
#' @param SchoolBuildings_count_missing Logical. Whether the function should return the percentage of NAs in the input school buildings database (see also \code{\link{Group_DB_MIUR}}). \code{FALSE} by default.
#' @param nstud_missing_to_1 Numeric. If \code{nstud == TRUE}, whether the number of classes should be imputed to 1 when it is missing and the number of students is below a threshold (argument \code{nstud_imputation_thresh}, see \code{\link{Util_nstud_wide}}). \code{FALSE} by default.
#' @param nstud_imputation_thresh Numeric. If \code{nstud_missing_to_1 == TRUE}, the minimum threshold below which the number of classes is imputed to 1 if missing;
#' see also \code{\link{Util_nstud_wide}}. \code{19} by default.
#' @param UB_nstud_byclass Numeric. Either a unique value for all school orders, or a vector of three order-specific values in the order: primary, middle, high.
#' If focus is on class size, the upper limit of the acceptable school-level (if \code{filter_by_grade == FALSE}) or grade-level (otherwise) average of the number of students by class.
#' If a whole school or any grade in a school respectively has a higher number of students by class, the record is considered an outlier and filtered out. \code{99} by default, i.e. no restriction is made.
#'  Please notice that boundaries are included in the acceptance interval.
#' @param LB_nstud_byclass Numeric. Either a unique value for all school orders, or a vector of three order-specific values in the order: primary, middle, wide.
#' If focus is on class size, the lower limit of the acceptable school-level (if \code{filter_by_grade == FALSE}) or grade_level (otherwise) average of the number of students by class.
#'  If a whole school or any grade in a school respectively has a smaller number of students by class, the record is considered an outlier and filtered out. \code{1} by default.
#'  Please notice that boundaries are included in the acceptance interval.
#' @param nstud_filter_by_grade Logical. If focus is on class size, whether to remove all school grades with average class size outside of the acceptance boundaries. \code{FALSE} by default.
#' @param UB_nstud_byclass_grade Numeric. IF \code{filter_by_grade == TRUE}, the upper limit of the acceptable grade-level average class size.
#' If \code{NULL} it is set equal to \code{UB_nstud_byclass}. \code{NULL} by default.
#' @param LB_nstud_byclass_grade Numeric. IF \code{filter_by_grade == TRUE}, the lowrer limit of the acceptable grade-level average class size.
#'  If \code{NULL} it is set equal to \code{LB_nstud_byclass}. \code{NULL} by default.
#' @param nstud_check Logical. If \code{nstud == TRUE}, whether to check the students number availability across all school included in the school registries (see \code{\link{Util_Check_nstud_availability}}). \code{TRUE} by default.
#' @param nstud_check_registry Character. If \code{nstud == TRUE} and \code{nstud_check == TRUE}, the school registries whose availability has to be checked. Either \code{"Registry_from_buildings"} (buildings registry), \code{"Registry_from_registry"} (proper registry), \code{"Any"} or \code{"Both"}. \code{"Any"} by default.
#' @param BroadBand_impute_missing Whether the schools not included in the Broadband dataset must be considered in the total of schools (i.e. the denominator to the Broadband availability indicator). \code{TRUE} by default.
#' @param Date Character or Date. The threshold date to broadband activation to consider it activated for a school, i.e. the date before which the works of broadband activation must be finished in order to consider a school as provided with the broadband. By default, September 1st at the beginning of the school year.
#' @param NA_autoRM Logical. Either \code{TRUE}, \code{FALSE} or \code{NULL}. If \code{TRUE}, the values missing in a single dataset are automatically deleted from the final DB. If \code{FALSE}, the missing observations are kept automatically. If \code{NULL}, the choice is left to the user by an interactive menu. \code{NULL} by default.
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param autoAbort Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param show_col_types Logical. If \code{TRUE}, if the \code{verbose} argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.
#' @param input_Invalsi_IS Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#' If \code{INVALSI == TRUE}, the raw Invalsi survey data, obtained as output of the \code{\link{Get_Invalsi_IS}} function.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default
#' @param input_Registry Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#' The school registry corresponding to the year in scope, obtained as output of the function \code{\link{Get_Registry}}.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default
#' @param input_SchoolBuildings Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. If \code{SchoolBuildings == TRUE}, the raw school buildings dataset obtained as output of the function \code{\link{Get_DB_MIUR}}.
#' If \code{NULL}, it will be downloaded automatically but not saved in the global environment. \code{NULL} by default.
#' @param input_nstud Object of class \code{list}, including two objects of class\code{tbl_df}, \code{tbl} and \code{data.frame}.
#' If \code{nstud == TRUE}, the students and classes counts, obtained as output of the function \code{\link{Get_nstud}} with default \code{filename} parameter.
#' If \code{NULL}, the function will download it automatically but it will not be saved in the global environment. \code{NULL} by default.
#' @param input_School2mun Object of class \code{list} with elements of class \code{tbl_df}, \code{tbl} and \code{data.frame}
#' If \code{nstud == TRUE}, the mapping from school codes to municipality (and province) codes. Needed only if \code{check == TRUE}, obtained as output of the function \code{\link{Get_School2mun}}.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.
#' @param input_AdmUnNames Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_AdmUnNames}}
#' If necessary,the ISTAT file including all the codes and the names of the administrative units for the year in scope. Required either if \code{nstud == TRUE & nstud_check == TRUE}, or if \code{SchoolBuildings == TRUE}, \code{input_DB_MIUR} is not provided, and the school year is one of 2015/16, 2017/18 or 1018/19
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.
#' @param input_InnerAreas Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#' If \code{InnerAreas == TRUE}, the classification of peripheral municipalities, obtained as output of the function \code{\link{Get_InnerAreas}}
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default
#' @param input_nteachers Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. If \code{nteachers == TRUE}, the number of teachers by province, obtained as output of the function \code{\link{Get_nteachers_prov}}. If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default
#' @param input_teachers4student Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. If \code{nteachers == TRUE} and \code{nstud = TRUE}, the number of teachers for studets by province. Please notice that
#' this object cannot be considered a substitute for the number of students by class since it provides no information on the number of schools in single educational grades but only at the school order level.
#' Obtained as output of the function \code{\link{Group_teachers4stud}}.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default.
#' @param input_BroadBand Object of classs \code{tbl_df}, \code{tbl} and \code{data.frame}. If BroadBand == TRUE, the raw Broadband connection dataset obtaned as output of the function \code{\link{Get_BroadBand}}
#' If \code{NULL}, it will be downloaded automatically but not saved in the global environment. \code{NULL} by default.
#'
#' @seealso \code{\link{Util_DB_MIUR_num}}, \code{\link{Group_DB_MIUR}}, \code{\link{Group_nstud}}, \code{\link{Util_Check_nstud_availability}}, \code{\link{Get_School2mun}}
#' for similar arguments.
#'
#' @return An object of  class \code{tbl_df}, \code{tbl} and \code{data.frame}
#'
#'
#'
#'
#' @examples
#'
#'
#'
#' DB23_prov <- Set_DB(Year = 2023, level = "NUTS-3",Invalsi_grade = c(5, 8, 13),
#'       Invalsi_subj = "Italian",nteachers = FALSE, BroadBand = FALSE,
#'       SchoolBuildings_count_missing = FALSE,NA_autoRM= TRUE,
#'       input_SchoolBuildings = example_input_DB23_MIUR[, -c(11:18, 10:27)],
#'       input_Invalsi_IS = example_Invalsi23_prov,
#'       input_nstud = example_input_nstud23,
#'       input_InnerAreas = example_InnerAreas,
#'       input_School2mun = example_School2mun23,
#'       input_AdmUnNames = example_AdmUnNames20220630)
#'
#'
#' DB23_prov
#'
#' summary(DB23_prov[, -c(22:62)])
#'
#'
#'
#'
#'
#' @export
Set_DB <- function( Year = 2023,
                    level = "LAU",
                    conservative = TRUE,
                    Invalsi = TRUE,
                    SchoolBuildings = TRUE,
                    nstud = TRUE,
                    nteachers = TRUE,
                    BroadBand = TRUE,
                    verbose = TRUE,
                    show_col_types = FALSE,
                    Invalsi_subj = c("ELI", "ERE", "ITA", "MAT"),
                    Invalsi_grade = c(2,5,8,10,13),
                    Invalsi_WLE = FALSE,
                    SchoolBuildings_certifications = FALSE,
                    SchoolBuildings_include_numerics = TRUE,
                    SchoolBuildings_include_qualitatives = FALSE,
                    SchoolBuildings_row_cutout = FALSE,
                    SchoolBuildings_unique_buildings = TRUE,
                    SchoolBuildings_col_cut_thresh = 2e+4,
                    SchoolBuildings_flag_outliers = TRUE,
                    SchoolBuildings_count_missing = FALSE,
                    nstud_imputation_thresh = 19,
                    nstud_missing_to_1 = FALSE,
                    UB_nstud_byclass = 99,
                    LB_nstud_byclass = 1,
                    UB_nstud_byclass_grade = NULL,
                    LB_nstud_byclass_grade = NULL,
                    nstud_filter_by_grade = FALSE,
                    InnerAreas = TRUE,
                    ord_InnerAreas = FALSE,
                    nstud_check = TRUE,
                    nstud_check_registry = "Any",
                    BroadBand_impute_missing = TRUE,
                    Date = as.Date(paste0(substr(year.patternA(Year),1,4), "-09-01")),
                    NA_autoRM = NULL,
                    input_Invalsi_IS = NULL,
                    input_Registry = NULL,
                    input_SchoolBuildings = NULL,
                    input_nstud = NULL,
                    input_School2mun = NULL,
                    input_AdmUnNames = NULL,
                    input_InnerAreas = NULL,
                    input_teachers4student = NULL,
                    input_nteachers = NULL,
                    input_BroadBand = NULL,
                    autoAbort = FALSE){

  start.zero <- Sys.time()
  YearMinus1 <- as.numeric(substr(year.patternA(Year),1,4))

  datasets <- list()

  if(BroadBand || is.null(input_School2mun) ||
     (SchoolBuildings && is.null(input_SchoolBuildings)) ||
     (nstud && is.null(input_nstud))){
    while(is.null(input_Registry)){
      input_Registry <- Get_Registry(Year = Year, show_col_types = show_col_types, autoAbort = autoAbort)
      if(is.null(input_Registry)){
        if(!autoAbort){
          holdOn <- ""
          message("Error during schools registry retrieving. Would you abort the whole operation or retry? \n",
                  "    - To abort the operation, press `A` \n",
                  "    - To retry data retrieving, press any other key \n")
          holdOn <- readline(prompt = "    ")
          if(toupper(holdOn) == "A"){
            cat("You chose to abort the operation \n")
            return(NULL)
          } else {
            cat("You chose to retry \n")
          }
        } else return(NULL)
      }
    }
  }

  if(SchoolBuildings || nstud || BroadBand){
    while(is.null(input_AdmUnNames)){
      AdmUnYear <- ifelse(any(year.patternA(Year) %in% c( year.patternA(2016), year.patternA(2018))), YearMinus1+1, YearMinus1)
      AdmUnDate <- ifelse(any(year.patternA(Year) %in%c(year.patternA(2016), year.patternA(2018))), "01-01", "09-01")
      input_AdmUnNames <- Get_AdmUnNames( Date = paste0(AdmUnYear, "-", AdmUnDate), autoAbort = autoAbort)
      if(is.null(input_AdmUnNames)){
        if(!autoAbort){
          holdOn <- ""
          message("Error during administrative codes retrieving. Would you abort the whole operation or retry? \n",
                  "    - To abort the operation, press `A` \n",
                  "    - To retry data retrieving, press any other key \n")
          holdOn <- readline(prompt = "    ")
          if(toupper(holdOn) == "A"){
            cat("You chose to abort the operation \n")
            return(NULL)
          } else {
            cat("You chose to retry \n")
          }
        } else return(NULL)
      }
    }
  }

  while(is.null(input_School2mun)){
    input_School2mun <- Get_School2mun(
      Year = Year, verbose = verbose, show_col_types = show_col_types,
      input_Registry = input_Registry, input_AdmUnNames = input_AdmUnNames,
      autoAbort = autoAbort)
    if(is.null(input_School2mun)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during mapping schools to municipalities. Would you abort the whole operation or retry? \n",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  while(Invalsi && is.null(input_Invalsi_IS)){
    input_Invalsi_IS <- Get_Invalsi_IS(level = level, verbose = verbose,
                                       show_col_types = show_col_types,
                                       multiple_out = FALSE, autoAbort = autoAbort)
    if(is.null(input_Invalsi_IS)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during Invalsi data retrieving. Would you abort this element or retry? \n",
                "    - To abort the element, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          Invalsi <- FALSE
        } else {
          cat("You chose to retry \n")
        }
      } else Invalsi <- FALSE
    }
  }

  while(SchoolBuildings && is.null(input_SchoolBuildings)){
    input_SchoolBuildings <-
      Get_DB_MIUR(Year = Year, verbose = verbose, show_col_types = show_col_types,
                  input_Registry = input_Registry, input_AdmUnNames = input_AdmUnNames,
                  certifications = SchoolBuildings_certifications, autoAbort = autoAbort)

    if(is.null(input_SchoolBuildings)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during school buildings DB retrieving. Would you abort this element or retry? \n",
                "    - To abort the element, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          SchoolBuildings <- FALSE
        } else {
          cat("You chose to retry \n")
        }
      } else SchoolBuildings <- FALSE
    }
    if(verbose) cat("\n")
  }

  while(InnerAreas && is.null(input_InnerAreas)){
    input_InnerAreas <- Get_InnerAreas(autoAbort)
    if(is.null(input_InnerAreas)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during inner areas classification retrieving. Would you abort this element or retry? \n",
                "    - To abort the element, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the element \n")
          InnerAreas <- FALSE
        } else {
          cat("You chose to retry \n")
        }
      } else InnerAreas <- FALSE
    }
  }

  while(nstud && is.null(input_nstud)){
    if(nstud_missing_to_1){
      nstud_filename <- c("ALUCORSOETASTA", "ALUCORSOINDCLASTA")
    } else {
      nstud_filename <- "ALUCORSOINDCLASTA"
    }
    input_nstud <- Get_nstud(Year = Year, verbose = verbose, filename = nstud_filename, autoAbort = autoAbort)
    if(is.null(input_nstud)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during students counts retrieving. Would you abort this element or retry? \n",
                "    - To abort the element, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          nstud <- FALSE
        } else {
          cat("You chose to retry \n")
        }
      } else nstud <- FALSE
    }
  }

  while(nteachers && is.null(input_nteachers)){
    input_nteachers <- Get_nteachers_prov(Year = Year, verbose = verbose,
                                          show_col_types = show_col_types, autoAbort = autoAbort)
    if(is.null(input_nteachers)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during teachers counts retrieving. Would you abort this element or retry? \n",
                "    - To abort the element, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the element \n")
          nteachers <- FALSE
        } else {
          cat("You chose to retry \n")
        }
      } else nteachers <- FALSE
    }
  }

  while(BroadBand && is.null(input_BroadBand)){
    input_BroadBand <- Get_BroadBand(verbose = verbose, Date = Date, autoAbort = autoAbort)
    if(is.null(input_BroadBand)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during broadband data retrieving. Would you abort this element or retry? \n",
                "    - To abort the element, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          BroadBand <- FALSE
        } else {
          cat("You chose to retry \n")
        }
      } else BroadBand <- FALSE
    }
  }

  if(verbose) cat("\n")

  if(!is.null(input_SchoolBuildings)){
    byschool_SchoolBuildings <-
      (is.data.frame(input_SchoolBuildings) &&
         "School_code" %in% names(input_SchoolBuildings)) ||
      (!is.data.frame(input_SchoolBuildings) &&
         "School_code" %in% names(input_SchoolBuildings[[1]]))
  }
  if(!is.null(input_nstud)){
    byschool_nstud <- (is.data.frame(input_nstud) &&
                         "School_code" %in% names(input_nstud))||
      "School_code" %in% names(input_nstud[[1L]])
  }

  if(! conservative){
    if(!is.null(input_SchoolBuildings) && byschool_SchoolBuildings){
      if(!is.null(input_nstud) %% byschool_nstud){
        input_nstud_byclass <- input_nstud$ALUCORSOINDCLASTA
        input_nstud_byclass <- filterCommonRows(input_nstud_byclass, input_SchoolBuildings, verbose)
        input_SchoolBuildings <- filterCommonRows(input_SchoolBuildings, input_nstud_byclass, verbose)
      }
      if(!is.null(input_BroadBand)){
        if(!"Municipality_code" %in% names(input_BroadBand)){
          input_BroadBand <- Util_BroadBand2mun(input_BroadBand, autoAbort = autoAbort)
        }
        input_BroadBand <- input_BroadBand %>%
          dplyr::filter(!grepl("[^A-Z]", substr(.data$School_code,1,4)) &
                        !grepl("X", substr(.data$School_code,1,4), ignore.case = TRUE)) %>%
          filterCommonRows(input_SchoolBuildings, verbose)
      }
    } else if(!is.null(input_nstud) && !is.null(input_BroadBand) &&
              byschool_nstud){
      input_BroadBand <- input_BroadBand %>%
        dplyr::filter(!grepl("[^A-Z]", substr(.data$School_code,1,4)) &
                      !grepl("X", substr(.data$School_code,1,4), ignore.case = TRUE)) %>%
        filterCommonRows(input_nstud_byclass, verbose)
    }
    if(!is.null(input_nstud)) input_nstud$ALUCORSOINDCLASTA <- input_nstud_byclass
    if(verbose) cat("\n")
  } else {
    if(!is.null(input_SchoolBuildings) && byschool_SchoolBuildings){
      input_SchoolBuildings <- input_SchoolBuildings %>%
        dplyr::filter(.data$School_code %in% input_School2mun$Registry_from_registry$School_code)
    }
    if(!is.null(input_nstud) && byschool_nstud){
      if(!is.data.frame(input_nstud)){
        input_nstud <- input_nstud %>% lapply(function(x){
          x <- x %>% dplyr::filter(.data$School_code %in% input_School2mun$Registry_from_registry$School_code)})
      } else {
        input_nstud <- input_nstud %>%
          dplyr::filter(.data$School_code %in% input_School2mun$Registry_from_registry$School_code)

      }
    }
    if(!is.null(input_BroadBand)){
      input_BroadBand <- input_BroadBand %>%
        dplyr::filter(!grepl("[^A-Z]", substr(.data$School_code,1,4)) &
                      !grepl("X", substr(.data$School_code,1,4), ignore.case = TRUE)) %>%
        dplyr::filter(.data$School_code %in% input_School2mun$Registry_from_registry$School_code)
    }
  }

  if(!is.null(input_SchoolBuildings)){

    if(byschool_SchoolBuildings){
      if(SchoolBuildings_certifications){
        SchoolBuildings_include_numerics <- TRUE
      }
      if(!is.data.frame(input_SchoolBuildings)){
        if("data" %in% names(input_SchoolBuildings)){
          input_SchoolBuildings <- input_SchoolBuildings$data
        } else input_SchoolBuildings <- input_SchoolBuildings[[1]]
      }

      fields_SchoolBuildings <- lapply(input_SchoolBuildings, is.numeric)

      if(!any(unlist(fields_SchoolBuildings[-which(names(fields_SchoolBuildings) %in% c(
        "Province_code", "Year", "Floors_number", "National_seismic_classification"))]))){
        DB_SchoolBuildings_num <- input_SchoolBuildings %>% Util_DB_MIUR_num(
          include_numerics = SchoolBuildings_include_numerics,
          include_qualitatives = SchoolBuildings_include_qualitatives,
          row_cutout = SchoolBuildings_row_cutout, flag_outliers = SchoolBuildings_flag_outliers,
          track_deleted = FALSE, unique_buildings = SchoolBuildings_unique_buildings,
          col_cut_thresh = SchoolBuildings_col_cut_thresh,
          verbose = verbose)
      } else {
        DB_SchoolBuildings_num <- input_SchoolBuildings
      }
      DB_SchoolBuildings <- DB_SchoolBuildings_num %>%
        Group_DB_MIUR(verbose = verbose, track_deleted = FALSE, Year = Year,
                      count_units = TRUE, countname = "nbuildings",
                      count_missing = SchoolBuildings_count_missing,
                      InnerAreas = InnerAreas, ord_InnerAreas = ord_InnerAreas,
                      input_InnerAreas = input_InnerAreas)
    } else{
      DB_SchoolBuildings <- input_SchoolBuildings
      if(is.data.frame(DB_SchoolBuildings)) SchoolBuildings_count_missing <- FALSE
    }

    if(toupper(level) %in% c("LAU", "MUNICIPALITY", "NUTS-4")){
      if(!SchoolBuildings_count_missing){
        DB_SchoolBuildings <- DB_SchoolBuildings$Municipality_data
      } else {
        DB_SchoolBuildings <- dplyr::left_join(
          DB_SchoolBuildings$Municipality_data,
          DB_SchoolBuildings$Municipality_missing,
          by = c("Municipality_code","Order"))
      }
    } else {
      if(!SchoolBuildings_count_missing){
        DB_SchoolBuildings <- DB_SchoolBuildings$Province_data
      } else{
        DB_SchoolBuildings <- dplyr::left_join(
          DB_SchoolBuildings$Province_data,
          DB_SchoolBuildings$Province_missing,
          by = c("Province_code","Order"))
      }
    }
      DB_SchoolBuildings <- DB_SchoolBuildings %>%
      dplyr::filter(!.data$Order %in% c("IC", "IS", "NR"))
      DB_SchoolBuildings <- DB_SchoolBuildings[, -which(
        names(DB_SchoolBuildings) %in% c("Year", "nbuildings_MP"))]

    datasets[["SchoolBuildings"]] <- DB_SchoolBuildings
  }

  if(!is.null(input_nstud)){
    nstud_InnerAreas <- InnerAreas && length(datasets) == 0
    nstud_ord_InnerAreas <- nstud_InnerAreas && ord_InnerAreas

    if(byschool_nstud){
      nstud_aggr <-
        Group_nstud(data = input_nstud, Year = Year, nstud_imputation_thresh = nstud_imputation_thresh,
                    missing_to_1 = nstud_missing_to_1, UB_nstud_byclass = UB_nstud_byclass,
                    LB_nstud_byclass = LB_nstud_byclass, filter_by_grade = nstud_filter_by_grade,
                    UB_nstud_byclass_grade = UB_nstud_byclass_grade,
                    LB_nstud_byclass_grade = LB_nstud_byclass_grade,
                    check = nstud_check, verbose = verbose,
                    check_registry = nstud_check_registry, InnerAreas = nstud_InnerAreas,
                    ord_InnerAreas = nstud_ord_InnerAreas,
                    input_Registry = input_Registry, input_InnerAreas = input_InnerAreas,
                    input_School2mun = input_School2mun, input_AdmUnNames = input_AdmUnNames)
    } else nstud_aggr <- input_nstud

    if(!is.data.frame(nstud_aggr)){
      if(toupper(level) %in% c("LAU", "MUNICIPALITY", "NUTS-4")){
        datasets[["nstud"]] <- nstud_aggr$Municipality_data
      } else datasets[["nstud"]] <- nstud_aggr$Province_data
    } else datasets[["nstud"]] <- nstud_aggr
  }

  if(!is.null(input_nteachers)){
    if(!is.null(input_nstud)){
      if(is.null(input_teachers4student)) {
        input_teachers4student <- Group_teachers4stud(
          Year = Year, input_nteachers = input_nteachers,
          verbose = verbose,
          input_nstud_aggr = nstud_aggr$Province_data,
          input_Registry = input_Registry,
          input_InnerAreas = input_InnerAreas,
          input_School2mun = input_School2mun)
      }
      datasets[["teachers4student"]] <- input_teachers4student %>%
        dplyr::mutate(Province_code = as.numeric(.data$Province_code))
    } else {
      datasets[["nteachers"]] <- input_nteachers
    }
  }

  if(!is.null(input_BroadBand)){

    if(BroadBand_impute_missing){
      BroadBand_missing <- input_School2mun$Registry_from_registry %>%
        School.order() %>%
        dplyr::filter(!.data$School_code %in% input_BroadBand$School_code) %>%
        dplyr::filter( !.data$Order %in% c("IC", "IS", "NR")) %>%
        dplyr::select(.data$School_code, .data$Order, .data$Province_code, .data$Province_initials,
                      .data$Municipality_code, .data$Municipality_description) %>%
        unique() %>% dplyr::mutate(BB_Activation_status = 0) %>%
        dplyr::left_join(dplyr::select(
          prov.names(), -.data$Province_initials),
          by = "Province_code")
      BroadBand_missing <- BroadBand_missing[,names(
        input_BroadBand[which(names(input_BroadBand) %in% names(BroadBand_missing))])]
      for(i in which(! names(input_BroadBand) %in% names(BroadBand_missing))){
        BroadBand_missing <- cbind(BroadBand_missing, rep(NA, nrow(BroadBand_missing)))
        names(BroadBand_missing)[ncol(BroadBand_missing)] <- names(input_BroadBand)[i]
      }
      BroadBand_missing <- BroadBand_missing %>% dplyr::select(names(input_BroadBand)) %>%
        dplyr::mutate(Province_description = stringr::str_to_title(.data$Province_description),
                      Region_description = stringr::str_to_title(.data$Region_description)) %>%
        structure(class = c("tbl_df", "tbl", "data.frame"))
      input_BroadBand <- rbind(input_BroadBand, BroadBand_missing) %>%
        dplyr::filter(.data$Order %in% c("Primary", "Middle", "High"))
    }

    BB <- Group_BroadBand(
      Date = Date, verbose = verbose, data = input_BroadBand )
    if(level %in% c("LAU", "Municipality")){
      datasets[["BroadBand"]] <- BB$Municipality_data %>%
        dplyr::select( -.data$Region_code, -.data$Region_description, -.data$nschools )
    } else {
      datasets[["BroadBand"]] <- BB$Province_data %>%
        dplyr::select(-.data$Region_code, -.data$Region_description, -.data$nschools)
    }
  }

  if(!is.null(input_Invalsi_IS)){

    if(!is.data.frame(input_Invalsi_IS) && is.list(input_Invalsi_IS)){
      if (toupper(level) %in% c("LAU", "NUTS-4", "MUNICIPALITY", "MUN")){
        input_Invalsi_IS <- input_Invalsi_IS$Municipality_data
      } else{
        input_Invalsi_IS <- input_Invalsi_IS$Province_data
      }
    }

    if(!any(grepl("_\\d+$", names(input_Invalsi_IS)))){
      Invalsi_IS <- Util_Invalsi_filter(
        data = input_Invalsi_IS, Year = Year, subj = Invalsi_subj,
        grade = Invalsi_grade, level = level,
        WLE = Invalsi_WLE, verbose = verbose)
    } else {
      Invalsi_IS <- input_Invalsi_IS
      if("Year" %in% names(Invalsi_IS)) {
        if(length(unique(Invalsi_IS$Year)>1L)){
        Invalsi_IS <- Invalsi_IS %>% dplyr::filter(.data$Year == year.patternA(Year))
      }}
    }

    SchoolOrder <- c(ifelse(any(Invalsi_grade < 6), "Primary", NA),
                     ifelse(8 %in% Invalsi_grade, "Middle", NA),
                     ifelse(any(Invalsi_grade>8), "High", NA))
    SchoolOrder <- SchoolOrder[!is.na(SchoolOrder)]

    Invalsi_primary <- Invalsi_IS  %>%
      dplyr::mutate(Order = NA) %>%
      dplyr::relocate(.data$Order, .after = 2) %>%
      dplyr::filter(!is.na(.data$Order))
    Invalsi_mid <- Invalsi_primary
    Invalsi_high <- Invalsi_primary

    if("Primary" %in% SchoolOrder){
      Invalsi_primary <- Invalsi_IS %>%
        dplyr::mutate(Order = "Primary") %>%
        dplyr::relocate(.data$Order, .after = 2)
      primcols <- grep(c("_2$|_5$"), names(Invalsi_primary))
      Invalsi_primary[, grep(c("_8|_10|_13"), names(Invalsi_primary))] <- NA
      Invalsi_primary <- Invalsi_primary[apply(
        Invalsi_primary[, primcols], 1, function(x) !all(is.na(x))), ]
    }
    if("Middle" %in% SchoolOrder){
      Invalsi_mid <- Invalsi_IS %>%
        dplyr::mutate(Order = "Middle") %>%
        dplyr::relocate(.data$Order, .after = 2)
      midcols <- grep(c("_8$"), names(Invalsi_mid))
      Invalsi_mid[, grep(c("_2$|_5$|_10$|_13$"), names(Invalsi_mid))] <- NA
      Invalsi_mid <- Invalsi_mid[apply(
        Invalsi_mid[, midcols], 1, function(x) !all(is.na(x))), ]
    }
    if("High" %in% SchoolOrder){
      Invalsi_high <- Invalsi_IS %>%
        dplyr::mutate(Order = "High") %>%
        dplyr::relocate(.data$Order, .after = 2)
      highcols <- grep(c("_10$|_13$"), names(Invalsi_high))
      Invalsi_high[, grep(c("_2$|_5$|_8$"), names(Invalsi_high))] <- NA
      Invalsi_high <- Invalsi_high[apply(
        Invalsi_high[, highcols], 1, function(x) !all(is.na(x))), ]
    }

    datasets[["Invalsi_IS"]] <-
      rbind(Invalsi_primary, Invalsi_mid, Invalsi_high)

    endtime <- Sys.time()
    if(verbose){
      cat(difftime(endtime, start.zero, units = "secs"), " seconds needed to import all input data \n \n")
    } #datasets <- datasets %>% lapply(function(x){#  if("Order" %in% colnames(x)) {#    x <- x %>% dplyr::filter(.data$Order %in% SchoolOrder)#    return(x)#  } else return(x)    #})
  }

  init <- input_School2mun$Registry_from_registry %>%
    School.order() %>%
    dplyr::filter(.data$Order %in% c("Primary", "Middle", "High")) %>%
    dplyr::distinct(.data$Province_code, .data$Province_initials,
                    .data$Municipality_code, .data$Municipality_description, .data$Order)
  datasets[["Registry"]] <- init

  datasets <- list(datasets$Registry, datasets$Invalsi_IS, datasets$SchoolBuildings,
                   datasets$nstud, datasets$BroadBand, datasets$teachers4student,
                   datasets$nteachers)
  names(datasets) <- c("Registry", "Invalsi_IS", "SchoolBuildings", "nstud", "BroadBand",
                       "teachers4student", "nteachers")
  datasets <- Filter(Negate(is.null), datasets)

  if(toupper(level) %in% c("LAU", "NUTS-4", "MUN", "MUNICIPALITY")){

    if(!is.null(datasets[["Invalsi_IS"]])){
      datasets[["Invalsi_IS"]] <- datasets[["Invalsi_IS"]] %>%
        dplyr::mutate(Province_code = as.numeric(substr(.data$Municipality_code, 1, 3))) %>%
        dplyr::relocate(.data$Province_code, .before = 1)
    }

    datasets[2:length(datasets)] <- datasets[2:length(datasets)] %>%
      lapply(function(x){
        if(!any(grepl("Teachers", names(x)))){
          x <- x[, which(!names(x) %in% c("Year",
                                          "Municipality_description", "Province_description", "Province_code",
                                          "Province_initials", "nschools",
                                          "Region_description", "Region_code"))]
        } else{
          if(is.null(input_nstud)){
            x <- x[, which(names(x) %in%  c(
              "Province_code", "Order", "Tot_teachers", "Tot_ATA", "Tot_Students", "Students_per_class_Tot",
              "Tot_Classes", "Teachers_per_student", "Teachers_per_class",
              "ATA_per_student", "ATA_per_class"))]
          } else {
            x <- x[, which(names(x) %in%  c(
              "Province_code", "Order", "Tot_teachers", "Teachers_per_student", "Teachers_per_class",
              "Tot_ATA", "ATA_per_student", "ATA_per_class"))]
          }
        }
      })

    res <- datasets[[1]]
    remaining <- list()
    for(i in (2:length(datasets))){
      ncol.old <- ncol(res)
      if("Municipality_code" %in% names(datasets[[i]])){
        if("Order" %in% names(datasets[[i]])){
          res <- res %>%
            dplyr::left_join(datasets[[i]],by = c("Municipality_code", "Order"))
          notfound <- which(apply(res[-c(1:ncol.old)], MARGIN = 1, function(x) all(is.na(x))))
          if(length(notfound)>0){
            if(is.null(NA_autoRM)){
              cat(paste(length(notfound)), " units in ",
                  names(datasets[i-1]), ifelse(i>2, " and in previous datasets", ""),
                  " are missing in ", names(datasets[i]), "\n")
            }
            res <- res %>% NA.join.manage(lastcol = ncol.old, NA_autoRM = NA_autoRM, verbose = verbose)
          }
        } else {
          remaining[[length(remaining)+1]] <- datasets[[i]]
          names(remaining)[[length(remaining)]] <- names(datasets[[i]])
        }
      } else if("Province_code" %in% names(datasets[[i]])){
        if("Order" %in% names(res) && !all(is.na(datasets[[i]]$Order))){
          res <- res %>% dplyr::left_join(datasets[[i]], by = c("Province_code", "Order"))
        } else {
          # WARNING: this is specific to nteachers - not robust to different data
          datasets[[i]] <- datasets[[i]][,which(
            names(datasets[[i]]) %in% c("Province_code", "Tot_Students", "Tot_Classes",
                                        "Tot_teachers", "Tot_ATA"))]
          startcol <- 1
          endcol <- ncol(datasets[[i]]) - 1
          datasets[[i]] <- datasets[[i]] %>%
            dplyr::group_by(.data$Province_code) %>%
            dplyr::summarise(dplyr::across(c(startcol:endcol), list(sum))) %>%
            dplyr::ungroup()
          names(datasets[[i]]) <- stringr::str_remove(names(datasets[[i]]), "_1")
          if("Tot_Students" %in% names(datasets[[i]])){
            if("Tot_teachers" %in% names(datasets[[i]])){
              datasets[[i]] <- datasets[[i]] %>%
                dplyr::mutate(Teachers_per_student = .data$Tot_teachers/.data$Tot_Students)
            }
            if("Tot_ATA" %in% names(datasets[[i]])){
              datasets[[i]] <- datasets[[i]] %>%
                dplyr::mutate(ATA_per_student = .data$Tot_ATA/.data$Tot_Students)
            }

            if("Tot_Classes" %in% names(datasets[[i]])){
              datasets[[i]] <- datasets[[i]] %>%
                dplyr::mutate(Students_per_class_Tot = .data$Tot_Students/.data$Tot_Classes)
            }
          }
          if("Tot_Classes" %in% names(datasets[[i]])){
            if("Tot_teachers" %in% names(datasets[[i]])){
              datasets[[i]] <- datasets[[i]] %>%
                dplyr::mutate(Teachers_per_class = .data$Tot_teachers/.data$Tot_Classes)
            }
            if("Tot_ATA" %in% names(datasets[[i]])){
              datasets[[i]] <- datasets[[i]] %>%
                dplyr::mutate(Teachers_per_class = .data$Tot_teachers/.data$Tot_Classes)

            }
          }
          res <- res %>% dplyr::left_join(datasets[[i]], by = "Province_code")
        }
        if(sum(is.na(res[,ncol.old+1]))>0){
          if(is.null(NA_autoRM)){
            cat(paste(sum(is.na(res[,ncol.old+1]))), " units in ",
                names(datasets[i-1]), ifelse(i>2, " and in previous datasets", ""),
                " are missing in ", names(datasets[i]), "\n")
          }
          res <- res %>% NA.join.manage(lastcol = ncol.old, NA_autoRM = NA_autoRM, verbose = verbose)
        }
      }
    }
    if(length(remaining) != 0){
      for(i in 1:length(remaining)){
        ncol.old <- ncol(res)
        res <- res %>% dplyr::left_join(remaining[[i]], by = "Municipality_code")
        if(sum(is.na(res[,ncol.old+1]))>0){
          if(is.null(NA_autoRM)){
            message(paste(sum(is.na(res[,ncol.old+1]))), " units in previous datasets",
                    " are missing in ", names(remaining)[i], "\n")
          }
          NA.join.manage(res, ncol.old, NA_autoRM = NA_autoRM, verbose = verbose)
        }
      }
    }
  } else {
    datasets[["Registry"]] <- datasets[["Registry"]] %>%
      dplyr::distinct(.data$Province_code, .data$Province_initials, .data$Order)

    datasets[2:length(datasets)] <- datasets[2:length(datasets)] %>%
      lapply(function(x){
        x <- x[,which(!names(x) %in% c(
          "Year","Municipality_description", "Province_description",
          "Municipality_code","Province_initials", "nschools",
          "Region_description", "Region_code"))] })
    res <- datasets[[1]]
    #remaining <- list() #CURRENTLY not needed. Currently.
    for(i in (2:length(datasets))){
      ncol.old <- ncol(res)
      if("Order" %in% names(datasets[[i]])){
        res <- res %>%  dplyr::left_join(datasets[[i]], by = c("Province_code", "Order"))
      } else {
        res <- res %>%  dplyr::left_join(datasets[[i]], by = "Province_code")
      }
      notfound <- which(apply(res[-c(1:ncol.old)], MARGIN = 1, function(x) all(is.na(x))))
      if(length(notfound)>0){
        if(is.null(NA_autoRM)){
          cat(paste(length(notfound)), " units in ",
              names(datasets[i-1]), ifelse(i>2, " and in previous datasets", ""),
              " are missing in ", names(datasets[i]), "\n")
        }
        res <- res %>% NA.join.manage(lastcol = ncol.old, NA_autoRM = NA_autoRM, verbose = verbose)
      }
    }
  }

  names(res) <- names(res) %>%  stringr::str_remove_all("\\.y") %>% stringr::str_remove_all("\\.x")
  res <- res[,!duplicated(colnames(res))]

  endtime <- Sys.time()
  if(verbose){
    cat("Total time needed to build the database: ",
        difftime(endtime, start.zero, units = "secs"), " seconds \n \n")
  }

  return(res)
}

#' @keywords internal
#'
cutout.default <- function(data, startcol=10, pattern.out = c(),
                           init.cutout = c()){
  cutout <- init.cutout
  for (j in (startcol:ncol(data))){
    if (nrow( unique(data[which(! data[,j][[1]] %in% pattern.out),j]) ) > 2){
      cutout <- c(cutout, names(data)[j])
    }
  }
  return(cutout)
}
#

#' Subset of Italian provinces shapefile
#'
#'
#' This is the shapefile for the provinces belonging to four regions: Molise, Campania, Apulia and Basilicata,
#' as of January 1st 2022. These are the latest administrative units boundaries relevant at the beginning of the school year 2022/23.
#' The whole shapefile can be retrieved with the command \code{Get_Shapefile(Year = 2022, level = "NUTS-3")}
#' @seealso \code{\link{Get_Shapefile}}
#'
#' @format ## `example_Prov22_shp`
#' A Spatial polygon data frame with 13 rows/polygons and 15 columns:
#' \itemize{
#'   \item \code{COD_RIP} Numeric; the code for the macroarea (1 for Northwest, 2 for Northeast, 3 for Center, 4 for South and 5 for Isles)
#'   \item \code{COD_REG} Numeric; the region (NUTS-2 administrative level) ID
#'   \item \code{COD_PROV} Numeric; the NUTS-3 administrative code
#'   \item \code{COD_CM} Numeric; the administrative code for Metropolitan Cities (which are always at the NUTS-3 level), obtained as 200 + NUTS-3 code, if the unit is a Metropolitan city; 0 otherwise.
#'   \item \code{COD_UTS} Numeric; the administrative code for Metropolitan cities if the unit is a Metropolitan City; the province code otherwise.
#'   \item \code{DEN_PROV} Character; the province (NUTS-3 administrative level) name, if the unit is not a Metropolitan City; blank otherwise.
#'   \item \code{DEN_CM} Character; the Metropolitan City (NUTS-3 administrative level) name, if the unit is a Metropolitan City; blank otherwise.
#'   \item \code{DEN_UTS} Character; the province or Metropolitan City (NUTS-3 administrative level) name.
#'   \item \code{SIGLA} Character; abbreviated NUTS-3 denomination.
#'   \item \code{TIPO_UTS} Character; the NUTS-3 type of the unit; either "Provincia" (Province) or "Citta metropolitana" (Metropolitan City)
#'   \item \code{Shape_Leng} Numeric; the polygon perimeter.
#'   \item \code{Shape_Area} Numeric; the polygon area.
#'   \item \code{geometry} the polygon geometry.
#'
#' }
#' @source <https://www.istat.it/it/archivio/222527>
"example_Prov22_shp"

#' Download the names and codes of Italian LAU and NUTS-3 administrative units
#'
#' @description This function downloads a file provided by the Italian National Institute of Statistics including all the codes of administrative units in Italy. As of today, it is the easiest way to map directly cadastral codes to municipality codes.
#'
#' @param Date Character. The date at which administrative unit codes are sought for. Important: must be in the format: "yyyy-mm-dd".
#'  Current date by default.
#' @param autoAbort Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#'
#' @return An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, including: NUTS-3 code, NUTS-3 abbreviation,
#' LAU code, LAU name (description) and cadastral code. All variables are characters except for the NUTS-3 code.
#'
#' @examples
#'  \donttest{
#'   AdmUnNames25 <- Get_AdmUnNames("2025-01-01", autoAbort = TRUE)
#'   # Codes for a randomly chosen province, say, Brindisi (Apulia)
#'   AdmUnNames25[which(AdmUnNames25$Province_code==74),]
#'  }
#'
#'
#'
#'
#' @source <https://situas.istat.it/web/#/territorio>
#'
#' @export



Get_AdmUnNames <- function(Date = Sys.Date(), autoAbort = FALSE){


  if(!Check_connection(autoAbort = autoAbort)) return(NULL)

  pattern0 <- c("https://situas.istat.it/ShibO2Module/api/Report/Spool/", "/61?&pdoctype=CSV")
  #date <- stringr::str_replace(date, "_", "-")
  #while(!date %in% c("01-01", "06-30", "09-01")) {
  # message("Please, choose either '01-01', '06-30' or '09-01' as date")
  #date <- readline(prompt = "  > ")
  #}
  while(!as.Date(Date) <= Sys.Date()){
    message("Please, choose a date prior to ", Sys.Date())
    Date <- readline(prompt = "  > ")
  }
  #Year <- 2000 + as.numeric(year.patternA(Year))%%100
  url<- paste0(pattern0[1], Date, pattern0[2])
  json_body <- "{}"

  status <- 0
  attempt <- 0
  while(status != 200 && attempt <= 10){

    response <- tryCatch({httr::POST(url,
                                     body = json_body,
                                     encode = "raw",
                                     httr::add_headers(
                                       `Content-Type` = "application/json-patch+json",
                                       `User-Agent` = "R (httr package)",
                                       `Origin` = "https://situas.istat.it",
                                       `Referer` = "https://situas.istat.it/web/"))
    }, error = function(e) return(NULL))

    status <- response$status_code
    if(is.null(response)) status <- 0
    if(status != 200){
      attempt <- attempt + 1
      message("Operation exited with status: ", status, "; operation repeated (",
              10 - attempt, " attempts left)")
    }
    if(attempt >= 10) {
      message("Maximum attempts reached. Abort. We apologise for the inconvenience")
      return(NULL)
    }
  }

  DB <- NULL
  attempt <- 0
  while(is.null(DB) && attempt <= 10){
    DB <- tryCatch({
      readr::read_delim(rawToChar(httr::content(response, type = "raw")), delim = ";", show_col_types = FALSE)
    }, error = function(e){
      message("Cannot read the administrative unit names file; ", 10 - attempt,
              " attempts left. If the problem persists, please contact the maintainer.\n")
      return(NULL)
    })
    attempt <- attempt + 1
  }
  if(is.null(DB)) return(NULL)

  res <- DB %>%
    dplyr::select(.data$`Codice Provincia (Storico)`, .data$`Sigla automobilistica`,
                  .data$`Codice Comune (alfanumerico)`, .data$`Comune`,
                  .data$`Codice catasto`)
  names(res) <- c("Province_code", "Province_initials","Municipality_code",
                  "Municipality_description", "Cadastral_code")

  res$Province_code <- as.numeric(res$Province_code)
  res$Province_initials[which(res$Province_code == 63)] <- "NA"

  return(res)
}

#' Display a map of Invalsi scores
#'
#' @description This function displays either a static or interactive map of the Invalsi scores, either at the municipality or province level.
#' It supports two kinds of map:
#' \itemize{
#'   \item Interactive map (default option), which allows the user to visualize all the data in scope through the interactive popup, and
#'   \item Static map (ggplot), which can be easily exported in \code{.pdf} objects.
#' }
#'
#' @param data Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#' The raw Invalsi survey data that has to be filtered, obtained as output of the \code{\link{Get_Invalsi_IS}} function.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default
#'
#' @param Year Numeric or character value. Reference school year for the data (last available is 2022/23).
#' Available in the formats: \code{2022}, \code{"2021/2022"}, \code{202122}, \code{20212022}. \code{2022} by default
#' @param subj_toplot Character. The school subject to display in the map,
#'  The school subject to include, one among:
#'  \code{"Englis_listening"}/\code{"ELI"}, \code{"English_reading"}/\code{"ERE"}, \code{"Italian"}/\code{"ITA"} and \code{"Mathematics"}/\code{"MAT"}. \code{"ITA"} (Italian) by default.
#' @param grade Numeric. The school grade to chose. Either \code{2} (2nd year of primary school), \code{5} (last year of primary school), \code{8} (last year of middle shcool), \code{10} (2nd year of high school) or \code{13} (last year of school). \code{8} by default
#' @param level Character. The level of aggregation of Invalsi census data. Either \code{"NUTS-3"}, \code{"Province"}, \code{"LAU"}, \code{"Municipality"}.
#' If an input dataframe is provided, please select the same level of aggregation.
#' \code{"LAU"} by default
#' @param main Character. A customary title to the map. If \code{NULL}, the title will mention: subject, year and school grade. Empty by default.
#' @param main_pos Character.Where the header should be placed if the \code{ggplot} mode is chosen.
#' The header is located on the top if \code{"top"} is given as input, and above the legend scale otherwise. \code{"top"} by default.
#' @param region_code Numeric. The NUTS-2 codes of the units that must be displayed.
#' If the level is set to \code{"LAU"}, choosing a limited number of regions is recommended.
#' By default, \code{c(1,3,5:20)}, i.e. all Italian regions except the provinces of Aosta, Trento and Bozen which have data availability issues.
#' @param plot Character. The type of map to display; either \code{"mapview"} for interactive maps, or \code{"ggplot"} for static maps. \code{"mapview"} by default.
#'  !!! Warning. As \code{mapview} turns out to have been taken away from CRAN,
#'  if it has not been previously installed by the user, \code{plot = "ggplot"} will be forced.
#' @param pal Character. The palette to use if the \code{"mapview"} mode is chose. \code{"viridis"} by default.
#' @param WLE Logical. Whether the variable to chose should be the average WLE score rather that the percentage of sufficient tests, if both are available. \code{FALSE} by default
#' @param col_rev Logical. Whether the scale of the colour palette should be reverted or not, if the \code{mapview} mode is chosen. \code{FALSE} by default
#' @param popup_height Numeric. The height of the popup table in terms of pixels if the \code{"mapview"} mode is chosen. \code{200} by default.
#' @param only_observed Logical. Whether to remove unobserved areas from the plot. \code{FALSE} by default.
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param input_shp Object of class \code{sf}, \code{tbl_df}, \code{tbl}, \code{data.frame}. The relevant shapefiles of Italian administrative boudaries,
#' at the selected level of detail (LAU or NUTS-3). If \code{NULL}, it is downloaded automatically but not saved in the global environment. \code{NULL} by default.
#' @param autoAbort Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#'
#'
#'
#' @return If \code{plot == "mapview"}, an object of class \code{mapview}. Otherwise, if \code{plot == "ggplot"}, an object of class \code{gg} and \code{ggplot}.
#'
#'
#' @examples
#'
#'
#'
#'
#'  Map_Invalsi(subj = "Italian", grade = 13, level = "NUTS-3", Year = 2023, WLE = FALSE,
#'   data = example_Invalsi23_prov, input_shp = example_Prov22_shp, plot = "ggplot")
#'
#'  Map_Invalsi(subj = "Italian", grade = 5, level = "NUTS-3", Year = 2023, WLE = TRUE,
#'   data = example_Invalsi23_prov, input_shp = example_Prov22_shp, plot = "ggplot")
#'
#'
#'
#'
#'
#' @export

Map_Invalsi <- function(data = NULL, Year = 2023, subj_toplot = "ITA", grade = 8, level = "LAU",
                        main = "", main_pos = "top", region_code = c(1:20), plot="mapview", pal = "viridis",
                        WLE = FALSE, col_rev = FALSE, popup_height = 200,
                        only_observed = FALSE, verbose = TRUE,
                        input_shp = NULL, autoAbort = FALSE){

  if (length(subj_toplot) > 1){
    warning("Only one subject can be selected for mapping. The first one will be plotted")
    subj_toplot <- subj_toplot[1L]
  }

  region_code <- as.numeric(region_code)
  grade <- as.numeric(grade)

  if(length(grade)>1L){
    message("It is possible to plot only one school grade. Only the first one will be selected")
    grade <- grade[1L]
  }

  while(is.null(data)){
    data <- Get_Invalsi_IS(level = level, verbose = verbose,
                           multiple_out = FALSE, autoAbort = autoAbort)
    if(is.null(data)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during Invalsi data retrieving. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  if(!is.data.frame(data) && is.list(data)){
    if (toupper(level) %in% c("LAU", "NUTS-4", "MUNICIPALITY", "MUN")){
      data <- data$Municipality_data
    } else{
      data <- data$Province_data
    }
    if(is.null(data)){
      message("No data provided at the ", level, " level")
      return(NULL)
    }
  }

  while(is.null(input_shp)){
    YearMinus1 <- as.numeric(year.patternA(Year))%/%100
    input_shp <- Get_Shapefile(Year =
      ifelse(year.patternA(Year)=="201819", Year, YearMinus1), level = level, autoAbort = autoAbort)
    if(is.null(input_shp)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during shapefile retrieving. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  if("Subject" %in% names(data)){
    dd <- Util_Invalsi_filter(data = data, Year = Year,
                               subj = c("ELI", "ERE", "ITA", "MAT"),
                               grade = grade, level = level, WLE=WLE,
                               verbose = verbose)
  } else dd <- data



  if(toupper(subj_toplot) %in% c("ELI", "ENGLISH L", "ENGLISH_L", "INGLESE L", "INGLESE_L", "ENGLISH LISTENING", "ENGLISH_LISTENING")){
    fieldname <-  "M_English_L"
  } else if(toupper(subj_toplot) %in% c("ERE", "ENGLISH R", "ENGLISH_R", "INGLESE R", "INGLESE_R", "ENGLISH_READING", "ENGLISH READING" )){
    fieldname <- "M_English_R"
  } else if (toupper(subj_toplot) %in% c("ITA", "ITALIANO", "ITALIAN")){
    fieldname <- "M_Italian"
  } else if (toupper(subj_toplot) %in% c("MAT", "MATEMATICA", "MATHEMATICS")){
    fieldname <- "M_Mathematics"
  }

  if (toupper(level) %in% c("MUNICIPALITY", "LAU", "NUTS-4") ){
    shp <- input_shp %>% dplyr::select(.data$COD_REG, .data$COD_PROV, .data$PRO_COM_T) %>%
      rename_by_idx(c(1,2,3), into=c("Region_code", "Province_code", "Municipality_code"))
  } else {
    shp <- input_shp %>% dplyr::select(.data$COD_REG, .data$COD_PROV) %>%
      rename_by_idx(c(1,2), into = c("Region_code", "Province_code"))
  }
  res <- dplyr::left_join(shp, dd, by = names(shp)[ncol(shp)-1]) %>%
    #dplyr::select(-.data$geometry) %>%
    dplyr::filter(.data$Region_code %in% region_code)
  fieldnum <- grep(fieldname, names(res))

  names(res)[fieldnum] <- "X"
  if(only_observed == TRUE){
    res <- res %>% dplyr::filter(!is.na(.data$X))
  }

  if(!"mapview" %in% rownames(utils::installed.packages())) plot <- "ggplot"

  if (plot =="mapview"){

    n <- length(unique(res$X)) - 1
    if( col_rev == FALSE){
      brew <- grDevices::hcl.colors(n, palette = pal)
    } else {
      brew <- rev(grDevices::hcl.colors(n, palette = pal))
    }


    names(res)[fieldnum] <- fieldname
    pop <- leafpop::popupTable(res)


    mapview::mapview(res, zcol=fieldname, col.regions = brew,
                     layer.name = ifelse(main == "", paste0(
                       fieldname, ", ", "Year: ", Year, ", ", "School grade: ", grade), main),
                     popup = paste0(set_popup_height(popup_height) , pop))

  } else {

    if (main_pos == "top"){
      ggplot2::ggplot() +
        ggplot2::geom_sf(data = res, ggplot2::aes(fill = .data$X)) +
        ggplot2::labs(fill = "") +
        ggplot2::ggtitle(ifelse(main == "", paste0(
          fieldname, ", ", "Year: ", Year, ", ", "School grade: ", grade), main)) +
        ggplot2::scale_fill_viridis_c(na.value = "white") + ggplot2::theme_minimal()
    } else {
      ggplot2::ggplot() +
        ggplot2::geom_sf(data = res, ggplot2::aes(fill = .data$X)) +
        ggplot2::labs(fill = ifelse(main == "",paste0(
          fieldname, ", ", "Year: ", Year, ", ", "School grade: ", grade), main))+
        ggplot2::scale_fill_viridis_c(na.value = "white") + ggplot2::theme_minimal()
    }
  }
}

#' Download the number of teachers in Italian schools by province
#'
#' @description  This functions downloads the number of teachers by province from the open website of the Italian Ministry of Education, University and Research.
#'
#' @param Year Numeric or character value. Reference school year for the school registry data (last available is 2023).
#' Available in the formats: \code{2022}, \code{"2021/2022"}, \code{202122}, \code{20212022.} \code{2023} by default
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param show_col_types Logical. If \code{TRUE}, if the 'verbose' argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.
#' @param autoAbort Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param filename Character. Which data to retrieve among the province counts of teachers/school personnel.
#' By default it is \code{c("DOCTIT", "DOCSUP")}, which are the file names used so far for the number of tenured and temporary teachers respectively.
#' Other file names are the following:
#'
#'
#' \code{"ATATIT"} for the number of tenured non-teaching personnel
#'
#'
#' \code{"ATASUP"} for the number of temporary non-teaching personnel
#' @param t_out Numeric. !! EXPERIMENTAL !! session timeout for scraping and download, in seconds. 3 seconds by default.
#' @param forceAbort Logic. Whether to force a limited number of attempts to scrape the provider webpage.
#' Not recommended to be switched to \code{TRUE}. Only insert to speedup function failure when
#' there is certainty that the provider website is out. \code{FALSE} by default.
#' Experimental, may be implemented in other \code{Get_} functions.
#'
#'
#'
#' @details Please notice that by default, the function returns the count of the number of tenured and temporary teachers.
#' If either the count of non-teaching personnel or the count of a single category of teaching personnel is needed, please adapt
#' the \code{filename} argument accordingly.
#'
#'
#' @return An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#'
#' @source <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Personale+Scuola>
#'
#' @examples
#'
#'
#' nteachers23 <- Get_nteachers_prov(Year=2023, filename="DOCTIT",
#'                autoAbort=TRUE, forceAbort = TRUE)
#' nteachers23[, c(3,4,5)]
#'
#'
#' @export
#'
#'
Get_nteachers_prov <- function(Year = 2023, verbose = TRUE, show_col_types = FALSE,
                               filename = c("DOCTIT", "DOCSUP"), t_out = 3,
                               autoAbort = FALSE, forceAbort = FALSE){

  if(!Check_connection(autoAbort)) return(NULL)
  options(dplyr.summarise.inform = FALSE)

  start.zero <- Sys.time()

  home.url <- "https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Personale%20Scuola"
  homepage <- NULL
  attempt <- 0
  if(!forceAbort) max_attempt <- 10 else max_attempt <- 1
  while(is.null(homepage) && attempt <= max_attempt){
    homepage <- tryCatch({
      httr::content(httr::GET(home.url, httr::timeout(t_out)))
    }, error = function(e){
      message("Cannot read the html; ", max_attempt - attempt,
              " attempts left. If the problem persists, please contact the maintainer.\n")
      return(NULL)
    })
    attempt <- attempt + 1
  }
  if(is.null(homepage)) return(NULL)

  name_pattern <- "([0-9]+)\\.(csv)$"
  pattern <- ifelse(year.patternA(Year)=="201516", "1516", year.patternA(Year) )

  links <- homepage %>% rvest::html_nodes("a") %>% rvest::html_attr("href") %>% unique()
  links <- links[which(!is.na(links))]
  if (!any(stringr::str_detect(links, pattern))){
    message("No data available for this year. We apologise for the inconvenience")
    return(NULL)
  }

  files_to_download <- c()
  for (string in links[grep(".csv", links)] ){
    num_numeric_digits <- sum(unlist(gregexpr("[0-9]", string) ) > 0)
    if (num_numeric_digits >= nchar(pattern)){
      chrpart <- gsub("csv", "", gsub("[^[:alpha:]]", "", string))
      numpart <- gsub("[[:alpha:]]", "", string)
      if (!is.na(numpart) & grepl(pattern, numpart) & !string %in% files_to_download &
          any(stringr::str_detect(chrpart, filename))){
        files_to_download <- append(files_to_download, string)
        if(verbose) cat("Found ", string, " as element ",length(files_to_download), "\n")
      }
    }
  }

  base.url <- dirname(home.url)

  input <- list()

  starttime <- Sys.time()
  for (link in files_to_download) {

    status <- 0
    attempt <- 0
    while(status != 200){
      file.url <- file.path(base.url, link)
      response <- tryCatch({
        httr::GET(file.url, httr::timeout(t_out))
      }, error = function(e) {
        message("Error occurred during scraping, attempt repeated ... \n")
        NULL
      })
      status <- response$status_code
      if(is.null(response)){
        status <- 0
      }
      if(status != 200){
        attempt <- attempt + 1
        message("Operation exited with status: ", status, "; operation repeated (",
                max_attempt - attempt, " attempts left)")
      }
      if(attempt >= max_attempt) {
        message("Maximum attempts reached. Abort. We apologise for the inconvenience")
        return(NULL)
      }
    }

    if (httr::http_type(response) %in% c("application/csv", "text/csv", "application/octet-stream")) {
      content <- rawToChar(response$content)
      if(nchar(content)==0){
        message("Empty file. Operation aborted.
        There seems to be something wrong with the website.
        Please contact the maintainer, maybe it could help. \n")
        return(NULL)
      } else {
        dat <- readr::read_csv(content, show_col_types = show_col_types)
        if(verbose) cat("CSV file downloaded:", link, " ... ")
        link.1 <- gsub("AS1516", "", link)
        element.name <- substr(link.1, 1, regexpr("[0-9]", link.1)-1)
        input[[element.name]] <- dat
        input[[element.name]] <- input[[element.name]][!duplicated(input[[element.name]]),]
      }
     } else {
      message(paste("Wrong file type:", httr::http_type(response)) )
      cat("Failed to download and process:", link, "\n")
    }
    endtime <- Sys.time()
    if(verbose){
      cat(round(difftime(endtime, starttime, units="secs") ,2), " seconds required \n")
    }
    starttime <- Sys.time()
  }
  if(length(input) == 0L) return(NULL)


  if("DOCTIT" %in% names(input)){
    input$DOCTIT <- input$DOCTIT %>%
      dplyr::group_by(.data$PROVINCIA, .data$ORDINESCUOLA) %>% dplyr::summarise(
        Tot_teachers = sum(.data$DOCENTITITOLARIFEMMINE) + sum(.data$DOCENTITITOLARIMASCHI)) %>% dplyr::ungroup()
  }

  if("DOCSUP" %in% names(input)){
    input$DOCSUP <- input$DOCSUP %>%
      dplyr::group_by(.data$PROVINCIA, .data$ORDINESCUOLA) %>% dplyr::summarise(
        Tot_teachers = sum(.data$DOCENTISUPPLENTIFEMMINE) + sum(.data$DOCENTISUPPLENTIMASCHI)) %>% dplyr::ungroup()
  }

  if("ATATIT" %in% names(input)){
    input$ATATIT <- input$ATATIT %>%
      dplyr::group_by(.data$PROVINCIA) %>% dplyr::summarise(
        Tot_ATA = sum(.data$ATATITOLARIFEMMINE) + sum(.data$ATATITOLARIMASCHI)) %>% dplyr::ungroup()
  }

  if("ATASUP" %in% names(input)){
    input$ATASUP <- input$ATASUP %>%
      dplyr::group_by(.data$PROVINCIA) %>% dplyr::summarise(
        Tot_ATA = sum(.data$ATASUPPLENTIFEMMINE) + sum(.data$ATASUPPLENTIMASCHI)) %>% dplyr::ungroup()
  }

  if(length(input) == 0L) return(NULL)

  nteachers <- input[c("DOCTIT", "DOCSUP")]
  nteachers <- Filter(Negate(is.null), nteachers)
  nATA <- input[c("ATATIT", "ATASUP")]
  nATA <- Filter(Negate(is.null), nATA)

  nteachers.Tot <- do.call(rbind, nteachers)

  nATA.Tot <- do.call(rbind, nATA)

  if(!is.null(nteachers.Tot)){
    nteachers.Tot <- nteachers.Tot %>% dplyr::group_by(.data$PROVINCIA, .data$ORDINESCUOLA) %>%
      dplyr::summarise(Tot_teachers = sum(.data$Tot_teachers)) %>% dplyr::ungroup()
  }
  if(!is.null(nATA.Tot)){
    nATA.Tot <- nATA.Tot %>% dplyr::group_by(.data$PROVINCIA) %>%
      dplyr::summarise(Tot_ATA = sum(.data$Tot_ATA)) %>% dplyr::ungroup()
  }

  if(!is.null(nteachers.Tot)){
    npers.Tot <- nteachers.Tot
    if(!is.null(nATA.Tot)){
      npers.Tot <- dplyr::left_join(npers.Tot, nATA.Tot, by = "PROVINCIA")
    }
  } else{
    npers.Tot <- nATA.Tot %>% dplyr::mutate(Order = NA) %>%
      dplyr::relocate(.data$Order, .after = .data$PROVINCIA)
  }

  npers.Tot <- npers.Tot %>% rename_by_idx(c(1,2), c("Province_description", "Order")) %>%
    dplyr::filter(is.na(.data$Order) | .data$Order != "SCUOLA INFANZIA")

  npers.Tot$Order <- npers.Tot$Order %>%
    stringr::str_replace_all("SCUOLA PRIMARIA", "Primary") %>%
    stringr::str_replace_all("SCUOLA SECONDARIA I GRADO", "Middle") %>%
    stringr::str_replace_all("SCUOLA SECONDARIA II GRADO", "High")


  names(npers.Tot) <- stringr::str_to_title(names(npers.Tot))

  npers.Tot$Province_description <- stringr::str_to_title(npers.Tot$Province_description) %>%
    stringr::str_replace_all("Monza E Brianza", "Monza E Della Brianza") %>%
    stringr::str_replace_all("Barletta-Adria-Trani", "Barletta-Andria-Trani") %>%
    stringr::str_replace_all("Medio-Campitano", "Medio Campidano") %>%
    stringr::str_replace_all("Pesaro-Urbino", "Pesaro E Urbino")
  npers.Tot$Province_description[which(npers.Tot$Province_description=="Massa")] <- "Massa-Carrara"

  provnames <- prov.names() %>% dplyr::select(-.data$Region_code, -.data$Region_description) %>%
    dplyr::mutate(dplyr::across(.data$Province_description, ~ stringr::str_to_title(.)))


  npers.Tot <- npers.Tot %>% dplyr::left_join(provnames, by = "Province_description") %>%
      dplyr::relocate(.data$Province_code, .before = 1) %>%
      dplyr::relocate(.data$Province_initials, .after = "Province_code")


  return(npers.Tot)

}

#' @keywords internal
#'
NA.join.manage <- function(data, lastcol, NA_autoRM = NULL, verbose = TRUE){
  na.rm <- NULL
  if(!is.null(NA_autoRM)){
    if(NA_autoRM %in% c("Y", "y", "YES", "yes", "Yes", 1, "1", TRUE, "TRUE")){
      na.rm <- "Y"
    } else na.rm <- "N"
  } else {
    cat("Do you want to clean NA values? \n",
          "    - To remove NAs, please press 'Y'\n",
          "    - To abort the operation and cancel the join, please press 'A'\n",
          "    - To keep the NAs, press any other key \n",
          "(please, do not use quotes in the prompt) \n")
    na.rm <- readline(prompt = "  > ")
  }

  if (toupper(na.rm) == "Y") {
    if(verbose) cat("NAs deleted \n")
    data <- data[-which(apply(data[-c(1:lastcol)], MARGIN = 1, function(x) all(is.na(x)))), ]
  } else if (toupper(na.rm) == "A"){
    data <- data[,c(1:lastcol)]
    if(verbose) cat("You chose to abort the join \n")
  } else {
    if(verbose) cat("NAs kept in \n")
  }

  return(data)
}

#' @keywords internal
#'
Clean_DB <- function(data, cutout = NULL, col_cut_thresh = 10^3, IDcol = c(2,5), startcol = 10,
                     pattern.out = c(), init.cutout = c(), track_deleted = TRUE, verbose = TRUE) {

  Nuisance_int <- Nuisance(data, startcol = startcol,
                           missing.pattern = toupper(pattern.out), partial.pattern = "")

  #column cutout

  DB <- data %>% dplyr::select( - rownames(Nuisance_int)[which(
    Nuisance_int$Non_definiti > col_cut_thresh )])
  if(is.null(cutout)){
    cutout <- cutout.default(data, startcol = startcol, init.cutout = init.cutout, pattern.out = pattern.out)
  }
  cols_in <- names(DB)[which(!names(DB) %in% cutout)]
  DB <- DB %>%  dplyr::select(cols_in)

  #row cutout
  deleted <- data.frame(matrix(nrow = 0, ncol = length(IDcol)))
  names(deleted) <- names(data)[IDcol]

  for (j in  c(startcol:ncol(DB))){
    deleted.new <- DB %>% dplyr::filter((!!rlang::sym(names(DB)[j])) %in% pattern.out) %>%
      dplyr::select(IDcol)

    DB <- DB %>% dplyr::anti_join(deleted.new, colnames(deleted.new))

    if (nrow(deleted.new) > 0 & verbose == TRUE){
      cat("deleted", nrow(deleted.new), "units whose field", names(DB)[j], "is missing \n")
    }
    deleted <- rbind(deleted, deleted.new)
  }

  if(track_deleted == TRUE){
    res <- list (data = DB, deleted = deleted)
  } else res <- DB

  return(res)
}

#' @keywords internal
#'
Nuisance <- function(data, startcol = 10,
                     missing.pattern = c("NON DEFINITO", "-","INFORMAZIONE_ASSENTE", "NON_COMUNICATO"),
                     partial.pattern = "IN PARTE"){

  res <- as.data.frame( cbind(
    apply( data[,-c(1:(startcol - 1))],MARGIN = 2,
           FUN = function(x){
             x <- toupper(x)
             return (sum (x %in% toupper(missing.pattern)))
           } ),
    apply( data[,-c(1:(startcol - 1))],MARGIN = 2,
           FUN = function(x){
             x <- toupper(x)
             return (sum (x %in% toupper(partial.pattern)))
           } ) ) )
  names(res) = c("Non_definiti", "In_parte")
  return(res)
}

#' Filter the Invalsi data by subject, school grade and year.
#'
#' @description  This function filters the database of Invalsi scores (see \code{\link{Get_Invalsi_IS}}) by school year, education grade and subject and returns a dataframe in wide format.
#' Each row corresponds to one territorial unit (either municipality or province); the numerical variables are three (the mean score, the score's standard deviation and the students coverage percentage) for each selected subject.
#'
#' @param data Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#' The raw Invalsi survey data that has to be filtered, obtained as output of the \code{\link{Get_Invalsi_IS}} function.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default
#' @param subj Character. The school subject(s) to include, among \code{"Englis_listening"}/\code{"ELI"}, \code{"English_reading"}/\code{"ERE"}, \code{"Italian"}/\code{"ITA"} and \code{"Mathematics"}/\code{"MAT"}. All four by default.
#' @param grade Numeric. The school grade to chose. Either \code{2} (2nd year of primary school), \code{5} (last year of primary school), \code{8} (last year of middle shcool), \code{10} (2nd year of high school) or \code{13} (last year of school). \code{8} by default
#' @param level Character. The level of aggregation of Invalsi census data. Either \code{"NUTS-3"}, \code{"Province"}, \code{"LAU"}, \code{"Municipality"}.
#' If an input dataframe is provided, please select the same level of aggregation. \code{"LAU"} by default
#' @param WLE Logical. Whether the variable to choose should be the average WLE score rather that the percentage of sufficient tests, if both are available. \code{FALSE} by default
#' @param Year Numeric or character value. Reference school year for the data (last available is 2022/23).
#' Available in the formats: \code{2022}, \code{"2021/2022"}, \code{202122}, \code{20212022}. \code{2023} by default
#' @param verbose Logical. If \code{TRUE}, the function informs about the time needed. \code{TRUE} by default.
#' @param autoAbort Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#'
#'
#'
#'
#' @return An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. For all subjects and school grades, the variables indicate:
#'
#' \itemize{
#' \item \code{M} The mean score, either WLE or percentage of sufficient tests
#' \item \code{S} The standard deviation of the score
#' \item \code{C} The students coverage percentage (expressed in the scale 1 - 100)
#' }
#'
#'
#'
#'
#' @examples
#'
#'
#'
#' Util_Invalsi_filter(subj = c("Italian", "Mathematics"), grade = 5, level = "NUTS-3", Year = 2023,
#'                    WLE = FALSE, data = example_Invalsi23_prov)
#'
#' Util_Invalsi_filter(subj = c("Italian", "Mathematics"), grade = 5, level = "NUTS-3", Year = 2023,
#'                     WLE = TRUE, data = example_Invalsi23_prov)
#'
#' Invalsi23_high <- Util_Invalsi_filter(subj = "Italian", grade = c(10,13), level = "NUTS-3",
#'                                       Year = 2023, data = example_Invalsi23_prov)
#'
#'
#'  summary(Invalsi23_high)
#'
#' @export

Util_Invalsi_filter <- function(data = NULL, subj=c("ELI", "ERE", "ITA", "MAT"), grade = 8, level = "LAU", WLE=FALSE,
                                Year = 2023, verbose = TRUE, autoAbort = FALSE){

  starttime <- Sys.time()

  while(is.null(data)){
    data <- Get_Invalsi_IS(level = level, verbose = verbose,  multiple_out = FALSE,
                           autoAbort = autoAbort, category = FALSE)
    if(is.null(data)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during Invalsi data retrieving. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  if(!is.data.frame(data) && is.list(data)){
    if (toupper(level) %in% c("LAU", "NUTS-4", "MUNICIPALITY", "MUN")){
      data <- data$Municipality_data
    } else{
      data <- data$Province_data
    }
    if(is.null(data)){
      message("No data provided at the ", level, " level")
      return(NULL)
    }
  }

  Y <- sapply(Year, year.patternA)

  for(i in 1:length(subj)){
    if(toupper(subj[i]) %in% c("ELI", "ENGLISH L", "ENGLISH_L", "INGLESE L", "INGLESE_L",
                               "ENGLISH LISTENING", "ENGLISH_LISTENING")){
      subj[i] <- "English_R"
    } else if(toupper(subj[i]) %in% c("ERE", "ENGLISH R", "ENGLISH_R", "INGLESE R", "INGLESE_R",
                                      "ENGLISH READING", "ENGLISH_READING", "INGLESE SCRITTO", "INGLESE_SCRITTO")){
      subj[i] <- "English_L"
    } else if (toupper(subj[i]) %in% c("ITA", "ITALIANO", "ITALIAN")){
      subj[i] <- "Italian"
    } else if (toupper(subj[i]) %in% c("MAT", "MATEMATICA", "MATHS", "MATHEMATICS")){
      subj[i] <- "Mathematics"
    }
  }

  Invalsi_IS <- data %>%
    dplyr::mutate(Year =  stringr::str_remove_all(.data$Year, "-")) %>%
    dplyr::mutate(Subject =  stringr::str_replace_all(.data$Subject, " ", "_")) %>%
    dplyr::filter(.data$Year %in% Y) %>%
    dplyr::filter(.data$Grade %in% grade) %>%
    dplyr::filter(.data$Subject %in% subj)

  if(nrow(Invalsi_IS) == 0){
    message("No Invalsi data found for year: ", Year, ", grade: ", grade, ", subject: ", subj,
      ". We apologise for the inconvenience. \n")
    return(NULL)
  }

  if (13 %in% grade | any(as.numeric(substr(Y, 1, 4)) > 2016) & any(grade >5)){
    WLE <- TRUE
  }

  if (toupper(level) %in% c("LAU", "NUTS-4", "MUNICIPALITY", "MUN")){

    doub <- which(duplicated(dplyr::select(Invalsi_IS, .data$Municipality_code, .data$Year,
                                           .data$Subject, .data$Grade)))
    if(length(doub) > 0) Invalsi_IS <- Invalsi_IS[-doub,]

    if (WLE){
      DB.Invalsi <- Invalsi_IS %>%
        dplyr::select(.data$Municipality_code, .data$Municipality_description, .data$Grade, .data$Subject,
                      .data$Year, .data$WLE_average_score, .data$Std_dev_WLE_score, .data$Students_coverage)
    } else {
      DB.Invalsi <- Invalsi_IS %>%
        dplyr::select(.data$Municipality_code, .data$Municipality_description, .data$Grade, .data$Subject,
                      .data$Year, .data$Average_percentage_score, .data$Std_dev_percentage_score, .data$Students_coverage)
    }

    names(DB.Invalsi)[c(6,7,8)] <- c("Mean", "StDev", "Coverage")
    DB.Invalsi <- DB.Invalsi %>% dplyr::filter(.data$Mean != 999 & .data$StDev != 999)


    # Necessary since in some cases different names are associated to the same municipality code
    # (for the same year) and this ambiguity would cause inconvenience when the dataframe is spreaded
    Mun.rename <- DB.Invalsi %>% dplyr::select(.data$Municipality_code, .data$Municipality_description) %>%
      dplyr::filter(!duplicated(.data$Municipality_code))
    DB.Invalsi <- DB.Invalsi %>% dplyr::select(-.data$Municipality_description) %>%
      dplyr::left_join(Mun.rename, by = "Municipality_code") %>%
      dplyr::relocate(.data$Municipality_description, .after = "Municipality_code")

    DB.Invalsi.m <- DB.Invalsi %>% dplyr::select(-.data$StDev, -.data$Coverage) %>%
      tidyr::spread(key = .data$Subject, value = .data$Mean)
    DB.Invalsi.s <- DB.Invalsi %>% dplyr::select(-.data$Mean, -.data$Coverage) %>%
      tidyr::spread(key = .data$Subject, value = .data$StDev) %>%
      dplyr::select(-.data$Municipality_description)
    DB.Invalsi.c <- DB.Invalsi %>% dplyr::select(-.data$Mean, -.data$StDev) %>%
      tidyr::spread(key = .data$Subject, value = .data$Coverage) %>%
      dplyr::select(-.data$Municipality_description)

    names(DB.Invalsi.m)[c(5:8)] <- paste ("M",names(DB.Invalsi.m)[c(5:8)],sep="_")
    names(DB.Invalsi.s)[c(4:7)] <- paste ("S",names(DB.Invalsi.s)[c(4:7)],sep="_")
    names(DB.Invalsi.c)[c(4:7)] <- paste ("C",names(DB.Invalsi.c)[c(4:7)],sep="_")

    DB.Invalsi <- DB.Invalsi.m %>%
      dplyr::left_join(DB.Invalsi.s, by = c("Municipality_code", "Grade", "Year")) %>%
      dplyr::left_join(DB.Invalsi.c, by = c("Municipality_code", "Grade", "Year"))
    values_from <- names(DB.Invalsi)[min(which(grepl("M_", names(DB.Invalsi)))):ncol(DB.Invalsi)]
    DB.Invalsi <- DB.Invalsi %>%
      tidyr::pivot_wider(id_cols = c(.data$Municipality_code, .data$Municipality_description, .data$Year),
                         names_from = .data$Grade,
                         values_from = values_from)

  } else if (toupper(level) %in% c("PROVINCE", "PROV", "NUTS-3") ) {

    if("Province_initials" %in% names(Invalsi_IS)){
      # The abbreviation for the province of Naples is confused with "Not Available"
      #Invalsi_IS$Province_initials <-
        #Invalsi_IS$Province_initials %>% stringr::str_replace_na("NA")

      Invalsi_IS <- Invalsi_IS %>%
        dplyr::select(-.data$Province_initials)

    }
    doub <- which(duplicated(dplyr::select(Invalsi_IS, .data$Province_description,
                                           .data$Year, .data$Subject, .data$Grade)))

    if(length(doub) > 0) Invalsi_IS <- Invalsi_IS[-doub,]

    if (!WLE){
      DB.Invalsi <- Invalsi_IS %>%
        dplyr::select(.data$Year, .data$Grade, .data$Subject, .data$Province_code,
                      .data$Province_description, .data$Average_percentage_score, .data$Std_dev_percentage_score, .data$Students_coverage)
    } else {
      DB.Invalsi <- Invalsi_IS %>%
        dplyr::select(.data$Year, .data$Grade, .data$Subject, .data$Province_code,
                      .data$Province_description, .data$WLE_average_score, .data$Std_dev_WLE_score, .data$Students_coverage)
    }
    names(DB.Invalsi)[c(6,7,8)] <- c("Mean", "StDev", "Coverage")

    DB.Invalsi <- DB.Invalsi %>% dplyr::filter(.data$Mean != 999 & .data$StDev != 999)
    DB.Invalsi.m <- DB.Invalsi %>% dplyr::select(-.data$StDev, -.data$Coverage) %>%
      tidyr::spread(key = .data$Subject, value=.data$Mean)
    DB.Invalsi.s <- DB.Invalsi %>% dplyr::select(-.data$Mean, -.data$Province_description, -.data$Coverage) %>%
      tidyr::spread(key = .data$Subject, value=.data$StDev)
    DB.Invalsi.c <- DB.Invalsi %>% dplyr::select(-.data$Mean, -.data$Province_description, -.data$StDev) %>%
      tidyr::spread(key = .data$Subject, value = .data$Coverage)

    names(DB.Invalsi.m)[c(5:8)] <- paste ("M",names(DB.Invalsi.m)[c(5:8)],sep="_")
    names(DB.Invalsi.s)[c(4:7)] <- paste ("S",names(DB.Invalsi.s)[c(4:7)],sep="_")
    names(DB.Invalsi.c)[c(4:7)] <- paste ("C",names(DB.Invalsi.c)[c(4:7)],sep="_")

    DB.Invalsi <- DB.Invalsi.m %>%
      dplyr::left_join(DB.Invalsi.s, by = c("Province_code", "Grade", "Year")) %>%
      dplyr::left_join(DB.Invalsi.c, by = c("Province_code", "Grade", "Year"))
    values_from <- names(DB.Invalsi)[min(which(grepl("M_", names(DB.Invalsi)))):ncol(DB.Invalsi)]
    DB.Invalsi <- DB.Invalsi %>%
      tidyr::pivot_wider(id_cols = c(.data$Province_code, .data$Province_description, .data$Year),
                         names_from = .data$Grade,
                         values_from = values_from)

  } else {
    message("Please, select either Municipality/LAU or Province/NUTS-3 as administrative level")
    return(NULL)
  }

  if(nrow(DB.Invalsi)==0){
    stop("No Invalsi data available", .call = TRUE)
  }
  #if(length(Y)==1L) DB.Invalsi <- dplyr::select(DB.Invalsi, -.data$Year)

  DB.Invalsi <- DB.Invalsi[, which(!unlist(lapply(DB.Invalsi, function(x) all(is.na(x)))))]
  DB.Invalsi <- DB.Invalsi[,c(1, 2, 3, 3 + c(order(as.numeric(stringr::str_extract(names(DB.Invalsi)[-c(1,2,3)], "\\d+$")))))]
  endtime <- Sys.time()

  if(verbose) {
    cat(paste("Total running time to", ifelse(is.null(data), "retrieve and", ""),
              "process Invalsi", level, "data", "for year",
              Y, "subject", paste(subj, collapse = ", "),"School year n.",
              paste(grade, collapse = ", "), ": \n"))
    cat(paste(round(difftime(endtime, starttime, units="secs") ,2), "seconds \n"))

  }

  return(DB.Invalsi)

}

#' @keywords internal
#'

gsub.bool <- function(data, startcol){
  DB_repl <- lapply(data[,-c(1:(startcol - 1))], function(x){
    gsub(
      "SI", 1, gsub(
        "NO", 0, gsub(
          "Esiste", 1, ignore.case = TRUE, gsub(
            "Non Esiste", 0, ignore.case = TRUE, gsub(
              "IN PARTE", 1, ignore.case = TRUE, gsub(
                "ND", 0, ignore.case = TRUE, gsub(
                  "Non Definito", 0, ignore.case = TRUE, gsub(
                    "Non Comunicato", 0, ignore.case = TRUE, gsub(
                      "Non Richiesto", 0, ignore.case = TRUE, gsub(
                        "^-$", 0 , gsub("^NA$", 0 , x)))))))))))})
  DB_repl <- lapply(DB_repl,  FUN=as.numeric ) %>% as.data.frame()
  res <- data
  res[-c(1:(startcol - 1))] <- DB_repl
  return(res)
}



#res <- data
#DB_repl <- lapply(data[,-c(1:(startcol - 1))], function(x){
#  gsub("SI", 1,
#       gsub("NO", 0,
#            gsub("NON DEFINITO",0,
#                 gsub("IN PARTE", 1,
#                      gsub("Esiste", 1,
#                           gsub("Non Esiste", 0 ,
#                                gsub("ND", 0, x)))))))
#}) %>% as.data.frame()
#DB_repl <- lapply(DB_repl,  FUN=as.numeric ) %>% as.data.frame()
#res[-c(1:(startcol - 1))] <- DB_repl
#return(res)

#' @keywords internal
#'
tabrename.nstud <- function(){
  manualdf <- data.frame(
    Input = c("ANNOSCOLASTICO", "CODICESCUOLA", "ORDINESCUOLA", "ANNOCORSO", "FASCIAETA",
              "ALUNNI", "CLASSI", "ALUNNIMASCHI", "ALUNNIFEMMINE",
              "ALUNNICITTADINANZAITALIANA", "ALUNNICITTADINANZANONITALIANA",
              "ALUNNICITTADINANZANONITALIANAPAESIUE", "ALUNNICITTADINANZANONITALIANAPAESINONUE",
              "TIPOPERCORSO", "PERCORSO", "INDIRIZZO", "TEMPOSCUOLA"),
    Output = c("Year", "School_code", "Order", "Grade", "Age_class",
                 "Students", "Classes", "Male_students", "Female_students",
               "ITA_students", "Foreign_students", "EU_Foreign_students", "Non_EU_students",
               "Curriculum_lv1", "Curriculum_lv2", "Curriculum_lv3", "Running_time") )
  return(manualdf)
}

#' Download the classification of peripheral municipalities
#'
#' @description
#' Retrieves the classification of Italian municipalities into six categories; classes D, E, and F are the so-called internal/inner areas; classes A, B and C are the central areas.
#'
#' @param autoAbort Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param verbose Logical. Whether to keep track of computational time. \code{TRUE} by default.
#'
#' @return An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#'
#' @details
#' Classes are defined according to these criteria; see the methodological note (in Italian) for more detail:
#'
#' \itemize{
#'   \item A - Standalone pole municipalities, the highest degree of centrality;
#'   they are characterised by a thorough and self-sufficient combined endowment of school, health and transport infrastructure,
#'    i.e. there are at least a lyceum and a technical high school; a railway station of medium dimensions and a hospital provided with an emergency ward.
#'   \item B - Intermunicipality poles; the endowment of such infrastructures is complete if a small set of contiguous municipalities is considered
#' }
#' The remaining classes are defined in terms of the national distribution of the road distances from a municipality to the closest pole:
#' \itemize{
#'   \item C - Belt municipalities, travel time below the median (< 27'42'') .
#'   \item D - Intermediate municipalities, travel time between the median and the third quartile (27'42'' - 40'54'').
#'   \item E - Peripheral municipalities, travel time between the third quartile and 97.5th percentile (40'54'' - 1h 6' 54'').
#'   \item F - Ultra-peripheral municipalities, travel time over the 97.5th percentile (>1h 6' 54'').
#' }
#' For more information regarding the dataset, it is possible to check the ISTAT methodological note (in Italian) available at <https://www.istat.it/it/files//2022/07/FOCUS-AREE-INTERNE-2021.pdf>
#'
#'
#' @examples
#'
#' \donttest{
#' InnerAreas <- Get_InnerAreas(autoAbort = TRUE)
#' InnerAreas[, c(1,9,13)]
#' }
#'
#' @source  <https://www.istat.it/notizia/la-geografia-delle-aree-interne-nel-2020-vasti-territori-tra-potenzialita-e-debolezze/>
#'
#'
#'
#' @export


Get_InnerAreas <- function(verbose = TRUE, autoAbort = FALSE){

  # This version does not require readxl to reduce the number of dependencies.
  # For a faster version requiring it, please ask the maintainer.
  if(!Check_connection(autoAbort)) return(NULL)

  starttime <- Sys.time()

  home.ISTAT <-"https://www.istat.it/notizia/la-geografia-delle-aree-interne-nel-2020-vasti-territori-tra-potenzialita-e-debolezze/"
  homepage <- NULL
  attempt <- 0
  while(is.null(homepage) && attempt <= 10){
    homepage <- tryCatch({
      xml2::read_html(home.ISTAT)
    }, error = function(e){
      message("Cannot read the html; ", 10 - attempt,
              " attempts left. If the problem persists, please contact the maintainer.\n")
      return(NULL)
    })
    attempt <- attempt + 1
  }
  if(is.null(homepage)) {
    message("Maximum attempts reached. Abort. We apologise for the inconvenience")
    return(NULL)
  }

  name_pattern <- "Elenco_Comuni_Classi_di_Aree_Interne"
  link <- homepage %>% rvest::html_nodes("a") %>% rvest::html_attr("href") %>% unique()
  link <- grep(name_pattern, link, value = TRUE)

  temp <- tempdir()
  if(!dir.exists(temp)){
    dir.create(temp)
  }

  status <- 0
  attempt <- 0
  while(status != 200 && attempt <= 10){
    base.url <- dirname(home.ISTAT)
    file.url <- xml2::url_absolute(link, base.url)
    response <- httr::GET(file.url, httr::write_disk(tempfile(fileext = ".xlsx")),  httr::config(timeout = 60))
    status <- response$status_code
    if(is.null(response)){
      status <- 0
    }
    if(status != 200){
      attempt <- attempt + 1
      message("Operation exited with status: ", status, "; operation repeated (",
              10 - attempt, " attempts left)")
    }
    if(attempt >= 10) {
      message("Maximum attempts reached. Abort. We apologise for the inconvenience")
      return(NULL)
    }
  }

  excel <- response$request$output$path

  utils::unzip(excel, exdir = temp)

  sheet_path <- file.path(temp, "xl", "worksheets", "sheet1.xml")
  shared_strings_path <- file.path(temp, "xl", "sharedStrings.xml")

  strings0 <- xml2::read_xml(shared_strings_path)

  ns <- xml2::xml_ns(strings0)

  shared_strings <- strings0 %>%
    xml2::xml_find_all(".//d1:si/d1:t", ns = ns) %>%
    xml2::xml_text()

  sheet <- xml2::read_xml(sheet_path)
  rows <- xml2::xml_find_all(sheet, "//d1:row", ns = xml2::xml_ns(sheet))

  lst <- lapply(rows, function(X){
    cells <- xml2::xml_find_all(X, "d1:c", ns = ns)
    strings <- as.vector(unlist(lapply(cells, function(Y) {
      id <- xml2::xml_find_first(Y, "d1:v", ns = ns) %>%
        xml2::xml_text()
      if(!is.na(xml2::xml_attr(Y, "t"))){
        return(gsub("\\s+$", "", as.character(shared_strings[as.numeric(id) + 1])))
      } else {
        return(id)
      }
    })))
    if(length(strings) > 16) strings <- strings[c(1:16)]
    return(strings)
  })
  lst <- lst[which(lapply(lst, length)==16)]
  res <- data.frame(do.call(rbind, lst[-1]))

  names(res) <-
    c("Municipality_code", "Municipality_code_numeric", "Cadastral_code", "Region_code",
      "Region_description", "Province_code", "Province_initials", "Province_description", "Municipality_description",
      "Inner_area_code_2014_2020", "Inner_area_description_2014_2020",
      "Inner_area_code_2021_2027", "Inner_area_description_2021_2027",
      "Destination_municipality_code", "Destination_municipality_description",
      "Destination_pole_code")

  res <- res %>%
    dplyr::mutate(Province_code = as.numeric(.data$Municipality_code)%/%1000) %>%
    dplyr::mutate(Municipality_code_numeric = as.numeric(.data$Municipality_code_numeric)) %>%
    structure(class = c("tbl_df", "tbl", "data.frame"))

  if(dir.exists(temp)){
    temp.contents <- list.files(temp, full.names = T)
    for(i in (1:length(temp.contents))) {
      unlink(temp.contents[i], recursive = TRUE)
    }
  }
  endtime <- Sys.time()
  if(verbose){
    cat(paste("Time needed to download inner areas taxomony:",
              round(difftime(endtime, starttime, units="secs"), 2), "seconds \n"  ))
  }

  return(res)
}

#' Subset of the school registry in school year 2022/23
#'
#'
#' This dataframe includes the schools directly identifiable as primary, middle or high school, from four regions: Molise, Campania, Apulia and Basilicata.
#' Only the first 10 columns are included.
#' The whole dataset can be retrieved with the command \code{Get_Registry(2023)}
#' @seealso \code{\link{Get_Registry}}
#'
#' @format ## `example_input_Registry23`
#' A data frame with 5929 rows and 10 columns:
#' \itemize{
#'   \item \code{Year} Numeric; the school year.
#'   \item \code{Area} Character; the macro-area of the municipality, i.e. North, Center or South.
#'   \item \code{Region_description} Character; the region (NUTS-2 administrative level) name.
#'   \item \code{Province_description} Character; the province (NUTS-3 administrative level) name.
#'   \item \code{Reference_institute_code} Character; the ID of the reference institute.
#'   \item \code{School_code} Character; the school ID.
#'   \item \code{Cadastral_code} Character; a LAU - level ID code, different from the official LAU municipality code.
#'                               The Italian Ministry of Education does provide this code in the place of the LAU code for both the Schools registry and the early school buildings DBs.
#'   \item \code{Municipality_description} Character; the municipality name.
#'   \item \code{School_address} Character; the school physical address.
#'   \item \code{Postal_code} Character; the ZIP code, slightly finer than municipality boundaries for big municipalities.

#' }
#' @source <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Scuole&datasetId=DS0400SCUANAGRAFESTAT>
#'
"example_input_Registry23"

#' Download students' number data
#'
#' @description   This functions downloads the data regarding the number of students, from the open website of the Italian Ministry of Education, University and Research
#'
#'
#'
#' @param Year Numeric or character. Reference school year  (last available is 2023).
#' Available in the formats: \code{2022}, \code{"2021/2022"}, \code{202122}, \code{20212022}. \code{2023} by default
#' @param filename Character. A string included in the name of the file to download.
#' By default it is \code{c("ALUCORSOETASTA", "ALUCORSOINDCLASTA")}, which are the file names used so far for the number of students by age and the number of studentsin public schools  by age and class.
#'
#'
#'
#'
#' Other file names are the following. The output is not currently supported by the remainder of the functions involving the number of students.
#'
#'
#' \code{"ALUITASTRACITSTA"} for the number of Italian and foreign students in public schools
#'
#'
#' \code{"ALUSECGRADOINDSTA"} for the number of students of public schools by high school address
#'
#'
#' \code{"ALUTEMPOSCUOLASTA"} for the number of students of public schools by school running time
#'
#' \code{"ALUCORSOETAPAR"}, \code{"ALUCORSOINDCLAPAR"}, \code{"ALUITASTRACITPAR"}, \code{"ALUSECGRADOINDPAR"}, \code{"ALUTEMPOSCUOLAPAR"} for the data of the previous file but referring to private schools.
#'
#'
#'
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param show_col_types Logical. If \code{TRUE}, if the \code{verbose} argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.
#' @param autoAbort Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#'
#' @source <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Studenti>
#'
#'
#' @examples
#'
#' \donttest{
#'   Get_nstud(2023, filename = "ALUCORSOINDCLASTA", autoAbort = TRUE)
#' }
#'
#'
#'
#' @return By default, a list of two \code{tbl_df}, \code{tbl} and \code{data.frame} objects:
#'
#' \itemize{
#'   \item \code{$ALUCORSOETASTA}: The number of students by school, school grade and age. It provides a higher number of school than the other element
#'   \item \code{$ALUCORSOINDCLASTA}: The number of students and classes by school and school grade. This is a long-format dataframe.
#'
#' }
#'
#' @export

Get_nstud <- function(Year = 2023, filename = c("ALUCORSOETASTA", "ALUCORSOINDCLASTA"),
                      verbose = TRUE, show_col_types = FALSE, autoAbort = FALSE){

  if(!Check_connection(autoAbort)) return(NULL)

  start.zero <- Sys.time()

  home.url <- "https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Studenti"
  homepage <- NULL
  attempt <- 0
  while(is.null(homepage) && attempt <= 10){
    homepage <- tryCatch({
      httr::content(httr::GET(home.url))
    }, error = function(e){
      message("Cannot read the html; ", 10 - attempt,
              " attempts left. If the problem persists, please contact the maintainer.\n")
      return(NULL)
    })
    attempt <- attempt + 1
  }
  if(is.null(homepage)) return(NULL)
  name_pattern <- "([0-9]+)\\.(csv)$"
  pattern <- year.patternA(Year)

  links <- homepage %>% rvest::html_nodes("a") %>% rvest::html_attr("href") %>% unique()
  links <- links[which(!is.na(links))]
  if (!any(stringr::str_detect(links, pattern))){
    message("No data available for this year. We apologise for the inconvenience")
    return(NULL)
  }


  files_to_download <- c()
  for (string in links[grep(".csv", links)] ) {
    num_numeric_digits <- sum(unlist(gregexpr("[0-9]", string) ) > 0)
    if (num_numeric_digits >= nchar(pattern) ){
      first_nchar <- stringr::str_extract(string, paste0("[0-9]{", nchar(pattern), "}"))
      if (!is.na(first_nchar) & pattern == first_nchar & ! string %in% files_to_download &
          #substr(string, nchar(string)-num_numeric_digits-6, nchar(string)-num_numeric_digits-4) == "STA" &
          substr(string,1,3) == "ALU" &
          substr(string, 1, nchar(string)-num_numeric_digits-4 ) %in% filename){
        files_to_download <- append(files_to_download, string)
      }
    }
  }

  input.nstud0 <- list()

  starttime <- Sys.time()
  for (link in files_to_download) {
    attempt <- 0
    status <- 0
    while(status != 200){
      base.url <- dirname(home.url)
      file.url <- file.path(base.url, link)
      response <- tryCatch({
        httr::GET(file.url)
      }, error = function(e) {
        message("Error occurred during scraping, attempt repeated ... \n")
        NULL
      })
      status <- response$status_code
      if(is.null(response)){
        status <- 0
      }
      if(status != 200){
        attempt <- attempt + 1
        message("Operation exited with status: ", status, "; operation repeated (",
                10 - attempt, " attempts left)")
      }
      if(attempt >= 10) {
        message("Maximum attempts reached. Abort. We apologise for the inconvenience")
        return(NULL)
      }
    }

    if (httr::http_type(response) %in% c("application/csv", "text/csv", "application/octet-stream")) {
      content <- rawToChar(response$content)
      if(nchar(content)==0){
        message("Empty file. Operation aborted.
        There seems to be something wrong with the website.
        Please contact the maintainer, maybe it could help. \n")
        return(NULL)
      } else {
        dat <- readr::read_csv(rawToChar(response$content), show_col_types = show_col_types)
        if(verbose) cat("CSV file downloaded: ", link, "...")
        element.name <- substr(link,1, regexpr("[0-9]", link)-1)
        input.nstud0[[element.name]] <- dat
        input.nstud0[[element.name]] <- input.nstud0[[element.name]][!duplicated(input.nstud0[[element.name]]),]
        endtime <- Sys.time()
        if(verbose) {cat(round(difftime(endtime, starttime, units="secs") ,2),
                         "seconds required \n ")}
        starttime <- Sys.time()
      }
    } else {
      message(paste("Wrong file type:", httr::http_type(response)) )
      message("Failed to download and process:", link, "\n")
    }
  }

  if(length(input.nstud0) == 0L) return(NULL)

  input.nstud0 <- lapply(input.nstud0, function(x){
    if ("ANNOCORSOCLASSE" %in% names(x)){
      x <- x %>% dplyr::rename(ANNOCORSO = .data$ANNOCORSOCLASSE)
    } else return(x)
  })

  input.nstud0  <- lapply(input.nstud0, function(x){
    if("ANNOCORSO" %in% names(x)){
      x <- x %>% dplyr::filter(.data$ANNOCORSO <= 5) %>%
        dplyr::mutate(ANNOCORSO = .data$ANNOCORSO + dplyr::case_when(
          .data$ORDINESCUOLA == "SCUOLA PRIMARIA" ~ 0,
          .data$ORDINESCUOLA == "SCUOLA SECONDARIA I GRADO" ~ 5 ,
          .data$ORDINESCUOLA == "SCUOLA SECONDARIA II GRADO" ~ 8 ) )
    }
    if("TEMPOSCUOLA" %in% names(x)){
      x <- x %>% dplyr::mutate(TEMPOSCUOLA = dplyr::case_when(
        .data$TEMPOSCUOLA == "TEMPO NORMALE" ~ "Ordinary",
        .data$TEMPOSCUOLA == "INDIRIZZO ORDINARIO" ~ "Ordinary",
        .data$TEMPOSCUOLA == "INDIRIZZO MUSICALE" ~ "Music_oriented",
        .data$TEMPOSCUOLA == "TEMPO PIENO" ~ "Full_time"))
    }
    x <- x %>% dplyr::mutate(ORDINESCUOLA = dplyr::case_when(
      .data$ORDINESCUOLA == "SCUOLA PRIMARIA" ~ "Primary",
      .data$ORDINESCUOLA == "SCUOLA SECONDARIA I GRADO" ~ "Middle",
      .data$ORDINESCUOLA == "SCUOLA SECONDARIA II GRADO" ~ "High"))

    return(x)
  })

  tabrename.nstud <- tabrename.nstud()

  input.nstud0 <- lapply(input.nstud0, function(x){
    for (j in (1:ncol(x))){
      if (names(x)[j] %in% tabrename.nstud$Input){
        names(x)[j] <- tabrename.nstud[which(tabrename.nstud$Input == names(x)[j]), 2]
      }
    }
    return(x)
  })

  endtime <- Sys.time()
  if(verbose) cat(paste(difftime(endtime, start.zero, units="secs"), " seconds needed to retrieve students number data \n" ) )

  return(input.nstud0)
}

#' @keywords internal
#'
set_popup_height <- function(x) {
  str1 <- '
   <style>
      .leaflet-popup-content {
        max-height: '

  str2 <- 'px;
        overflow-y: auto;
      }
    </style>
    '
  return(paste(str1, x, str2, sep=""))
}

#' @keywords internal
#'

Group_BroadBand <- function(data = NULL, Date = Sys.Date(), input_School2mun = NULL, verbose=TRUE,  autoAbort = FALSE){
  if(is.null(data)){
    data <- Get_BroadBand(Date = Date, verbose = verbose, autoAbort = autoAbort)
  }
  if(is.null(data)) return(NULL)
  data <- data %>% dplyr::mutate(
    Province_code = as.numeric(substr(.data$Municipality_code, 1, 3)))

  broadband.mun <- data %>% dplyr::filter(.data$Order != "NR") %>%
    dplyr::group_by(.data$Region_code, .data$Region_description, .data$Province_code,
                    .data$Municipality_code, .data$Order) %>%
    dplyr::summarise(nschools = dplyr::n(), Province_description = .data$Province_description[1L],
                     Municipality_description = .data$Municipality_description[1L],
                     BB_Activation_status = mean(.data$BB_Activation_status, na.rm = TRUE)) %>%
    dplyr::ungroup()

  broadband.prov <- data %>% dplyr::filter(.data$Order != "NR") %>%
    dplyr::group_by(.data$Region_code, .data$Region_description, .data$Province_code,  .data$Order) %>%
    dplyr::summarise(nschools = dplyr::n(), Province_description = .data$Province_description[1L],
                     BB_Activation_status = mean(.data$BB_Activation_status, na.rm = TRUE)) %>%
    dplyr::ungroup()

  res <- list(Municipality_data = broadband.mun, Province_data = broadband.prov)
  return(res)

}

#' @keywords internal

str_detect.general <- function(string, pattern, negate = FALSE){
    if(length(pattern)==1){
      return(stringr::str_detect(string, pattern, negate = negate))
      } else if (length(pattern)>1){
        res <- c()
        for(i in (1:length(pattern))){
          newvector <- stringr::str_detect(string, pattern[i], negate = negate)
          res <- c(res,newvector)
          }
    return(res)
  }
}

#' Download the Invalsi census survey data
#'
#' @description  Downloads the full database of the Invalsi scores, detailed either at the municipality or province level.
#'
#'
#'
#' @param multiple_out Logical. Wheter keeping
#' multiple dataframes as outputs (thus overriding the \code{level} argument) or not.
#' We advise to keep this option active in order not to lose information. \code{TRUE} by default.
#' @param level Character. The level of aggregation of Invalsi census data, such as \code{"NUTS-3"}, \code{"Province"} for provinces; or \code{"LAU"}, \code{"Municipality"} for muncipalities.
#' Only useful when \code{multiple_out == FALSE}. \code{"LAU"} by default.
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param show_col_types Logical. If \code{TRUE}, if the \code{verbose} argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.
#' @param autoAbort Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' \code{TRUE} by default.
#' @param category Logical. Whether to focus on a specific category of students participating to the census survey. Warning: experimental. \code{FALSE} by default.
#'
#' @return If \code{multiple_out == FALSE}, an object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#' Otherwise, a list including objects of the aforementioned classes
#'
#' @source  <https://serviziostatistico.invalsi.it/en/archivio-dati/?_sft_invalsi_ss_data_collective=open-data>
#'
#' @details
#' Numeric variables provided are:
#' \itemize{
#'   \item \code{Average_percentage_score} Average direct score (percentage of sufficient tests)
#'   \item \code{Std_dev_percentage_score} Standard deviation of the direct score
#'   \item \code{WLE_average_score} Average WLE score. The WLE score is calculated through the Rasch's psychometric model and is suitable for middle and high schools in that it is cleaned from the effect of cheating  (which would affect both the average score and the score variability). By construction it has a mean around 200 points.
#'   \item \code{Std_dev_WLE_score} Standard deviation of the WLE score. By construction it ranges around 40 points at the school level.
#'   \item \code{Students_coverage} Students coverage percentage
#' }
#' Additional numeric variables, not always available for all observational units, are:
#' \itemize{
#'   \item Mean and SD of ESCS indicator
#'   \item \code{First-Fifth_Level}: Distribution of the proficiency level of students
#'   \item \code{Targets_percentage}: Percentage of students reaching targets
#' }
#' Numeric codes \code{888} and \code{999} denote not applicable and not available fields respectively.
#'
#' If \code{multiple_out == TRUE}, provides the following datasets:
#' \itemize{
#'   \item \code{Municipality_data}: LAU-level data
#'   \item \code{Province_data}: NUTS-3-level data
#'   \item \code{Region_data}: NUTS-2-level data
#'   \item \code{LLS_data}: data at the level of local labour systems
#'   (Sistemi Locali del Lavoro; see \href{https://www.istat.it/statistiche-per-temi/focus/informazioni-territoriali-e-cartografiche/statistiche-sul-territorio/sistemi-locali-del-lavoro-e-distretti-industriali/}{ISTAT webpage} for details)
#'   \item \code{Inner_Areas_2021_data} aggregated data for inner areas according to the 2020 taxonomy
#'   \item \code{Inner_Areas_2014_data} aggregated data for inner areas according to the former 2014 taxonomy
#'   \item \code{Macroarea_data} data aggregated for North-West, North-East, Center, South and Islands
#' }
#'
#' @examples
#' \donttest{
#' Data_Invalsi <- Get_Invalsi_IS(level = "NUTS-3", autoAbort = TRUE, verbose = FALSE)
#' Data_Invalsi$Province_data
#' }
#'
#'
#' @export


Get_Invalsi_IS <- function(level = "LAU", verbose = TRUE, show_col_types = FALSE,
                           multiple_out = TRUE, autoAbort = FALSE, category = FALSE){

  if(!Check_connection(autoAbort)) return(NULL)

  starttime <- Sys.time()
  if (verbose) cat("Retrieving Invalsi census data ... \n")
  url.invalsi <- "https://serviziostatistico.invalsi.it/en/invalsi_ss_data/punteggi-e-percentuale-di-studenti-nei-livelli-di-competenza-per-ripartizioni-territoriali-e-caratteristiche-di-contesto/"

  homepage <- tryCatch({
    httr::content(httr::GET(url.invalsi))
  }, error = function(e){
    NULL
  })
  if(!is.null(homepage)){
    name_pattern <- "report_generale"
  } else {
    message("It seems INVALSI changed address. \n Please contact the maintainer and ask him to fix.")
    return(NULL)
  }

  links <- homepage %>% rvest::html_nodes("a") %>% rvest::html_attr("href")
  # Warning: this works for the current website architecture - may
  # change in the future
  links <- links[grepl(paste0(name_pattern), links)]
  link <- unique(links[grepl("zip", links)])
  if(length(link)==0L){
    message("It seems INVALSI changed address. \n Please contact the maintainer and ask him to fix.")
    return(NULL)
  }
  base.url <- dirname(url.invalsi)
  file.url <- xml2::url_absolute(link, base.url)
  temp1 <- tempfile(fileext=".zip")
  timeout.default <- options("timeout")$timeout
  on.exit(options(timeout = timeout.default))
  options(timeout = max(timeout.default, 180))

  attempt <- 0
  x <- NULL
  while(is.null(x)){
    x <- tryCatch({
      utils::download.file(url = file.url, destfile = temp1, mode = "wb", quiet = TRUE)},
      error = function(e) {
        NULL
      })
    if(is.null(x)){
      attempt <- attempt + 1
      message("Error in file downloading, ", 10-attempt, " attempts left.\n")
    }
    if(attempt >= 10){
      unlink(temp1, recursive = TRUE, force = TRUE)
      message("Unable to download data; max attempts reached.")
      message("Abort. Please contact the maintiner and ask him to fix.\n")
      return(NULL)
    }
  }

  options(timeout = timeout.default)

  attempt <- 0
  Invalsi_IS <- NULL
  while(is.null(Invalsi_IS)){
    zip.content <- utils::unzip(temp1, list = TRUE)
    content.csv <- zip.content[which(grepl("\\.csv$", zip.content$Name, ignore.case = TRUE)),]
    if(nrow(content.csv) > 0L){
      csv.max <- which.max(zip.content$Length)
      filename <- zip.content$Name[csv.max]
      con <- unz(temp1, filename)
      Invalsi_IS <- tryCatch({
        readr::read_delim( con, delim = ";", na = c("", " ", "N.A."),
                           locale = readr::locale(decimal_mark = ",",
                                                  encoding = "ISO-8859-1"),
                           show_col_types = show_col_types )},
        error = function(e){
          message("For some reason, R cannot open the .csv file... \n")
          return(NULL)
        })
    }
    if(is.null(Invalsi_IS)) {
      message("Retry operation. ", 10-attempt, " attempts left... \n")
      attempt <- attempt + 1
    }
    if(attempt >= 10) {
      message("Impossible to read Invalsi file; max attempts reached. Abort. \n",
      "Please contact the maintainer and ask him to fix. \n")
      unlink(temp1, recursive = TRUE, force = TRUE)
      return(NULL)
    }
  }

  names(Invalsi_IS) <- names(Invalsi_IS) %>%
    stringr::str_replace_all("Aggregato_territoriale","Territorial_aggregate") %>%
    stringr::str_replace_all("aggregazione","Level") %>%
    stringr::str_replace_all("Denominazione", "Description") %>%
    stringr::str_replace_all("Codice_provincia", "Province_code") %>%
    stringr::str_replace_all("Nome_provincia", "Province_description") %>%
    stringr::str_replace_all("Sigla_provincia", "Province_initials") %>%
    stringr::str_replace_all("id", "ID") %>%
    stringr::str_replace_all("Comune", "Municipality_description") %>%
    stringr::str_replace_all("Grado", "Grade") %>%
    stringr::str_replace_all("Materia", "Subject") %>%
    stringr::str_replace_all("Anno", "Year") %>%
    stringr::str_replace_all("Punteggio_wle_medio", "WLE_average_score") %>%
    stringr::str_replace_all("punteggio_medio_wle", "WLE_average_score") %>%
    stringr::str_replace_all("deviazione_standard_wle", "Std_dev_WLE_score") %>%
    stringr::str_replace_all("Dev_Std_Punteggio_wle", "Std_dev_WLE_score") %>%
    stringr::str_replace_all("perc_copertura_stu", "Students_coverage") %>%
    stringr::str_replace_all("Punteggio_percentuale_medio", "Average_percentage_score") %>%
    stringr::str_replace_all("Punteggio_medio$", "Average_percentage_score") %>%
    stringr::str_replace_all("punteggio_medio$", "Average_percentage_score") %>%
    stringr::str_replace_all("deviazione_standard$", "Std_dev_percentage_score") %>%
    stringr::str_replace_all("Deviazione_standard$", "Std_dev_percentage_score") %>%
    stringr::str_replace_all("Dev_Std_Punteggio_Percentuale", "Std_dev_percentage_score") %>%
    stringr::str_replace_all("media_ESCS", "ESCS_average") %>%
    stringr::str_replace_all("Dev_Std_ESCS", "ESCS_Std_dev") %>%
    stringr::str_replace_all("Primo_Livello", "First_level") %>%
    stringr::str_replace_all("Secondo_Livello", "Second_level") %>%
    stringr::str_replace_all("Terzo_Livello", "Third_level") %>%
    stringr::str_replace_all("Quarto_Livello", "Fourth_level") %>%
    stringr::str_replace_all("Quinto_Livello", "Fifth_level") %>%
    stringr::str_replace_all("Perc_traguardi", "Targets_percentage") %>%
    stringr::str_replace_all("Percentuale_partecipazione", "Students_coverage")

  Invalsi_IS$Subject <- Invalsi_IS$Subject %>%
    stringr::str_replace_all("Matematica", "Mathematics") %>%
    stringr::str_replace_all("Italiano", "Italian") %>%
    stringr::str_replace_all("Inglese", "English")




  Invalsi_IS <- Invalsi_IS %>%
    dplyr::mutate(Description = stringr::str_to_title(.data$Description))

  # This is a loss in information next package versions shall overcome:
  if(!category){
    Invalsi_IS <- Invalsi_IS %>%
      dplyr::filter(.data$Level == "Totale") %>%
      dplyr::select(-.data$Level)
  } else{
    categories <- unique(Invalsi_IS$Level)
    categories_mat <- apply(cbind(c(1:length(categories)), categories), 1,
                            function(X) paste0(X, collapse = ": "))
    cat("Please, choose one or more numbers \n corresponding to the following categories;
         \n use the semicolon (;) as separator \n",
         paste(categories_mat, collapse = "\n"))
    nn_categories <- as.numeric(unlist(strsplit(readline(prompt = "  > "), split = ";")))
    categories_in <- categories[nn_categories]
    Invalsi_IS <- Invalsi_IS %>%
      dplyr::filter(.data$Level %in% categories_in) %>%
      dplyr::rename( Category = .data$Level)
    }


  dd_mun <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Comune plesso") %>%
    dplyr::rename(Municipality_code = .data$ID) %>%
    dplyr::rename(Municipality_description = .data$Description) %>%
    dplyr::select(-.data$Territorial_aggregate)%>%
    dplyr::mutate(Municipality_code = sprintf("%06d", .data$Municipality_code))
  dd_prov <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Provincia") %>%
    dplyr::rename(Province_code = .data$ID) %>%
    dplyr::rename(Province_description = .data$Description) %>%
    dplyr::select(-.data$Territorial_aggregate)
  dd_reg <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Regione") %>%
    dplyr::rename(Region_code = .data$ID) %>%
    dplyr::rename(Region_description = .data$Description) %>%
    dplyr::select(-.data$Territorial_aggregate)
  dd_LLS <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Sistemi locali del lavoro (SLL)") %>%
    dplyr::rename(LLS_code = .data$ID) %>%
    dplyr::rename(LLS_description = .data$Description) %>%
    dplyr::select(-.data$Territorial_aggregate)
  dd_InnerAreas2014 <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Aree interne SNAI 2014") %>%
    dplyr::rename(Inner_area_code = .data$ID) %>%
    dplyr::rename(Inner_area_description = .data$Description) %>%
    dplyr::select(-.data$Territorial_aggregate) # Not sure this is relevant
  dd_InnerAreas2021 <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Aree interne SNAI 2021") %>%
    dplyr::rename(Inner_area_code = .data$ID) %>%
    dplyr::rename(Inner_area_description = .data$Description) %>%
    dplyr::select(-.data$Territorial_aggregate)
  dd_macroarea <- Invalsi_IS %>% dplyr::filter(.data$Territorial_aggregate == "Area geografica") %>%
    dplyr::rename(Macroarea_code = .data$ID) %>%
    dplyr::rename(Macroarea_description = .data$Description) %>%
    dplyr::select(-.data$Territorial_aggregate) %>%
    dplyr::mutate(dplyr::across(
      .data$Macroarea_description, ~ stringr::str_replace_all(.x, c(
         "Nord Ovest" = "North-West",
        "Nord Est" = "North-East",
        "Centro" = "Center",
        "Sud E Isole" = "South and Islands",
        "Sud" = "South"))))

  if(multiple_out){
    Invalsi_IS <- list(Municipality_data = dd_mun, Province_data = dd_prov,
                       Region_data = dd_reg, LLS_data = dd_LLS,
                       Inner_Areas_2014_data = dd_InnerAreas2014,
                       Inner_Areas_2021_data = dd_InnerAreas2021,
                       Macroarea_data = dd_macroarea)
  } else{
      if (toupper(level) %in% c("MUNICIPALITY", "MUN", "LAU", "NUTS-4")){
        Invalsi_IS <- dd_mun
      } else if(toupper(level) %in% c("PROV", "PROVINCE", "NUTS-3")){
        Invalsi_IS <- dd_prov
      } else if(toupper(level) %in% c("REG", "REGION", "NUTS-2")){
        Invalsi_IS <- dd_reg
      } else if(toupper(level) %in% c("INNERAREAS", "INNER_AREAS", "INNERAREAS2020", "INNERAREAS20",
                                      "INNER_AREAS2020", "INNER_AREAS20", "INNERAREAS_2020", "INNERAREAS_20",
                                      "INNER_AREAS_2020", "INNER_AREAS_20", "INNERAREAS2021", "INNERAREAS21",
                                      "INNER_AREAS2021", "INNER_AREAS21", "INNERAREAS_2021", "INNERAREAS_21",
                                      "INNER_AREAS_2021", "INNER_AREAS_21")){
        Invalsi_IS <- dd_InnerAreas2021
      } else if(toupper(level) %in% c("INNERAREAS_OLD", "INNER_AREAS_OLD",  "INNERAREAS2014", "INNERAREAS14",
                                      "INNER_AREAS2014", "INNER_AREAS14", "INNERAREAS_2014", "INNERAREAS_14",
                                      "INNER_AREAS_2014", "INNER_AREAS_14")){
        Invalsi_IS <- dd_InnerAreas2014
      } else if(toupper(level) %in% c("LWS", "LOCAL WORKING SYSTEM", "LOCAL WORK SYSTEM", "LOCAL LABOUR SYSTEM",
                                      "LOCAL LABOR SYSTEM",
                                      "LLS", "SISTEMA LOCALE DEL LAVORO")){
        Invalsi_IS <- dd_LLS
      } else if(toupper(level) %in% c("MACROAREA", "NUTS-1", "NCS", "NORTHCENTERSOUTH", "NORTHCENTRESOUTH")){
        Invalsi_IS <- dd_macroarea
      } else{
        message("Level not recognised: `", level, "`; municipality data returned by default. Sorry. \n")
        Invalsi_IS <- dd_mun
      }
  }


  unlink(temp1, recursive = TRUE, force = TRUE)


  endtime <- Sys.time()
  if(verbose) {
    cat(paste("Total running time to retrieve Invalsi", level, "data:",
              paste(round(difftime(endtime, starttime, units="secs") ,2)), "seconds \n"))
  }
  return(Invalsi_IS)
}

#' Subset of the students and classes counts in school year 2022/23
#'
#'
#' This dataframe includes students and classes counts for the schools from four regions: Molise, Campania, Apulia and Basilicata.
#' The whole dataset can be retrieved with the command \code{Get_nstud(2023, filename = "ALUCORSOINDCLASTA")}
#' @seealso \code{\link{Get_nstud}}
#'
#' @format ## `example_input_nstud23`
#' A data frame with 21208 rows and 7 columns:
#' \itemize{
#'   \item \code{Year} Numeric; the school year.
#'   \item \code{School_code} Character; the school ID.
#'   \item \code{Order} Character; the school order, either primary, middle or high school.
#'   \item \code{Grade} Numeric; the school grade.
#'   \item \code{Classes} Numeric; the count of classes of a given grade in each school
#'   \item \code{Male_students} Numeric; the count of male students in all classes of a given educational grade in each school
#'   \item \code{Female_students} Numeric; the count of female students in all classes of a given educational grade in each school
#' }
#' @source <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Studenti&datasetId=DS0030ALUCORSOINDCLASTA>
#'
"example_input_nstud23"

#' Subset of the Invalsi scores in school year 2022/23
#'
#'
#' This dataframe includes the Invalsi scores of the schools from four regions: Molise, Campania, Apulia and Basilicata, for the school year 2022/23.
#' The whole dataset can be retrieved with the command \code{Get_Invalsi_IS(level = "NUTS-3")}
#' @seealso \code{\link{Get_Invalsi_IS}}
#'
#' @format ## `example_Invalsi23_prov`
#' A data frame with 240 rows and 11 columns:
#' \itemize{
#'   \item \code{Year} Character; the school year.
#'   \item \code{Grade} Numeric; the school grade; only includes the school grades subjected to the Invalsi survey. Either 2, 5, 8, 10 or 13.
#'   \item \code{Subject} Character; the school subject in which the test is taken; either Italian, Mathematics, English reading or English listening.
#'   \item \code{Province_code} Numeric; the NUTS-3 administrative code.
#'   \item \code{Province_initials} Character; abbreviated NUTS-3 denomination.
#'   \item \code{Province_description} Character; the province (NUTS-3 administrative level) denomination.
#'   \item \code{Average_percentage_score} Numeric; the province-level percentage of sufficient tests, only for primary schools; ranges 0-100.
#'   \item \code{Std_dev_percentage_score} Numeric; the standard deviation of the percentage of sufficient tests, only for primary schools.
#'   \item \code{WLE_average_score} Numeric; the province-level average WLE (Weighted Likelihood Estimator) score.
#'   \item \code{Std_dev_WLE_score} Numeric; the standard deviation of WLE scores.
#'   \item \code{Students_coverage} Numeric; the percentage of students for which the Invalsi tests are reported.
#' }
#' @source <https://serviziostatistico.invalsi.it/en/archivio-dati/?_sft_invalsi_ss_data_collective=open-data>
#'
#'
#'
"example_Invalsi23_prov"

#' Clean and convert the raw school buildings data to Boolean variables
#'
#' @description This function cleans the output of the \code{\link{Get_DB_MIUR}} function from missing values in two steps:
#' \itemize{
#' \item First, it deletes both the columns exceeding a threshold of missing values (1000 by default) and the columns that cannot be converted into Boolean variables
#' \item Then, it deletes the rows in which missing values remain
#' }
#'
#'  Finally, the remaining data are converted into Boolean variables.
#'  It is possible to keep track of the deleted rows.
#'
#' @param data Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. Input data obtaned through the function \code{\link{Get_DB_MIUR}}.
#' If \code{NULL} it will be downloaded automatically with the appropriate arguments, but not saved in the global environment. \code{NULL} by default.
#' @param cutout Character. The columns to cut out. If \code{NULL}, it will be determined automatically.
#'  \code{NULL} by default.
#' @param col_cut_thresh Numeric. The threshold of missing values allowed for each variable.
#' If a variable as a higher number of missing observations, then it is cut out. \code{1000} by default.
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. TRUE by default.
#' @param track_deleted Logical. If \code{TRUE}, the function returns the names of the school not included in the output dataframe. \code{TRUE} by default.
#' @param autoAbort Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#'
#'
#' @return If \code{track_deleted == TRUE}, An object of class \code{list} including two objects:
#' \itemize{
#'   \item \code{$data}: object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, the output dataframe. All variables besides the first 8 ones (which identify the record) are numeric.
#'   \item \code{$deleted}: character. The school codes corresponding to deleted rows
#'
#' }
#'
#' If \code{track_deleted == FALSE}, the output is only the first element of the list.
#'
#'
#'
# @examples
#'
#'
#'
# library(magrittr)
# input_DB23_MIUR <- Get_DB_MIUR(2023, verbose = FALSE)
# DB23_MIUR_Bool <- input_DB23_MIUR %>% SchoolDataIT:::Util_DB_MIUR_bool(track_deleted = FALSE)
# DB23_MIUR[,-c(1,4,6,9)]
#'
#'
#'
#' @keywords internal



Util_DB_MIUR_bool <- function(data = NULL, cutout = NULL, col_cut_thresh = 10^3,
                              verbose = TRUE, track_deleted = TRUE, autoAbort = autoAbort, ...){

  starttime <- Sys.time()

  init.cutout = c("Address_type","Address_name","Civic_number", "Building_status")
  pattern.out = c("NON DEFINITO","-", "Informazione assente", "Non Comunicato", "ND", "Non richiesto", "NA")

  while(is.null(data)){
    data <- Get_DB_MIUR(autoAbort = autoAbort, verbose = verbose, ...)
    if(is.null(data)){
      holdOn <- ""
      message("Error during school buildings DB retrieving. Would you abort the whole operation or retry?",
              "    - To abort the operation, press `A` \n",
              "    - To retry data retrieving, press any other key \n")
      holdOn <- readline(prompt = "    ")
      if(toupper(holdOn) == "A"){
        cat("You chose to abort the operation \n")
        return(NULL)
      } else {
        cat("You chose to retry \n")
      }
    }
  }

  if(is.null(cutout)){
    cutout <- cutout.default(data, startcol = 10,
                             init.cutout = init.cutout,
                             pattern.out = pattern.out)
  }

  m <-  sum(names(data)[c(1:9)] %in% init.cutout)
  input <- data %>% Clean_DB(cutout = cutout, col_cut_thresh = col_cut_thresh,
                             IDcol = c(2,5), startcol = 10,
                             init.cutout = init.cutout,
                             pattern.out = pattern.out,
                             track_deleted = track_deleted, verbose = verbose)

  if(track_deleted == TRUE){
    DB <- input$data
  } else DB <- input

  DB_repl <- lapply(DB[,-c(1:(9 - m))], function(x){
    gsub(
      "^SI$", 1, gsub(
        "^NO$", 0, gsub(
          "^Esiste$", 1, ignore.case = TRUE, gsub(
            "^Non Esiste$", 0, ignore.case = TRUE, gsub(
              "^IN PARTE$", 1, ignore.case = TRUE, gsub(
                "^ND$", 0, ignore.case = TRUE, gsub(
                  "^Non Definito$", 0, ignore.case = TRUE, gsub(
                    "^Non Comunicato$", 0, ignore.case = TRUE, gsub(
                      "^Non Richiesto$", 0, ignore.case = TRUE, gsub(
                        "^-$", 0 , gsub("^NA$", 0 , x)))))))))))})
  DB_repl <- lapply(DB_repl,  FUN=as.numeric ) %>% as.data.frame()
  DB[-c(1:(9 - m))] <- DB_repl


  DB <- DB %>%
    dplyr::mutate(Province_code = as.numeric(substr(
      .data$Municipality_code, 1, nchar(.data$Municipality_code)-3)) ) %>%
    dplyr::relocate(.data$Province_code, .after = "Municipality_description")

  if(track_deleted == TRUE){
    res <- list(data = DB, deleted = input$deleted)
  } else res <- DB

  endtime <- Sys.time()
  if(verbose){
    cat("Total running time needed to convert data to Boolean variables:",
        round(difftime(endtime, starttime, units="secs") ,2), "seconds \n"  )
  }

  return(res)
}

#' @keywords internal
#'

filterCommonRows <- function(df1, df2, verbose = TRUE){
  nrow.old <- nrow(df1)
  df1 <- df1 %>% dplyr::filter(.data$School_code %in% df2$School_code)
  nrow.new <- nrow(df1)
  args <- as.list(match.call())[-1]
  args$df1 <- as.character(args$df1)
  args$df2 <- as.character(args$df2)
  if(verbose){cat("Deleted ", nrow.old - nrow.new, " rows of ",
                  args$df1, " not included in ", args$df2, " \n")}
  return(df1)
}

#' Association of the municipality code to a subset of public schools 2022/23
#'
#'
#' This list maps the IDs of the schools from four regions (Molise, Campania, Apulia and Basilicata) to the corresponding LAU codes.
#' The whole dataset can be retrieved with the command \code{Get_School2mun(2023)}
#' @seealso \code{\link{Get_School2mun}}
#'
#' @format ## `example_School2mun23`
#' A list of four elements
#' \itemize{
#'   \item \code{Registry_from_buildings} A data frame of 5527 rows and 5 columns, including the schools listed in the buildings registry.
#'   \item \code{Registry_from_registry} A data frame of 5929 rows and 5 columns, including the schools listed in the schools registry.
#'   \item \code{Any} A data frame of 5954 rows and 5 columns, including schools listed in any of the registryes
#'   \item \code{Both} A data frame of 5510 rows and 5 columns, including schools listed in both registries
#' }
#' For each element, rows correspond to school IDs; the columns are:
#' \itemize{
#'   \item \code{School_code} Character; the school ID.
#'   \item \code{Province_code} Numeric; the NUTS-3 administrative code.
#'   \item \code{Province_initials} Character; abbreviated NUTS-3 denomination.
#'   \item \code{Municipality_code} Character; the ISTAT LAU (municipality) ID.
#'   \item \code{Municipality_description} Character; the municipality name.
#' }
#'
#'
#' @source See:
#' \itemize{
#'   \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0101EDIANAGRAFESTA2021> For Buildings registry (2021 onwards);
#'   \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0200EDIANAGRAFESTA> For Buindings registry(until 2019);
#'   \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Scuole> For Schools registry
#'  }
#'
"example_School2mun23"

#' Clean the raw dataframe of the number of students and arrange it in a wide format
#'
#' @description  This function rearranges the output of the \code{\link{Get_nstud}} function in such a way to represent the
#' counts of students and, if required, either the number of students by class and number of classes, or
#' the counts of students per school timetable (running time) in a unique observation per school.
#' If the focus is on class size, this function  firstly cleans the data from the outliers in terms of
#' average number of students by class at the school level and imputates the number of classes to 1 when missing.
#'
#'
#'
#'
#'
#' @param data Object of class \code{list}, including two objects of class \code{tbl_df},  \code{tbl} and \code{data.frame}, obtainded as output of the \code{\link{Get_nstud}} function with the default \code{filename} parameter.
#' If \code{NULL}, the function will download it automatically but it will not be saved in the global environment. \code{NULL} by default.
#' @param missing_to_1 Logical. If focus is on class size, whether the number of classes should be imputed to 1 when it is missing and the number of students is below a threshold (argument \code{nstud_imputation_thresh}). \code{TRUE} by default.
#' @param nstud_imputation_thresh Numeric. If focus is on class size, the minimum threshold below which the number of classes is imputed to 1 if missing, if \code{missing_to_1 == TRUE}.
#'  E.g. if the threshold is 19, for all the schools in which there are 19 or less students in a given grade but the number of classes for that grade is missing, the number of classes is imputated to 1. \code{19} by default.
#' @param UB_nstud_byclass Numeric. Either a unique value for all school orders, or a vector of three order-specific values in the order: primary, middle, high.
#' If focus is on class size, the upper limit of the acceptable school-level (if \code{filter_by_grade == FALSE}) or grade-level (otherwise) average of the number of students by class.
#' If a whole school or any grade in a school respectively has a higher number of students by class, the record is considered an outlier and filtered out. \code{99} by default, i.e. no restriction is made.
#'  Please notice that boundaries are included in the acceptance interval.
#' @param LB_nstud_byclass Numeric. Either a unique value for all school orders, or a vector of three order-specific values in the order: primary, middle, wide.
#' If focus is on class size, the lower limit of the acceptable school-level (if \code{filter_by_grade == FALSE}) or grade_level (otherwise) average of the number of students by class.
#'  If a whole school or any grade in a school respectively has a smaller number of students by class, the record is considered an outlier and filtered out. \code{1} by default.
#'  Please notice that boundaries are included in the acceptance interval.
#' @param filter_by_grade Logical. If focus is on class size, whether to remove all school grades with average class size outside of the acceptance boundaries. \code{FALSE} by default.
#' @param UB_nstud_byclass_grade Numeric. IF \code{filter_by_grade == TRUE}, the upper limit of the acceptable grade-level average class size.
#' If \code{NULL} it is set equal to \code{UB_nstud_byclass}. \code{NULL} by default.
#' @param LB_nstud_byclass_grade Numeric. IF \code{filter_by_grade == TRUE}, the lowrer limit of the acceptable grade-level average class size.
#'  If \code{NULL} it is set equal to \code{LB_nstud_byclass}. \code{NULL} by default.
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param autoAbort Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param ... Arguments to \code{\link{Get_nstud}}, needed if \code{data} is not provided.
#'
#'
#'
#' @return An object of class  \code{tbl_df}, \code{tbl} and \code{data.frame}
#'
#'
#' @details In the example, we compare the dataframe obtained with the default settings
#'  and the one imposed setting narrow inclusion criteria
#'
#' @examples
#'
#'
#' nstud.default <- Util_nstud_wide(example_input_nstud23)
#'
#'
#' nstud.narrow <- Util_nstud_wide(example_input_nstud23,
#'   UB_nstud_byclass = 35, LB_nstud_byclass = 5 )
#'
#' nrow(nstud.default)
#' nrow(nstud.narrow)
#'
#' nstud.default
#'
#' summary(nstud.default)
#'
#'
#' @export


Util_nstud_wide <- function(data = NULL, missing_to_1 = FALSE,
                            nstud_imputation_thresh = 19,
                            UB_nstud_byclass = 99, LB_nstud_byclass = 1,
                            filter_by_grade = FALSE,
                            UB_nstud_byclass_grade = NULL,
                            LB_nstud_byclass_grade = NULL,
                            verbose = TRUE, autoAbort = FALSE, ...){

  options(dplyr.summarise.inform = FALSE)

  while (is.null(data)){
    if(verbose) cat("Downloading input data \n")
    #if(missing_to_1){
    #  filename <- c("ALUCORSOETASTA", "ALUCORSOINDCLASTA")
    #} else {
    #  filename <- "ALUCORSOINDCLASTA"
    #}
    data  <- Get_nstud(verbose = verbose, autoAbort = autoAbort, ...)
    if(is.null(data)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during students counts retrieving. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  if(is.data.frame(data) || length(data) == 1L ||
     (!is.data.frame(data) && !"ALUCORSOINDCLASTA" %in% names(data))) missing_to_1 <- FALSE

  if (is.data.frame(data)){
    nstud.wide <- data %>%
      dplyr::mutate(Students = .data$Male_students + .data$Female_students) %>%
      dplyr::select(-.data$Male_students, -.data$Female_students)
  }  else if(length(data) == 1L){
    nstud.wide <- data[[1L]] %>%
      dplyr::mutate(Students = .data$Male_students + .data$Female_students) %>%
      dplyr::select(-.data$Male_students, -.data$Female_students)
  } else {
    data <- data %>% lapply(function(x){
      if(! "ID" %in% colnames(data)){
        tidyr::unite(x, "ID", c(.data$School_code, .data$Grade), remove = FALSE)
      }} )
    nstud.wide <- data[names(data)[which(names(data)!="ALUCORSOETASTA")]][[1]] %>%
      dplyr::mutate(Students = .data$Male_students + .data$Female_students) %>%
      dplyr::select(-.data$Male_students, -.data$Female_students)

    data$ALUCORSOETASTA <- data$ALUCORSOETASTA %>% dplyr::group_by(
      .data$Year, .data$ID, .data$School_code, .data$Order, .data$Grade) %>%
      dplyr::summarise(Students = sum(.data$Students)) %>% dplyr::ungroup()

    probl.ID <- data$ALUCORSOETASTA %>% dplyr::filter(! .data$ID %in% nstud.wide$ID) %>%
      dplyr::select(.data$ID) %>% unique() %>% unlist() %>% as.vector()

    if(missing_to_1){

      if(verbose){
        cat("Imputating missing number of classes to 1 for school years with ",
            nstud_imputation_thresh, "students or less\n")
      }

      probl <- data$ALUCORSOETASTA %>%
        dplyr::filter(.data$ID %in% probl.ID & .data$Students <= nstud_imputation_thresh) %>%
        dplyr::mutate(Classes = 1) %>% dplyr::relocate(.data$Students, .after ="Classes")
      nrow.old <- nrow(nstud.wide)
      nstud.wide <- rbind(nstud.wide, probl)
      nrow.new <- nrow(nstud.wide)
      if(verbose) {
        message(paste("Missing number of classes imputated to 1 for",
                      nrow.new - nrow.old, "schools"))
      }
    }
    nstud.wide <- nstud.wide %>%
      dplyr::select(-.data$ID)
  }

  if("Classes" %in% names (nstud.wide)){

    if(length(UB_nstud_byclass) != 3L){
      UB_nstud_byclass <- rep(UB_nstud_byclass[1L], 3)
    }
    if(length(LB_nstud_byclass) != 3L){
      LB_nstud_byclass <- rep(LB_nstud_byclass[1L], 3)
    }

    if(filter_by_grade){
      if(is.null(LB_nstud_byclass_grade)) LB_nstud_byclass_grade <- LB_nstud_byclass
      if(is.null(UB_nstud_byclass_grade)) UB_nstud_byclass_grade <- UB_nstud_byclass
      nrow.old <- nrow(nstud.wide)
      nstud.wide <- nstud.wide %>%
        dplyr::filter(.data$Order == "Primary" &
                        dplyr::between(.data$Students/.data$Classes,
                                       LB_nstud_byclass_grade[1L],
                                       UB_nstud_byclass_grade[1L]) |
                        .data$Order == "Middle" &
                        dplyr::between(.data$Students/.data$Classes,
                                       LB_nstud_byclass_grade[2L],
                                       UB_nstud_byclass_grade[2L]) |
                        .data$Order == "High" &
                        dplyr::between(.data$Students/.data$Classes,
                                       LB_nstud_byclass_grade[3L],
                                       UB_nstud_byclass_grade[3L]))
      nrow.new <- nrow(nstud.wide)
      if(verbose & nrow.new < nrow.old){
        message(paste("Filtered out", nrow.old - nrow.new, "schools with either less than",
                      LB_nstud_byclass_grade[1L], "(primary),",
                      LB_nstud_byclass_grade[2L], "(middle),",
                      LB_nstud_byclass_grade[3L], "(high)\n",
                      "or more than",
                      UB_nstud_byclass_grade[1L], "(primary),",
                      UB_nstud_byclass_grade[2L], "(middle),",
                      UB_nstud_byclass_grade[3L], "(high)",
                      "students per class in any grade"))
      }
    }

    nstud.wide <- nstud.wide %>% dplyr::select(-.data$Year) %>%
      dplyr::filter(.data$Order != "Primary" | .data$Grade < 6) %>%
      dplyr::filter(.data$Grade < 14) %>%
      dplyr::mutate(dplyr::across(.data$Grade,  ~paste0("grade_", .data$Grade))) %>%
      tidyr::pivot_wider(names_from = .data$Grade, values_from = c(.data$Classes, .data$Students))

    nn <- paste(rep(c("Students_grade_", "Classes_grade_"),13),
                sort(as.numeric(gsub("\\D", "", names(nstud.wide)[3:28] )) ), sep = "")

    nstud.wide <- nstud.wide[,c(1,2,match(nn, names(nstud.wide)))]

    nstud.wide[is.na(nstud.wide)] <- 0

    nstud.wide <- nstud.wide %>%
      dplyr::mutate(Tot_Students = rowSums(nstud.wide[,which(grepl("Students", names(nstud.wide)))])) %>%
      dplyr::mutate(Tot_Classes =rowSums(nstud.wide[,which(grepl("Classes", names(nstud.wide)))]))

    for (i in (1:14)){
      j <- 2*i + 1
      nstud.wide <- nstud.wide %>%
        dplyr::mutate(xx = as.numeric(unlist(dplyr::select(nstud.wide, j)/dplyr::select(nstud.wide,j+1) ) ) )
      names(nstud.wide)[ncol(nstud.wide)] <- paste("Students_per_class_", ifelse(i<14, i,"Tot"), sep = "")
    }
    nstud.wide[is.na(nstud.wide)] <- 0

    for (i in c(1:13)){
      j <- i + 30
      k <- 3*i + 1
      nstud.wide <- nstud.wide %>% dplyr::relocate(j, .after = k)
    }

    nrow.old <- nrow(nstud.wide)
    nstud.wide <- nstud.wide %>%
      dplyr::filter(.data$Order == "Primary" &
                      dplyr::between(.data$Students_per_class_Tot,
                                     LB_nstud_byclass[1L],
                                     UB_nstud_byclass[1L]) |
                      .data$Order == "Middle" &
                      dplyr::between(.data$Students_per_class_Tot,
                                     LB_nstud_byclass[2L],
                                     UB_nstud_byclass[2L]) |
                      .data$Order == "High" &
                      dplyr::between(.data$Students_per_class_Tot,
                                     LB_nstud_byclass[3L],
                                     UB_nstud_byclass[3L]))
    nrow.new <- nrow(nstud.wide)
    if(verbose & nrow.new < nrow.old){
      message(paste("Filtered out", nrow.old - nrow.new, "schools with either less than",
                    LB_nstud_byclass[1L], "(primary),",
                    LB_nstud_byclass[2L], "(middle),",
                    LB_nstud_byclass[3L], "(high)\n",
                    "or more than",
                    UB_nstud_byclass[1L], "(primary),",
                    UB_nstud_byclass[2L], "(middle),",
                    UB_nstud_byclass[3L], "(high)",
                    "students per class on average"))
    }
  } else if("Running_time" %in% names(nstud.wide)){
    nstud.wide <- nstud.wide %>% dplyr::select(-.data$Year) %>%
      dplyr::filter(.data$Order != "Primary" | .data$Grade < 6) %>%
      dplyr::filter(.data$Grade < 14) %>%
      dplyr::mutate(dplyr::across(.data$Grade,  ~ paste0("Students_G_", .data$Grade))) %>%
      tidyr::pivot_wider(names_from = c(.data$Grade, .data$Running_time), values_from = c(.data$Students))

    nn <- paste(rep("Students_G_", 8),
                sort(as.numeric(gsub("\\D", "", names(nstud.wide)[3:18] )) ),
                c(rep(c("_Ordinary", "_Full_time"),5),  rep(c("_Ordinary", "_Music_oriented"),3)),
                sep = "")

    nstud.wide <- nstud.wide[,c(1,2,match(nn, names(nstud.wide)))]

    nstud.wide[is.na(nstud.wide)] <- 0

    nstud.wide <- nstud.wide %>%
      dplyr::mutate(Tot_Students_ordinary =
                      rowSums(nstud.wide[which(grepl("Ordinary", names(nstud.wide)))])) %>%
      dplyr::mutate(Tot_Students_full_time =
                      rowSums(nstud.wide[which(grepl("Full_time", names(nstud.wide)))])) %>%
      dplyr::mutate(Tot_Students_music_oriented =
                      rowSums(nstud.wide[which(grepl("Music_oriented", names(nstud.wide)))])) %>%
      dplyr::mutate(Tot_Students = .data$Tot_Students_ordinary +
                      .data$Tot_Students_full_time +
                      .data$Tot_Students_music_oriented)
  }


  return(nstud.wide)
}

#' Check how many schools in the school registries are included in the students count dataframe
#'
#' @description  This function checks for which schools listed in the two registries (the buildings registry and the properly said schools registry)
#' the count of students is available. The first registry is referred to as as \code{Registry_from_buildings} and the second one as \code{Registry_from_registry}.
#'
#'
#'
#'
#'
#' @param data Object of class  \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the \code{\link{Util_nstud_wide}} function
#' @param Year Numeric or character value. Reference school year.
#' Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023}.
#' @param cutout Character. The types of schools not to be taken into account (because not relevant or because they are out of scope in the students number section). By default \code{c("IC", "IS", "NR")} , i.e. the check does not regard comprehensive institutes, superior institutes, and all the schools that cannot be classified either as primary, middle or high schools.
#' @param ggplot Logical. If \code{TRUE}, the function displays a static map of the availability of the students number by province (but it does not save the ggplot object into the global environment). \code{TRUE} by default.
#' @param toplot_registry Character. If the \code{ggplot} option is chosen, the students number availability of which registry must be plotted; either \code{"Registry_from_buildings"}, \code{"Registry_from_registry"}, \code{"Any"} or \code{"Both"}. \code{"Any"} by default.
#' @param InnerAreas Logical. Whether it must be checked if municipalities belong to inner areas or not. \code{TRUE} by default.
#' @param ord_InnerAreas Logical. Whether the inner areas classification should be treated as an ordinal variable rather than as a categorical one (see \code{\link{Get_InnerAreas}} for the classification).
#' \code{FALSE} by default.
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param input_Registry Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_Registry}}
#' The school registry from the registry section.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default
#' @param input_InnerAreas Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#' The classification of peripheral municipalities, obtained as output of the \code{\link{Get_InnerAreas}} function.
#' Needed only if the \code{InnerAreas} option is chosen.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default
#' @param input_Prov_shp Object of class \code{sf}, \code{tbl_df}, \code{tbl}, \code{data.frame}. The relevant shapefile of Italian municipalities, if the \code{ggplot} option is chosen.
#'  If \code{NULL} it is downloaded automatically but not saved in the global environment. \code{NULL} by default.
#' @param input_AdmUnNames Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_AdmUnNames}}
#' The ISTAT file including all the codes and the names of the administrative units for the year in scope.
#' Only needed if the argument input_School2mun is \code{NULL} and has to be computed.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.
#' @param input_School2mun Object of class \code{list} with elements of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_School2mun}}.
#' The mapping from school codes to municipality (and province) codes. If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.
#' @param autoAbort Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#'
#'
#'
#'
#' @return An object of class \code{list} including two elements:
#' \itemize{
#'   \item \code{$Municipality_data}
#'   \item \code{$Province_data}
#' }
#' Both the elements are objects of class \code{list} including four elements:
#'
#' \itemize{
#'   \item \code{$Registry_from_buildings}: object of class of class \code{tbl_df}, \code{tbl} and \code{data.frame}: the availability of the number of students in the schools listed in the buildings section.
#'   \item \code{$Registry_from_registry}: object of class of class \code{tbl_df}, \code{tbl} and \code{data.frame}: the availability of the number of students in the schools listed in the registry section.
#'   \item \code{$Any}: object of class of class \code{tbl_df}, \code{tbl} and \code{data.frame}: the availability of the number of students in the schools listed anywhere.
#'   \item \code{$Both}: object of class of class \code{tbl_df}, \code{tbl} and \code{data.frame}: the availability of the number of students in the schools listed in both sections.
#' }
#'
#' @source See \itemize{
#'   \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0101EDIANAGRAFESTA2021> For Buildings Registry;
#'   \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Scuole&datasetId=DS0400SCUANAGRAFESTAT> For Schools Registry.

#' }
#'
#'
#'
#'
#'
#' @examples
#'
#' nstud23 <- Util_nstud_wide(example_input_nstud23, verbose = FALSE)
#'
#' Util_Check_nstud_availability(nstud23, Year = 2023,
#'   input_Registry = example_input_Registry23, InnerAreas = FALSE,
#'   input_School2mun = example_School2mun23, input_Prov_shp = example_Prov22_shp)
#'
#'
#'
#'
#'
#' @export



Util_Check_nstud_availability <- function(data, Year,
                                          cutout = c("IC", "IS", "NR"), verbose = TRUE,
                                          ggplot = TRUE, toplot_registry = "Any",
                                          InnerAreas = TRUE, ord_InnerAreas = FALSE,
                                          input_Registry = NULL, input_InnerAreas = NULL,
                                          input_Prov_shp = NULL, input_AdmUnNames = NULL,
                                          input_School2mun = NULL, autoAbort = FALSE){

  options(dplyr.summarise.inform = FALSE)

  while(is.null(input_Prov_shp) && ggplot){
    if(verbose) cat("Downloading the shapefile (since ggplot has been required) \n")
    input_Prov_shp <- Get_Shapefile(
      Year = as.numeric(year.patternA(Year))%/%100, level = "NUTS-3", autoAbort = autoAbort, lightShp = TRUE)
    if(is.null(input_Prov_shp)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during shapefile retrieving. Would you abort this element or retry? \n",
                "    - To abort the element, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the element \n")
          ggplot <- FALSE
        } else {
          cat("You chose to retry \n")
        }
      } else ggplot <- FALSE
    }
  }

  while(is.null(input_School2mun)){
    input_School2mun <- Get_School2mun(
      Year = Year, verbose = verbose, input_AdmUnNames = input_AdmUnNames,
      input_Registry = input_Registry, autoAbort = autoAbort)
    if(is.null(input_School2mun)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during mapping schools to municipalities. Would you abort the whole operation or retry? \n",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  while(InnerAreas && is.null(input_InnerAreas)){
    if(verbose) cat("Retrieving the classification of inner areas \n")
    input_InnerAreas <- Get_InnerAreas(autoAbort = autoAbort)
    if(is.null(input_InnerAreas)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during inner areas retrieving. Would you abort this element or retry? \n",
                "    - To abort the element, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the element \n")
          InnerAreas <- FALSE
        } else {
          cat("You chose to retry \n")
        }
      } else InnerAreas <- FALSE
    }
  }
  if(InnerAreas){
    if(dplyr::between(as.numeric(substr(year.patternA(Year),1,4))+1, 2021, 2027)){
      InnerAreas.R <- input_InnerAreas %>%
        dplyr::select(.data$Municipality_code, .data$Inner_area_code_2021_2027) %>%
        dplyr::mutate(Inner_area = ifelse(
          .data$Inner_area_code_2021_2027 %in% c("A", "B", "C"), 0, 1)) %>%
        dplyr::rename(Inner_area_code = .data$Inner_area_code_2021_2027)
    } else {
      InnerAreas.R <- input_InnerAreas %>%
        dplyr::select(.data$Municipality_code, .data$Inner_area_code_2014_2020) %>%
        dplyr::mutate(Inner_area = ifelse(
          .data$Inner_area_code_2014_2020 %in% c("A", "B", "C"), 0, 1)) %>%
        dplyr::rename(Inner_area_code = .data$Inner_area_code_2014_2020)
    }

    if(ord_InnerAreas){
      InnerAreas.R <- InnerAreas.R %>% dplyr::mutate(
        A_mun = as.numeric(.data$Inner_area_code == "A"),
        B_mun = as.numeric(.data$Inner_area_code == "B"),
        C_mun = as.numeric(.data$Inner_area_code == "C"),
        D_mun = as.numeric(.data$Inner_area_code == "D"),
        E_mun = as.numeric(.data$Inner_area_code == "E"),
        F_mun = as.numeric(.data$Inner_area_code == "F") )
    }
    InnerAreas.R <- InnerAreas.R %>% dplyr::select(-.data$Inner_area_code)
  }

  suppressWarnings(Names.to.find <- data %>% dplyr::select(.data$School_code) %>% unlist() %>% as.vector())

  Schools <- input_School2mun %>% lapply (function(x){
    suppressWarnings(
      x <- x %>%
        dplyr::mutate(Available = as.numeric(.data$School_code %in% Names.to.find)) %>%
        School.order(field = "School_code") %>%
        dplyr::filter(!.data$Order %in% cutout) %>%
        dplyr::filter(!.data$Province_initials %in% c("AO", "BZ", "TN"))
      # Schools in the Valle d'Aosta region have problematic data and are therefore out of scope
    )
    if(InnerAreas) x <- x %>% dplyr::left_join(InnerAreas.R, by = "Municipality_code")
    return(x)
  })

  Municipality_data <- Schools %>% lapply(function(x){
    if(InnerAreas){
      if(ord_InnerAreas){
        suppressWarnings(
          x <- x %>% dplyr::filter(!is.na(.data$Inner_area)) %>%
            dplyr::group_by(.data$Order, .data$Municipality_code, .data$Municipality_description,
                            .data$Province_code, .data$Province_initials) %>%
            dplyr::summarise(
              Available = sum(.data$Available),
              Unavailable = dplyr::n()-sum(.data$Available),
              Inner_area =  mean(.data$Inner_area),
              A_mun = mean(.data$A_mun),
              B_mun = mean(.data$B_mun),
              C_mun = mean(.data$C_mun),
              D_mun = mean(.data$D_mun),
              E_mun = mean(.data$E_mun),
              F_mun = mean(.data$F_mun)) %>%
            dplyr::ungroup () %>%
            dplyr::mutate(Availability = .data$Available/(.data$Available + .data$Unavailable)))
      } else{
        suppressWarnings(
          x <- x %>% dplyr::filter(!is.na(.data$Inner_area)) %>%
            dplyr::group_by(.data$Order, .data$Municipality_code, .data$Municipality_description,
                            .data$Province_code, .data$Province_initials) %>%
            dplyr::summarise(
              Available = sum(.data$Available),
              Unavailable = dplyr::n()-sum(.data$Available),
              Inner_area =  mean(.data$Inner_area)) %>%
            dplyr::ungroup () %>%
            dplyr::mutate(Availability = .data$Available/(.data$Available+.data$Unavailable)))
      }
    } else {
      suppressWarnings(
        x <- x %>% dplyr::group_by(.data$Order, .data$Municipality_code, .data$Municipality_description,
                               .data$Province_code, .data$Province_initials) %>%
          dplyr::summarise(
            Available = sum(.data$Available),
            Unavailable = dplyr::n()-sum(.data$Available)) %>%
          dplyr::ungroup () %>%
          dplyr::mutate(Availability = .data$Available/(.data$Available+.data$Unavailable)))
    }
  })

  Province_data <- Schools %>% lapply( function(x){
    if(InnerAreas){
      if(ord_InnerAreas){
        suppressWarnings(
          x <- x %>% dplyr::filter(!is.na(.data$Inner_area)) %>%
            dplyr::group_by(.data$Order, .data$Province_code, .data$Province_initials) %>%
            dplyr::summarise(
              Available = sum(.data$Available),
              Unavailable = dplyr::n()-sum(.data$Available),
              Inner_area =  mean(.data$Inner_area),
              A_mun = mean(.data$A_mun),
              B_mun = mean(.data$B_mun),
              C_mun = mean(.data$C_mun),
              D_mun = mean(.data$D_mun),
              E_mun = mean(.data$E_mun),
              F_mun = mean(.data$F_mun)) %>%
            dplyr::ungroup () %>%
            dplyr::mutate(Availability = .data$Available/(.data$Available+.data$Unavailable)))
      } else {
        suppressWarnings(
          x <- x %>% dplyr::filter(!is.na(.data$Inner_area)) %>%
            dplyr::group_by(.data$Order, .data$Province_code, .data$Province_initials) %>%
            dplyr::summarise(
              Available = sum(.data$Available),
              Unavailable = dplyr::n()-sum(.data$Available),
              Inner_area =  mean(.data$Inner_area)) %>%
            dplyr::ungroup () %>%
            dplyr::mutate(Availability = .data$Available/(.data$Available+.data$Unavailable)))
      }
    } else{
      suppressWarnings(
        x <- x %>% dplyr::group_by(.data$Order, .data$Province_code, .data$Province_initials) %>%
          dplyr::summarise(
            Available = sum(.data$Available),
            Unavailable = dplyr::n()-sum(.data$Available)) %>%
          dplyr::ungroup() %>%
          dplyr::mutate(Availability = .data$Available/(.data$Available+.data$Unavailable)))
    }
    return(x)
  })

  if(ggplot){
    while (!toplot_registry %in% names(Municipality_data)){
      message(paste("Please, choose a valid register source to plot among:", paste(names(Municipality_data), collapse = ", ")))
      toplot_registry <- readline(prompt = "    ")
    }

    R.Availability <- Province_data[[toplot_registry]] %>%
      dplyr::filter(.data$Order %in% c("Primary", "Middle", "High")) %>%
      dplyr::select(.data$Order, .data$Province_code, .data$Availability)

    dat.plot.long <- input_Prov_shp %>% dplyr::select(.data$COD_PROV) %>%
      dplyr::rename(Province_code = .data$COD_PROV) %>%
      dplyr::left_join(R.Availability, by = "Province_code") %>%
      dplyr::filter(!is.na(.data$Order))

     toplot <- ggplot2::ggplot(dat.plot.long, ggplot2::aes(fill = .data$Availability)) + ggplot2::geom_sf() +
      ggplot2::facet_wrap(~ .data$Order) + ggplot2::labs(title = paste("% Students number coverage by province in year ", Year))

    plot(toplot)
  }

  return(list(Municipality_data = Municipality_data, Province_data = Province_data))
}

#' Subset of the school buildings database in school year 2022/23
#'
#'
#' This dataframe includes the schools directly identifiable as primary, middle or high school, from four regions: Molise, Campania, Apulia and Basilicata.
#' Only the first 35 columns are included. Some strings including accents in fields \code{Other_disturbances_proximity},
#' \code{Other_specific_criticalities} and \code{Other} have been forced to ASCII.
#' The whole dataset can be retrieved with the command \code{Get_DB_MIUR(2023)}
#' @seealso \code{\link{Get_DB_MIUR}}
#'
#' @format ## `example_input_DB23_MIUR`
#' A data frame with 7479 rows and 35 columns:
#' \itemize{
#'   \item \code{Year} Numeric; the school year.
#'   \item \code{School_code} Character; the school ID.
#'   \item \code{Order} Character; the school order, either primary, middle or high school.
#'   \item \code{Reference_institute_code} Character; the ID of the reference institute.
#'   \item \code{Building_code} Character; the building ID; the first 6 digits usually identify the municipality.
#'   \item \code{Municipality_code} Character; the ISTAT LAU (municipality) ID.
#'   \item \code{Municipality_description} Character; the municipality name.
#'   \item \code{Province_initials} Character; abbreviated NUTS-3 denomination.
#'   \item \code{Postal_code} Character; the ZIP code; slightly finer than municipality boundaries. for big municipalities.
#'   \item \code{Context_without_disturbances} Character; whether the school belongs to an environment devoid of disturbances; otherwise, the types of disturbances are listed in columns 11 - 18.
#'   \item \code{Dumps_proximity} Character; whether the school is close to dumps (disturbance element).
#'   \item \code{Pollutant_industries_proximity} Character; whether the school is close to pollutant industries (disturbance element).
#'   \item \code{Pollutant_waters_proximity} Character; whether the school is close to pollutant or stagnant streams or ponds (disturbance element).
#'   \item \code{Air_pollution_sourcer_proximity} Character; whether the school is close to sources of air pollution (disturbance element).
#'   \item \code{Acoustic_pollution_sourcer_proximity} Character; whether the school is close to sources of acoustic pollution (disturbance element).
#'   \item \code{Electromagnetic_radiation_sources_proximity} Character; whether the school is close to sources of electromagnetic radiation (disturbance element).
#'   \item \code{Graveyards_proximity} Character; whether the school is close to a graveyard (disturbance element).
#'   \item \code{Other_disturbances_proximity} Character; other disturbance elements to which the school is close, other than those already listed.
#'   \item \code{School_area_specific_criticalities} Character; whether any specific criticality element occurs inside the school area; specified in columns 20 - 27.
#'   \item \code{Layby absence} Character; whether the access to the area pertaining to the school building lacks a lay-by or pitch (school area criticality element).
#'   \item \code{Unfenced area} Character; whether the school building area lacks fences or enclosures (school area criticality element).
#'   \item \code{Large_traffic} Character; whether the school area is close to large traffic streams (school area criticality element).
#'   \item \code{Railway_traffic} Character; whether the school area is close to railway traffic streams (school area criticality element).
#'   \item \code{Abandoned_industries} Character; whether the school area is located in pre-existences of abandoned industries (school area criticality element).
#'   \item \code{Decayed_urban_area} Character; whether the school belongs or is close to a decayed area (school area criticality element).
#'   \item \code{Risky_industries_proximity} Character; whether the school is close to perilous industrial areas  (school area criticality element).
#'   \item \code{Other_specific_criticalities} Character; specific criticality elements regarding the school area, other than those already listed.
#'   \item \code{School_bus} Character; whether the school is reached by school-bus service.
#'   \item \code{Urban_public_transport} Character; whether the school is served by a urban public transport station in the range of 250 meters.
#'   \item \code{Interurban_public_transport} Character; whether the school is served by a inter-urban public transport station in the range of 500 meters.
#'   \item \code{Railway_transport} Character; whether the school ranges 500 meters or less from a train station.
#'   \item \code{Private_transport} Character; whether the school can be reached by private transport.
#'   \item \code{Disabled_people_transport} Character; whether the school is provided with disabled people specific transport.
#'   \item \code{Bicycle_lane} Character; whether the building is in proximity of a bicycle/bike lane.
#'   \item \code{Other} Character; whether the building can be reached in any other specific way.
#' }
#' @source
#'
#'   <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Edilizia+Scolastica>. More in detail, the dataset blocks are downloaded respectively from
#'   \itemize{
#'   \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0111EDIAMBIENTESTA2021> cols 10-18;
#'   \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0121EDICOLLEGAMENTISTA2021> cols 20-27;
#'   \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0121EDICOLLEGAMENTISTA2021> cols 28-35
#' }
#'
#'
#' #'
"example_input_DB23_MIUR"

#' Aggregate the database of Italian public schools buildings at the municipality and province level
#'
#' @description This function transforms the output of the \code{\link{Util_DB_MIUR_num}} function (which is detailed at the level of single school buildings) at the municipality/LAU and province/NUTS-3 level.
#' It also allows the user to classify the grade of centrality of municipalities through the variable  \code{Inner_area}.
#'
#'
#'
#'
#' @param data Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. The database of school buildings, preferably already converted to numeric, obtained via \code{\link{Util_DB_MIUR_num}}
#' @param Year Numeric or Character. The reference school year, if either \code{data} or \code{input_InnerAreas} must be retrieved.
#' Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023}.
#' Important: use the same \code{Year} argument used to retrieve the input school buildings data if they are provided as input. \code{2023} by default
#' @param count_units Logical. Whether the rows to aggregate at each level must be counted or not. True by default.
#' @param countname character. The name of the variable indicating the number of schools included in each municipality of province,
#' if the argument 'count' is \code{TRUE}. \code{"nbuildings"}  by default.
#' @param count_missing Logical. Whether the function should return two dataframes including the percentage of NAs in the \code{data} object at the territorial level. \code{TRUE} by default
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param track_deleted Logical. If \code{TRUE}, the function returns the IDs of schools not included. \code{TRUE} by default.
#' @param InnerAreas Logical. Whether an indicator of the percentage of schools belonging to peripheral (Inner) areas mus be included or not.
#' @param ord_InnerAreas Logical. Whether the Inner areas classification should be treated as an ordinal variable rather than as a binary one (see \code{\link{Get_InnerAreas}} for the classification).
#' Please notice than the function creates a column for each class, and if this database must be used in a statistical model, one of the 6 resulting columns must be dropped.
#' False by default.
#' @param input_InnerAreas Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#' The classification of peripheral municipalities, needed only if \code{InnerAreas == TRUE}, obtained as output of the \code{\link{Get_InnerAreas}} function.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default
#' @param autoAbort Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param ... Additional arguments to the function \code{\link{Util_DB_MIUR_num}} in case no data are provided or data.
#'
#'
#' @return An object of class \code{list} including:
#'
#' \itemize{
#'   \item \code{$Municipality_data}:
#'    object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, the output dataframe detailed at the municipality level;
#'    all variables besides the first 5 (which identify the record) are numeric
#'
#'   \item \code{$Province_data}: object of class 'tbl_df', 'tbl' and 'data.frame', the output dataframe detailad at the province level;
#'   all variables besides the first 3 (which identify the record) are numeric
#'
#'   \item \code{$Municipality_missing} (Only if \code{count_missing == TRUE}); object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, the percentage of NAs in each variable at the municipality level.
#'
#'   \item \code{$Province_missing}: (Only if \code{count_missing == TRUE}); object of class 'tbl_df', 'tbl' and 'data.frame', the percentage of NAs in each variable at the province level.

#'   \item \code{$deleted}: character vector. The schools removed from the original dataframe for data quality reasons. This object is returned only if \code{track_deleted == TRUE}
#'  }
#'
#'
#' @details Numerical variables are summarised by the mean; Boolean variables are summarised by the mean as well, thus they become frequency indicators.
#' Qualitative values, if included, are summarised by the mode. Summary measures do not include NAs.
#' The output dataframes are also detailed at the school order level (i.e. Primary, Midde, High school, or different orders). This means that rows are unique combinations of territorial unities and school order.
#'
#'
#'
#' @examples
#'
#'
#' library(magrittr)
#' DB23_MIUR <- example_input_DB23_MIUR %>% Util_DB_MIUR_num(verbose = FALSE) %>%
#'     Group_DB_MIUR(InnerAreas = FALSE)
#'
#'
#'
#' DB23_MIUR$Municipality_data[, -c(1,2,4)]
#' summary(DB23_MIUR$Municipality_data)
#'
#' DB23_MIUR$Province_data[, -c(1,3)]
#' summary(DB23_MIUR$Province_data)
#'
#'
#'
#'
#'
#' @export


Group_DB_MIUR <- function(data = NULL, Year = 2023,
                          count_units = TRUE, countname="nbuildings",
                          count_missing = TRUE, verbose = TRUE,
                          track_deleted = TRUE, InnerAreas = TRUE, ord_InnerAreas = FALSE,
                          input_InnerAreas = NULL, autoAbort = FALSE, ...){

  options(dplyr.summarise.inform = FALSE)

  # No input provided: data are downloaded
  while(is.null(data)){
    data <- Get_DB_MIUR(Year = Year, verbose = verbose, autoAbort = autoAbort) %>%
      Util_DB_MIUR_num(track_deleted = track_deleted, ...)
    if(is.null(data)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during school buildings DB retrieving. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  # To evaluate if data are the raw dataset (if data are the numeric dataset the fields list becomes trivial)
  fields <- lapply(data, is.numeric)

  if(is.data.frame(data) && ! any(unlist(fields[-which(names(fields) %in% c(
    "Province_code", "Year", "Floors_number", "National_seismic_classification"))]))){
    data <- data %>% Util_DB_MIUR_num(track_deleted = track_deleted, verbose = verbose, ...)
  }

  if(is.data.frame(data)){
    DB <- data
    track_deleted <- FALSE
  } else{
    DB <- data$data
    deleted <- data$deleted
  }

  if("Postal_code" %in% names(DB)) {
    DB <- DB %>% dplyr::select(-.data$Postal_code)
  }

  if(InnerAreas){
    if(is.null(input_InnerAreas)) {
      if(verbose) cat("Retrieving Inner areas classification: \n")
      input_InnerAreas <- Get_InnerAreas(autoAbort)
      if(is.null(input_InnerAreas)){
        message("Failed to import inner areas classification")
      }
    }
    if(!is.null(input_InnerAreas)){
      if(dplyr::between(as.numeric(substr(year.patternA(Year),1,4))+1, 2021, 2027)){
        InnerAreas.R <- input_InnerAreas %>%
          dplyr::select(.data$Municipality_code, .data$Inner_area_code_2021_2027) %>%
          dplyr::mutate(Inner_area = ifelse(
            .data$Inner_area_code_2021_2027 %in% c("A", "B", "C"), 0, 1)) %>%
          dplyr::rename(Inner_area_code = .data$Inner_area_code_2021_2027)
      } else {
        InnerAreas.R <- input_InnerAreas %>%
          dplyr::select(.data$Municipality_code, .data$Inner_area_code_2014_2020) %>%
          dplyr::mutate(Inner_area = ifelse(
            .data$Inner_area_code_2014_2020 %in% c("A", "B", "C"), 0, 1)) %>%
          dplyr::rename(Inner_area_code = .data$Inner_area_code_2014_2020)
      }

      if(ord_InnerAreas){
        InnerAreas.R <- InnerAreas.R %>% dplyr::mutate(
          A_mun = as.numeric(.data$Inner_area_code == "A"),
          B_mun = as.numeric(.data$Inner_area_code == "B"),
          C_mun = as.numeric(.data$Inner_area_code == "C"),
          D_mun = as.numeric(.data$Inner_area_code == "D"),
          E_mun = as.numeric(.data$Inner_area_code == "E"),
          F_mun = as.numeric(.data$Inner_area_code == "F") )
      }
      InnerAreas.R <-
        InnerAreas.R %>% dplyr::select(-.data$Inner_area_code)

      DB <- DB %>% dplyr::left_join(InnerAreas.R, by = "Municipality_code")

    }
   }

  groupcol.mun = c("Year", "Municipality_code", "Municipality_description", "Province_code", "Province_initials", "Order" )
  groupcol.prov = c("Year", "Province_code", "Province_initials", "Order" )

  Municipality_data <- DB %>%
    Group_Count(groupcol = groupcol.mun,  startgroup = 10,
                count = count_units, countname = countname, FUN = MeanOrMode)

  Province_data <- DB %>%
    Group_Count(groupcol = groupcol.prov, startgroup = 10,
                count = count_units, countname = countname, FUN = MeanOrMode)

  Municipality_missing <- DB %>%
    Group_Count(groupcol = c("Municipality_code", "Order"),  startgroup = 10,
                count = count_units, countname = countname, FUN = function(x){sum(is.na(x))/length(x)})
  names(Municipality_missing)[-c(1,2)] <-
    paste0(names(Municipality_missing)[-c(1,2)], "_MP")

  Province_missing <- DB %>%
    Group_Count(groupcol = c("Province_code", "Order"), startgroup = 10,
                count = count_units, countname = countname, FUN = function(x){sum(is.na(x))/length(x)})
  names(Province_missing)[-c(1,2)] <-
    paste0(names(Province_missing)[-c(1,2)], "_MP")

  res <- list(Municipality_data = Municipality_data, Province_data = Province_data)

  if(track_deleted) res$deleted <- deleted
  if(count_missing){
    res$Municipality_missing <- Municipality_missing
    res$Province_missing <- Province_missing
  }

  return(res)
}

#' @keywords internal
#'
tabrename.manual <- function(){
  manualdf <- data.frame(
    Input = c("ANNOSCOLASTICO", "ACCESSODAESTERNOCONRAMPE","ACCESSOPIAZZOLA","ACCESSORAMPE","ACCORGIMENTIRIDUZIONECONSUMIENERGETICI",
              "ACCORGIMENTISUPERAMENTOBARRIEREARCHITETTONICHE", "ADEGUAMENTOSISMICO",
              "ALTRECRITICITASPECIFICHE","ALTRIACCORGIMENTI","ALTRIACCORGIMENTIDISABILI",
              "ALTRIACCORGIMENTIRIDUZIONECONSUMI","ALTRIUSI","ALTRO","ALTROIMPIANTORISCALDAMENTO","ALTROSPECIFICARE","ALTROTIPOISOLAMENTO",
              "ANNOADATTAMENTO","ANNOCOSTRUZIONE","AREAALTREMOTIVAZIONI","AREAGRANDETRAFFICO","AREAINDUSTRIEABBANDONATE",
              "AREAINDUSTRIERISCHIO","AREANONRECINTATA","AREASICURA","AREATRANSITIFERROVIARI","AREAURBANADEGRADATA","ASCENSOREDISABILI",
              "ASCENSORETRASPORTODISABILI","ASSENZAPIAZZOLAADEGUATA","ATTESTAZIONERINNOVOPERIODICOCONFORMITAANTINCENDIO","AUDITORIUM",
              "AULAMAGNA","AULE","AULEINFORMATICHE","AULETECNICHE","CAP","CENTRALIZZATOARIA","CENTRALIZZATOGASOLIO","CENTRALIZZATOGPL",
              "CENTRALIZZATOMETANO","CENTRALIZZATOOLIOCOMBUSTIBILE","CENTRALIZZATOOLIOCOMBUSTIBILI","CERTIFICATOAGIBILITAABITABILITA",
              "CERTIFICATOCOLLAUDOIMPIANTOSPEGNIMENTO","CERTIFICATOCOLLAUDOSTATICO","CERTIFICATOOMOLOGAZIONECENTRALETERMICA",
              "CERTIFICATOPREVENZIONEINCENDI","CERTIFICATOPREVENZIONEINCENDICPI","CERTIFICATOSEGNALAZIONEAGIBILITA",
              "CLASSIFICAZIONESISMICANAZIONALE","CODICECOMUNE","CODICEEDIFICIO","CODICESCUOLA","CONDIZIONAMENTOVENTILAZIONE",
              "CONTESTOSENZADISTURBI","CORPISCALDANTIELETTRICIAUTONOMI","CRITICITASPECIFICHEAREASCOLASTICA","DENOMINAZIONEINDIRIZZO",
              "DESCRIZIONECOMUNE","DOCUMENTOVALUTAZIONERISCHI","DOCUMENTOVALUTAZIONERISCHIO","DOPPIVETRISERRAMENTI","EDIFICIOTUTELATO",
              "EDIFICIOVETUSTO","FASCIAETAADATTAMENTO","FASCIAETACOSTRUZIONE","IMPIANTOFOTOVOLTAICO","IMPIANTORISCALDAMENTO",
              "IMPIANTOSOLARETERMICO","ISOLAMENTOACUSTICO","ISOLAMENTOACUSTICORUMOROSITAESTERNA","ISOLAMENTOCOPERTURA",
              "ISOLAMENTOCOPERTURE","ISOLAMENTOINTERNO","ISOLAMENTOINTERNOAULE","ISOLAMENTOINTERNOPIANI","ISOLAMENTOINTERNOPIANIDIVERSI",
              "ISOLAMENTOPARETIESTERNE","ISOLAMENTORUMORIESTERNI","LIBRETTOOMOLOGAZIONEINAILCENTRALETERMICA","MENSA",
              "MEZZIPRIVATI", "MIGLIORAMENTOSISMICO", "NULLAOSTAPROVVISORIOPREVENZIONEINCENDINOP","NUMEROCIVICO","NUMEROCOMPLESSIVOPIANI","NUMEROPIANI",
              "PALESTRA","PALESTRAPISCINA","PANNELLISOLARI","PERCORSIESTERNI","PERCORSIESTERNIDISABILI","PERCORSIINTERNI",
              "PERCORSIINTERNIDISABILI","PERIODOADATTAMENTO","PERIODOCOSTRUZIONE","PIANOEMERGENZA","PIANOEVACUAZIONE",
              "PIATTAFORMAELEVATRICE","PISCINA","PISTACICLABILE","PORTELARGHEZZADISABILI","PORTELARGHEZZAMINIMA",
              "PREESISTENZEINDUSTRIALIABBANDONATE","PRESENZAGRANDETRAFFICO","PRESENZATRANSITOFERROVIARIO",
              "PROGETTAZIONEANTISISMICA","PROGETTATONORMATIVATECNICAANTISISMICA",
              "RISCALDAMENTOALTRANATURA","SCALEANORMA","SCALENORMA","SCIAANTINCENDIO",
              "SCUOLABUS","SERVIZIIGIENICIDISABILI","SERVIZIIGIENICISPECIFICINORMADISABILI","SERVOSCALAPIATTAFORMAELEVATRICE",
              "SIGLAPROVINCIA","SPAZIAMMINISTRATIVI","SPAZIAMMINISTRAZIONE","SPAZICOLLETTIVI","SPAZIDIDATTICA",
              "SPAZIDIDATTICI","SPERFICIEAREALIBERA","SPERFICIEAREATOTALE","STATOEDIFICIO","SUPERAMENTOBARRIEREARCH",
              "SUPERFICIETOTALEAREALIBERA","SUPERFICIETOTALEAREASCOLASTICA","TELERISCALDAMENTO","TIPOLOGIAINDIRIZZO",
              "TRASPORTIFERROVIARI","TRASPORTIPUBBLICIINTERURBANI","TRASPORTIPUBBLICIURBANI","TRASPORTODISABILI",
              "USOSCOLASTICO","VETRIDOPPISERRAMENTIDOPPI","VICINANZAACQUEINQUINANTI","VICINANZAALTRIDISTURBI",
              "VICINANZACIMITERI","VICINANZADISCARICA","VICINANZAFONTIINQUINAMENTOACUSTICO","VICINANZAFONTIINQUINAMENTOATMOSFERICO",
              "VICINANZAINDUSTRIEARISCHIO","VICINANZAINDUSTRIEINQUINANTI","VICINANZAINQUINAMENTOACUSTICO",
              "VICINANZAINQUINAMENTOATMOSFERICO","VICINANZARADIAZIONIELETTROMAGNETICHE","VICINANZASORGENTIRADIAZIONIELETTROMAGNETICHE",
              "VINCOLIIDROGEOLOGICI","VINCOLIPAESAGGIO","VINCOLOIDROGEOLOGICO","VINCOLOINTERESSECULTURALE",
              "VINCOLOPAESAGGISTICO","VOLUME","VOLUMELORDOEDIFICIO","ZONASISMICA","ZONAURBANADEGRADATA","ZONIZZAZIONEIMPIANTOTERMICO"),
    file1 =c("ANNO_SCOLASTICO","ACCESSO_DA_ESTERNO_CON_RAMPE","","ACCESSO_DA_ESTERNO_CON_RAMPE","ACCORGIMENTI_RIDUZIONE_CONSUMI_ENERGETICI",
             "ACCORGIMENTI_SUPERAMENTO_BARRIERE_ARCHITETTONICHE","ADEGUAMENTO_SISMICO", "ALTRE_CRITICITA_SPECIFICHE","ALTRI_ACCORGIMENTI",
             "ALTRI_ACCORGIMENTI","","ALTRI_USI","ALTRO","ALTRO_IMPIANTO_RISCALDAMENTO","ALTRO_SPECIFICARE","",
             "ANNO_ADATTAMENTO","ANNO_COSTRUZIONE","","PRESENZA_GRANDE_TRAFFICO","PREESISTENZE_INDUSTRIALI_ABBANDONATE",
             "VICINANZA_INDUSTRIE_A_RISCHIO","AREA_NON_RECINTATA","","PRESENZA_TRANSITO_FERROVIARIO","ZONA_URBANA_DEGRADATA",
             "ASCENSORE_TRASPORTO_DISABILI","ASCENSORE_TRASPORTO_DISABILI","ASSENZA_PIAZZOLA_ADEGUATA",
             "ATTESTAZIONE_RINNOVO_PERIODICO_CONFORMITA_ANTINCENDIO","AUDITORIUM","","AULE","AULE_INFORMATICHE",
             "AULE_TECNICHE","CAP","","CENTRALIZZATO_GASOLIO","CENTRALIZZATO_GPL","CENTRALIZZATO_METANO",
             "CENTRALIZZATO_OLIO_COMBUSTIBILE","CENTRALIZZATO_OLIO_COMBUSTIBILE","","","","","CERTIFICATO_PREVENZIONE_INCENDI",
             "CERTIFICATO_PREVENZIONE_INCENDI","CERTIFICATO_SEGNALAZIONE_AGIBILITA","CLASSIFICAZIONE_SISMICA_NAZIONALE",
             "Municipality_code","CODICE_EDIFICIO","School_code","CONDIZIONAMENTO_VENTILAZIONE","CONTESTO_SENZA_DISTURBI",
             "CORPI_SCALDANTI_ELETTRICI_AUTONOMI","CRITICITA_SPECIFICHE_AREA_SCOLASTICA","DENOMINAZIONE_INDIRIZZO",
             "DESCRIZIONE_COMUNE","DOCUMENTO_VALUTAZIONE_RISCHI","DOCUMENTO_VALUTAZIONE_RISCHIO","VETRI_DOPPI_SERRAMENTI_DOPPI",
             "EDIFICIO_TUTELATO","","","","IMPIANTO_FOTOVOLTAICO","IMPIANTO_RISCALDAMENTO","IMPIANTO_SOLARE_TERMICO",
             "ISOLAMENTO_ACUSTICO","ISOLAMENTO_ACUSTICO_RUMOROSITA_ESTERNA","ISOLAMENTO_COPERTURE","ISOLAMENTO_COPERTURE",
             "ISOLAMENTO_INTERNO","","","ISOLAMENTO_INTERNO_PIANI_DIVERSI","ISOLAMENTO_PARETI_ESTERNE","",
             "LIBRETTO_OMOLOGAZIONE_INAIL_CENTRALE_TERMICA","MENSA","MEZZI_PRIVATI","MIGLIORAMENTO_SISMICO","","NUMERO_CIVICO","NUMERO_COMPLESSIVO_PIANI",
             "NUMERO_COMPLESSIVO_PIANI","PALESTRA","","","PERCORSI_ESTERNI","PERCORSI_ESTERNI","PERCORSI_INTERNI",
             "PERCORSI_INTERNI","PERIODO_ADATTAMENTO","PERIODO_COSTRUZIONE","","PIANO_EVACUAZIONE","",
             "PISCINA","PISTA_CICLABILE","PORTE_LARGHEZZA_MINIMA","PORTE_LARGHEZZA_MINIMA","PREESISTENZE_INDUSTRIALI_ABBANDONATE",
             "PRESENZA_GRANDE_TRAFFICO","PRESENZA_TRANSITO_FERROVIARIO","","","","SCALE_A_NORMA","SCALE_A_NORMA",
             "SCIA_ANTINCENDIO","SCUOLABUS","SERVIZI_IGIENICI_SPECIFICI_NORMA_DISABILI","SERVIZI_IGIENICI_SPECIFICI_NORMA_DISABILI",
             "SERVOSCALA_PIATTAFORMA_ELEVATRICE","SIGLA_PROVINCIA","SPAZI_AMMINISTRAZIONE","SPAZI_AMMINISTRAZIONE",
             "SPAZI_COLLETTIVI","SPAZI_DIDATTICA","SPAZI_DIDATTICA","SUPERFICIE_TOTALE_AREA_LIBERA","SUPERFICIE_TOTALE_AREA_SCOLASTICA",
             "","","SUPERFICIE_TOTALE_AREA_LIBERA","SUPERFICIE_TOTALE_AREA_SCOLASTICA","TELERISCALDAMENTO",
             "TIPOLOGIA_INDIRIZZO","TRASPORTI_FERROVIARI","TRASPORTI_PUBBLICI_INTERURBANI","TRASPORTI_PUBBLICI_URBANI",
             "TRASPORTO_DISABILI","USO_SCOLASTICO","VETRI_DOPPI_SERRAMENTI_DOPPI","VICINANZA_ACQUE_INQUINANTI",
             "VICINANZA_ALTRI_DISTURBI","VICINANZA_CIMITERI","VICINANZA_DISCARICA","VICINANZA_FONTI_INQUINAMENTO_ACUSTICO",
             "VICINANZA_FONTI_INQUINAMENTO_ATMOSFERICO","VICINANZA_INDUSTRIE_A_RISCHIO","VICINANZA_INDUSTRIE_INQUINANTI",
             "VICINANZA_FONTI_INQUINAMENTO_ACUSTICO","VICINANZA_FONTI_INQUINAMENTO_ATMOSFERICO",
             "VICINANZA_SORGENTI_RADIAZIONI_ELETTROMAGNETICHE","VICINANZA_SORGENTI_RADIAZIONI_ELETTROMAGNETICHE",
             "VINCOLO_IDROGEOLOGICO","VINCOLO_PAESAGGISTICO","VINCOLO_IDROGEOLOGICO","VINCOLO_INTERESSE_CULTURALE",
             "VINCOLO_PAESAGGISTICO","VOLUME_LORDO_EDIFICIO","VOLUME_LORDO_EDIFICIO","","ZONA_URBANA_DEGRADATA","ZONIZZAZIONE_IMPIANTO_TERMICO"),
    file2 = c("ANNO_SCOLASTICO","","ACCESSO_PIAZZOLA","ACCESSO_RAMPE","","","","","","ALTRI_ACCORGIMENTI_DISABILI",
              "ALTRI_ACCORGIMENTI_RIDUZIONE_CONSUMI","","","","","ALTRO_TIPO_ISOLAMENTO","","","AREA_ALTRE_MOTIVAZIONI",
              "AREA_GRANDE_TRAFFICO","AREA_INDUSTRIE_ABBANDONATE","AREA_INDUSTRIE_RISCHIO","","AREA_SICURA","AREA_TRANSITI_FERROVIARI",
              "AREA_URBANA_DEGRADATA","ASCENSORE_DISABILI","","","","","AULA_MAGNA","","","","","CENTRALIZZATO_ARIA","","","","",
              "CENTRALIZZATO_OLIO_COMBUSTIBILI","CERTIFICATO_AGIBILITA_ABITABILITA","CERTIFICATO_COLLAUDO_IMPIANTO_SPEGNIMENTO",
              "CERTIFICATO_COLLAUDO_STATICO","CERTIFICATO_OMOLOGAZIONE_CENTRALE_TERMICA","","CERTIFICATO_PREVENZIONE_INCENDI_CPI",
              "","","","","","","","","","","","","DOCUMENTO_VALUTAZIONE_RISCHIO","DOPPI_VETRI_SERRAMENTI","","EDIFICIO_VETUSTO",
              "FASCIA_ETA_ADATTAMENTO","FASCIA_ETA_COSTRUZIONE","","","","","","ISOLAMENTO_COPERTURA","","","ISOLAMENTO_INTERNO_AULE",
              "ISOLAMENTO_INTERNO_PIANI","","","ISOLAMENTO_RUMORI_ESTERNI","","","","","NULLA_OSTA_PROVVISORIO_PREVENZIONE_INCENDI_NOP","","",
              "NUMERO_PIANI","","PALESTRA_PISCINA","PANNELLI_SOLARI","","PERCORSI_ESTERNI_DISABILI","","PERCORSI_INTERNI_DISABILI","","",
              "PIANO_EMERGENZA","","PIATTAFORMA_ELEVATRICE","","","PORTE_LARGHEZZA_DISABILI","","","","","","PROGETTAZIONE_ANTISISMICA",
              "RISCALDAMENTO_ALTRA_NATURA","","SCALE_NORMA","","","SERVIZI_IGIENICI_DISABILI","","","","SPAZI_AMMINISTRATIVI","","","",
              "SPAZI_DIDATTICI","SPERFICIE_AREA_LIBERA","SPERFICIE_AREA_TOTALE","STATO_EDIFICIO","SUPERAMENTO_BARRIERE_ARCH","","","",
              "","","","","","","","","","","","","","","","VICINANZA_INQUINAMENTO_ACUSTICO","VICINANZA_INQUINAMENTO_ATMOSFERICO",
              "VICINANZA_RADIAZIONI_ELETTROMAGNETICHE","","VINCOLI_IDROGEOLOGICI","VINCOLI_PAESAGGIO","","","","VOLUME","","ZONA_SISMICA","",""),
    Readable = c("School_year", "External_access_ramps", "Layby_access", "Outside_access_ramps",
                 "Energy_saving_devices", "Architectural_barriers_overcoming_devices", "Seismic_adaptation",
                 "Other_specific_criticalities", "Other_devices", "Other_devices", "Other_consumption_reduction_devices",
                 "Other_uses", "Other", "Other_heating_system", "Other_specify", "Other_insulation_type",
                 "Adaptation_year", "Building_year", "Area_other_motivations", "Large_traffic",
                 "Abandoned_industries", "Risky_industries_proximity", "Unenclosed_area",
                 "Secure_area", "Railway_traffic", "Decayed_urban_area", "Disabled_people_elevator",
                 "Disabled_people_elevator", "Layby_absence", "Periodic_fire_safety_compliance_renewal",
                 "Auditorium", "Main_hall", "Classrooms", "IT_classrooms", "Technology_classrooms", "Postal_code", "Central_air_system",
                 "Centralised_diesel_oil_system", "Centralised_LPG_system", "Centralised_methane_system", "Centralised_fuel_oil_system",
                 "Centralised_fuel_oil_system", "Occupancy_certificate", "Shutdown_system_testing_certification",
                 "Static_testing_certification", "Thermal_plant_approval_certification", "Fire_prevention_certification",
                 "Fire_prevention_certification", "Occupancy_report_certification", "National_seismic_classification",
                 "Municipality_code", "Building_code", "School_code", "Ventilation_conditioning", "Undisturbed_context",
                 "Autonomous_electric_heating_elements", "School_area_specific_criticalities", "Address_name",
                 "Municipality_description", "Risk_assessment_document", "Risk_assesment_document",
                 "Double_windows_glazing", "Protected_building", "Ancient_building", "Adaptation_age_range",
                 "Building_age_range", "Photovoltaic_system", "Heating_system", "Solar_heating_system",
                 "Acoustic_insulation", "Acoustic_insulation_outside_noise", "Covers_insulation",
                 "Covers_insulation", "Internal_insulation", "Classrooms_internal_insulation", "Floors_internal_insulation",
                 "Different_floors_internal_insulation", "Outside_walls_insulation", "Outside_noise_insulation",
                 "Thermal_plant_INAIL_certification_booklet", "Canteen", "Private_transport", "Seismic_improvement", "Provisional_fire_prevention_clearance",
                 "Civic_number", "Floors_number", "Floors_number", "Gymnasium", "Gymnasium_swimming_pool",
                 "Solar_panels", "External_routes", "External_routes", "Internal_routes", "Internal_routes",
                 "Adaptation_period", "Building_period", "Emergency_plan", "Evacuation_plan",
                 "Elevator_platform", "Swimming_pool", "Bicycle_lane", "Weelchair_accessible_doors_width",
                 "Weelchair_accessible_doors_width", "Abandoned_industries", "Large_traffic",
                 "Railway_traffic", "Seismic_design", "Seismic_design", "Other_heating_system",
                 "Stairs_up_to_standard", "Stairs_up_to_standard", "Fire_prevention_activity_starting_notification", "School_bus",
                 "Disabled_people_compliant_sanitation","Disabled_people_compliant_sanitation", "Elevator_stairlift",
                 "Province_initials", "Administration_spaces", "Administration_spaces", "Collective_spaces",
                 "Teaching_spaces", "Teaching_spaces", "Free_area_surface", "School_area_surface",
                 "Building_status", "Architectural_barriers_overcoming_devices", "Free_area_surface", "School_area_surface",
                 "District_heating", "Address_type", "Railway_transport", "Interurban_public_transport",
                 "Urban_public_transport", "Disabled_people_transport", "School_use", "Double_windows_glazing",
                 "Polluted_waters_proximity", "Other_disturbances_proximity", "Graveyards_proximity", "Dumps_proximity",
                 "Acoustic_pollution_sources_proximity", "Air_pollution_sources_proximity",
                 "Hazardous_industries_proximity", "Pollutant_industries_proximity", "Acoustic_pollution_sources_proximity",
                 "Air_pollution_sources_proximity", "Electromagnetic_radiation_sources_proximity",
                 "Electromagnetic_radiation_sources_proximity", "Hydrogeological_restrictions", "Landscape_restrictions",
                 "Hydrogeological_restrictions", "Cultural_heritage_restrictions", "Landscape_restrictions", "Gross_building_volume",
                 "Gross_building_volume", "Seismic_zone", "Decayed_urban_area", "Heating_system_zoning")

  )
  return(manualdf)
}

#' Display data fom the school buildings database
#'
#' @description  This function displays a map of the data downloaded trough the \code{\link{Get_DB_MIUR}} function.
#' It supports two kinds of map:
#' \itemize{
#'   \item Interactive map (default option), which allows the user to visualize all the data in scope through the interactive popup, and
#'   \item Static map (ggplot), which can be easily exported in \code{.pdf} objects.
#' }
#'
#'
#'
#' @param data Object of class \code{list} or \code{tbl_df}, \code{tbl} and \code{data.frame}. Input data obtained as output of the function \code{\link{Group_DB_MIUR}}
#' If NULL, it will be downloaded automatically but not saved in the global environment. \code{NULL} by default.
#' @param field Character. The variable to display in the map.
#' @param order Character. The school order. Either \code{"Primary"}, \code{"Middle"}, or \code{"High"} (high school).
#' If \code{NULL}, an average of the three school orders will be displayed for the target variable. \code{NULL} by default.
#' @param level Character. The administrative level of detailed at which the target variable must be displayed.
#' Either \code{"LAU"}/\code{"Municipality"} or \code{"NUTS-3"}/\code{"Province"}. \code{"LAU"} by default.
#' @param region_code Numeric. The NUTS-2 codes of the units that must be displayed.
#' If the level is set to \code{"LAU"}, choosing a limited number of regions is recommended.
#' By default, \code{c(1:20)}, i.e. all Italian regions.
#' @param plot Character. The type of map to display; either \code{"mapview"} for interactive maps, or \code{"ggplot"} for static maps. \code{"mapview"} by default.
#'  !!! Warning. As \code{mapview} turns out to have been taken away from CRAN,
#'  if it has not been previously installed by the user, \code{plot = "ggplot"} will be forced.
#' @param pal Character. The palette to use if the \code{"mapview"} mode is chose. \code{"viridis"} by default.
#' @param col_rev Logical. Whether the scale of the colour palette should be reverted or not, if the \code{"mapview"} mode is chosen. \code{FALSE} by default
#' @param popup_height Numeric. The height of the popup table in terms of pixels if the \code{"mapview"} mode is chosen. \code{200} by default.
#' @param main_pos Character. Where the header should be placed if the \code{ggplot} mode is chosen.
#' The header is located on the top if \code{"top"} is given as input, and above the legend scale otherwise. \code{"top"} by default.
#' @param main Character. The customary title to display in the \code{"ggplot"} rendering options
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.s
#' @param input_shp Object of class \code{sf}, \code{tbl_df}, \code{tbl}, \code{data.frame}. The relevant shapefiles of Italian administrative boudaries,
#' at the selected level of detail (LAU or NUTS-3). If \code{NULL} it is downloaded automatically but not saved in the global environment. \code{NULL} by default.
#' @param only_observed Logical. Whether to remove unobserved areas from the plot. \code{FALSE} by default.
#' @param autoAbort Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param ... If \code{data} is not provided, the arguments to \code{\link{Group_DB_MIUR}}.
#'
#'
#'
#' @return If \code{plot == "mapview"}, an object of class \code{mapview}. Otherwise, if \code{plot == "ggplot"}, an object of class \code{gg} and \code{ggplot}.
#'
#'
#'
#' @examples
#'
#'
#'
#'
#'
#'   library(magrittr)
#'
#'   DB23_MIUR <- example_input_DB23_MIUR %>%
#'     Util_DB_MIUR_num(track_deleted = FALSE) %>%
#'     Group_DB_MIUR(InnerAreas = FALSE, count_missing = FALSE)
#'
#'   DB23_MIUR %>% Map_School_Buildings(field = "School_bus",
#'      order = "Primary",level = "NUTS-3",  plot = "ggplot",
#'      input_shp = example_Prov22_shp)
#'
#'   DB23_MIUR %>% Map_School_Buildings(field = "Railway_transport",
#'      order = "High",level = "NUTS-3", plot = "ggplot",
#'      input_shp = example_Prov22_shp)
#'
#'   DB23_MIUR %>% Map_School_Buildings(field = "Context_without_disturbances",
#'      order = "Middle",level = "NUTS-3", plot = "ggplot",
#'      input_shp = example_Prov22_shp, col_rev = TRUE)
#'
#'
#'
#'
#'
#' @export


Map_School_Buildings <- function (data = NULL, field, order = NULL,  level = "LAU",
                                  region_code = c(1:20), plot = "mapview", pal = "viridis",
                                  col_rev = FALSE, popup_height = 200,
                                  main_pos = "top", main = "", only_observed = FALSE,
                                  verbose = TRUE, input_shp = NULL, autoAbort = FALSE, ... ) {
  options(dplyr.summarise.inform = FALSE)

  while(is.null(data)){
    if(verbose) cat("Loading input data: \n")
    data <- Group_DB_MIUR(autoAbort = autoAbort, verbose = verbose, ...)
    if(is.null(data)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during school buildings DB retrieving. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  if((is.data.frame(data) && "School_code" %in% names(data)) ||
     (!is.data.frame(data) && "School_code" %in% names(data[[1]]))){
       data <- Group_DB_MIUR(data, verbose = verbose, ...)
     }

  if(!is.data.frame(data)){
    if(any(grepl("missing", names(data)))){
      if(toupper(level) %in% c("LAU", "MUNICIPALITY", "NUTS-4")){
        DB <- data$Municipality_data %>%
          dplyr::left_join(data$Municipality_missing, by = c("Municipality_code", "Order"))
      } else {
        DB <- data$Province_data %>%
          dplyr::left_join(data$Province_missing, by = c("Province_code", "Order"))
      }
    } else {
      if(toupper(level) %in% c("LAU", "MUNICIPALITY", "NUTS-4")){
        DB <- data$Municipality_data
      } else DB <- data$Province_data
    }
  } else DB <- data

  Year <- as.numeric(DB$Year[1])%/%100 + 1
  YearMinus1 <- Year - 1

  while(is.null(input_shp)){
    if (verbose) cat("Loading shapefile: \n")
    input_shp <- Get_Shapefile(Year = ifelse(
      any(year.patternA(Year) %in% c(year.patternA(2016), year.patternA(2018))), Year, YearMinus1),
      level = level, autoAbort = autoAbort)
    if(is.null(input_shp)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during shapefile retrieving. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  if(verbose) cat("Setting target variables... \n")

  if (toupper(level) %in% c("LAU", "MUNICIPALITY", "NUTS-4")){
    dat.R <- DB %>% dplyr::filter(.data$Order != "NR")

    # formerly across(all_of(c(3:ncol(.)-4)))
    if (is.null(order)){
      dat.R <- dat.R %>%
        dplyr::filter(.data$Order != "IC") %>%
        dplyr::filter(.data$Order != "IS") %>%
        dplyr::group_by(
        .data$Municipality_code, .data$Municipality_description,
        .data$Province_code, .data$Province_initials) %>%
        dplyr::summarise(nbuildings = sum(.data$nbuildings),
                         dplyr::across(c(3:(ncol(dat.R)-4)), list(MeanOrMode))) %>%
        dplyr::ungroup()
      names(dat.R) <- names(dat.R) %>% stringr::str_remove_all("_1")
    } else {
      dat.R <- dat.R %>%
        dplyr::filter(.data$Order == order) %>%
        dplyr::select(-.data$Order)
    }

    res <- input_shp %>% dplyr::select(.data$COD_REG, .data$PRO_COM_T) %>%
      rename_by_idx(2, "Municipality_code") %>%
      dplyr::filter(.data$COD_REG %in% region_code) %>%
      dplyr::left_join(dat.R, by = "Municipality_code")

  } else {

    if("Municipality_code" %in% names(DB)){
      startgroup <- min(
        which(
          unlist(
            lapply(dplyr::select(DB, -.data$Province_code),
                   function(x) any(is.numeric(x)))))) + 1
      DB <- DB %>% Group_Count(
        groupcol = c("Province_code", "Province_initials", "Order"), FUN =MeanOrMode,
        startgroup = startgroup, countname = "nmun")
      if("nbuildings" %in% names(DB)){
        DB <- DB %>% dplyr::mutate(nmun = .data$nmun * .data$nbuildings) %>%
          dplyr::select(-.data$nbuildings) %>%
          dplyr::rename(nbuildings = .data$nmun)
      } else {
        DB <- DB %>% dplyr::select(-.data$nmun)
      }
    }

    dat.R <- DB %>% dplyr::filter(.data$Order != "NR")

    if (is.null(order)){
      if(verbose){message("No school order has been selected. A global average will be displayed")}
      dat.R <- dat.R %>%
        dplyr::filter(.data$Order != "IC") %>%
        dplyr::filter(.data$Order != "IS") %>%
        dplyr::group_by(.data$Province_code, .data$Province_initials) %>%
        dplyr::summarise(nbuildings = sum(.data$nbuildings), dplyr::across(c(3:(ncol(dat.R)-2)), list(MeanOrMode))) %>%
        dplyr::ungroup() %>%
        dplyr::mutate(Province_code = as.numeric(.data$Province_code))
      names(dat.R) <- names(dat.R) %>% stringr::str_remove_all("_1")
    } else {
      dat.R <- dat.R %>%
        dplyr::filter(.data$Order == order) %>%
        dplyr::select(-.data$Order) %>%
        dplyr::mutate(Province_code = as.numeric(.data$Province_code))
    }

    res <- input_shp %>% dplyr::select(.data$COD_REG, .data$COD_PROV)  %>%
      rename_by_idx(2, "Province_code")  %>%
      dplyr::filter(.data$COD_REG %in% region_code) %>%
      dplyr::left_join(dat.R , by ="Province_code")
  }

  fieldname <- ifelse(is.numeric(field), names(res)[field], field)
  nfield <- ifelse(is.numeric(field), field, match(field, names(res)))

  if(only_observed){
    res <- res[which(!is.na(res[, nfield])),]
  }

  while(! fieldname %in% names(res)){
    message(paste("The variable", field, "does not seem to belong to the current database.
                  Please insert another one (do not use quotes in the prompt)"))
    field <- readline(prompt = "  > ")

    nfield <- ifelse(is.numeric(field), field, match(field, names(res)))
    fieldname <- ifelse(is.numeric(field), names(res)[field], field)
  }

  layername <- ifelse(main == "", paste(
    fieldname, ifelse(is.null(order),"",paste(order, "School"))), main)

  if(!"mapview" %in% rownames(utils::installed.packages())) plot <- "ggplot"

  if(verbose) cat("Rendering:")

  if(plot == "ggplot"){

    if(main_pos == "top"){
      ggplot2::ggplot() + ggplot2::geom_sf(data = res, ggplot2::aes(fill = !!rlang::sym(fieldname))) +
        ggplot2::labs(fill = "") + ggplot2::ggtitle(layername) +
        ggplot2::scale_fill_viridis_c(na.value = "white",
                                      direction = 2*(1/2 - as.numeric(col_rev))) +
        ggplot2::theme_minimal()
    } else {
      ggplot2::ggplot() + ggplot2::geom_sf(data = res, ggplot2::aes(fill = !!rlang::sym(fieldname))) +
        ggplot2::labs(fill = fieldname) +
        ggplot2::scale_fill_viridis_c(na.value = "white",
                                      direction = 2*(1/2 - as.numeric(col_rev))) +
        ggplot2::theme_minimal()
    }
  } else {

    #n <- length(unique(unlist(sf::st_drop_geometry(res[,nfield])))) - 1
    n <- length(unique(unlist(dplyr::select(dplyr::filter(
      dat.R, .data$Province_code %in% res$Province_code), field)))) - 1
    if (col_rev == FALSE) {
      brew <- grDevices::hcl.colors(n, palette = pal)
    } else {
      brew <- rev(grDevices::hcl.colors(n, palette = pal))
    }
    pop = leafpop::popupTable(res)
    suppressWarnings(
      mapview::mapview(res, zcol = fieldname, col.regions = brew,
                       layer.name = layername,
                       popup = paste0(set_popup_height(popup_height) , pop) )
    )
  }
}

#' @keywords internal
#'
prov.names <- function(){

  res <- data.frame(
    Region_code = c(
      rep(1,6),2, rep(7,4), rep(3,9), rep(4,2), rep(5,7), rep(6,3), rep(8,8), rep(11,4),
      rep(9,9), rep(10,2), rep(12,5), rep(15,5), rep(13,4), 14, rep(16,5), rep(17,2), rep(18,3),
      rep(19,9), rep(20,3), 6, 14, 20, 1, 3, 3, 8, 9, 18, 18, 1, 20, 20, 20, 20, 3, 11, 16, 20),
    Province_code = seq(1:111),
    Region_description = c(
      rep("PIEMONTE",6),"VALLE D'AOSTA", rep("LIGURIA",4), rep("LOMBARDIA", 9),
      rep("TRENTINO ALTO ADIGE",2), rep("VENETO",7), rep("FRIULI VENEZIA GIULIA",3),
      rep("EMILIA ROMAGNA",8), rep("MARCHE",4),rep("TOSCANA",9), rep("UMBRIA",2), rep("LAZIO",5),
      rep("CAMPANIA",5), rep("ABRUZZO",4), "MOLISE", rep("PUGLIA",5), rep("BASILICATA",2), rep("CALABRIA",3),
      rep("SICILIA",9), rep("SARDEGNA",3), "FRIULI VENEZIA GIULIA", "MOLISE", "SARDEGNA",
      "PIEMONTE", "LOMBARDIA", "LOMBARDIA", "EMILIA ROMAGNA", "TOSCANA", "CALABRIA",
      "CALABRIA", "PIEMONTE", "SARDEGNA", "SARDEGNA", "SARDEGNA", "SARDEGNA", "LOMBARDIA",
      "MARCHE", "PUGLIA", "SARDEGNA"),
    Province_initials = c(
      "TO", "VC", "NO", "CN", "AT", "AL", "AO", "IM", "SV", "GE", "SP", "VA", "CO",
      "SO", "MI", "BG", "BS", "PV", "CR", "MN", "BZ", "TN", "VR", "VI", "BL", "TV",
      "VE", "PD", "RO", "UD", "GO", "TS", "PC", "PR", "RE", "MO", "BO", "FE", "RA",
      "FC", "PU", "AN", "MC", "AP", "MS", "LU", "PT", "FI", "LI", "PI", "AR", "SI",
      "GR", "PG", "TR", "VT", "RI", "RM", "LT", "FR", "CE", "BN", "NA", "AV", "SA",
      "AQ", "TE", "PE", "CH", "CB", "FG", "BA", "TA", "BR", "LE", "PZ", "MT", "CS",
      "CZ", "RC", "TP", "PA", "ME", "AG", "CL", "EN", "CT", "RG", "SR", "SS", "NU",
      "CA", "PN", "IS", "OR", "BI", "LC", "LO", "RN", "PO", "KR", "VV", "VB", "OT",
      "OG", "VS", "CI", "MB", "FM", "BT", "SU" ) ,
    Province_description =c(
      "TORINO", "VERCELLI", "NOVARA", "CUNEO", "ASTI", "ALESSANDRIA", "AOSTA", "IMPERIA",
      "SAVONA", "GENOVA", "LA SPEZIA","VARESE","COMO", "SONDRIO", "MILANO", "BERGAMO",
      "BRESCIA", "PAVIA",  "CREMONA", "MANTOVA", "BOLZANO", "TRENTO", "VERONA", "VICENZA",
      "BELLUNO", "TREVISO", "VENEZIA", "PADOVA", "ROVIGO", "UDINE", "GORIZIA", "TRIESTE",
      "PIACENZA", "PARMA", "REGGIO EMILIA", "MODENA", "BOLOGNA", "FERRARA", "RAVENNA",
      "FORLI'-CESENA", "PESARO E URBINO","ANCONA", "MACERATA", "ASCOLI PICENO", "MASSA-CARRARA",
      "LUCCA", "PISTOIA", "FIRENZE", "LIVORNO",  "PISA", "AREZZO", "SIENA", "GROSSETO",
      "PERUGIA", "TERNI", "VITERBO", "RIETI", "ROMA", "LATINA", "FROSINONE", "CASERTA",
      "BENEVENTO", "NAPOLI", "AVELLINO", "SALERNO", "L'AQUILA", "TERAMO", "PESCARA",
      "CHIETI", "CAMPOBASSO", "FOGGIA", "BARI","TARANTO","BRINDISI", "LECCE", "POTENZA",
      "MATERA", "COSENZA", "CATANZARO", "REGGIO CALABRIA",   "TRAPANI", "PALERMO", "MESSINA",
      "AGRIGENTO", "CALTANISSETTA", "ENNA", "CATANIA", "RAGUSA", "SIRACUSA", "SASSARI",
      "NUORO", "CAGLIARI", "PORDENONE", "ISERNIA", "ORISTANO", "BIELLA", "LECCO", "LODI",
      "RIMINI", "PRATO", "CROTONE", "VIBO VALENTIA", "VERBANO-CUSIO-OSSOLA", "OLBIA-TEMPIO",
      "OGLIASTRA", "MEDIO CAMPIDANO", "CARBONIA-IGLESIAS", "MONZA E DELLA BRIANZA", "FERMO",
      "BARLETTA-ANDRIA-TRANI","SUD SARDEGNA" ))
  return(res)
}

#' Subset of the school registry in school year 2022/23
#'
#'
#' This dataframe includes the classification of municipalities , from four regions: Molise, Campania, Apulia and Basilicata.
#' Only the first 10 columns are included;
#' some strings in field \code{Municipality_description} including accents have been forced to ASCII.
#' The whole dataset can be retrieved with the command \code{Get_InnerAreas()}.
#' For the definition of ISTAT inner areas class, see \code{\link{Get_InnerAreas}}
#'
#' @seealso \code{\link{Get_InnerAreas}}
#'
#' @format ## `example_InnerAreas`
#' A data frame with 1074 rows and 10 columns:
#' \itemize{
#'   \item \code{Municipality_code} Character; the ISTAT LAU (municipality) ID.
#'   \item \code{Municipality_code_numeric} Numeric; the ISTAT LAU (municipality) ID in numeric format.
#'   \item \code{Cadastral_code} Character; a LAU - level ID code, different from the official ISTAT municipality code.
#'   \item \code{Region_code} Numeric; the region (NUTS-2 administrative level) ID
#'   \item \code{Region_description} Character; the region (NUTS-2 administrative level) name.
#'   \item \code{Province_code} Numeric; the NUTS-3 administrative code.
#'   \item \code{Province_initials} Character; abbreviated NUTS-3 denomination.
#'   \item \code{Province_description} Character; the province (NUTS-3 administrative level) denomination.
#'   \item \code{Municipality_description} Character; the municipality name.
#'   \item \code{Inner_area_code_2014_2020} Character; the ISTAT inner areas classification between 2014 and 2020.
#'   \item \code{Inner_area_description_2014_2020} Character; the description of the classes identified in the previous column
#'   \item \code{Inner_area_code_2021_2027} Character; the ISTAT inner areas classification between 2021 and 2027.
#'   \item \code{Inner_area_description_2021_2027} Character; the description of the classes identified in the previous column
#'   \item \code{Destination_municipality_code} Character; For non-central municipalities (classes C, D, E, F), the ID of the closest pole municipality according to the 2021-2027 classification
#'   \item \code{Destination_municipality_code} Character; The denomination of the municipalities in the previous column
#'   \item \code{Destination_pole_code} Character; An internal ID convention for the destination poles; it includes a letter
#'   (the class of the destination pole, either A or B); a number of two digits (the region code of the destination pole) and the progressive number of poles within a region.
#' }
#' @source <https://www.istat.it/it/archivio/273176>
#'
"example_InnerAreas"

#' @keywords internal
#'
School.order <- function(data, field = "School_code"){
  res <- data %>% dplyr::mutate(
    Order = ifelse(substr(!!rlang::sym(field),3,4) %in% c("EE", "1E") , "Primary", ifelse(
      substr(!!rlang::sym(field), 3, 4) %in% c("MM", "1M") , "Middle", ifelse(
        substr(!!rlang::sym(field), 3, 4) %in%
          c ("PC", "PL", "PM","PQ","PS","RA","RC","RE","RF","RH","RI","RM", "RR", "RS","RV","SD",
             "SL","TA","TB","TD","TE","TF","TH","TL","TN"), "High", ifelse(
               substr(!!rlang::sym(field),3,4) %in% c("IC", "IP"), "IC",  ifelse(
                 substr(!!rlang::sym(field),3,4) == "IS", "IS", "NR")))))) %>%
    dplyr::relocate(.data$Order, .after = field)
  return(res)
}

#' Download the registry of Italian public schools from the school registry section
#'
#' @description This function returns two main pieces of information regarding Italian schools, namely:
#' \itemize{
#'   \item The denomination of the region, province and municipality to which the school belongs.
#'   \item The mechanographical code to the reference institute of each school.
#' }
#' It is possible to access schools in all the national territory, including the autonomous provinces of Aosta, Trento and Bozen.
#'
#'
#'
#' @param Year Numeric or character. Reference school year (last available is 2025).
#' Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023}. \code{2023} by default.
#' @param show_col_types Logical. If \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.
#' @param autoAbort Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param filename Character. A string included in the name of the file to download, identifying the schools included.
#' By default it is \code{c("SCUANAGRAFESTAT", "SCUANAAUTSTAT")}, i.e. the file names used for public school registries,
#' respectively across all the national territory except for the autonomous provinces of Aosta, Trento or Bozen, and only in the three
#' If instead the registry of the private schools is needed, please insert \code{"SCUANAGRAFEPAR"} and/or \code{"SCUANAAUTPAR"}.
#'
#' For the registry of private schools, either in all the national territory except for the aforementioned provinces, and for these provinces, please use \code{"SCUANAGRAFEPAR"} and \code{"SCUANAAUTPAR"} respectively. Please notice that data regarding private schools are not available for most functions in this package.
#'
#' @return An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#'
#' @details Schools different from primary, middle or high schools are classified as \code{"NR"}.
#'
#' @source <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Scuole>
#' @examples
#'
#' \donttest{
#'   Get_Registry(2024, filename = "SCUANAGRAFESTAT", autoAbort = TRUE)
#' }
#'
#'
#'
#'
#' @export



Get_Registry <- function(Year = 2023, filename = c("SCUANAGRAFESTAT", "SCUANAAUTSTAT"),
                         show_col_types = FALSE, autoAbort = FALSE){

  if(!Check_connection(autoAbort)) return(NULL)

  home.url <-"https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Scuole"
  homepage <- NULL
  attempt <- 0
  while(is.null(homepage) && attempt <= 10){
    homepage <- tryCatch({
      httr::content(httr::GET(home.url))
    }, error = function(e){
      message("Cannot read the html; ", 10 - attempt,
              " attempts left. If the problem persists, please contact the maintainer.\n")
      return(NULL)
    })
    attempt <- attempt + 1
  }
  if(is.null(homepage)) {
    message("Maximum attempts reached. Abort. We apologise for the inconvenience")
    return(NULL)
  }
  name_pattern2 <- "([0-9]+)\\.(csv)$"
  links <- homepage %>% rvest::html_nodes("a") %>% rvest::html_attr("href") %>% unique()

  pattern <- year.patternA(Year)

  while(! all(filename %in% c("SCUANAGRAFESTAT", "SCUANAGRAFEPAR", "SCUANAAUTSTAT", "SCUANAAUTPAR"))){
    notfound <- which(!filename %in% c(
      "SCUANAGRAFESTAT", "SCUANAGRAFEPAR", "SCUANAAUTSTAT", "SCUANAAUTPAR"))[1L]
    message(paste0("No file corresponds to the filename `", filename[notfound],"`"))
    cat("Please, choose one among the following (do not use quotes or hyphens in the prompt): \n" )
    cat("    - SCUANAGRAFESTAT for public schools in all regions except the provinces of Aosta, Trento and Bozen \n")
    cat("    - SCUANAGRAFEPAR for private schools in the aforementioned territory \n")
    cat("    - SCUANAAUTSTAT for public schools in the provinces of Aosta, Trento and Bozen \n")
    cat("    - SCUANAAUTPAR for private schools in the aforementioned provinces \n")
    filename[notfound] <- readline(prompt = "    ")
  }

  out <- list()

  for(obj in filename){

    for (string in links[grep(".csv", links)] ) {
      num_numeric_digits <- sum(unlist(gregexpr("[0-9]", string) ) > 0)
      if ( num_numeric_digits >= nchar(pattern) ){
        first_nchar <- stringr::str_extract(string, paste0("[0-9]{", nchar(pattern), "}"))
        if (!is.na(first_nchar) & first_nchar == pattern & length(grep(obj, string)) > 0 ) {
          file_to_download <- string
          break
        }
      }
    }

    if (!"file_to_download" %in% ls()){
      message("Schools registry not available for this year. We apologise for the inconvenience.")
      return(NULL)
    }
    attempt <- 0
    status <- 0
    while(status != 200){
      base.url <- dirname(home.url)
      file.url <- file.path(base.url, file_to_download)
      response <- tryCatch({
        httr::GET(file.url)
      }, error = function(e) {
        message("Error occurred during scraping, attempt repeated ... \n")
        NULL
      })
      status <- response$status_code
      if(is.null(response)){
        status <- 0
      }
      if(status != 200){
        attempt <- attempt + 1
        message("Operation exited with status: ", status, "; operation repeated (",
                10 - attempt, " attempts left)")
      }
      if(attempt >= 10) {
        message("Maximum attempts reached. Abort. We apologise for the inconvenience")
        return(NULL)
      }
    }

    if (httr::http_type(response) %in% c("application/csv", "text/csv", "application/octet-stream")) {

      content <- rawToChar(response$content)
      if(nchar(content)==0){
        message("Empty file. Operation aborted.
        There seems to be something wrong with the website.
        Please contact the maintainer, maybe it could help. \n")
        return(NULL)
      } else {

        dat <- readr::read_csv(content, show_col_types = show_col_types) %>%
          dplyr::mutate(CAPSCUOLA = as.character(.data$CAPSCUOLA))
        names(dat) <- names(dat) %>% stringr::str_replace_all("ANNOSCOLASTICO", "Year") %>%
          stringr::str_replace_all("AREAGEOGRAFICA", "Area") %>%
          stringr::str_replace_all("REGIONE", "Region_description") %>%
          stringr::str_replace_all("PROVINCIA", "Province_description") %>%
          stringr::str_replace_all("CODICEISTITUTORIFERIMENTO", "Reference_institute_code") %>%
          stringr::str_replace_all("DENOMINAZIONEISTITUTORIFERIMENTO", "Reference_institute_name") %>%
          stringr::str_replace_all("CODICESCUOLA", "School_code") %>%
          stringr::str_replace_all("DENOMINAZIONESCUOLA", "School_name") %>%
          stringr::str_replace_all("INDIRIZZOSCUOLA", "School_address") %>%
          stringr::str_replace_all("CAPSCUOLA", "Postal_code") %>%
          stringr::str_replace_all("CODICECOMUNESCUOLA", "Cadastral_code") %>%
          stringr::str_replace_all("DESCRIZIONECOMUNE", "Municipality_description") %>%
          stringr::str_replace_all("DESCRIZIONECARATTERISTICASCUOLA", "School_feature_description") %>%
          stringr::str_replace_all("DESCRIZIONETIPOLOGIAGRADOISTRUZIONESCUOLA", "Order_description") %>%
          stringr::str_replace_all("INDICAZIONESEDEDIRETTIVO", "Headquarters_indication") %>%
          stringr::str_replace_all("INDICAZIONESEDEOMNICOMPRENSIVO", "Comprehensive_institute_venue_information") %>%
          stringr::str_replace_all("INDIRIZZOEMAILSCUOLA", "email_address") %>%
          stringr::str_replace_all("INDIRIZZOPECSCUOLA", "Certified_email_address") %>%
          stringr::str_replace_all("SITOWEBSCUOLA", "Website") %>%
          stringr::str_replace_all("SEDESCOLASTICA", "Venue")
        out[[obj]] <- dat
      }

    } else{
      message("Wrong file type:", httr::http_type(response))
    }
  }
  if(length(out) == 0L) return(NULL)

  res <- do.call(dplyr::bind_rows, out) %>%
    dplyr::relocate(.data$Postal_code, .after = "Municipality_description") %>%
    dplyr::relocate(.data$Reference_institute_name, .after = "Postal_code")%>%
    dplyr::relocate(.data$School_name, .after = "Reference_institute_name") %>%
    dplyr::relocate(.data$School_address, .after = "Municipality_description") %>%
    dplyr::mutate(Area = stringr::str_to_title(.data$Area),
                  Region_description = stringr::str_to_title(.data$Region_description),
                  Province_description = stringr::str_to_title(.data$Province_description),
                  Municipality_description = stringr::str_to_title(.data$Municipality_description),
                  Reference_institute_name = stringr::str_to_title(.data$Reference_institute_name),
                  School_name = stringr::str_to_title(.data$School_name),
                  School_address = stringr::str_to_title(.data$School_address),
                  School_feature_description = stringr::str_to_title(.data$School_feature_description),
                  Order_description = stringr::str_to_title(.data$Order_description))

  return(res)
}

#' Download the database of Italian public schools buildings
#'
#' @description  This function downloads the School Buildings Open Database provided by the Italian Ministry of Education, University and Research.
#'
#'
#' It is one of the main sources of information regarding the infrastructure system of public schools in Italy.
#' For a given year, all available data are downloaded (except for the structural units section, which has a different level of detail) and gathered into a unique dataframe.
#'
#' @param Year Numeric or character value. Reference school year (last available is 2023).
#' Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023}. \code{2022} by default (other databases are not currently available for 2023).
#' @param input_Registry Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#' The school registry corresponding to the year in scope, obtained as output of the  function \code{\link{Get_Registry}}.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
#' \code{NULL} by default.
#' @param certifications Logical. From year 2021/22 onwards, whether to include some safety certifications in the database.
#' Given the particular level of definition of this file, it requires extra computational time (other than the downloading time). \code{FALSE} by default.
#' @param input_AdmUnNames Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#' The ISTAT file including all the codes and all the names of the administrative units for the year in scope, obtained as output of the function \code{\link{Get_AdmUnNames}}.
#' Only  necessary for school years 2015/16, 2017/18 and 2018/19.
#' If \code{NULL} and required, it will be downloaded automatically but not saved in the global environment. \code{NULL} by default.
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param autoAbort Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param show_col_types Logical. If \code{TRUE}, if the \code{verbose} argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.
#' @param t_out Numeric. !! EXPERIMENTAL !! session timeout for scraping and download, in seconds. 900 seconds by default.
#'
#' @source  <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Edilizia+Scolastica>
#'
#' @details
#' This function downloads the raw data; missing observations are not edited; all variables are characters.
#' Since certifications are defined at the level of structural units of the single buildings, here
#' the fields read as the percentage of structural units in a building having a given certificate.
#' To edit the output of this function and convert the relevant variables to numeric or Boolean, please \code{\link{Util_DB_MIUR_num}}.
#' Schools different from primary, middle or high schools are classified as \code{"NR"}. In the example, the data for school year 2022/23 are retrieved.
#'
#'
#' @return An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#'
#' @examples
#'
#' \donttest{
#'   input_DB23_MIUR <- Get_DB_MIUR(2023, autoAbort = TRUE, t_out = 20)
#'
#'   input_DB23_MIUR[-c(1,4,6,9)]
#'
#' }
#'
#'
#' @export


Get_DB_MIUR <- function(Year = 2023, verbose = TRUE, input_Registry = NULL,
                        input_AdmUnNames = NULL, show_col_types = FALSE,
                        certifications = FALSE, autoAbort = FALSE, t_out = 900){

  start.zero <- Sys.time()

  if(!Check_connection(autoAbort)) return(NULL)

  # Link retrieving
  home.url <-"https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Edilizia%20Scolastica"
  homepage <- NULL
  attempt <- 0
  while(is.null(homepage) && attempt <= 10){
    homepage <- tryCatch({
      httr::content(httr::GET(home.url))
    }, error = function(e){
      message("Cannot read the html; ", 10 - attempt,
              " attempts left. If the problem persists, please contact the maintainer.\n")
      return(NULL)
    })
    attempt <- attempt + 1
  }
  if(is.null(homepage)) return(NULL)
  name_pattern <- "([0-9]+)\\.(csv)$"
  pattern <- year.patternB(Year)
  patternA <- year.patternA(Year)
  links <- homepage %>% rvest::html_nodes("a") %>% rvest::html_attr("href") %>% unique()
  links <- links[which(!is.na(links))]
  if (!any(str_detect.general(links, pattern))){
    message("No data available for this year. We apologise for the inconvenience")
    return(NULL)
  }

  files_to_download <- c()
  for (string in links[grep(".csv", links)] ) {
    num_numeric_digits <- sum(unlist(gregexpr("[0-9]", string) ) > 0)
    nchar_min <- min(nchar(pattern))
    nchar_minA <- nchar(patternA)
    nchar_max <- max(nchar(pattern))
    if (num_numeric_digits >= nchar_min && !grepl("EDIUNITASTRUTSTA", string)){
      first_nchar_min <- stringr::str_extract(string, paste0("[0-9]{", nchar_min, "}"))
      first_nchar_max <- stringr::str_extract(string, paste0("[0-9]{", nchar_max, "}"))
      if (!is.na(first_nchar_min) && !is.na(first_nchar_max) && any(pattern %in% c(first_nchar_min, first_nchar_max)) &&
          ! string %in% files_to_download) {
        files_to_download <- append(files_to_download, string)
      }
    } else if(stringr::str_extract(string, paste0("[0-9]{", nchar_minA, "}")) == patternA && grepl("EDIUNITASTRUTSTA", string)){
      if(certifications) files_to_download <- append(files_to_download, string)
    }
  }

  # Scraping
  base.url <- dirname(home.url)
  input_MIUR <- list()
  starttime <- Sys.time()
  for (link in files_to_download) {
    file.url <- file.path(base.url, link)
    status <- 0
    attempt <- 0
    while(status != 200){
      response <- tryCatch({
        httr::GET(file.url, httr::timeout(t_out))
      }, error = function(e) {
        message("Error occurred during scraping, attempt repeated ... \n")
        NULL
      })
      status <- response$status_code
      if(is.null(response)){
        status <- 0
      }
      if(status != 200){
        attempt <- attempt + 1
        message("Operation exited with status: ", status, "; operation repeated (",
                10 - attempt, " attempts left)")
      }
      if(attempt >= 10) {
        message("Maximum attempts reached. Abort. We apologise for the inconvenience")
        return(NULL)
      }
    }

    if (httr::http_type(response) %in% c("application/csv", "text/csv", "application/octet-stream")) {
      content <- rawToChar(response$content)
      if(nchar(content)==0){
        #This way the whole routine aborts even if one file is missing
        message("Empty file. Operation aborted.
      There seems to be something wrong with the website.
      Please contact the maintainer, maybe it could help. \n")
        return(NULL)
      } else{
        if(verbose){
          dat <- readr::read_csv(content, show_col_types = FALSE)
          cat("CSV file downloaded:", link, " ... ")
        } else {
          suppressMessages(dat <- readr::read_csv(content))
        }
      }
      if(grepl("UNITASTRUTSTA", link)){
        dat[, -c(1:4)] <- lapply(dat[, -c(1:4)], function(x){
          gsub(
            "SI", 1, gsub(
              "NO", 0, gsub(
                "Esiste", 1, ignore.case = TRUE, gsub(
                  "Non Esiste", 0, ignore.case = TRUE, gsub(
                    "IN PARTE", NA, ignore.case = TRUE, gsub(
                      "ND", NA, ignore.case = TRUE, gsub(
                        "Non Definito", NA, ignore.case = TRUE, gsub(
                          "Non Comunicato", NA, ignore.case = TRUE, gsub(
                            "Non Richiesto", NA, ignore.case = TRUE, gsub(
                              "^-$", NA, x))))))))))}) %>%
          as.data.frame()
        dat <- dat %>%
          Group_Count(groupcol = c("ANNOSCOLASTICO", "CODICESCUOLA", "CODICEEDIFICIO"),
                      startgroup = 5, count = FALSE, FUN = MeanOrMode)

        dat <- dat %>%
          dplyr::mutate(dplyr::across(names(dat)[unlist(lapply(dat, is.numeric))], as.character))
      }
      input_MIUR[[link]] <- dat
      input_MIUR[[link]] <- input_MIUR[[link]] %>% dplyr::select(-.data$ANNOSCOLASTICO)
      input_MIUR[[link]] <- input_MIUR[[link]][!duplicated(input_MIUR[[link]]),]
      #input_MIUR[[link]] <- input_MIUR[[link]] %>% tidyr::unite(ID, .data$CODICESCUOLA, .data$CODICEEDIFICIO)
     } else {
      if(verbose){
        message(paste("Wrong file type:", httr::http_type(response)) )
        message("Failed to download and process:", link)
      }
    }
    endtime <- Sys.time()
    if(verbose){
      cat(paste(round(difftime(endtime, starttime, units="secs"),
                      2),"seconds required to join it \n ") )
    }
    starttime <- Sys.time()
  }
  if(length(input_MIUR) == 0L) return(NULL)

  # Joining tables
  mapping_MIUR <- input_MIUR[[grep("ANAGRAFE", names(input_MIUR))]]
  DB_MIUR.R <- mapping_MIUR %>% dplyr::select(
    .data$CODICESCUOLA, .data$CODICEEDIFICIO, .data$CODICECOMUNE, .data$DESCRIZIONECOMUNE,
    .data$SIGLAPROVINCIA, .data$CAP)


  # This is for the municipality of Bladen/Plodn/Sappada which changed it province in 2018
  if(!any(pattern %in% year.patternB(2016))){
    DB_MIUR.R <- DB_MIUR.R %>% dplyr::mutate(dplyr::across(.data$SIGLAPROVINCIA, ~ dplyr::case_when(
      toupper(.data$DESCRIZIONECOMUNE) == "SAPPADA" ~ "UD",
      TRUE ~ .data$SIGLAPROVINCIA
    )))
  }

  for ( i in c(1:length(input_MIUR))) {
    if(length(grep("ANAGRAFE", names(input_MIUR)[i]))==0){
      if (nrow(input_MIUR[[i]]) != nrow(DB_MIUR.R) & verbose == TRUE){
        warning(paste("Expected", nrow(DB_MIUR.R), "rows but in",
                      names(input_MIUR)[i], "there are:", nrow(input_MIUR[[i]])))
      }
      DB_MIUR.R <- dplyr::left_join(DB_MIUR.R, input_MIUR[[i]], by = c("CODICESCUOLA", "CODICEEDIFICIO"))
    }
  }

  names(DB_MIUR.R) <- names(DB_MIUR.R) %>% stringr::str_remove_all(".y") %>% stringr::str_remove_all(".x")
  DB_MIUR.R <- DB_MIUR.R[,!duplicated(colnames(DB_MIUR.R))]

  # This is for the province of Naples which happens to have "NA" as abbreviation
  DB_MIUR.R$SIGLAPROVINCIA <- stringr::str_replace_na(DB_MIUR.R$SIGLAPROVINCIA, "NA")

  #DB_MIUR.R <- DB_MIUR.R %>%tidyr::separate(col = .data$ID, into=c("CODICESCUOLA", "CODICEEDIFICIO"), sep="_")

  tabrename <- tabrename.manual()
  for (j in (1:ncol(DB_MIUR.R))){
    if (names(DB_MIUR.R)[j] %in% tabrename$Input){
      names(DB_MIUR.R)[j] <- tabrename[which(tabrename$Input == names(DB_MIUR.R)[j]),4]
    }
  }

  #This is for old data where the municipality is identified through the cadastral code
  if (any(pattern %in% c(year.patternB(2016), year.patternB(2018), year.patternB(2019)))) {

    YearMinus1 <- as.numeric(substr(year.patternA(Year),1,4))
    if(is.null(input_AdmUnNames)) {
      cat("Mapping cadastral codes to municipality (LAU) codes:")
      AdmUnYear <- ifelse(any(pattern %in% c(year.patternB(2016), year.patternB(2018))), YearMinus1+1, YearMinus1)
      AdmUnDate <- ifelse(any(pattern %in% c(year.patternB(2016), year.patternB(2018))), "01-01", "09-01")
      input_AdmUnNames <- Get_AdmUnNames(Date = paste0(AdmUnYear, "-", AdmUnDate), autoAbort = autoAbort)
    }
    CodMun.R <- input_AdmUnNames %>% dplyr::select(.data$Cadastral_code, .data$Municipality_code)

    DB_MIUR.R <- DB_MIUR.R %>% dplyr::rename(Cadastral_code = .data$Municipality_code) %>%
      dplyr::left_join(CodMun.R, by = "Cadastral_code") %>%
      dplyr::relocate(.data$Municipality_code, .after = "Building_code") %>%
      dplyr::select(-.data$Cadastral_code) %>%
      fixMun.manual(Year)
  }


  attempt.registry <- 0
  while (is.null(input_Registry) && attempt.registry == 0) {
    input_Registry <- Get_Registry(Year = Year, autoAbort = autoAbort)
    attempt.registry <- 1
  }
  if(is.null(input_Registry)){
    message("Warning: problem in downloading school registries data, called from
    Get_DB_MIUR. Despite all the work done, at this stage we are forced to abort.
    Please, try downloading & providing
    schools registry separately with Get_Registry, then run Get_DB_MIUR again.
    We apologise for the inconvenience. \n")
    return(NULL)
  }

  left <- input_Registry[,c(1,6,5)] %>% dplyr::filter(.data$School_code %in% DB_MIUR.R$School_code)

  DB_MIUR <- dplyr::left_join(left, DB_MIUR.R, by = "School_code") %>%
    School.order() %>%
    dplyr::mutate(Municipality_description = stringr::str_to_title(.data$Municipality_description))

  enditme <- Sys.time()
  if(verbose){
    cat(paste("Total running time needed to import school buildings data:",
              round(difftime(endtime, start.zero, units="secs"), 2), "seconds \n"  ))
  }

  return(DB_MIUR)
}

#' Convert the raw school buildings data to numeric or Boolean variables
#'
#' @description This function transforms the output variables of the \code{\link{Get_DB_MIUR}} into Boolean or Numeric.
#' Additionally, it removes the columns with an excessive number of missing observations (20.000 by default), and if required it may also delete the rows including missing fields.
#'  In this case, it is possible to keep track of the deleted rows.
#'
#' @param data Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. Input data obtained through the function \code{\link{Get_DB_MIUR}}.
#' If \code{NULL} it will be downloaded automatically with the appropriate arguments, but not saved in the global environment. \code{NULL} by default.
#' @param include_numerics Logical. Whether to include strictly numeric variables alongside with Boolean ones. \code{TRUE} by default.
#' @param include_qualitatives Logical. Whether to include qualitative variables alongside with Boolean ones. \code{FALSE} by default.
#' @param row_cutout Logical. Whether to filter out rows including missing fields. \code{FALSE} by default.
#' @param col_cut_thresh Numeric. The threshold of missing values allowed for each variable.
#' If a variable as a higher number of missing observations, then it is cut out. \code{20.000} by default.
#' Warning: if the option \code{row_cutout} is active, please select a lower threshold (e.g. \code{1000})
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. TRUE by default.
#' @param track_deleted Logical. If \code{TRUE}, the function returns the names of the schools not included in the output dataframe. \code{TRUE} by default.
#' @param unique_buildings Logical. Whether to remove records in which the building code is duplicated and all other fields are as well.
#' As rows are combinations of building ID and school ID, if a school is hosted by several buildings, and each field other than
#' \code{School_code} are duplicated, then only one row is retained. \code{TRUE} by default.
#' @param flag_outliers Logical. Whether to assign NA to outliers in numeric variables. \code{TRUE} by default.
#' @param autoAbort Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param ... Additional arguments to the function \code{Get_DB_MIUR} if \code{data} is not provided.
#'
#' @details The outliers to be set to \code{NA} if \code{flag_outliers} is active are defined as follows: School area or free area surface of less than 50 squared meters,
#' building volume of less than 150 cubic meters, 0 floors in the building.
#'
#'
#'
#' @return If \code{track_deleted == TRUE}, An object of class \code{list} including two objects:
#' \itemize{
#'   \item \code{$data}: object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, the output dataframe.
#'   \item \code{$deleted}: object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. The school IDs of the deleted units.
#' }
#'
#' If \code{track_deleted == FALSE}, the output is only the first element of the list.
#'
#'
#'
#'
#'
#'
#' @examples
#'
#' library(magrittr)
#'
#' DB23_MIUR_num <- example_input_DB23_MIUR %>% Util_DB_MIUR_num(track_deleted = FALSE)
#'
#'
#' DB23_MIUR_num[, -c(1,4,6,8,9,10)]
#' summary(DB23_MIUR_num)
#'
#'
#' @export

Util_DB_MIUR_num <- function(data = NULL, include_numerics = TRUE, include_qualitatives = FALSE,
         row_cutout = FALSE, track_deleted = TRUE, verbose = TRUE, col_cut_thresh = 2e+4,
         unique_buildings = TRUE, flag_outliers = TRUE, autoAbort = FALSE, ...){

  starttime <- Sys.time()

  while(is.null(data)) {
    data <- Get_DB_MIUR(autoAbort = autoAbort, verbose = verbose, ...)
    if(is.null(data)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during school buildings DB retrieving. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }
  y <- as.numeric(data$Year[1])%/%100+1
  startcol <- 10
  uncomma <- function(c, na.rm = FALSE){
    if(na.rm){
      c <- c[which(!is.na(c))]
    }
    if(any(grepl(",", c))){
      c <- stringr::str_remove_all(c, "\\.")
      c <- stringr::str_replace_all(c, ",", ".")
    }
    return(c)
  }
  nc <- ncol(data)
  #init.cutout = c("Address_type","Address_name","Civic_number", "Building_status")
  pattern.out = c("NON DEFINITO","-", "Informazione assente", "Non Comunicato", "ND", "Non richiesto", "IN PARTE")

  starttime <- Sys.time()
  if("Adaptation_year" %in% names(data)){
    data <- data %>% dplyr::mutate(
      Adaptation_year = y - suppressWarnings(as.numeric(.data$Adaptation_year))) %>%
      dplyr::rename(Adaptation_age = .data$Adaptation_year)
  }
  if("Building_year" %in% names(data)){
    data <- data %>% dplyr::mutate(
      Building_year = y - suppressWarnings(as.numeric(.data$Building_year))) %>%
      dplyr::rename(Building_age = .data$Building_year)
  }

  booleans <- c()
  numerics <- c()
  qualitatives <- c()

  for(j in (startcol:nc)){
    C <- unlist(unique(data[which(! data[,j][[1]] %in% pattern.out),j]))
    if(all(toupper(C[which(!is.na(C))]) %in% c ("SI", "NO", "ESISTE", "NON ESISTE"))){
      booleans <- c(booleans,j)
    } else if(any(is.na(suppressWarnings(as.numeric(uncomma(C, na.rm = TRUE)))))){
      qualitatives <- c(qualitatives, j)
    } else {
      numerics <- c(numerics, j)
    }
  }

  if("National_seismic_classification" %in% names(data)){
    SC.idx <- which(names(data) == "National_seismic_classification")
    numerics <- numerics[which(numerics != SC.idx)]
    qualitatives <- c(qualitatives, SC.idx)
  }

  if(verbose) cat("Converting dichotomic variables to Boolean:\n")
  dat.bool <- lapply(data[, booleans], function(x){
    suppressWarnings(as.numeric(gsub(
      "^SI$", 1, gsub(
        "^NO$", 0, gsub(
          "Esiste", 1, ignore.case = TRUE, gsub(
            "Non Esiste", 0 , ignore.case = TRUE, x))))))}) %>%
    as.data.frame()

  if(include_numerics){
    if(verbose) cat("Setting numeric variables: \n")
    dat.num <- lapply(data[, numerics], function(x){
      suppressWarnings(as.numeric(uncomma(x)))
    }) %>% as.data.frame()
    if(flag_outliers){
      if("School_area_surface" %in% names(dat.num)){
        dat.num$School_area_surface[which(dat.num$School_area_surface < 50)] <- NA
      }
      if("Free_area_surface" %in% names(dat.num)){
        dat.num$Free_area_surface[which(dat.num$Free_area_surface < 50)] <- NA
      }
      if("Gross_building_volume" %in% names(dat.num)){
        dat.num$Gross_building_volume[which(dat.num$Gross_building_volume < 150)] <- NA
      }
      if("Floors_number" %in% names(dat.num)){
        dat.num$Floors_number[which(dat.num$Floors_number == 0)] <- NA
      }
      #if("Building_age" %in% names(dat.num)){
      #  dat.num$Building_age[which(dat.num$Building_age > 1500)] <- NA
      #}
    }
  }

  if(include_qualitatives){
    if(verbose) cat("Setting qualitative variables: \n")
    dat.qual <- data[, qualitatives] %>% apply(MARGIN = 2, function(x){
      x[which(x %in% pattern.out)] <- NA
      return(x)
    }) %>% as.data.frame()
  }

  DB <- data
  DB[, booleans] <- dat.bool

  if(include_numerics & include_qualitatives){
    DB[, numerics] <- dat.num
    DB[, qualitatives] <- dat.qual
  } else if(include_numerics & !include_qualitatives){
    DB[, numerics] <- dat.num
    DB <- DB[-qualitatives]
  } else if(!include_numerics & include_qualitatives){
    DB[, qualitatives] <- dat.qual
    DB <- DB[-numerics]
  } else DB <- DB[-c(qualitatives, numerics)]

  NAs <- data.frame(
    NAs = apply(DB[startcol:ncol(DB)], MARGIN = 2,function(x){
      sum(is.na(x))}))

  # Column cutout
  DB <- DB %>% dplyr::select( -rownames(
    dplyr::filter(NAs, .data$NAs > col_cut_thresh)))

  #Row cutout
  IDcol <- c(2,5)
  deleted <- data.frame(matrix(nrow = 0, ncol = length(IDcol)))
  if(row_cutout) {

    names(deleted) <- names(data)[IDcol]
    IDcolName <- names(DB)[IDcol]
    for(j in c(startcol:ncol(DB))){
      deleted.new <- DB %>% dplyr::filter(is.na(!!rlang::sym(names(DB)[j]))) %>%
        dplyr::select(IDcol)
      DB <- DB %>% dplyr::anti_join(deleted.new, colnames(deleted.new))
      if (nrow(deleted.new) > 0 & verbose == TRUE){
        cat("deleted", nrow(deleted.new), "units whose field", names(DB)[j], "is missing \n")
      }
      deleted <- rbind(deleted, deleted.new)
    }
  }

  DB <- DB %>%
    dplyr::mutate(Province_code = as.numeric(substr(
      .data$Municipality_code, 1, nchar(.data$Municipality_code)-3)) ) %>%
    dplyr::relocate(.data$Province_code, .after = "Municipality_description")

  if(unique_buildings){
    DB <- DB %>%
      dplyr::distinct(dplyr::across(
        setdiff(names(DB), "School_code")), .keep_all = TRUE)
  }


  if(track_deleted & row_cutout){
    res <- list(data = DB, deleted = deleted)
  } else res <- DB

  endtime <- Sys.time()
  if(verbose){
    cat("Total running time needed to convert data to numeric variables:",
        round(difftime(endtime, starttime, units="secs") ,2), "seconds \n"  )
  }

  return(res)
}

#' Download the data regarding the broad band connection activation in Italian schools
#'
#' @description
#' Retrieves the data regarding the activation date of the ultra-broadband connection in schools
#' and indicates whether the connection was activated or not at a certain date.
#'
#'
#' @param Date Object of class \code{Date}. The date at which it is required
#' to determine if the broad band connection has been activated or not.
#' By default, the first day of last month, which we assume to be the last update of the dataset.
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.
#' @param autoAbort Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param include_municipality_code Logical. Whether to include municipality codes.
#'  \code{TRUE} by default.
#' @param input_School2mun Object of class \code{list} obtained with \code{\link{Get_School2mun}}.
#' If \code{include_municipality_code == TRUE}, the mapping from school codes to municipality (and province) codes.
#' If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.
#' @param input_Registry If \code{input_School2mun} is required, an object of class \code{tbl_df}, \code{tbl} and \code{data.frame}
#' corresponding to the national school registry (preferably of the last year, i.e. 2024/2025) obtained with \code{\link{Get_Registry}}. NULL by default.
#' @param input_AdmUnNames If \code{input_School2mun} is required, an object of class \code{tbl_df}, \code{tbl} and \code{data.frame}
#' corresponding to the statistical codes of administrative units (preferably referring to the period corresponding to school year 2024/2025)
#'  obtained with \code{\link{Get_AdmUnNames}}. NULL by default.
#' @return An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
#'  The variables \code{BB_Activation_date} and \code{BB_Activation_staus} indicate
#'  the activation date and activation status of the broadband connection at the selected date.
#'
#' @source  Broadband dashboard: <https://bandaultralarga.italia.it/scuole-voucher/dashboard-scuole/>
#'
#'
#' @details Ultra - Broadband is defined as everlasting internet connection with a
#' maximum speed of 1 gigabit per second, with a minimum guaranteed speed of
#' 100 megabits/second both on the uploading and downloading operations, until
#' the peering point is reached, as declared on the data provider's website:
#' <https://bandaultralarga.italia.it/scuole-voucher/progetto-scuole/>.
#' In the example the broadband availability at the beginning of school  year 2022/23 (1st september 2022) is shown.
#'
#' @examples
#'
#' \donttest{
#'
#'   Broadband_220901 <- Get_BroadBand(Date = as.Date("2022-09-01"), autoAbort = TRUE)
#'
#'   Broadband_220901
#'
#'   Broadband_220901[, c(9,6,13,14)]
#'
#' }
#'
#'
#'
#'
#' @export


Get_BroadBand <- function(Date = as.Date(format(as.Date(format(Sys.Date(), "%Y-%m-01"))-1, "%Y-%m-01")),
                          include_municipality_code = TRUE,
                          input_School2mun = NULL,
                          input_Registry = NULL,
                          input_AdmUnNames = NULL,
                          verbose=TRUE, autoAbort = FALSE){

  if(!Check_connection(autoAbort)) return(NULL)

  starttime <- Sys.time()

  home.url <- "https://bandaultralarga.italia.it/scuole-voucher/dashboard-scuole/"
  homepage <- NULL
  attempt <- 0
  while(is.null(homepage) && attempt <= 10){
    homepage <- tryCatch({
      xml2::read_html(home.url)
    }, error = function(e){
      message("Cannot read the html; ", 10 - attempt,
              " attempts left. If the problem persists, please check if the provider's website is working
              or contact the package maintainer.\n")
      return(NULL)
    })
    attempt <- attempt + 1
  }
  if(is.null(homepage)) return(NULL)

  links <- homepage %>% rvest::html_nodes("a") %>% rvest::html_attr("href") %>% unique()
  link <- links[grep("Report", links, ignore.case = TRUE)]
  base.url <- dirname(home.url)

  file.url <- xml2::url_absolute(link, base.url)

  status <- 0
  attempt <- 0
  while(status != 200){
    response <- tryCatch({
      httr::GET(file.url)
    }, error = function(e) {
      message("Error occurred during scraping, attempt repeated ... \n")
      NULL
    })
    status <- response$status_code
    if(is.null(response)){
      status <- 0
    }
    if(status != 200){
      attempt <- attempt + 1
      message("Operation exited with status: ", status, "; operation repeated (",
              10 - attempt, " attempts left)")
    }
    if(attempt >= 10) {
      message("Maximum attempts reached. Abort. We apologise for the inconvenience")
      return(NULL)
    }
  }
  if(is.null(response$content)){
    message("It seems that Broadband data are not available.
            We apologise for the inconvenience")
    return(NULL)
  }

  temp <- tempdir()
  if(!dir.exists(temp)){
    dir.create(temp)
  }

  excel <- paste0(temp, "/tempdata.xlsx")
  writeBin(as.vector(response$content), excel)

  if(! "readxl" %in% rownames(utils::installed.packages())){
    if(verbose){
      cat("Parsing broadband data from the xml file ...\n")
    }
    utils::unzip(excel, exdir = temp)
    templist <- list.files(temp, full.names = TRUE)
    xl.link <- templist[which(substr(templist, nchar(templist)-1, nchar(templist))=="xl")]
    ws.link <- grep("worksheets", list.files(xl.link, full.names = TRUE) , value = TRUE)
    #ws.link <- xl.link %>% list.files(full.names = TRUE) %>% grep("worksheets", ., value = TRUE)
    xml.link <-  grep(".xml", list.files(ws.link, full.names = TRUE), value = TRUE)

    worksheet1 <- xml2::read_xml(xml.link[1])
    descr.d1 <- xml2::xml_ns(worksheet1)[["d1"]]

    rows <- xml2::xml_find_all(worksheet1, ".//d1:sheetData/d1:row", namespaces = c(d1 = descr.d1))

    data_list <- lapply(rows, function(X) {
      cells <- xml2::xml_find_all(X, "./d1:c", ns = c(d1 = descr.d1))

      row.ls <- sapply(cells, function(Y) {
        attr <- xml2::xml_attr(Y, "t")
        val_node <- xml2::xml_find_first(Y, ".//d1:v", ns = c(d1 = descr.d1))

        if (!is.na(attr) && attr == "inlineStr") {
          return(xml2::xml_text(
            xml2::xml_find_first(Y, ".//d1:t", ns = c(d1 = descr.d1))))
        } else if (!is.na(val_node)) {
          return(xml2::xml_text(val_node))
        } else return(NA)
      })
      row.value <- as.vector(unlist(row.ls))

      cell_refs <- xml2::xml_attr(cells, "r")
      row.idx <- as.numeric(regmatches(cell_refs[1], gregexpr("[0-9]+", cell_refs[1] )))
      if(!is.na(row.idx)){
        if(verbose && row.idx%%5000 == 0){
          cat("Parsed the first ", row.idx, "rows \n")
        }
      }

      if(length(row.value)>0){
        if(!is.na(row.value[1])){
          return(row.value)
        } else return (NA)
      }
    })

    dd <- as.data.frame(do.call(rbind, data_list[-1]))
    names(dd) <- data_list[[1]]
    dd <- dd %>% dplyr::filter(!is.na(.data$Comune)) %>%
      dplyr::group_by(.data$`Codice univoco Infratel`) %>%
      dplyr::ungroup() %>%
      dplyr::mutate(Latitudine = as.numeric(.data$Latitudine)) %>%
      dplyr::mutate(Longitudine = as.numeric(.data$Longitudine))

  } else {
    dd <- readxl::read_xlsx(excel, col_types = "text")
  }

  while(!"Date" %in% class(Date)){
    message("Please, provide a date in format `yyyy-mm-dd` (do not insert quotes in the prompt)")
    Date.new <- readline(prompt = "")
    Date <- as.Date(Date.new)
  }

  broadband <- dd %>% dplyr::filter(substr(.data$`Codice Sede`,1,2) != "XX") %>%
    dplyr::select(.data$Regione, .data$Provincia, .data$Comune, .data$`Codice Sede`,
                  .data$`Codice univoco Infratel`, .data$Denominazione,
                  .data$Latitudine, .data$Longitudine,
                  .data$`Date prevista attivazione`, .data$`Data effettiva attivazione`)%>%
    dplyr::mutate(`Date prevista attivazione` =
                    as.numeric(.data$`Date prevista attivazione` )- 25569) %>%
    dplyr::mutate(`Data effettiva attivazione` =
                    as.numeric(.data$`Data effettiva attivazione`) - 25569) %>%
    dplyr::mutate(
      BB_Activation_date =as.Date(
        dplyr::coalesce(.data$`Data effettiva attivazione`,
                        .data$`Date prevista attivazione`))) %>%
    dplyr::mutate(Provincia = toupper(.data$Provincia)) %>%
    dplyr::select(-.data$`Data effettiva attivazione`,
                  -.data$`Date prevista attivazione`) %>%
    dplyr::rename(Region_description = .data$Regione) %>%
    dplyr::rename(Province_description = .data$Provincia) %>%
    dplyr::rename(Municipality_description = .data$Comune) %>%
    dplyr::rename(School_code = .data$`Codice Sede`) %>%
    dplyr::rename(School_name = .data$Denominazione) %>%
    dplyr::rename(Infratel_code = .data$`Codice univoco Infratel`) %>%
    dplyr::rename(Latitude = .data$Latitudine) %>%
    dplyr::rename(Longitude = .data$Longitudine) %>%
    School.order() %>%
    dplyr::mutate(BB_Activation_status = as.numeric(.data$BB_Activation_date <= Date)) %>%
    dplyr::mutate(Province_description = ifelse(
      grepl("-CESENA$", .data$Province_description),
      "FORLI'-CESENA", .data$Province_description)) %>%
    dplyr::left_join(prov.names()[,-c(3, 4)], by = "Province_description") %>%
    dplyr::relocate(.data$Region_code, .before = 1) %>%
    dplyr::relocate(.data$Province_code, .after = .data$Region_description)

  if(dir.exists(temp)){
    temp.contents <- list.files(temp, full.names = T)
    for(i in (1:length(temp.contents))) {
      unlink(temp.contents[i], recursive = TRUE)
    }
  }

  if(include_municipality_code){
    broadband <- Util_BroadBand2mun(broadband, input_School2mun = input_School2mun,
                                    input_Registry = input_Registry, input_AdmUnNames = input_AdmUnNames,
                                    autoAbort = autoAbort, verbose = FALSE)
  }

  endtime <- Sys.time()
  if (verbose && !is.null(broadband)) {
    cat(paste(round(difftime(endtime, starttime, units="secs") ,2),
              "seconds required to download ultra-broadband activation data \n") )
  }
  return(broadband)
}

#' Subset of the administrative codes of municipalities
#'
#'
#' This table includes the administrative codes of the municipalities from four regions: Molise, Campania, Apulia and Basilicata,
#' as of June 30th 2022; some strings in field \code{Municipality_description} including accents have been forced to ASCII.
#' The whole dataset can be retrieved with the command \code{Get_AdmUnNames(Date = "2022-06-30")}
#' @seealso \code{\link{Get_AdmUnNames}}
#'
#' @format ## `example_AdmUnNames20220630`
#' A data frame with 1,074 rows and 5 columns:
#' \itemize{
#'   \item \code{Province_code} Numeric; the NUTS-3 administrative code
#'   \item \code{Province_initials} Character;abbreviated NUTS-3 denomination.
#'   \item \code{Municipality_code} Character; the ISTAT LAU (municipality) ID.
#'   \item \code{Municipality_description} Character; the municipality name.
#'   \item \code{Cadastral_code} Character; a LAU - level ID code, different from the official ISTAT municipality code.
#'   It is used in the school registry (see \code{\link{example_input_Registry23}})
#' }
#' @source <https://www.istat.it/it/archivio/6789>
"example_AdmUnNames20220630"

#' Arrange the number of teachers per students in public Italian schools at the province level
#'
#' @description  This function provides the average number of teachers per students in Italian public schools at the province level.
#'
#'
#'
#'
#' @param Year Numeric or character value. Reference school year for the school registry data (last available is 2022).
#' Available in the formats: \code{2022}, \code{"2021/2022"}, \code{202122}, \code{20212022}. \code{2023} by default
#' @param input_nteachers Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. The number of teachers by province, obtained as output of the function \code{\link{Get_nteachers_prov}}. If \code{NULL}, the function will download it automatically but it will not be saved in the global environment. \code{NULL} by default.
#' @param input_nstud_raw Object of class 'list', including two objects of class \code{tbl_df}', \code{tbl} and \code{data.frame}, obtainded as output of the \code{\link{Get_nstud}} function with the default \code{filename} parameter.
#' Not necessary if the argument \code{input_nstud_aggr} is provided. If \code{NULL}, the function will download it automatically but it will not be saved in the global environment. \code{NULL} by default.
#' @param input_nstud_aggr Object of class \code{list}, including two objects of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Group_nstud}}. If \code{NULL}, the function will compute it manually but it will not be saved in the global environment. \code{NULL} by default.
#' @param nteachers_filename Character. If \code{input_nteachers} is not provided, which data to retrieve regarding the number of teachers/personnel; see \code{\link{Get_nteachers_prov}}
#'  \code{c("DOCTIT", "DOCSUP")} by default, i.e. tenured teachers and temporary teachers.
#' @param verbose Logical. If \code{TRUE}, the user keeps track of the main underlying operations. TRUE by default.
#' @param autoAbort Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param ... Arguments to \code{\link{Group_nstud}} if argument \code{input_nstud_aggr} is not provided
#'
#'
#'
#'
#'
#' @return An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}
#'
#'
#'
#' @examples
#'
#'
#' \donttest{
#'   input_nstud23 <- Get_nstud(2023, filename ="ALUCORSOINDCLASTA", autoAbort = TRUE)
#'   Registry23 <- Get_Registry(2023, autoAbort = TRUE)
#'   School2mun23 <- Get_School2mun(2023, input_Registry = Registry23, autoAbort = TRUE)
#'
#'
#'   nstud23.aggr <- Group_nstud(Year = 2023, data = input_nstud23,
#'     input_Registry = Registry23, input_School2mun = School2mun23,
#'     autoAbort = TRUE)
#'
#'   input_nteachers23 <- Get_nteachers_prov(2023, autoAbort = TRUE)
#'
#'   teachers4stud <- Group_teachers4stud(Year = 2023,
#'                    input_nteachers = input_nteachers23,
#'                    input_nstud_aggr = nstud23.aggr, autoAbort = TRUE)
#'
#'   teachers4stud[, -c(1, 2, 10, 11)]
#'
#'   summary(teachers4stud)
#'}
#'
#' @export



Group_teachers4stud <- function(Year = 2023, input_nteachers = NULL,
                                nteachers_filename = c("DOCTIT", "DOCSUP"),
                                verbose = TRUE,input_nstud_raw = NULL,
                                input_nstud_aggr = NULL, autoAbort = FALSE, ...){

  while(is.null(input_nteachers)){
    if(verbose) cat("Retrieving number of teachers by province")
    input_nteachers <- Get_nteachers_prov(Year = Year, verbose = verbose,
                                          filename = nteachers_filename, autoAbort = autoAbort)
    if(is.null(input_nteachers)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during teachers counts retrieving. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  while (is.null(input_nstud_aggr)){
    if(is.null(input_nstud_raw)){
      cat("Retrieving and aggregating number of students by province \n" )} else {
        cat("Aggregating number of students by province \n")
      }
    input_nstud_aggr <- Group_nstud(Year = Year, data = input_nstud_raw, autoAbort = autoAbort, ...)
    if(is.null(input_nstud_aggr)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during students counts processing Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  if(!is.data.frame(input_nstud_aggr)){
    input_nstud_aggr <- input_nstud_aggr$Province_data
  }
  nstud <- input_nstud_aggr[, which(names(input_nstud_aggr)%in% c(
      "Province_code", "Order", "nschools", "Tot_Students", "Tot_Classes",
      "Students_per_class_Tot", "Availability", "Availabili", "Inner_area"))]
  if("Availability"%in% names(nstud)){
    names(nstud) <- names(nstud) %>%
      stringr::str_replace("Availability", "nstud_availability")
  }

  if("TOT_TEACHERS" %in% toupper(names(input_nteachers))){
    nstud4teacher <- input_nteachers %>% dplyr::left_join(nstud, by = c("Province_code", "Order")) %>%
      dplyr::mutate(Teachers_per_student = .data$Tot_teachers/.data$Tot_Students,
                    Teachers_per_class = .data$Tot_teachers/.data$Tot_Classes)

    if("TOT_ATA" %in% toupper(names(input_nteachers))){
      nstud4teacher <- nstud4teacher %>%
        dplyr::mutate(ATA_per_student = .data$Tot_ATA/.data$Tot_Students,
                      ATA_per_class = .data$Tot_ATA/.data$Tot_Classes)
    }

  } else {
    nstud4teacher <- input_nteachers %>% dplyr::left_join(nstud, by = "Province_code") %>%
      dplyr::mutate(ATA_per_student = .data$Tot_ATA/.data$Tot_Students,
                    ATA_per_class = .data$Tot_ATA/.data$Tot_Classes)
  }



  return(nstud4teacher)
}

#' @keywords internal
#'
Check_connection <- function(autoAbort = FALSE){
  if(!autoAbort){
    while(!curl::has_internet()){
      message("There seems to be no internet connection. Would you wait or abort the operation? \n",
              "    - To hold on for any number of seconds, press the corresponding number \n",
              "    - To abort the operation, press `A`\n")

      holdOn <- readline(prompt = "    ")
      if (toupper(holdOn) == "A") {
        message("You chose to abort the operation. Please try later \n")
        return(FALSE)
      } else {
        n <- as.numeric(holdOn)
        if(is.na(n)) n <- 10
        cat("You chose to wait ", n, " seconds \n")
        Sys.sleep(n)
      }
    }
    return(TRUE)
  } else {
    res <- curl::has_internet()
    if(!res) message("There seems to be no internet connection. Please try later \n")
    return(res)
  }
}

#' Map school data
#'
#' @description This function displays a map of the data arranged trough the function \code{\link{Set_DB}}.
#' It supports two kinds of map:
#' \itemize{
#'   \item Interactive map (default option), which allows the user to visualize all the data in scope through the interactive popup, and
#'   \item Static map (ggplot), which can be easily exported in \code{.pdf} objects.
#' }
#'
#'
#' The user must select a variable to display.
#' It is possible to insert either a readily-downloaded database obtained through the function \code{\link{Set_DB}} or the basic inputs to plug in that function, other than an input shapefile. Relevant arguments not provided by the user will be download automatically, but not saved into the global environment. However we suggest to plug in at least some inputs, as otherwise the running time may be long.
#' This function generalises the functionalities of the more data-specific functions \code{\link{Map_School_Buildings}} and \code{\link{Map_Invalsi}}.
#'
#' @param data Object of class \code{tbl.df}, \code{tbl} and \code{data.frame}, obtained as output of the \code{\link{Set_DB}} function. If NULL, it will be arranged automatically but not saved into the global environment. NULL by default.
#' @param field Character. The variable to display in the map.
#' @param level Character. The administrative level of detailed at which the target variable must be displayed. Either \code{"LAU"}/\code{"Municipality"} or \code{"NUTS-3"}/\code{"Province"}. If the \code{"data"} argument is plugged in, please select the same level. \code{"LAU"} by default.
#' @param plot Character. The type of map to display; either \code{"mapview"} for interactive maps, or \code{"ggplot"} for static maps. \code{"mapview"} by default.
#'  !!! Warning. As \code{mapview} turns out to have been taken away from CRAN,
#'  if it has not been previously installed by the user, \code{plot = "ggplot"} will be forced.
#' @param popup_height Numeric. The height of the popup table in terms of pixels if the \code{"mapview"} mode is chosen. \code{200} by default.
#' @param main_pos Character.Where the header should be placed if the \code{ggplot} mode is chosen.
#' The header is located on the top if \code{"top"} is given as input, and above the legend scale otherwise. \code{"top"} by default.
#' @param main Character. The title to display in the \code{"ggplot"} rendering options.
#' @param col_rev Logical. Whether the scale of the colour palette should be reverted or not. \code{FALSE} by default.
#' @param pal Character. The palette to use if the \code{"mapview"} mode is chose. \code{"viridis"} by default.
#' @param Year Numeric or Character. The reference school year, needed if either \code{data} or \code{input_shp} are not provided.
#' Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023}.  \code{2023} by default.
#' @param input_shp Object of class \code{sf}, \code{tbl.df}, \code{tbl} and \code{data.frame}. The relevant shapefiles of Italian administrative boundaries,
#' at the selected level of detail (LAU or NUTS-3). If \code{NULL}, it is downloaded automatically but not saved in the global environment. \code{NULL} by default.
#' @param region_code Numeric. The NUTS-2 codes of the units that must be displayed.
#' If the level is set to \code{"LAU"}, choosing a limited number of regions is recommended.
#' By default, \code{c(1,3,5:20)}, i.e. all Italian regions except the provinces of Aosta, Trento and Bozen which have data availability issues.
#' @param order Character. The educational level. Either \code{"Primary"} (primary school), \code{"Middle"} (middle school), or \code{"High"} (high school).
#' If the data include the Invalsi census survey, please select a level consistent with the chosen educational grade. \code{"Media"} by default.
#  @param Invalsi Logical. whether the data to map include the Invalsi survey. \code{TRUE} by default.
#  @param Invalsi.subj Character. If \code{Invalsi == TRUE}, the school subject(s) to include, among \code{"English_listening"}/\code{"ELI"}, \code{"English_reading"}/\code{"ERE"}, \code{"Italian"}/\code{"Ita"} and \code{"Mathematics"}/\code{"MAT"}. All four by default.
#' @param autoAbort Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
#' @param only_observed Logical. Whether to remove unobserved areas from the plot. \code{FALSE} by default.
#' @param ... Additional arguments for the input database, if not provided; see \code{\link{Set_DB}}
#'
#'
#' @return If \code{plot == "mapview"}, an object of class \code{mapview}. Otherwise, if \code{plot == "ggplot"}, an object of class \code{gg} and \code{ggplot}.
#'
#'
#' @examples
#'
#'
#'
#' DB23 <- Set_DB(Year = 2023, level = "NUTS-3",
#'        Invalsi_grade = c(10,13), NA_autoRM = TRUE,
#'        input_Invalsi_IS = example_Invalsi23_prov, input_nstud = example_input_nstud23,
#'        input_InnerAreas = example_InnerAreas,
#'        input_School2mun = example_School2mun23,
#'        input_AdmUnNames = example_AdmUnNames20220630,
#'        nteachers = FALSE, BroadBand = FALSE, SchoolBuildings = FALSE)
#'
#'
#'
#'
#' Map_DB(DB23, field = "Students_per_class_13", input_shp = example_Prov22_shp, level = "NUTS-3",
#'  col_rev = TRUE, plot = "ggplot")
#'
#' Map_DB(DB23, field = "Inner_area", input_shp = example_Prov22_shp, order = "High",
#'  level = "NUTS-3",col_rev = TRUE, plot = "ggplot")
#'
#' Map_DB(DB23, field = "M_Mathematics_10", input_shp = example_Prov22_shp, level = "NUTS-3",
#'  plot = "ggplot")
#'
#'
#' @import sf
#'
#' @export

Map_DB <- function(
    data = NULL,
    Year = 2023,
    field,
    level = "LAU",
    plot = "mapview",
    popup_height = 200,
    col_rev = FALSE,
    pal = "viridis",
    input_shp = NULL,
    region_code = c(1:20),
    main_pos = "top",
    main = "",
    order = NULL,
    autoAbort = FALSE,
    only_observed = FALSE,
    ...){

  #rlang::check_installed("sf", reason = "Package \"sf\" must be installed to manage geometries in shapefiles.")
  #library(sf)

  Year.n <- as.numeric(substr(year.patternA(Year),1,4)) + 1
  YearMinus1.n <- Year - 1
  while(is.null(input_shp)){
    input_shp <- Get_Shapefile(Year = ifelse(
      Year.n %in% c(2016, 2018), Year.n, YearMinus1.n), level = level, lightShp = TRUE,
      autoAbort = autoAbort)
    if(is.null(input_shp)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during shapefile retrieving. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  #args <- as.list(match.call())[-1]
  #args <- args[which(names(args) %in% names(formals(Set_DB)))]

  while(is.null(data)) {
    data <- Set_DB(Year = Year, level = level, autoAbort = autoAbort, ...)  #do.call(Set_DB, args)
    if(is.null(data)){
      if(!autoAbort){
        holdOn <- ""
        message("Error during data retrieving. Would you abort the whole operation or retry?",
                "    - To abort the operation, press `A` \n",
                "    - To retry data retrieving, press any other key \n")
        holdOn <- readline(prompt = "    ")
        if(toupper(holdOn) == "A"){
          cat("You chose to abort the operation \n")
          return(NULL)
        } else {
          cat("You chose to retry \n")
        }
      } else return(NULL)
    }
  }

  while(! field %in% names(data)){
    message(paste("The variable", field, "does not seem to belong to the current database.
                  Please insert another one (do not use quotes in the prompt)"))
    field <- readline(prompt = "  > ")
  }

  #if(!any(grep(c("M_Italian|M_Mathematics|M_English_L|M_English_R|S_Italian|S_Mathematics|S_English_L|S_English_R"), names(data)))){#}

  if(length(unique(data$Order)) == 1L) {
    order <- unique(data$Order)[1L]
  }

  if(any(grep("_2$|_3$|_4$|_5$", field))){
    order <- "Primary"
  } else if(any(grep("_6$|_7$|_8$", field))){
    order <- "Middle"
  }else if(any(grep("_9$|_10$|_11$|_12$|_13$", field))) {
    order <- "High"
  }

  data <- data %>% dplyr::filter(.data$Order == order)


  if(toupper(level) %in% c("LAU", "MUNICIPALITY", "NUTS-4")){
    res <- input_shp %>% dplyr::select(.data$COD_REG, .data$PRO_COM_T) %>%
      rename_by_idx(c(1,2), into = c("Region_code", "Municipality_code")) %>%
      dplyr::left_join(data, by= "Municipality_code") %>%
      dplyr::filter(.data$Region_code %in% region_code)
  } else {

    if("Municipality_code" %in% names(data)){
      startgroup <- min(
        which(
          unlist(
            lapply(dplyr::select(data, -.data$Province_code),
                   function(x) any(is.numeric(x)))))) + 1
      data <- data %>% Group_Count(
        groupcol = c("Province_code", "Province_initials", "Order"), FUN =MeanOrMode,
        startgroup = startgroup, countname = "nmun")
      if("nbuildings" %in% names(data)){
        data <- data %>% dplyr::mutate(nmun = .data$nmun * .data$nbuildings) %>%
          dplyr::select(-.data$nbuildings) %>%
          dplyr::rename(nbuildings = .data$nmun)
      }
      else if("nschools" %in% names(data)){
        data <- data %>% dplyr::mutate(nmun = .data$nmun * .data$nschools) %>%
          dplyr::select(-.data$nschools) %>%
          dplyr::rename(nschools = .data$nmun)
      } else {
        data <- data %>% dplyr::select(-.data$nmun)
      }
    }

    res <- input_shp %>% dplyr::select(.data$COD_REG, .data$COD_PROV) %>%
      rename_by_idx(c(1,2), into = c("Region_code", "Province_code")) %>%
      dplyr::left_join(data,by = "Province_code") %>%
      dplyr::filter(.data$Region_code %in% region_code)
  }

  nfield <- which(names(res) == field)

  if(only_observed){
    res <- res[which(!is.na(res[,nfield])),]
  }

  if(main == ""){
    layername <- paste0(field, ", year: ", Year, ", ", order, " schools")
  } else layername <- main

  if(!"mapview" %in% rownames(utils::installed.packages())) plot <- "ggplot"


  if(plot == "ggplot"){

    #res.nospatial <- sf::st_drop_geometry(res)

    if(main_pos == "top"){
      ggplot2::ggplot() + ggplot2::geom_sf(data = res, ggplot2::aes(
        fill = !!rlang::sym(field))) +
        ggplot2::labs(fill = "") + ggplot2::ggtitle(layername) +
        ggplot2::scale_fill_viridis_c(na.value = "white",
                                      direction = 2*(1/2 - as.numeric(col_rev))) +
        ggplot2::theme_minimal()
    } else {
      ggplot2::ggplot() + ggplot2::geom_sf(data = res, ggplot2::aes(
        fill = !!rlang::sym(field))) +
        ggplot2::labs(fill = field)  +
        ggplot2::scale_fill_viridis_c(na.value = "white",
                                      direction = 2*(1/2 - as.numeric(col_rev))) +
        ggplot2::theme_minimal()
    }

  } else {

    n <- length(unique(unlist(dplyr::select(dplyr::filter(
      data, .data$Province_code %in% res$Province_code), field)))) - 1

    if (col_rev == FALSE) {
      brew <- grDevices::hcl.colors(n, palette = pal)
    } else {
      brew <- rev(grDevices::hcl.colors(n, palette = pal))
    }
    pop = leafpop::popupTable(res)
    suppressWarnings( mapview::mapview(
      res, zcol = field, col.regions = brew,layer.name = layername,
      popup = paste0(set_popup_height(popup_height) , pop) ) )
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Get_nteachers_prov.R
\name{Get_nteachers_prov}
\alias{Get_nteachers_prov}
\title{Download the number of teachers in Italian schools by province}
\source{
<https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Personale+Scuola>
}
\usage{
Get_nteachers_prov(
  Year = 2023,
  verbose = TRUE,
  show_col_types = FALSE,
  filename = c("DOCTIT", "DOCSUP"),
  t_out = 3,
  autoAbort = FALSE,
  forceAbort = FALSE
)
}
\arguments{
\item{Year}{Numeric or character value. Reference school year for the school registry data (last available is 2023).
Available in the formats: \code{2022}, \code{"2021/2022"}, \code{202122}, \code{20212022.} \code{2023} by default}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{show_col_types}{Logical. If \code{TRUE}, if the 'verbose' argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.}

\item{filename}{Character. Which data to retrieve among the province counts of teachers/school personnel.
By default it is \code{c("DOCTIT", "DOCSUP")}, which are the file names used so far for the number of tenured and temporary teachers respectively.
Other file names are the following:


\code{"ATATIT"} for the number of tenured non-teaching personnel


\code{"ATASUP"} for the number of temporary non-teaching personnel}

\item{t_out}{Numeric. !! EXPERIMENTAL !! session timeout for scraping and download, in seconds. 3 seconds by default.}

\item{autoAbort}{Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}

\item{forceAbort}{Logic. Whether to force a limited number of attempts to scrape the provider webpage.
Not recommended to be switched to \code{TRUE}. Only insert to speedup function failure when
there is certainty that the provider website is out. \code{FALSE} by default.
Experimental, may be implemented in other \code{Get_} functions.}
}
\value{
An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
}
\description{
This functions downloads the number of teachers by province from the open website of the Italian Ministry of Education, University and Research.
}
\details{
Please notice that by default, the function returns the count of the number of tenured and temporary teachers.
If either the count of non-teaching personnel or the count of a single category of teaching personnel is needed, please adapt
the \code{filename} argument accordingly.
}
\examples{


nteachers23 <- Get_nteachers_prov(Year=2023, filename="DOCTIT",
               autoAbort=TRUE, forceAbort = TRUE)
nteachers23[, c(3,4,5)]


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Util_Invalsi_filter.R
\name{Util_Invalsi_filter}
\alias{Util_Invalsi_filter}
\title{Filter the Invalsi data by subject, school grade and year.}
\usage{
Util_Invalsi_filter(
  data = NULL,
  subj = c("ELI", "ERE", "ITA", "MAT"),
  grade = 8,
  level = "LAU",
  WLE = FALSE,
  Year = 2023,
  verbose = TRUE,
  autoAbort = FALSE
)
}
\arguments{
\item{data}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
The raw Invalsi survey data that has to be filtered, obtained as output of the \code{\link{Get_Invalsi_IS}} function.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default}

\item{subj}{Character. The school subject(s) to include, among \code{"Englis_listening"}/\code{"ELI"}, \code{"English_reading"}/\code{"ERE"}, \code{"Italian"}/\code{"ITA"} and \code{"Mathematics"}/\code{"MAT"}. All four by default.}

\item{grade}{Numeric. The school grade to chose. Either \code{2} (2nd year of primary school), \code{5} (last year of primary school), \code{8} (last year of middle shcool), \code{10} (2nd year of high school) or \code{13} (last year of school). \code{8} by default}

\item{level}{Character. The level of aggregation of Invalsi census data. Either \code{"NUTS-3"}, \code{"Province"}, \code{"LAU"}, \code{"Municipality"}.
If an input dataframe is provided, please select the same level of aggregation. \code{"LAU"} by default}

\item{WLE}{Logical. Whether the variable to choose should be the average WLE score rather that the percentage of sufficient tests, if both are available. \code{FALSE} by default}

\item{Year}{Numeric or character value. Reference school year for the data (last available is 2022/23).
Available in the formats: \code{2022}, \code{"2021/2022"}, \code{202122}, \code{20212022}. \code{2023} by default}

\item{verbose}{Logical. If \code{TRUE}, the function informs about the time needed. \code{TRUE} by default.}

\item{autoAbort}{Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}
}
\value{
An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. For all subjects and school grades, the variables indicate:

\itemize{
\item \code{M} The mean score, either WLE or percentage of sufficient tests
\item \code{S} The standard deviation of the score
\item \code{C} The students coverage percentage (expressed in the scale 1 - 100)
}
}
\description{
This function filters the database of Invalsi scores (see \code{\link{Get_Invalsi_IS}}) by school year, education grade and subject and returns a dataframe in wide format.
Each row corresponds to one territorial unit (either municipality or province); the numerical variables are three (the mean score, the score's standard deviation and the students coverage percentage) for each selected subject.
}
\examples{



Util_Invalsi_filter(subj = c("Italian", "Mathematics"), grade = 5, level = "NUTS-3", Year = 2023,
                   WLE = FALSE, data = example_Invalsi23_prov)

Util_Invalsi_filter(subj = c("Italian", "Mathematics"), grade = 5, level = "NUTS-3", Year = 2023,
                    WLE = TRUE, data = example_Invalsi23_prov)

Invalsi23_high <- Util_Invalsi_filter(subj = "Italian", grade = c(10,13), level = "NUTS-3",
                                      Year = 2023, data = example_Invalsi23_prov)


 summary(Invalsi23_high)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Get_School2mun.R
\name{Get_School2mun}
\alias{Get_School2mun}
\title{Associate a Municipality (LAU) code to each school}
\source{
\itemize{
   \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0101EDIANAGRAFESTA2021> For Buildings registry (2021 onwards);
   \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0200EDIANAGRAFESTA> For Buindings registry(until 2019);
   \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Scuole> For Schools registry
  }
}
\usage{
Get_School2mun(
  Year = 2023,
  show_col_types = FALSE,
  verbose = TRUE,
  input_AdmUnNames = NULL,
  input_Registry = NULL,
  autoAbort = FALSE
)
}
\arguments{
\item{Year}{Numeric or character value (last available is 2023).
Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023.} \code{2023} by default.}

\item{show_col_types}{Logical. If \code{TRUE}, if the \code{verbose} argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{input_AdmUnNames}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_AdmUnNames}}
The ISTAT file including all the administrative units codes for the year in scope.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.}

\item{input_Registry}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_Registry}}
The school registry corresonding to the year in scope.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default}

\item{autoAbort}{Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}
}
\value{
An object of class \code{list}, including 4 elements:
\itemize{
  \item \code{$Registry_from_buildings}: Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}: the schools listed in the buildings registry
  \item \code{$Registry_from_registry}: Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}: the schools listed in the schools registry
  \item \code{$Any}: Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}: schools listed anywhere
  \item \code{$Both}: Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}: schools listed in both the sections

}
}
\description{
This function associates the relevant municipality codes to all the schools listed in the two main registries provided by the Italian Ministry of Education, University and Research, namely:
 \itemize{
   \item The registry of school buildings, here referred to as \code{Registry_from_buildings} (\code{\link{Get_DB_MIUR}})
   \item The official schools registry, here referred to as \code{Registry_from_registry} (see \code{\link{Get_Registry}})
 }
}
\examples{

\donttest{
Get_School2mun(Year = 2023, autoAbort = TRUE)
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Get_DB_MIUR.R
\name{Get_DB_MIUR}
\alias{Get_DB_MIUR}
\title{Download the database of Italian public schools buildings}
\source{
<https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Edilizia+Scolastica>
}
\usage{
Get_DB_MIUR(
  Year = 2023,
  verbose = TRUE,
  input_Registry = NULL,
  input_AdmUnNames = NULL,
  show_col_types = FALSE,
  certifications = FALSE,
  autoAbort = FALSE,
  t_out = 900
)
}
\arguments{
\item{Year}{Numeric or character value. Reference school year (last available is 2023).
Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023}. \code{2022} by default (other databases are not currently available for 2023).}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{input_Registry}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
The school registry corresponding to the year in scope, obtained as output of the  function \code{\link{Get_Registry}}.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default.}

\item{input_AdmUnNames}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
The ISTAT file including all the codes and all the names of the administrative units for the year in scope, obtained as output of the function \code{\link{Get_AdmUnNames}}.
Only  necessary for school years 2015/16, 2017/18 and 2018/19.
If \code{NULL} and required, it will be downloaded automatically but not saved in the global environment. \code{NULL} by default.}

\item{show_col_types}{Logical. If \code{TRUE}, if the \code{verbose} argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.}

\item{certifications}{Logical. From year 2021/22 onwards, whether to include some safety certifications in the database.
Given the particular level of definition of this file, it requires extra computational time (other than the downloading time). \code{FALSE} by default.}

\item{autoAbort}{Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}

\item{t_out}{Numeric. !! EXPERIMENTAL !! session timeout for scraping and download, in seconds. 900 seconds by default.}
}
\value{
An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
}
\description{
This function downloads the School Buildings Open Database provided by the Italian Ministry of Education, University and Research.


It is one of the main sources of information regarding the infrastructure system of public schools in Italy.
For a given year, all available data are downloaded (except for the structural units section, which has a different level of detail) and gathered into a unique dataframe.
}
\details{
This function downloads the raw data; missing observations are not edited; all variables are characters.
Since certifications are defined at the level of structural units of the single buildings, here
the fields read as the percentage of structural units in a building having a given certificate.
To edit the output of this function and convert the relevant variables to numeric or Boolean, please \code{\link{Util_DB_MIUR_num}}.
Schools different from primary, middle or high schools are classified as \code{"NR"}. In the example, the data for school year 2022/23 are retrieved.
}
\examples{

\donttest{
  input_DB23_MIUR <- Get_DB_MIUR(2023, autoAbort = TRUE, t_out = 20)

  input_DB23_MIUR[-c(1,4,6,9)]

}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_InnerAreas.R
\docType{data}
\name{example_InnerAreas}
\alias{example_InnerAreas}
\title{Subset of the school registry in school year 2022/23}
\format{
## `example_InnerAreas`
A data frame with 1074 rows and 10 columns:
\itemize{
  \item \code{Municipality_code} Character; the ISTAT LAU (municipality) ID.
  \item \code{Municipality_code_numeric} Numeric; the ISTAT LAU (municipality) ID in numeric format.
  \item \code{Cadastral_code} Character; a LAU - level ID code, different from the official ISTAT municipality code.
  \item \code{Region_code} Numeric; the region (NUTS-2 administrative level) ID
  \item \code{Region_description} Character; the region (NUTS-2 administrative level) name.
  \item \code{Province_code} Numeric; the NUTS-3 administrative code.
  \item \code{Province_initials} Character; abbreviated NUTS-3 denomination.
  \item \code{Province_description} Character; the province (NUTS-3 administrative level) denomination.
  \item \code{Municipality_description} Character; the municipality name.
  \item \code{Inner_area_code_2014_2020} Character; the ISTAT inner areas classification between 2014 and 2020.
  \item \code{Inner_area_description_2014_2020} Character; the description of the classes identified in the previous column
  \item \code{Inner_area_code_2021_2027} Character; the ISTAT inner areas classification between 2021 and 2027.
  \item \code{Inner_area_description_2021_2027} Character; the description of the classes identified in the previous column
  \item \code{Destination_municipality_code} Character; For non-central municipalities (classes C, D, E, F), the ID of the closest pole municipality according to the 2021-2027 classification
  \item \code{Destination_municipality_code} Character; The denomination of the municipalities in the previous column
  \item \code{Destination_pole_code} Character; An internal ID convention for the destination poles; it includes a letter
  (the class of the destination pole, either A or B); a number of two digits (the region code of the destination pole) and the progressive number of poles within a region.
}
}
\source{
<https://www.istat.it/it/archivio/273176>
}
\usage{
example_InnerAreas
}
\description{
This dataframe includes the classification of municipalities , from four regions: Molise, Campania, Apulia and Basilicata.
Only the first 10 columns are included;
some strings in field \code{Municipality_description} including accents have been forced to ASCII.
The whole dataset can be retrieved with the command \code{Get_InnerAreas()}.
For the definition of ISTAT inner areas class, see \code{\link{Get_InnerAreas}}
}
\seealso{
\code{\link{Get_InnerAreas}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_input_DB23_MIUR.R
\docType{data}
\name{example_input_DB23_MIUR}
\alias{example_input_DB23_MIUR}
\title{Subset of the school buildings database in school year 2022/23}
\format{
## `example_input_DB23_MIUR`
A data frame with 7479 rows and 35 columns:
\itemize{
  \item \code{Year} Numeric; the school year.
  \item \code{School_code} Character; the school ID.
  \item \code{Order} Character; the school order, either primary, middle or high school.
  \item \code{Reference_institute_code} Character; the ID of the reference institute.
  \item \code{Building_code} Character; the building ID; the first 6 digits usually identify the municipality.
  \item \code{Municipality_code} Character; the ISTAT LAU (municipality) ID.
  \item \code{Municipality_description} Character; the municipality name.
  \item \code{Province_initials} Character; abbreviated NUTS-3 denomination.
  \item \code{Postal_code} Character; the ZIP code; slightly finer than municipality boundaries. for big municipalities.
  \item \code{Context_without_disturbances} Character; whether the school belongs to an environment devoid of disturbances; otherwise, the types of disturbances are listed in columns 11 - 18.
  \item \code{Dumps_proximity} Character; whether the school is close to dumps (disturbance element).
  \item \code{Pollutant_industries_proximity} Character; whether the school is close to pollutant industries (disturbance element).
  \item \code{Pollutant_waters_proximity} Character; whether the school is close to pollutant or stagnant streams or ponds (disturbance element).
  \item \code{Air_pollution_sourcer_proximity} Character; whether the school is close to sources of air pollution (disturbance element).
  \item \code{Acoustic_pollution_sourcer_proximity} Character; whether the school is close to sources of acoustic pollution (disturbance element).
  \item \code{Electromagnetic_radiation_sources_proximity} Character; whether the school is close to sources of electromagnetic radiation (disturbance element).
  \item \code{Graveyards_proximity} Character; whether the school is close to a graveyard (disturbance element).
  \item \code{Other_disturbances_proximity} Character; other disturbance elements to which the school is close, other than those already listed.
  \item \code{School_area_specific_criticalities} Character; whether any specific criticality element occurs inside the school area; specified in columns 20 - 27.
  \item \code{Layby absence} Character; whether the access to the area pertaining to the school building lacks a lay-by or pitch (school area criticality element).
  \item \code{Unfenced area} Character; whether the school building area lacks fences or enclosures (school area criticality element).
  \item \code{Large_traffic} Character; whether the school area is close to large traffic streams (school area criticality element).
  \item \code{Railway_traffic} Character; whether the school area is close to railway traffic streams (school area criticality element).
  \item \code{Abandoned_industries} Character; whether the school area is located in pre-existences of abandoned industries (school area criticality element).
  \item \code{Decayed_urban_area} Character; whether the school belongs or is close to a decayed area (school area criticality element).
  \item \code{Risky_industries_proximity} Character; whether the school is close to perilous industrial areas  (school area criticality element).
  \item \code{Other_specific_criticalities} Character; specific criticality elements regarding the school area, other than those already listed.
  \item \code{School_bus} Character; whether the school is reached by school-bus service.
  \item \code{Urban_public_transport} Character; whether the school is served by a urban public transport station in the range of 250 meters.
  \item \code{Interurban_public_transport} Character; whether the school is served by a inter-urban public transport station in the range of 500 meters.
  \item \code{Railway_transport} Character; whether the school ranges 500 meters or less from a train station.
  \item \code{Private_transport} Character; whether the school can be reached by private transport.
  \item \code{Disabled_people_transport} Character; whether the school is provided with disabled people specific transport.
  \item \code{Bicycle_lane} Character; whether the building is in proximity of a bicycle/bike lane.
  \item \code{Other} Character; whether the building can be reached in any other specific way.
}
}
\source{
<https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Edilizia+Scolastica>. More in detail, the dataset blocks are downloaded respectively from
  \itemize{
  \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0111EDIAMBIENTESTA2021> cols 10-18;
  \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0121EDICOLLEGAMENTISTA2021> cols 20-27;
  \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0121EDICOLLEGAMENTISTA2021> cols 28-35
}


#'
}
\usage{
example_input_DB23_MIUR
}
\description{
This dataframe includes the schools directly identifiable as primary, middle or high school, from four regions: Molise, Campania, Apulia and Basilicata.
Only the first 35 columns are included. Some strings including accents in fields \code{Other_disturbances_proximity},
\code{Other_specific_criticalities} and \code{Other} have been forced to ASCII.
The whole dataset can be retrieved with the command \code{Get_DB_MIUR(2023)}
}
\seealso{
\code{\link{Get_DB_MIUR}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Group_DB_MIUR.R
\name{Group_DB_MIUR}
\alias{Group_DB_MIUR}
\title{Aggregate the database of Italian public schools buildings at the municipality and province level}
\usage{
Group_DB_MIUR(
  data = NULL,
  Year = 2023,
  count_units = TRUE,
  countname = "nbuildings",
  count_missing = TRUE,
  verbose = TRUE,
  track_deleted = TRUE,
  InnerAreas = TRUE,
  ord_InnerAreas = FALSE,
  input_InnerAreas = NULL,
  autoAbort = FALSE,
  ...
)
}
\arguments{
\item{data}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. The database of school buildings, preferably already converted to numeric, obtained via \code{\link{Util_DB_MIUR_num}}}

\item{Year}{Numeric or Character. The reference school year, if either \code{data} or \code{input_InnerAreas} must be retrieved.
Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023}.
Important: use the same \code{Year} argument used to retrieve the input school buildings data if they are provided as input. \code{2023} by default}

\item{count_units}{Logical. Whether the rows to aggregate at each level must be counted or not. True by default.}

\item{countname}{character. The name of the variable indicating the number of schools included in each municipality of province,
if the argument 'count' is \code{TRUE}. \code{"nbuildings"}  by default.}

\item{count_missing}{Logical. Whether the function should return two dataframes including the percentage of NAs in the \code{data} object at the territorial level. \code{TRUE} by default}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{track_deleted}{Logical. If \code{TRUE}, the function returns the IDs of schools not included. \code{TRUE} by default.}

\item{InnerAreas}{Logical. Whether an indicator of the percentage of schools belonging to peripheral (Inner) areas mus be included or not.}

\item{ord_InnerAreas}{Logical. Whether the Inner areas classification should be treated as an ordinal variable rather than as a binary one (see \code{\link{Get_InnerAreas}} for the classification).
Please notice than the function creates a column for each class, and if this database must be used in a statistical model, one of the 6 resulting columns must be dropped.
False by default.}

\item{input_InnerAreas}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
The classification of peripheral municipalities, needed only if \code{InnerAreas == TRUE}, obtained as output of the \code{\link{Get_InnerAreas}} function.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default}

\item{autoAbort}{Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}

\item{...}{Additional arguments to the function \code{\link{Util_DB_MIUR_num}} in case no data are provided or data.}
}
\value{
An object of class \code{list} including:

\itemize{
  \item \code{$Municipality_data}:
   object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, the output dataframe detailed at the municipality level;
   all variables besides the first 5 (which identify the record) are numeric

  \item \code{$Province_data}: object of class 'tbl_df', 'tbl' and 'data.frame', the output dataframe detailad at the province level;
  all variables besides the first 3 (which identify the record) are numeric

  \item \code{$Municipality_missing} (Only if \code{count_missing == TRUE}); object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, the percentage of NAs in each variable at the municipality level.

  \item \code{$Province_missing}: (Only if \code{count_missing == TRUE}); object of class 'tbl_df', 'tbl' and 'data.frame', the percentage of NAs in each variable at the province level.
  \item \code{$deleted}: character vector. The schools removed from the original dataframe for data quality reasons. This object is returned only if \code{track_deleted == TRUE}
 }
}
\description{
This function transforms the output of the \code{\link{Util_DB_MIUR_num}} function (which is detailed at the level of single school buildings) at the municipality/LAU and province/NUTS-3 level.
It also allows the user to classify the grade of centrality of municipalities through the variable  \code{Inner_area}.
}
\details{
Numerical variables are summarised by the mean; Boolean variables are summarised by the mean as well, thus they become frequency indicators.
Qualitative values, if included, are summarised by the mode. Summary measures do not include NAs.
The output dataframes are also detailed at the school order level (i.e. Primary, Midde, High school, or different orders). This means that rows are unique combinations of territorial unities and school order.
}
\examples{


library(magrittr)
DB23_MIUR <- example_input_DB23_MIUR \%>\% Util_DB_MIUR_num(verbose = FALSE) \%>\%
    Group_DB_MIUR(InnerAreas = FALSE)



DB23_MIUR$Municipality_data[, -c(1,2,4)]
summary(DB23_MIUR$Municipality_data)

DB23_MIUR$Province_data[, -c(1,3)]
summary(DB23_MIUR$Province_data)





}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Get_Invalsi_IS.R
\name{Get_Invalsi_IS}
\alias{Get_Invalsi_IS}
\title{Download the Invalsi census survey data}
\source{
<https://serviziostatistico.invalsi.it/en/archivio-dati/?_sft_invalsi_ss_data_collective=open-data>
}
\usage{
Get_Invalsi_IS(
  level = "LAU",
  verbose = TRUE,
  show_col_types = FALSE,
  multiple_out = TRUE,
  autoAbort = FALSE,
  category = FALSE
)
}
\arguments{
\item{level}{Character. The level of aggregation of Invalsi census data, such as \code{"NUTS-3"}, \code{"Province"} for provinces; or \code{"LAU"}, \code{"Municipality"} for muncipalities.
Only useful when \code{multiple_out == FALSE}. \code{"LAU"} by default.}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{show_col_types}{Logical. If \code{TRUE}, if the \code{verbose} argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.}

\item{multiple_out}{Logical. Wheter keeping
multiple dataframes as outputs (thus overriding the \code{level} argument) or not.
We advise to keep this option active in order not to lose information. \code{TRUE} by default.}

\item{autoAbort}{Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.
\code{TRUE} by default.}

\item{category}{Logical. Whether to focus on a specific category of students participating to the census survey. Warning: experimental. \code{FALSE} by default.}
}
\value{
If \code{multiple_out == FALSE}, an object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
Otherwise, a list including objects of the aforementioned classes
}
\description{
Downloads the full database of the Invalsi scores, detailed either at the municipality or province level.
}
\details{
Numeric variables provided are:
\itemize{
  \item \code{Average_percentage_score} Average direct score (percentage of sufficient tests)
  \item \code{Std_dev_percentage_score} Standard deviation of the direct score
  \item \code{WLE_average_score} Average WLE score. The WLE score is calculated through the Rasch's psychometric model and is suitable for middle and high schools in that it is cleaned from the effect of cheating  (which would affect both the average score and the score variability). By construction it has a mean around 200 points.
  \item \code{Std_dev_WLE_score} Standard deviation of the WLE score. By construction it ranges around 40 points at the school level.
  \item \code{Students_coverage} Students coverage percentage
}
Additional numeric variables, not always available for all observational units, are:
\itemize{
  \item Mean and SD of ESCS indicator
  \item \code{First-Fifth_Level}: Distribution of the proficiency level of students
  \item \code{Targets_percentage}: Percentage of students reaching targets
}
Numeric codes \code{888} and \code{999} denote not applicable and not available fields respectively.

If \code{multiple_out == TRUE}, provides the following datasets:
\itemize{
  \item \code{Municipality_data}: LAU-level data
  \item \code{Province_data}: NUTS-3-level data
  \item \code{Region_data}: NUTS-2-level data
  \item \code{LLS_data}: data at the level of local labour systems
  (Sistemi Locali del Lavoro; see \href{https://www.istat.it/statistiche-per-temi/focus/informazioni-territoriali-e-cartografiche/statistiche-sul-territorio/sistemi-locali-del-lavoro-e-distretti-industriali/}{ISTAT webpage} for details)
  \item \code{Inner_Areas_2021_data} aggregated data for inner areas according to the 2020 taxonomy
  \item \code{Inner_Areas_2014_data} aggregated data for inner areas according to the former 2014 taxonomy
  \item \code{Macroarea_data} data aggregated for North-West, North-East, Center, South and Islands
}
}
\examples{
\donttest{
Data_Invalsi <- Get_Invalsi_IS(level = "NUTS-3", autoAbort = TRUE, verbose = FALSE)
Data_Invalsi$Province_data
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Get_Shapefile.R
\name{Get_Shapefile}
\alias{Get_Shapefile}
\title{Download the shapefiles of Italian NUTS-3 and LAU administrative units}
\source{
<https://www.istat.it/it/archivio/222527>
}
\usage{
Get_Shapefile(
  Year,
  level = "LAU",
  lightShp = TRUE,
  autoAbort = FALSE,
  centroids = FALSE
)
}
\arguments{
\item{Year}{Numeric. Reference year for the administrative units.}

\item{level}{Character. Either  \code{"LAU"/"Municipality"}, \code{"NUTS-3"/"Province"},  \code{"NUTS-2"/"Region"}, . \code{"LAU"} by default}

\item{lightShp}{Logical. If \code{TRUE}, the function downloads a generalised, i.e.less detailed, and lighter version of the shapefiles.
\code{TRUE} by default.}

\item{autoAbort}{Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}

\item{centroids}{Logical. Whether to switch from polygon geometry to point geometry. In the latter case, the point is located at the centroid of the relevant area. \code{FALSE} by default.}
}
\value{
A spatial data frame of class \code{data.frame} and \code{sf}.
}
\description{
Downloads either the boundaries or the centroids of the relevant administrative units,
either provinces or municipalities, from the ISTAT website. Geometries are expressed in meters.
}
\examples{

\donttest{
  library(magrittr)


  Prov23_shp <- Get_Shapefile(2023, lightShp = TRUE, level = "NUTS-3", autoAbort = TRUE)
  ggplot2::ggplot() + ggplot2::geom_sf(data = Prov23_shp) +
    ggplot2::ggtitle("Italian provinces in 2023/01/01")
 }


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Get_AdmUnNames.R
\name{Get_AdmUnNames}
\alias{Get_AdmUnNames}
\title{Download the names and codes of Italian LAU and NUTS-3 administrative units}
\source{
<https://situas.istat.it/web/#/territorio>
}
\usage{
Get_AdmUnNames(Date = Sys.Date(), autoAbort = FALSE)
}
\arguments{
\item{Date}{Character. The date at which administrative unit codes are sought for. Important: must be in the format: "yyyy-mm-dd".
Current date by default.}

\item{autoAbort}{Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}
}
\value{
An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, including: NUTS-3 code, NUTS-3 abbreviation,
LAU code, LAU name (description) and cadastral code. All variables are characters except for the NUTS-3 code.
}
\description{
This function downloads a file provided by the Italian National Institute of Statistics including all the codes of administrative units in Italy. As of today, it is the easiest way to map directly cadastral codes to municipality codes.
}
\examples{
 \donttest{
  AdmUnNames25 <- Get_AdmUnNames("2025-01-01", autoAbort = TRUE)
  # Codes for a randomly chosen province, say, Brindisi (Apulia)
  AdmUnNames25[which(AdmUnNames25$Province_code==74),]
 }




}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Group_nstud.R
\name{Group_nstud}
\alias{Group_nstud}
\title{Aggregate the students number data by class at the municipality and province level}
\usage{
Group_nstud(
  data = NULL,
  Year = 2023,
  check = TRUE,
  verbose = TRUE,
  check_registry = "Any",
  InnerAreas = TRUE,
  ord_InnerAreas = FALSE,
  check_ggplot = FALSE,
  missing_to_1 = FALSE,
  input_Registry = NULL,
  input_InnerAreas = NULL,
  input_Prov_shp = NULL,
  input_School2mun = NULL,
  input_AdmUnNames = NULL,
  autoAbort = FALSE,
  ...
)
}
\arguments{
\item{data}{Either an object of class \code{list}, obtained as output of the \code{\link{Get_nstud}} function,
or an object of class class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the \code{\link{Util_nstud_wide}} function,
if \code{NULL}, the function will download it automatically but it will not be saved in the global environment. \code{NULL} by default.}

\item{Year}{Numeric or character value. The reference school year, if either of the \code{input_} arguments must be retrieved.
Available in the formats: \code{2022}, \code{"2022/2023"}, \code{"202223"}, \code{"20222023"}. \code{2023} by default}

\item{check}{Logical. If \code{TRUE}, the function runs the test of the students number availability across all school included in the school registries (see \code{\link{Util_Check_nstud_availability}}). \code{TRUE} by default}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{check_registry}{Character. If \code{check == TRUE}, the school registries included in the \code{input_School2mun} object (see \code{\link{Get_School2mun}}) whose availability has to be checked. Either \code{"Registry_from_buildings"} (buildings section), \code{"Registry_from_registry"} (registry section), \code{"Any"} or \code{"Both"}. \code{"Any"} by default.}

\item{InnerAreas}{Logical. If \code{check == TRUE}, Whether it must be checked if municipalities belong to Inner areas or not. \code{TRUE} by default.}

\item{ord_InnerAreas}{Logical. If \code{check == TRUE} and \code{InnerAreas == TRUE}, whether the Inner areas classification should be treated as an ordinal variable rather than as a categorical one (see \code{\link{Get_InnerAreas}} for the classification).
\code{FALSE} by default.}

\item{check_ggplot}{Logical. If \code{check == TRUE}, whether to display or not a static map of the availability of the students number by province; see also \code{\link{Util_Check_nstud_availability}}. \code{TRUE} by default.}

\item{missing_to_1}{Logical. Only needed if \code{data} is not provided in wide format. Whether the number of classes should be imputed to 1 when it is missing; see \code{\link{Util_nstud_wide}}. \code{FALSE} by default.}

\item{input_Registry}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_Registry}}
If \code{check == TRUE}, the school registry (the properly said one, from the registry section).
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default}

\item{input_InnerAreas}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
The classification of peripheral municipalities, obtained as output of the \code{\link{Get_InnerAreas}} function.
Needed only if \code{check == TRUE} and \code{InnerAreas == TRUE}.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default}

\item{input_Prov_shp}{Object of class \code{sf}, \code{tbl_df}, \code{tbl}, \code{data.frame}. The relevant shapefile of Italian municipalities, if both the \code{check} and \code{check_ggplot} options are chosen.
If \code{NULL} it is downloaded automatically but not saved in the global environment. \code{NULL} by default.}

\item{input_School2mun}{Object of class \code{list} with elements of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_School2mun}}.
The mapping from school codes to municipality (and province) codes. Needed only if `check == TRUE`.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.}

\item{input_AdmUnNames}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_AdmUnNames}}
The ISTAT file including all the codes and the names of the administrative units for the year in scope.
Only needed if \code{check == TRUE} and the argument \code{input_School2mun} is \code{NULL}.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.}

\item{autoAbort}{Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}

\item{...}{Additional arguments to the function \code{\link{Util_nstud_wide}} if \code{data} is not provided.}
}
\value{
An object of class \code{list} including:

\itemize{
  \item \code{$Municipality_data}:
   object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, the output dataframe detailed at the municipality level

  \item \code{$Province_data}: object of class 'tbl_df', 'tbl' and 'data.frame', the output dataframe detailad at the province level
 }
}
\description{
This function creates two dataframes with the number of students, classes and students by class, aggregated at the province and municipality level
}
\details{
Numerical variables are summarised by the mean; Boolean variables are summarised by the mean as well, thus they become frequency indicators.
Qualitative values, if included, are summarised by the mode. Summary measures do not include NAs.
}
\examples{
Year <- 2023

nstud23_aggr <- Group_nstud(data = example_input_nstud23, Year = Year,
                           input_Registry = example_input_Registry23,
                           InnerAreas = FALSE,
                           input_School2mun = example_School2mun23)

summary(nstud23_aggr$Municipality_data[,c(46,47,48)])

summary(nstud23_aggr$Province_data[,c(44,45,46)])


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SchoolDataIT-package.R
\docType{package}
\name{SchoolDataIT-package}
\alias{SchoolDataIT}
\alias{SchoolDataIT-package}
\title{SchoolDataIT: Retrieve, Harmonise and Map Open Data Regarding the Italian School System}
\description{
Compiles and displays the available data sets regarding the Italian school system, with a focus on the infrastructural aspects. Input datasets are downloaded from the web, with the aim of updating everything to real time. The functions are divided in four main modules, namely 'Get', to scrape raw data from the web 'Util', various utilities needed to process raw data 'Group', to aggregate data at the municipality or province level 'Map', to visualize the output datasets.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/lcef97/SchoolDataIT}
}

}
\author{
\strong{Maintainer}: Leonardo Cefalo \email{leonardo.cefalo@uniba.it} (\href{https://orcid.org/0009-0001-7776-723X}{ORCID})

Other contributors:
\itemize{
  \item Alessio Pollice \email{alessio.pollice@uniba.it} (\href{https://orcid.org/0000-0002-2818-9373}{ORCID}) [contributor, thesis advisor]
  \item Paolo Maranzano \email{pmaranzano.ricercastatistica@gmail.com} (\href{https://orcid.org/0000-0002-9228-2759}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_input_nstud23.R
\docType{data}
\name{example_input_nstud23}
\alias{example_input_nstud23}
\title{Subset of the students and classes counts in school year 2022/23}
\format{
## `example_input_nstud23`
A data frame with 21208 rows and 7 columns:
\itemize{
  \item \code{Year} Numeric; the school year.
  \item \code{School_code} Character; the school ID.
  \item \code{Order} Character; the school order, either primary, middle or high school.
  \item \code{Grade} Numeric; the school grade.
  \item \code{Classes} Numeric; the count of classes of a given grade in each school
  \item \code{Male_students} Numeric; the count of male students in all classes of a given educational grade in each school
  \item \code{Female_students} Numeric; the count of female students in all classes of a given educational grade in each school
}
}
\source{
<https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Studenti&datasetId=DS0030ALUCORSOINDCLASTA>
}
\usage{
example_input_nstud23
}
\description{
This dataframe includes students and classes counts for the schools from four regions: Molise, Campania, Apulia and Basilicata.
The whole dataset can be retrieved with the command \code{Get_nstud(2023, filename = "ALUCORSOINDCLASTA")}
}
\seealso{
\code{\link{Get_nstud}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Util_DB_MIUR_bool.R
\name{Util_DB_MIUR_bool}
\alias{Util_DB_MIUR_bool}
\title{Clean and convert the raw school buildings data to Boolean variables}
\usage{
Util_DB_MIUR_bool(
  data = NULL,
  cutout = NULL,
  col_cut_thresh = 10^3,
  verbose = TRUE,
  track_deleted = TRUE,
  autoAbort = autoAbort,
  ...
)
}
\arguments{
\item{data}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. Input data obtaned through the function \code{\link{Get_DB_MIUR}}.
If \code{NULL} it will be downloaded automatically with the appropriate arguments, but not saved in the global environment. \code{NULL} by default.}

\item{cutout}{Character. The columns to cut out. If \code{NULL}, it will be determined automatically.
\code{NULL} by default.}

\item{col_cut_thresh}{Numeric. The threshold of missing values allowed for each variable.
If a variable as a higher number of missing observations, then it is cut out. \code{1000} by default.}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. TRUE by default.}

\item{track_deleted}{Logical. If \code{TRUE}, the function returns the names of the school not included in the output dataframe. \code{TRUE} by default.}

\item{autoAbort}{Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}
}
\value{
If \code{track_deleted == TRUE}, An object of class \code{list} including two objects:
\itemize{
  \item \code{$data}: object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, the output dataframe. All variables besides the first 8 ones (which identify the record) are numeric.
  \item \code{$deleted}: character. The school codes corresponding to deleted rows

}

If \code{track_deleted == FALSE}, the output is only the first element of the list.
}
\description{
This function cleans the output of the \code{\link{Get_DB_MIUR}} function from missing values in two steps:
\itemize{
\item First, it deletes both the columns exceeding a threshold of missing values (1000 by default) and the columns that cannot be converted into Boolean variables
\item Then, it deletes the rows in which missing values remain
}

 Finally, the remaining data are converted into Boolean variables.
 It is possible to keep track of the deleted rows.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Get_Registry.R
\name{Get_Registry}
\alias{Get_Registry}
\title{Download the registry of Italian public schools from the school registry section}
\source{
<https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Scuole>
}
\usage{
Get_Registry(
  Year = 2023,
  filename = c("SCUANAGRAFESTAT", "SCUANAAUTSTAT"),
  show_col_types = FALSE,
  autoAbort = FALSE
)
}
\arguments{
\item{Year}{Numeric or character. Reference school year (last available is 2025).
Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023}. \code{2023} by default.}

\item{filename}{Character. A string included in the name of the file to download, identifying the schools included.
By default it is \code{c("SCUANAGRAFESTAT", "SCUANAAUTSTAT")}, i.e. the file names used for public school registries,
respectively across all the national territory except for the autonomous provinces of Aosta, Trento or Bozen, and only in the three
If instead the registry of the private schools is needed, please insert \code{"SCUANAGRAFEPAR"} and/or \code{"SCUANAAUTPAR"}.

For the registry of private schools, either in all the national territory except for the aforementioned provinces, and for these provinces, please use \code{"SCUANAGRAFEPAR"} and \code{"SCUANAAUTPAR"} respectively. Please notice that data regarding private schools are not available for most functions in this package.}

\item{show_col_types}{Logical. If \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.}

\item{autoAbort}{Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}
}
\value{
An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
}
\description{
This function returns two main pieces of information regarding Italian schools, namely:
\itemize{
  \item The denomination of the region, province and municipality to which the school belongs.
  \item The mechanographical code to the reference institute of each school.
}
It is possible to access schools in all the national territory, including the autonomous provinces of Aosta, Trento and Bozen.
}
\details{
Schools different from primary, middle or high schools are classified as \code{"NR"}.
}
\examples{

\donttest{
  Get_Registry(2024, filename = "SCUANAGRAFESTAT", autoAbort = TRUE)
}




}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Map_School_Buildings.R
\name{Map_School_Buildings}
\alias{Map_School_Buildings}
\title{Display data fom the school buildings database}
\usage{
Map_School_Buildings(
  data = NULL,
  field,
  order = NULL,
  level = "LAU",
  region_code = c(1:20),
  plot = "mapview",
  pal = "viridis",
  col_rev = FALSE,
  popup_height = 200,
  main_pos = "top",
  main = "",
  only_observed = FALSE,
  verbose = TRUE,
  input_shp = NULL,
  autoAbort = FALSE,
  ...
)
}
\arguments{
\item{data}{Object of class \code{list} or \code{tbl_df}, \code{tbl} and \code{data.frame}. Input data obtained as output of the function \code{\link{Group_DB_MIUR}}
If NULL, it will be downloaded automatically but not saved in the global environment. \code{NULL} by default.}

\item{field}{Character. The variable to display in the map.}

\item{order}{Character. The school order. Either \code{"Primary"}, \code{"Middle"}, or \code{"High"} (high school).
If \code{NULL}, an average of the three school orders will be displayed for the target variable. \code{NULL} by default.}

\item{level}{Character. The administrative level of detailed at which the target variable must be displayed.
Either \code{"LAU"}/\code{"Municipality"} or \code{"NUTS-3"}/\code{"Province"}. \code{"LAU"} by default.}

\item{region_code}{Numeric. The NUTS-2 codes of the units that must be displayed.
If the level is set to \code{"LAU"}, choosing a limited number of regions is recommended.
By default, \code{c(1:20)}, i.e. all Italian regions.}

\item{plot}{Character. The type of map to display; either \code{"mapview"} for interactive maps, or \code{"ggplot"} for static maps. \code{"mapview"} by default.
!!! Warning. As \code{mapview} turns out to have been taken away from CRAN,
if it has not been previously installed by the user, \code{plot = "ggplot"} will be forced.}

\item{pal}{Character. The palette to use if the \code{"mapview"} mode is chose. \code{"viridis"} by default.}

\item{col_rev}{Logical. Whether the scale of the colour palette should be reverted or not, if the \code{"mapview"} mode is chosen. \code{FALSE} by default}

\item{popup_height}{Numeric. The height of the popup table in terms of pixels if the \code{"mapview"} mode is chosen. \code{200} by default.}

\item{main_pos}{Character. Where the header should be placed if the \code{ggplot} mode is chosen.
The header is located on the top if \code{"top"} is given as input, and above the legend scale otherwise. \code{"top"} by default.}

\item{main}{Character. The customary title to display in the \code{"ggplot"} rendering options}

\item{only_observed}{Logical. Whether to remove unobserved areas from the plot. \code{FALSE} by default.}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.s}

\item{input_shp}{Object of class \code{sf}, \code{tbl_df}, \code{tbl}, \code{data.frame}. The relevant shapefiles of Italian administrative boudaries,
at the selected level of detail (LAU or NUTS-3). If \code{NULL} it is downloaded automatically but not saved in the global environment. \code{NULL} by default.}

\item{autoAbort}{Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}

\item{...}{If \code{data} is not provided, the arguments to \code{\link{Group_DB_MIUR}}.}
}
\value{
If \code{plot == "mapview"}, an object of class \code{mapview}. Otherwise, if \code{plot == "ggplot"}, an object of class \code{gg} and \code{ggplot}.
}
\description{
This function displays a map of the data downloaded trough the \code{\link{Get_DB_MIUR}} function.
It supports two kinds of map:
\itemize{
  \item Interactive map (default option), which allows the user to visualize all the data in scope through the interactive popup, and
  \item Static map (ggplot), which can be easily exported in \code{.pdf} objects.
}
}
\examples{





  library(magrittr)

  DB23_MIUR <- example_input_DB23_MIUR \%>\%
    Util_DB_MIUR_num(track_deleted = FALSE) \%>\%
    Group_DB_MIUR(InnerAreas = FALSE, count_missing = FALSE)

  DB23_MIUR \%>\% Map_School_Buildings(field = "School_bus",
     order = "Primary",level = "NUTS-3",  plot = "ggplot",
     input_shp = example_Prov22_shp)

  DB23_MIUR \%>\% Map_School_Buildings(field = "Railway_transport",
     order = "High",level = "NUTS-3", plot = "ggplot",
     input_shp = example_Prov22_shp)

  DB23_MIUR \%>\% Map_School_Buildings(field = "Context_without_disturbances",
     order = "Middle",level = "NUTS-3", plot = "ggplot",
     input_shp = example_Prov22_shp, col_rev = TRUE)





}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Util_nstud_wide.R
\name{Util_nstud_wide}
\alias{Util_nstud_wide}
\title{Clean the raw dataframe of the number of students and arrange it in a wide format}
\usage{
Util_nstud_wide(
  data = NULL,
  missing_to_1 = FALSE,
  nstud_imputation_thresh = 19,
  UB_nstud_byclass = 99,
  LB_nstud_byclass = 1,
  filter_by_grade = FALSE,
  UB_nstud_byclass_grade = NULL,
  LB_nstud_byclass_grade = NULL,
  verbose = TRUE,
  autoAbort = FALSE,
  ...
)
}
\arguments{
\item{data}{Object of class \code{list}, including two objects of class \code{tbl_df},  \code{tbl} and \code{data.frame}, obtainded as output of the \code{\link{Get_nstud}} function with the default \code{filename} parameter.
If \code{NULL}, the function will download it automatically but it will not be saved in the global environment. \code{NULL} by default.}

\item{missing_to_1}{Logical. If focus is on class size, whether the number of classes should be imputed to 1 when it is missing and the number of students is below a threshold (argument \code{nstud_imputation_thresh}). \code{TRUE} by default.}

\item{nstud_imputation_thresh}{Numeric. If focus is on class size, the minimum threshold below which the number of classes is imputed to 1 if missing, if \code{missing_to_1 == TRUE}.
E.g. if the threshold is 19, for all the schools in which there are 19 or less students in a given grade but the number of classes for that grade is missing, the number of classes is imputated to 1. \code{19} by default.}

\item{UB_nstud_byclass}{Numeric. Either a unique value for all school orders, or a vector of three order-specific values in the order: primary, middle, high.
If focus is on class size, the upper limit of the acceptable school-level (if \code{filter_by_grade == FALSE}) or grade-level (otherwise) average of the number of students by class.
If a whole school or any grade in a school respectively has a higher number of students by class, the record is considered an outlier and filtered out. \code{99} by default, i.e. no restriction is made.
 Please notice that boundaries are included in the acceptance interval.}

\item{LB_nstud_byclass}{Numeric. Either a unique value for all school orders, or a vector of three order-specific values in the order: primary, middle, wide.
If focus is on class size, the lower limit of the acceptable school-level (if \code{filter_by_grade == FALSE}) or grade_level (otherwise) average of the number of students by class.
 If a whole school or any grade in a school respectively has a smaller number of students by class, the record is considered an outlier and filtered out. \code{1} by default.
 Please notice that boundaries are included in the acceptance interval.}

\item{filter_by_grade}{Logical. If focus is on class size, whether to remove all school grades with average class size outside of the acceptance boundaries. \code{FALSE} by default.}

\item{UB_nstud_byclass_grade}{Numeric. IF \code{filter_by_grade == TRUE}, the upper limit of the acceptable grade-level average class size.
If \code{NULL} it is set equal to \code{UB_nstud_byclass}. \code{NULL} by default.}

\item{LB_nstud_byclass_grade}{Numeric. IF \code{filter_by_grade == TRUE}, the lowrer limit of the acceptable grade-level average class size.
If \code{NULL} it is set equal to \code{LB_nstud_byclass}. \code{NULL} by default.}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{autoAbort}{Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}

\item{...}{Arguments to \code{\link{Get_nstud}}, needed if \code{data} is not provided.}
}
\value{
An object of class  \code{tbl_df}, \code{tbl} and \code{data.frame}
}
\description{
This function rearranges the output of the \code{\link{Get_nstud}} function in such a way to represent the
counts of students and, if required, either the number of students by class and number of classes, or
the counts of students per school timetable (running time) in a unique observation per school.
If the focus is on class size, this function  firstly cleans the data from the outliers in terms of
average number of students by class at the school level and imputates the number of classes to 1 when missing.
}
\details{
In the example, we compare the dataframe obtained with the default settings
 and the one imposed setting narrow inclusion criteria
}
\examples{


nstud.default <- Util_nstud_wide(example_input_nstud23)


nstud.narrow <- Util_nstud_wide(example_input_nstud23,
  UB_nstud_byclass = 35, LB_nstud_byclass = 5 )

nrow(nstud.default)
nrow(nstud.narrow)

nstud.default

summary(nstud.default)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_School2mun23.R
\docType{data}
\name{example_School2mun23}
\alias{example_School2mun23}
\title{Association of the municipality code to a subset of public schools 2022/23}
\format{
## `example_School2mun23`
A list of four elements
\itemize{
  \item \code{Registry_from_buildings} A data frame of 5527 rows and 5 columns, including the schools listed in the buildings registry.
  \item \code{Registry_from_registry} A data frame of 5929 rows and 5 columns, including the schools listed in the schools registry.
  \item \code{Any} A data frame of 5954 rows and 5 columns, including schools listed in any of the registryes
  \item \code{Both} A data frame of 5510 rows and 5 columns, including schools listed in both registries
}
For each element, rows correspond to school IDs; the columns are:
\itemize{
  \item \code{School_code} Character; the school ID.
  \item \code{Province_code} Numeric; the NUTS-3 administrative code.
  \item \code{Province_initials} Character; abbreviated NUTS-3 denomination.
  \item \code{Municipality_code} Character; the ISTAT LAU (municipality) ID.
  \item \code{Municipality_description} Character; the municipality name.
}
}
\source{
See:
\itemize{
  \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0101EDIANAGRAFESTA2021> For Buildings registry (2021 onwards);
  \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0200EDIANAGRAFESTA> For Buindings registry(until 2019);
  \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Scuole> For Schools registry
 }
}
\usage{
example_School2mun23
}
\description{
This list maps the IDs of the schools from four regions (Molise, Campania, Apulia and Basilicata) to the corresponding LAU codes.
The whole dataset can be retrieved with the command \code{Get_School2mun(2023)}
}
\seealso{
\code{\link{Get_School2mun}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Get_nstud.R
\name{Get_nstud}
\alias{Get_nstud}
\title{Download students' number data}
\source{
<https://dati.istruzione.it/opendata/opendata/catalogo/elements1/?area=Studenti>
}
\usage{
Get_nstud(
  Year = 2023,
  filename = c("ALUCORSOETASTA", "ALUCORSOINDCLASTA"),
  verbose = TRUE,
  show_col_types = FALSE,
  autoAbort = FALSE
)
}
\arguments{
\item{Year}{Numeric or character. Reference school year  (last available is 2023).
Available in the formats: \code{2022}, \code{"2021/2022"}, \code{202122}, \code{20212022}. \code{2023} by default}

\item{filename}{Character. A string included in the name of the file to download.
By default it is \code{c("ALUCORSOETASTA", "ALUCORSOINDCLASTA")}, which are the file names used so far for the number of students by age and the number of studentsin public schools  by age and class.




Other file names are the following. The output is not currently supported by the remainder of the functions involving the number of students.


\code{"ALUITASTRACITSTA"} for the number of Italian and foreign students in public schools


\code{"ALUSECGRADOINDSTA"} for the number of students of public schools by high school address


\code{"ALUTEMPOSCUOLASTA"} for the number of students of public schools by school running time

\code{"ALUCORSOETAPAR"}, \code{"ALUCORSOINDCLAPAR"}, \code{"ALUITASTRACITPAR"}, \code{"ALUSECGRADOINDPAR"}, \code{"ALUTEMPOSCUOLAPAR"} for the data of the previous file but referring to private schools.}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{show_col_types}{Logical. If \code{TRUE}, if the \code{verbose} argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.}

\item{autoAbort}{Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}
}
\value{
By default, a list of two \code{tbl_df}, \code{tbl} and \code{data.frame} objects:

\itemize{
  \item \code{$ALUCORSOETASTA}: The number of students by school, school grade and age. It provides a higher number of school than the other element
  \item \code{$ALUCORSOINDCLASTA}: The number of students and classes by school and school grade. This is a long-format dataframe.

}
}
\description{
This functions downloads the data regarding the number of students, from the open website of the Italian Ministry of Education, University and Research
}
\examples{

\donttest{
  Get_nstud(2023, filename = "ALUCORSOINDCLASTA", autoAbort = TRUE)
}



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Get_BroadBand.R
\name{Get_BroadBand}
\alias{Get_BroadBand}
\title{Download the data regarding the broad band connection activation in Italian schools}
\source{
Broadband dashboard: <https://bandaultralarga.italia.it/scuole-voucher/dashboard-scuole/>
}
\usage{
Get_BroadBand(
  Date = as.Date(format(as.Date(format(Sys.Date(), "\%Y-\%m-01")) - 1, "\%Y-\%m-01")),
  include_municipality_code = TRUE,
  input_School2mun = NULL,
  input_Registry = NULL,
  input_AdmUnNames = NULL,
  verbose = TRUE,
  autoAbort = FALSE
)
}
\arguments{
\item{Date}{Object of class \code{Date}. The date at which it is required
to determine if the broad band connection has been activated or not.
By default, the first day of last month, which we assume to be the last update of the dataset.}

\item{include_municipality_code}{Logical. Whether to include municipality codes.
\code{TRUE} by default.}

\item{input_School2mun}{Object of class \code{list} obtained with \code{\link{Get_School2mun}}.
If \code{include_municipality_code == TRUE}, the mapping from school codes to municipality (and province) codes.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.}

\item{input_Registry}{If \code{input_School2mun} is required, an object of class \code{tbl_df}, \code{tbl} and \code{data.frame}
corresponding to the national school registry (preferably of the last year, i.e. 2024/2025) obtained with \code{\link{Get_Registry}}. NULL by default.}

\item{input_AdmUnNames}{If \code{input_School2mun} is required, an object of class \code{tbl_df}, \code{tbl} and \code{data.frame}
corresponding to the statistical codes of administrative units (preferably referring to the period corresponding to school year 2024/2025)
 obtained with \code{\link{Get_AdmUnNames}}. NULL by default.}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{autoAbort}{Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}
}
\value{
An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
 The variables \code{BB_Activation_date} and \code{BB_Activation_staus} indicate
 the activation date and activation status of the broadband connection at the selected date.
}
\description{
Retrieves the data regarding the activation date of the ultra-broadband connection in schools
and indicates whether the connection was activated or not at a certain date.
}
\details{
Ultra - Broadband is defined as everlasting internet connection with a
maximum speed of 1 gigabit per second, with a minimum guaranteed speed of
100 megabits/second both on the uploading and downloading operations, until
the peering point is reached, as declared on the data provider's website:
<https://bandaultralarga.italia.it/scuole-voucher/progetto-scuole/>.
In the example the broadband availability at the beginning of school  year 2022/23 (1st september 2022) is shown.
}
\examples{

\donttest{

  Broadband_220901 <- Get_BroadBand(Date = as.Date("2022-09-01"), autoAbort = TRUE)

  Broadband_220901

  Broadband_220901[, c(9,6,13,14)]

}




}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_input_Registry23.R
\docType{data}
\name{example_input_Registry23}
\alias{example_input_Registry23}
\title{Subset of the school registry in school year 2022/23}
\format{
## `example_input_Registry23`
A data frame with 5929 rows and 10 columns:
\itemize{
  \item \code{Year} Numeric; the school year.
  \item \code{Area} Character; the macro-area of the municipality, i.e. North, Center or South.
  \item \code{Region_description} Character; the region (NUTS-2 administrative level) name.
  \item \code{Province_description} Character; the province (NUTS-3 administrative level) name.
  \item \code{Reference_institute_code} Character; the ID of the reference institute.
  \item \code{School_code} Character; the school ID.
  \item \code{Cadastral_code} Character; a LAU - level ID code, different from the official LAU municipality code.
                              The Italian Ministry of Education does provide this code in the place of the LAU code for both the Schools registry and the early school buildings DBs.
  \item \code{Municipality_description} Character; the municipality name.
  \item \code{School_address} Character; the school physical address.
  \item \code{Postal_code} Character; the ZIP code, slightly finer than municipality boundaries for big municipalities.
}
}
\source{
<https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Scuole&datasetId=DS0400SCUANAGRAFESTAT>
}
\usage{
example_input_Registry23
}
\description{
This dataframe includes the schools directly identifiable as primary, middle or high school, from four regions: Molise, Campania, Apulia and Basilicata.
Only the first 10 columns are included.
The whole dataset can be retrieved with the command \code{Get_Registry(2023)}
}
\seealso{
\code{\link{Get_Registry}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Set_DB.R
\name{Set_DB}
\alias{Set_DB}
\title{Build up a comprehensive database regarding the school system}
\usage{
Set_DB(
  Year = 2023,
  level = "LAU",
  conservative = TRUE,
  Invalsi = TRUE,
  SchoolBuildings = TRUE,
  nstud = TRUE,
  nteachers = TRUE,
  BroadBand = TRUE,
  verbose = TRUE,
  show_col_types = FALSE,
  Invalsi_subj = c("ELI", "ERE", "ITA", "MAT"),
  Invalsi_grade = c(2, 5, 8, 10, 13),
  Invalsi_WLE = FALSE,
  SchoolBuildings_certifications = FALSE,
  SchoolBuildings_include_numerics = TRUE,
  SchoolBuildings_include_qualitatives = FALSE,
  SchoolBuildings_row_cutout = FALSE,
  SchoolBuildings_unique_buildings = TRUE,
  SchoolBuildings_col_cut_thresh = 20000,
  SchoolBuildings_flag_outliers = TRUE,
  SchoolBuildings_count_missing = FALSE,
  nstud_imputation_thresh = 19,
  nstud_missing_to_1 = FALSE,
  UB_nstud_byclass = 99,
  LB_nstud_byclass = 1,
  UB_nstud_byclass_grade = NULL,
  LB_nstud_byclass_grade = NULL,
  nstud_filter_by_grade = FALSE,
  InnerAreas = TRUE,
  ord_InnerAreas = FALSE,
  nstud_check = TRUE,
  nstud_check_registry = "Any",
  BroadBand_impute_missing = TRUE,
  Date = as.Date(paste0(substr(year.patternA(Year), 1, 4), "-09-01")),
  NA_autoRM = NULL,
  input_Invalsi_IS = NULL,
  input_Registry = NULL,
  input_SchoolBuildings = NULL,
  input_nstud = NULL,
  input_School2mun = NULL,
  input_AdmUnNames = NULL,
  input_InnerAreas = NULL,
  input_teachers4student = NULL,
  input_nteachers = NULL,
  input_BroadBand = NULL,
  autoAbort = FALSE
)
}
\arguments{
\item{Year}{Numeric or Character. The relevant school year. Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023}.
Important: if input datasets are plugged in, please select the same \code{Year} argument used to download the input data. \code{2023} by default.}

\item{level}{Character. The administrative level of detail at which data must be aggregated.
Either \code{"LAU"}/\code{"Municipality"} or \code{"NUTS-3"}/\code{"Province"}. \code{"LAU"} by default.}

\item{conservative}{Logical. If \code{FALSE}, only the schools included in all the datasets are taken as input. \code{TRUE} by default.}

\item{Invalsi}{Logical. Whether the Invalsi census data must be included (see \code{\link{Get_Invalsi_IS}}. \code{TRUE} by default.}

\item{SchoolBuildings}{Logical. Whether the school buildings dataset must be included (see \code{link{Get_DB_MIUR}}, \code{\link{Util_DB_MIUR_num}}. \code{TRUE} by default.}

\item{nstud}{Logical. Whether the students number per class must be included (see \code{\link{Get_nstud}}. \code{TRUE} by default.}

\item{nteachers}{Logical. Whether the number of teachers by province must be included (see \code{link{Get_nteachers_prov}}). \code{TRUE} by default.}

\item{BroadBand}{Logical. Whether the broadband availability in schools must be included (see \code{\link{Get_BroadBand}}). \code{TRUE} by default}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{show_col_types}{Logical. If \code{TRUE}, if the \code{verbose} argument is also \code{TRUE}, the columns of the raw dataset are shown during the download. \code{FALSE} by default.}

\item{Invalsi_subj}{Character. If \code{Invalsi == TRUE}, the school subject(s) to include, among \code{"Englis_listening"}/\code{"ELI"}, \code{"English_reading"}/\code{"ERE"}, \code{"Italian"}/\code{"Ita"} and \code{"Mathematics"}/\code{"MAT"}. All four by default.}

\item{Invalsi_grade}{Numeric. If \code{Invalsi == TRUE}, the educational grade to choose. Either \code{2} (2nd year of primary school), \code{5} (last year of primary school), \code{8} (last year of middle shcool), \code{10} (2nd year of high school) or \code{13} (last year of school). All by default.}

\item{Invalsi_WLE}{Logical. Whether to express Invalsi scores as averagev WLE score rather that the percentage of sufficient tests, if both are Invalsi_grade is either or \code{2} \code{5}. \code{FALSE} by default}

\item{SchoolBuildings_certifications}{Logical. If the school buldings database has to be downloaded, whether to include safety certifications.  Only relevant from schol year 2020/21 onwards (see \code{\link{Get_DB_MIUR}}). \code{FALSE} by default}

\item{SchoolBuildings_include_numerics}{Logical. Whether to include strictly numeric variables alongside with Boolean ones in the school buildings database (see \code{\link{Util_DB_MIUR_num}}). \code{TRUE} by default.}

\item{SchoolBuildings_include_qualitatives}{Logical. Whether to include qualitative variables alongside with Boolean ones in the school buildings database (see \code{\link{Util_DB_MIUR_num}}). \code{FALSE} by default.}

\item{SchoolBuildings_row_cutout}{Logical. Whether to filter out rows including missing fields in the school buildings database (see \code{\link{Util_DB_MIUR_num}}). \code{FALSE} by default.}

\item{SchoolBuildings_unique_buildings}{Logical. If school buildings DB is included at the building level,
whether to remove records in which the building code is duplicated and all other fields are as well.
As rows are combinations of building ID and school ID, if a school is hosted by several buildings, and each field other than
\code{School_code} are duplicated, then only one row is retained. \code{TRUE} by default. See \code{\link{Util_DB_MIUR_num}}.}

\item{SchoolBuildings_col_cut_thresh}{Numeric. The threshold of missing values allowed for each variable in the school buildings database (see \code{\link{Util_DB_MIUR_num}}).
If a variable as a higher number of missing observations, then it is cut out. \code{20.000} by default.
Warning: if the option \code{SchoolBuildings_row_cutout} is active, please select a lower threshold (e.g. \code{1000})}

\item{SchoolBuildings_flag_outliers}{Logical. Whether to assign NA to outliers in numeric variables; see \code{\link{Util_DB_MIUR_num}} for more details. \code{TRUE} by default.}

\item{SchoolBuildings_count_missing}{Logical. Whether the function should return the percentage of NAs in the input school buildings database (see also \code{\link{Group_DB_MIUR}}). \code{FALSE} by default.}

\item{nstud_imputation_thresh}{Numeric. If \code{nstud_missing_to_1 == TRUE}, the minimum threshold below which the number of classes is imputed to 1 if missing;
see also \code{\link{Util_nstud_wide}}. \code{19} by default.}

\item{nstud_missing_to_1}{Numeric. If \code{nstud == TRUE}, whether the number of classes should be imputed to 1 when it is missing and the number of students is below a threshold (argument \code{nstud_imputation_thresh}, see \code{\link{Util_nstud_wide}}). \code{FALSE} by default.}

\item{UB_nstud_byclass}{Numeric. Either a unique value for all school orders, or a vector of three order-specific values in the order: primary, middle, high.
If focus is on class size, the upper limit of the acceptable school-level (if \code{filter_by_grade == FALSE}) or grade-level (otherwise) average of the number of students by class.
If a whole school or any grade in a school respectively has a higher number of students by class, the record is considered an outlier and filtered out. \code{99} by default, i.e. no restriction is made.
 Please notice that boundaries are included in the acceptance interval.}

\item{LB_nstud_byclass}{Numeric. Either a unique value for all school orders, or a vector of three order-specific values in the order: primary, middle, wide.
If focus is on class size, the lower limit of the acceptable school-level (if \code{filter_by_grade == FALSE}) or grade_level (otherwise) average of the number of students by class.
 If a whole school or any grade in a school respectively has a smaller number of students by class, the record is considered an outlier and filtered out. \code{1} by default.
 Please notice that boundaries are included in the acceptance interval.}

\item{UB_nstud_byclass_grade}{Numeric. IF \code{filter_by_grade == TRUE}, the upper limit of the acceptable grade-level average class size.
If \code{NULL} it is set equal to \code{UB_nstud_byclass}. \code{NULL} by default.}

\item{LB_nstud_byclass_grade}{Numeric. IF \code{filter_by_grade == TRUE}, the lowrer limit of the acceptable grade-level average class size.
If \code{NULL} it is set equal to \code{LB_nstud_byclass}. \code{NULL} by default.}

\item{nstud_filter_by_grade}{Logical. If focus is on class size, whether to remove all school grades with average class size outside of the acceptance boundaries. \code{FALSE} by default.}

\item{InnerAreas}{Logical. Whether the percentage of schools belonging to inner/internal areas must be included (see \code{\link{Get_InnerAreas}}). TRUE by default.}

\item{ord_InnerAreas}{Logical. If \code{check == TRUE} and \code{InnerAreas == TRUE}, whether the Inner areas classification should be treated as an ordinal variable rather than as a categorical one (see \code{\link{Get_InnerAreas}} for the classification). \code{FALSE} by default.}

\item{nstud_check}{Logical. If \code{nstud == TRUE}, whether to check the students number availability across all school included in the school registries (see \code{\link{Util_Check_nstud_availability}}). \code{TRUE} by default.}

\item{nstud_check_registry}{Character. If \code{nstud == TRUE} and \code{nstud_check == TRUE}, the school registries whose availability has to be checked. Either \code{"Registry_from_buildings"} (buildings registry), \code{"Registry_from_registry"} (proper registry), \code{"Any"} or \code{"Both"}. \code{"Any"} by default.}

\item{BroadBand_impute_missing}{Whether the schools not included in the Broadband dataset must be considered in the total of schools (i.e. the denominator to the Broadband availability indicator). \code{TRUE} by default.}

\item{Date}{Character or Date. The threshold date to broadband activation to consider it activated for a school, i.e. the date before which the works of broadband activation must be finished in order to consider a school as provided with the broadband. By default, September 1st at the beginning of the school year.}

\item{NA_autoRM}{Logical. Either \code{TRUE}, \code{FALSE} or \code{NULL}. If \code{TRUE}, the values missing in a single dataset are automatically deleted from the final DB. If \code{FALSE}, the missing observations are kept automatically. If \code{NULL}, the choice is left to the user by an interactive menu. \code{NULL} by default.}

\item{input_Invalsi_IS}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
If \code{INVALSI == TRUE}, the raw Invalsi survey data, obtained as output of the \code{\link{Get_Invalsi_IS}} function.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default}

\item{input_Registry}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
The school registry corresponding to the year in scope, obtained as output of the function \code{\link{Get_Registry}}.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default}

\item{input_SchoolBuildings}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. If \code{SchoolBuildings == TRUE}, the raw school buildings dataset obtained as output of the function \code{\link{Get_DB_MIUR}}.
If \code{NULL}, it will be downloaded automatically but not saved in the global environment. \code{NULL} by default.}

\item{input_nstud}{Object of class \code{list}, including two objects of class\code{tbl_df}, \code{tbl} and \code{data.frame}.
If \code{nstud == TRUE}, the students and classes counts, obtained as output of the function \code{\link{Get_nstud}} with default \code{filename} parameter.
If \code{NULL}, the function will download it automatically but it will not be saved in the global environment. \code{NULL} by default.}

\item{input_School2mun}{Object of class \code{list} with elements of class \code{tbl_df}, \code{tbl} and \code{data.frame}
If \code{nstud == TRUE}, the mapping from school codes to municipality (and province) codes. Needed only if \code{check == TRUE}, obtained as output of the function \code{\link{Get_School2mun}}.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.}

\item{input_AdmUnNames}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_AdmUnNames}}
If necessary,the ISTAT file including all the codes and the names of the administrative units for the year in scope. Required either if \code{nstud == TRUE & nstud_check == TRUE}, or if \code{SchoolBuildings == TRUE}, \code{input_DB_MIUR} is not provided, and the school year is one of 2015/16, 2017/18 or 1018/19
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.}

\item{input_InnerAreas}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
If \code{InnerAreas == TRUE}, the classification of peripheral municipalities, obtained as output of the function \code{\link{Get_InnerAreas}}
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default}

\item{input_teachers4student}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. If \code{nteachers == TRUE} and \code{nstud = TRUE}, the number of teachers for studets by province. Please notice that
this object cannot be considered a substitute for the number of students by class since it provides no information on the number of schools in single educational grades but only at the school order level.
Obtained as output of the function \code{\link{Group_teachers4stud}}.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default.}

\item{input_nteachers}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. If \code{nteachers == TRUE}, the number of teachers by province, obtained as output of the function \code{\link{Get_nteachers_prov}}. If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default}

\item{input_BroadBand}{Object of classs \code{tbl_df}, \code{tbl} and \code{data.frame}. If BroadBand == TRUE, the raw Broadband connection dataset obtaned as output of the function \code{\link{Get_BroadBand}}
If \code{NULL}, it will be downloaded automatically but not saved in the global environment. \code{NULL} by default.}

\item{autoAbort}{Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}
}
\value{
An object of  class \code{tbl_df}, \code{tbl} and \code{data.frame}
}
\description{
This function generates a unique dataframe of the school system data including a customary choice of available datasets. This function allows the user to aggregate the desired datasets, when available, among these:
\itemize{
  \item Invalsi census survey
  \item School buildings
  \item Number of students and school classes
  \item Number of teachers
  \item Broadband connection availability
}


To save as much time as possible it is possible to plug in ready-made input data; otherwise they will be downloaded automatically but not saved in the global environment
When a new dataset is joined to the existing ones, it is possible that some observations in this datasets are missing. In this case, by default, the choice of keeping as much observational units as possible, or to remove units with missing variables is left to the user.
}
\examples{



DB23_prov <- Set_DB(Year = 2023, level = "NUTS-3",Invalsi_grade = c(5, 8, 13),
      Invalsi_subj = "Italian",nteachers = FALSE, BroadBand = FALSE,
      SchoolBuildings_count_missing = FALSE,NA_autoRM= TRUE,
      input_SchoolBuildings = example_input_DB23_MIUR[, -c(11:18, 10:27)],
      input_Invalsi_IS = example_Invalsi23_prov,
      input_nstud = example_input_nstud23,
      input_InnerAreas = example_InnerAreas,
      input_School2mun = example_School2mun23,
      input_AdmUnNames = example_AdmUnNames20220630)


DB23_prov

summary(DB23_prov[, -c(22:62)])





}
\seealso{
\code{\link{Util_DB_MIUR_num}}, \code{\link{Group_DB_MIUR}}, \code{\link{Group_nstud}}, \code{\link{Util_Check_nstud_availability}}, \code{\link{Get_School2mun}}
for similar arguments.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_Invalsi23_prov.R
\docType{data}
\name{example_Invalsi23_prov}
\alias{example_Invalsi23_prov}
\title{Subset of the Invalsi scores in school year 2022/23}
\format{
## `example_Invalsi23_prov`
A data frame with 240 rows and 11 columns:
\itemize{
  \item \code{Year} Character; the school year.
  \item \code{Grade} Numeric; the school grade; only includes the school grades subjected to the Invalsi survey. Either 2, 5, 8, 10 or 13.
  \item \code{Subject} Character; the school subject in which the test is taken; either Italian, Mathematics, English reading or English listening.
  \item \code{Province_code} Numeric; the NUTS-3 administrative code.
  \item \code{Province_initials} Character; abbreviated NUTS-3 denomination.
  \item \code{Province_description} Character; the province (NUTS-3 administrative level) denomination.
  \item \code{Average_percentage_score} Numeric; the province-level percentage of sufficient tests, only for primary schools; ranges 0-100.
  \item \code{Std_dev_percentage_score} Numeric; the standard deviation of the percentage of sufficient tests, only for primary schools.
  \item \code{WLE_average_score} Numeric; the province-level average WLE (Weighted Likelihood Estimator) score.
  \item \code{Std_dev_WLE_score} Numeric; the standard deviation of WLE scores.
  \item \code{Students_coverage} Numeric; the percentage of students for which the Invalsi tests are reported.
}
}
\source{
<https://serviziostatistico.invalsi.it/en/archivio-dati/?_sft_invalsi_ss_data_collective=open-data>
}
\usage{
example_Invalsi23_prov
}
\description{
This dataframe includes the Invalsi scores of the schools from four regions: Molise, Campania, Apulia and Basilicata, for the school year 2022/23.
The whole dataset can be retrieved with the command \code{Get_Invalsi_IS(level = "NUTS-3")}
}
\seealso{
\code{\link{Get_Invalsi_IS}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Util_Check_nstud_availability.R
\name{Util_Check_nstud_availability}
\alias{Util_Check_nstud_availability}
\title{Check how many schools in the school registries are included in the students count dataframe}
\source{
See \itemize{
  \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Edilizia+Scolastica&datasetId=DS0101EDIANAGRAFESTA2021> For Buildings Registry;
  \item <https://dati.istruzione.it/opendata/opendata/catalogo/elements1/leaf/?area=Scuole&datasetId=DS0400SCUANAGRAFESTAT> For Schools Registry.
}
}
\usage{
Util_Check_nstud_availability(
  data,
  Year,
  cutout = c("IC", "IS", "NR"),
  verbose = TRUE,
  ggplot = TRUE,
  toplot_registry = "Any",
  InnerAreas = TRUE,
  ord_InnerAreas = FALSE,
  input_Registry = NULL,
  input_InnerAreas = NULL,
  input_Prov_shp = NULL,
  input_AdmUnNames = NULL,
  input_School2mun = NULL,
  autoAbort = FALSE
)
}
\arguments{
\item{data}{Object of class  \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the \code{\link{Util_nstud_wide}} function}

\item{Year}{Numeric or character value. Reference school year.
Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023}.}

\item{cutout}{Character. The types of schools not to be taken into account (because not relevant or because they are out of scope in the students number section). By default \code{c("IC", "IS", "NR")} , i.e. the check does not regard comprehensive institutes, superior institutes, and all the schools that cannot be classified either as primary, middle or high schools.}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{ggplot}{Logical. If \code{TRUE}, the function displays a static map of the availability of the students number by province (but it does not save the ggplot object into the global environment). \code{TRUE} by default.}

\item{toplot_registry}{Character. If the \code{ggplot} option is chosen, the students number availability of which registry must be plotted; either \code{"Registry_from_buildings"}, \code{"Registry_from_registry"}, \code{"Any"} or \code{"Both"}. \code{"Any"} by default.}

\item{InnerAreas}{Logical. Whether it must be checked if municipalities belong to inner areas or not. \code{TRUE} by default.}

\item{ord_InnerAreas}{Logical. Whether the inner areas classification should be treated as an ordinal variable rather than as a categorical one (see \code{\link{Get_InnerAreas}} for the classification).
\code{FALSE} by default.}

\item{input_Registry}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_Registry}}
The school registry from the registry section.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default}

\item{input_InnerAreas}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
The classification of peripheral municipalities, obtained as output of the \code{\link{Get_InnerAreas}} function.
Needed only if the \code{InnerAreas} option is chosen.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default}

\item{input_Prov_shp}{Object of class \code{sf}, \code{tbl_df}, \code{tbl}, \code{data.frame}. The relevant shapefile of Italian municipalities, if the \code{ggplot} option is chosen.
If \code{NULL} it is downloaded automatically but not saved in the global environment. \code{NULL} by default.}

\item{input_AdmUnNames}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_AdmUnNames}}
The ISTAT file including all the codes and the names of the administrative units for the year in scope.
Only needed if the argument input_School2mun is \code{NULL} and has to be computed.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.}

\item{input_School2mun}{Object of class \code{list} with elements of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Get_School2mun}}.
The mapping from school codes to municipality (and province) codes. If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.}

\item{autoAbort}{Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}
}
\value{
An object of class \code{list} including two elements:
\itemize{
  \item \code{$Municipality_data}
  \item \code{$Province_data}
}
Both the elements are objects of class \code{list} including four elements:

\itemize{
  \item \code{$Registry_from_buildings}: object of class of class \code{tbl_df}, \code{tbl} and \code{data.frame}: the availability of the number of students in the schools listed in the buildings section.
  \item \code{$Registry_from_registry}: object of class of class \code{tbl_df}, \code{tbl} and \code{data.frame}: the availability of the number of students in the schools listed in the registry section.
  \item \code{$Any}: object of class of class \code{tbl_df}, \code{tbl} and \code{data.frame}: the availability of the number of students in the schools listed anywhere.
  \item \code{$Both}: object of class of class \code{tbl_df}, \code{tbl} and \code{data.frame}: the availability of the number of students in the schools listed in both sections.
}
}
\description{
This function checks for which schools listed in the two registries (the buildings registry and the properly said schools registry)
the count of students is available. The first registry is referred to as as \code{Registry_from_buildings} and the second one as \code{Registry_from_registry}.
}
\examples{

nstud23 <- Util_nstud_wide(example_input_nstud23, verbose = FALSE)

Util_Check_nstud_availability(nstud23, Year = 2023,
  input_Registry = example_input_Registry23, InnerAreas = FALSE,
  input_School2mun = example_School2mun23, input_Prov_shp = example_Prov22_shp)





}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_AdmUnNames20220630.R
\docType{data}
\name{example_AdmUnNames20220630}
\alias{example_AdmUnNames20220630}
\title{Subset of the administrative codes of municipalities}
\format{
## `example_AdmUnNames20220630`
A data frame with 1,074 rows and 5 columns:
\itemize{
  \item \code{Province_code} Numeric; the NUTS-3 administrative code
  \item \code{Province_initials} Character;abbreviated NUTS-3 denomination.
  \item \code{Municipality_code} Character; the ISTAT LAU (municipality) ID.
  \item \code{Municipality_description} Character; the municipality name.
  \item \code{Cadastral_code} Character; a LAU - level ID code, different from the official ISTAT municipality code.
  It is used in the school registry (see \code{\link{example_input_Registry23}})
}
}
\source{
<https://www.istat.it/it/archivio/6789>
}
\usage{
example_AdmUnNames20220630
}
\description{
This table includes the administrative codes of the municipalities from four regions: Molise, Campania, Apulia and Basilicata,
as of June 30th 2022; some strings in field \code{Municipality_description} including accents have been forced to ASCII.
The whole dataset can be retrieved with the command \code{Get_AdmUnNames(Date = "2022-06-30")}
}
\seealso{
\code{\link{Get_AdmUnNames}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Group_teachers4stud.R
\name{Group_teachers4stud}
\alias{Group_teachers4stud}
\title{Arrange the number of teachers per students in public Italian schools at the province level}
\usage{
Group_teachers4stud(
  Year = 2023,
  input_nteachers = NULL,
  nteachers_filename = c("DOCTIT", "DOCSUP"),
  verbose = TRUE,
  input_nstud_raw = NULL,
  input_nstud_aggr = NULL,
  autoAbort = FALSE,
  ...
)
}
\arguments{
\item{Year}{Numeric or character value. Reference school year for the school registry data (last available is 2022).
Available in the formats: \code{2022}, \code{"2021/2022"}, \code{202122}, \code{20212022}. \code{2023} by default}

\item{input_nteachers}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. The number of teachers by province, obtained as output of the function \code{\link{Get_nteachers_prov}}. If \code{NULL}, the function will download it automatically but it will not be saved in the global environment. \code{NULL} by default.}

\item{nteachers_filename}{Character. If \code{input_nteachers} is not provided, which data to retrieve regarding the number of teachers/personnel; see \code{\link{Get_nteachers_prov}}
\code{c("DOCTIT", "DOCSUP")} by default, i.e. tenured teachers and temporary teachers.}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. TRUE by default.}

\item{input_nstud_raw}{Object of class 'list', including two objects of class \code{tbl_df}', \code{tbl} and \code{data.frame}, obtainded as output of the \code{\link{Get_nstud}} function with the default \code{filename} parameter.
Not necessary if the argument \code{input_nstud_aggr} is provided. If \code{NULL}, the function will download it automatically but it will not be saved in the global environment. \code{NULL} by default.}

\item{input_nstud_aggr}{Object of class \code{list}, including two objects of class \code{tbl_df}, \code{tbl} and \code{data.frame}, obtained as output of the function \code{\link{Group_nstud}}. If \code{NULL}, the function will compute it manually but it will not be saved in the global environment. \code{NULL} by default.}

\item{autoAbort}{Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}

\item{...}{Arguments to \code{\link{Group_nstud}} if argument \code{input_nstud_aggr} is not provided}
}
\value{
An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}
}
\description{
This function provides the average number of teachers per students in Italian public schools at the province level.
}
\examples{


\donttest{
  input_nstud23 <- Get_nstud(2023, filename ="ALUCORSOINDCLASTA", autoAbort = TRUE)
  Registry23 <- Get_Registry(2023, autoAbort = TRUE)
  School2mun23 <- Get_School2mun(2023, input_Registry = Registry23, autoAbort = TRUE)


  nstud23.aggr <- Group_nstud(Year = 2023, data = input_nstud23,
    input_Registry = Registry23, input_School2mun = School2mun23,
    autoAbort = TRUE)

  input_nteachers23 <- Get_nteachers_prov(2023, autoAbort = TRUE)

  teachers4stud <- Group_teachers4stud(Year = 2023,
                   input_nteachers = input_nteachers23,
                   input_nstud_aggr = nstud23.aggr, autoAbort = TRUE)

  teachers4stud[, -c(1, 2, 10, 11)]

  summary(teachers4stud)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_Prov22_shp.R
\docType{data}
\name{example_Prov22_shp}
\alias{example_Prov22_shp}
\title{Subset of Italian provinces shapefile}
\format{
## `example_Prov22_shp`
A Spatial polygon data frame with 13 rows/polygons and 15 columns:
\itemize{
  \item \code{COD_RIP} Numeric; the code for the macroarea (1 for Northwest, 2 for Northeast, 3 for Center, 4 for South and 5 for Isles)
  \item \code{COD_REG} Numeric; the region (NUTS-2 administrative level) ID
  \item \code{COD_PROV} Numeric; the NUTS-3 administrative code
  \item \code{COD_CM} Numeric; the administrative code for Metropolitan Cities (which are always at the NUTS-3 level), obtained as 200 + NUTS-3 code, if the unit is a Metropolitan city; 0 otherwise.
  \item \code{COD_UTS} Numeric; the administrative code for Metropolitan cities if the unit is a Metropolitan City; the province code otherwise.
  \item \code{DEN_PROV} Character; the province (NUTS-3 administrative level) name, if the unit is not a Metropolitan City; blank otherwise.
  \item \code{DEN_CM} Character; the Metropolitan City (NUTS-3 administrative level) name, if the unit is a Metropolitan City; blank otherwise.
  \item \code{DEN_UTS} Character; the province or Metropolitan City (NUTS-3 administrative level) name.
  \item \code{SIGLA} Character; abbreviated NUTS-3 denomination.
  \item \code{TIPO_UTS} Character; the NUTS-3 type of the unit; either "Provincia" (Province) or "Citta metropolitana" (Metropolitan City)
  \item \code{Shape_Leng} Numeric; the polygon perimeter.
  \item \code{Shape_Area} Numeric; the polygon area.
  \item \code{geometry} the polygon geometry.

}
}
\source{
<https://www.istat.it/it/archivio/222527>
}
\usage{
example_Prov22_shp
}
\description{
This is the shapefile for the provinces belonging to four regions: Molise, Campania, Apulia and Basilicata,
as of January 1st 2022. These are the latest administrative units boundaries relevant at the beginning of the school year 2022/23.
The whole shapefile can be retrieved with the command \code{Get_Shapefile(Year = 2022, level = "NUTS-3")}
}
\seealso{
\code{\link{Get_Shapefile}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Util_DB_MIUR_num.R
\name{Util_DB_MIUR_num}
\alias{Util_DB_MIUR_num}
\title{Convert the raw school buildings data to numeric or Boolean variables}
\usage{
Util_DB_MIUR_num(
  data = NULL,
  include_numerics = TRUE,
  include_qualitatives = FALSE,
  row_cutout = FALSE,
  track_deleted = TRUE,
  verbose = TRUE,
  col_cut_thresh = 20000,
  unique_buildings = TRUE,
  flag_outliers = TRUE,
  autoAbort = FALSE,
  ...
)
}
\arguments{
\item{data}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. Input data obtained through the function \code{\link{Get_DB_MIUR}}.
If \code{NULL} it will be downloaded automatically with the appropriate arguments, but not saved in the global environment. \code{NULL} by default.}

\item{include_numerics}{Logical. Whether to include strictly numeric variables alongside with Boolean ones. \code{TRUE} by default.}

\item{include_qualitatives}{Logical. Whether to include qualitative variables alongside with Boolean ones. \code{FALSE} by default.}

\item{row_cutout}{Logical. Whether to filter out rows including missing fields. \code{FALSE} by default.}

\item{track_deleted}{Logical. If \code{TRUE}, the function returns the names of the schools not included in the output dataframe. \code{TRUE} by default.}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. TRUE by default.}

\item{col_cut_thresh}{Numeric. The threshold of missing values allowed for each variable.
If a variable as a higher number of missing observations, then it is cut out. \code{20.000} by default.
Warning: if the option \code{row_cutout} is active, please select a lower threshold (e.g. \code{1000})}

\item{unique_buildings}{Logical. Whether to remove records in which the building code is duplicated and all other fields are as well.
As rows are combinations of building ID and school ID, if a school is hosted by several buildings, and each field other than
\code{School_code} are duplicated, then only one row is retained. \code{TRUE} by default.}

\item{flag_outliers}{Logical. Whether to assign NA to outliers in numeric variables. \code{TRUE} by default.}

\item{autoAbort}{Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}

\item{...}{Additional arguments to the function \code{Get_DB_MIUR} if \code{data} is not provided.}
}
\value{
If \code{track_deleted == TRUE}, An object of class \code{list} including two objects:
\itemize{
  \item \code{$data}: object of class \code{tbl_df}, \code{tbl} and \code{data.frame}, the output dataframe.
  \item \code{$deleted}: object of class \code{tbl_df}, \code{tbl} and \code{data.frame}. The school IDs of the deleted units.
}

If \code{track_deleted == FALSE}, the output is only the first element of the list.
}
\description{
This function transforms the output variables of the \code{\link{Get_DB_MIUR}} into Boolean or Numeric.
Additionally, it removes the columns with an excessive number of missing observations (20.000 by default), and if required it may also delete the rows including missing fields.
 In this case, it is possible to keep track of the deleted rows.
}
\details{
The outliers to be set to \code{NA} if \code{flag_outliers} is active are defined as follows: School area or free area surface of less than 50 squared meters,
building volume of less than 150 cubic meters, 0 floors in the building.
}
\examples{

library(magrittr)

DB23_MIUR_num <- example_input_DB23_MIUR \%>\% Util_DB_MIUR_num(track_deleted = FALSE)


DB23_MIUR_num[, -c(1,4,6,8,9,10)]
summary(DB23_MIUR_num)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Get_InnerAreas.R
\name{Get_InnerAreas}
\alias{Get_InnerAreas}
\title{Download the classification of peripheral municipalities}
\source{
<https://www.istat.it/notizia/la-geografia-delle-aree-interne-nel-2020-vasti-territori-tra-potenzialita-e-debolezze/>
}
\usage{
Get_InnerAreas(verbose = TRUE, autoAbort = FALSE)
}
\arguments{
\item{verbose}{Logical. Whether to keep track of computational time. \code{TRUE} by default.}

\item{autoAbort}{Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}
}
\value{
An object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
}
\description{
Retrieves the classification of Italian municipalities into six categories; classes D, E, and F are the so-called internal/inner areas; classes A, B and C are the central areas.
}
\details{
Classes are defined according to these criteria; see the methodological note (in Italian) for more detail:

\itemize{
  \item A - Standalone pole municipalities, the highest degree of centrality;
  they are characterised by a thorough and self-sufficient combined endowment of school, health and transport infrastructure,
   i.e. there are at least a lyceum and a technical high school; a railway station of medium dimensions and a hospital provided with an emergency ward.
  \item B - Intermunicipality poles; the endowment of such infrastructures is complete if a small set of contiguous municipalities is considered
}
The remaining classes are defined in terms of the national distribution of the road distances from a municipality to the closest pole:
\itemize{
  \item C - Belt municipalities, travel time below the median (< 27'42'') .
  \item D - Intermediate municipalities, travel time between the median and the third quartile (27'42'' - 40'54'').
  \item E - Peripheral municipalities, travel time between the third quartile and 97.5th percentile (40'54'' - 1h 6' 54'').
  \item F - Ultra-peripheral municipalities, travel time over the 97.5th percentile (>1h 6' 54'').
}
For more information regarding the dataset, it is possible to check the ISTAT methodological note (in Italian) available at <https://www.istat.it/it/files//2022/07/FOCUS-AREE-INTERNE-2021.pdf>
}
\examples{

\donttest{
InnerAreas <- Get_InnerAreas(autoAbort = TRUE)
InnerAreas[, c(1,9,13)]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Map_DB.R
\name{Map_DB}
\alias{Map_DB}
\title{Map school data}
\usage{
Map_DB(
  data = NULL,
  Year = 2023,
  field,
  level = "LAU",
  plot = "mapview",
  popup_height = 200,
  col_rev = FALSE,
  pal = "viridis",
  input_shp = NULL,
  region_code = c(1:20),
  main_pos = "top",
  main = "",
  order = NULL,
  autoAbort = FALSE,
  only_observed = FALSE,
  ...
)
}
\arguments{
\item{data}{Object of class \code{tbl.df}, \code{tbl} and \code{data.frame}, obtained as output of the \code{\link{Set_DB}} function. If NULL, it will be arranged automatically but not saved into the global environment. NULL by default.}

\item{Year}{Numeric or Character. The reference school year, needed if either \code{data} or \code{input_shp} are not provided.
Available in the formats: \code{2023}, \code{"2022/2023"}, \code{202223}, \code{20222023}.  \code{2023} by default.}

\item{field}{Character. The variable to display in the map.}

\item{level}{Character. The administrative level of detailed at which the target variable must be displayed. Either \code{"LAU"}/\code{"Municipality"} or \code{"NUTS-3"}/\code{"Province"}. If the \code{"data"} argument is plugged in, please select the same level. \code{"LAU"} by default.}

\item{plot}{Character. The type of map to display; either \code{"mapview"} for interactive maps, or \code{"ggplot"} for static maps. \code{"mapview"} by default.
!!! Warning. As \code{mapview} turns out to have been taken away from CRAN,
if it has not been previously installed by the user, \code{plot = "ggplot"} will be forced.}

\item{popup_height}{Numeric. The height of the popup table in terms of pixels if the \code{"mapview"} mode is chosen. \code{200} by default.}

\item{col_rev}{Logical. Whether the scale of the colour palette should be reverted or not. \code{FALSE} by default.}

\item{pal}{Character. The palette to use if the \code{"mapview"} mode is chose. \code{"viridis"} by default.}

\item{input_shp}{Object of class \code{sf}, \code{tbl.df}, \code{tbl} and \code{data.frame}. The relevant shapefiles of Italian administrative boundaries,
at the selected level of detail (LAU or NUTS-3). If \code{NULL}, it is downloaded automatically but not saved in the global environment. \code{NULL} by default.}

\item{region_code}{Numeric. The NUTS-2 codes of the units that must be displayed.
If the level is set to \code{"LAU"}, choosing a limited number of regions is recommended.
By default, \code{c(1,3,5:20)}, i.e. all Italian regions except the provinces of Aosta, Trento and Bozen which have data availability issues.}

\item{main_pos}{Character.Where the header should be placed if the \code{ggplot} mode is chosen.
The header is located on the top if \code{"top"} is given as input, and above the legend scale otherwise. \code{"top"} by default.}

\item{main}{Character. The title to display in the \code{"ggplot"} rendering options.}

\item{order}{Character. The educational level. Either \code{"Primary"} (primary school), \code{"Middle"} (middle school), or \code{"High"} (high school).
If the data include the Invalsi census survey, please select a level consistent with the chosen educational grade. \code{"Media"} by default.}

\item{autoAbort}{Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}

\item{only_observed}{Logical. Whether to remove unobserved areas from the plot. \code{FALSE} by default.}

\item{...}{Additional arguments for the input database, if not provided; see \code{\link{Set_DB}}}
}
\value{
If \code{plot == "mapview"}, an object of class \code{mapview}. Otherwise, if \code{plot == "ggplot"}, an object of class \code{gg} and \code{ggplot}.
}
\description{
This function displays a map of the data arranged trough the function \code{\link{Set_DB}}.
It supports two kinds of map:
\itemize{
  \item Interactive map (default option), which allows the user to visualize all the data in scope through the interactive popup, and
  \item Static map (ggplot), which can be easily exported in \code{.pdf} objects.
}


The user must select a variable to display.
It is possible to insert either a readily-downloaded database obtained through the function \code{\link{Set_DB}} or the basic inputs to plug in that function, other than an input shapefile. Relevant arguments not provided by the user will be download automatically, but not saved into the global environment. However we suggest to plug in at least some inputs, as otherwise the running time may be long.
This function generalises the functionalities of the more data-specific functions \code{\link{Map_School_Buildings}} and \code{\link{Map_Invalsi}}.
}
\examples{



DB23 <- Set_DB(Year = 2023, level = "NUTS-3",
       Invalsi_grade = c(10,13), NA_autoRM = TRUE,
       input_Invalsi_IS = example_Invalsi23_prov, input_nstud = example_input_nstud23,
       input_InnerAreas = example_InnerAreas,
       input_School2mun = example_School2mun23,
       input_AdmUnNames = example_AdmUnNames20220630,
       nteachers = FALSE, BroadBand = FALSE, SchoolBuildings = FALSE)




Map_DB(DB23, field = "Students_per_class_13", input_shp = example_Prov22_shp, level = "NUTS-3",
 col_rev = TRUE, plot = "ggplot")

Map_DB(DB23, field = "Inner_area", input_shp = example_Prov22_shp, order = "High",
 level = "NUTS-3",col_rev = TRUE, plot = "ggplot")

Map_DB(DB23, field = "M_Mathematics_10", input_shp = example_Prov22_shp, level = "NUTS-3",
 plot = "ggplot")


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Map_Invalsi.R
\name{Map_Invalsi}
\alias{Map_Invalsi}
\title{Display a map of Invalsi scores}
\usage{
Map_Invalsi(
  data = NULL,
  Year = 2023,
  subj_toplot = "ITA",
  grade = 8,
  level = "LAU",
  main = "",
  main_pos = "top",
  region_code = c(1:20),
  plot = "mapview",
  pal = "viridis",
  WLE = FALSE,
  col_rev = FALSE,
  popup_height = 200,
  only_observed = FALSE,
  verbose = TRUE,
  input_shp = NULL,
  autoAbort = FALSE
)
}
\arguments{
\item{data}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}.
The raw Invalsi survey data that has to be filtered, obtained as output of the \code{\link{Get_Invalsi_IS}} function.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment.
\code{NULL} by default}

\item{Year}{Numeric or character value. Reference school year for the data (last available is 2022/23).
Available in the formats: \code{2022}, \code{"2021/2022"}, \code{202122}, \code{20212022}. \code{2022} by default}

\item{subj_toplot}{Character. The school subject to display in the map,
The school subject to include, one among:
\code{"Englis_listening"}/\code{"ELI"}, \code{"English_reading"}/\code{"ERE"}, \code{"Italian"}/\code{"ITA"} and \code{"Mathematics"}/\code{"MAT"}. \code{"ITA"} (Italian) by default.}

\item{grade}{Numeric. The school grade to chose. Either \code{2} (2nd year of primary school), \code{5} (last year of primary school), \code{8} (last year of middle shcool), \code{10} (2nd year of high school) or \code{13} (last year of school). \code{8} by default}

\item{level}{Character. The level of aggregation of Invalsi census data. Either \code{"NUTS-3"}, \code{"Province"}, \code{"LAU"}, \code{"Municipality"}.
If an input dataframe is provided, please select the same level of aggregation.
\code{"LAU"} by default}

\item{main}{Character. A customary title to the map. If \code{NULL}, the title will mention: subject, year and school grade. Empty by default.}

\item{main_pos}{Character.Where the header should be placed if the \code{ggplot} mode is chosen.
The header is located on the top if \code{"top"} is given as input, and above the legend scale otherwise. \code{"top"} by default.}

\item{region_code}{Numeric. The NUTS-2 codes of the units that must be displayed.
If the level is set to \code{"LAU"}, choosing a limited number of regions is recommended.
By default, \code{c(1,3,5:20)}, i.e. all Italian regions except the provinces of Aosta, Trento and Bozen which have data availability issues.}

\item{plot}{Character. The type of map to display; either \code{"mapview"} for interactive maps, or \code{"ggplot"} for static maps. \code{"mapview"} by default.
!!! Warning. As \code{mapview} turns out to have been taken away from CRAN,
if it has not been previously installed by the user, \code{plot = "ggplot"} will be forced.}

\item{pal}{Character. The palette to use if the \code{"mapview"} mode is chose. \code{"viridis"} by default.}

\item{WLE}{Logical. Whether the variable to chose should be the average WLE score rather that the percentage of sufficient tests, if both are available. \code{FALSE} by default}

\item{col_rev}{Logical. Whether the scale of the colour palette should be reverted or not, if the \code{mapview} mode is chosen. \code{FALSE} by default}

\item{popup_height}{Numeric. The height of the popup table in terms of pixels if the \code{"mapview"} mode is chosen. \code{200} by default.}

\item{only_observed}{Logical. Whether to remove unobserved areas from the plot. \code{FALSE} by default.}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{input_shp}{Object of class \code{sf}, \code{tbl_df}, \code{tbl}, \code{data.frame}. The relevant shapefiles of Italian administrative boudaries,
at the selected level of detail (LAU or NUTS-3). If \code{NULL}, it is downloaded automatically but not saved in the global environment. \code{NULL} by default.}

\item{autoAbort}{Logical. In case any data must be retrieved, whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}
}
\value{
If \code{plot == "mapview"}, an object of class \code{mapview}. Otherwise, if \code{plot == "ggplot"}, an object of class \code{gg} and \code{ggplot}.
}
\description{
This function displays either a static or interactive map of the Invalsi scores, either at the municipality or province level.
It supports two kinds of map:
\itemize{
  \item Interactive map (default option), which allows the user to visualize all the data in scope through the interactive popup, and
  \item Static map (ggplot), which can be easily exported in \code{.pdf} objects.
}
}
\examples{




 Map_Invalsi(subj = "Italian", grade = 13, level = "NUTS-3", Year = 2023, WLE = FALSE,
  data = example_Invalsi23_prov, input_shp = example_Prov22_shp, plot = "ggplot")

 Map_Invalsi(subj = "Italian", grade = 5, level = "NUTS-3", Year = 2023, WLE = TRUE,
  data = example_Invalsi23_prov, input_shp = example_Prov22_shp, plot = "ggplot")





}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Util_BroadBand2mun.R
\name{Util_BroadBand2mun}
\alias{Util_BroadBand2mun}
\title{Map schools included in the ultra-broadband plan to their LAU codes.}
\source{
Broadband dashboard: <https://bandaultralarga.italia.it/scuole-voucher/dashboard-scuole/> .
ISTAT LAU codes: <https://situas.istat.it/web/#/territorio>
}
\usage{
Util_BroadBand2mun(
  data,
  input_School2mun = NULL,
  input_Registry = NULL,
  input_AdmUnNames = NULL,
  verbose = FALSE,
  autoAbort = FALSE
)
}
\arguments{
\item{data}{Object of class \code{tbl_df}, \code{tbl} and \code{data.frame}
obtained with the function \code{\link{Get_BroadBand}}}

\item{input_School2mun}{Object of class \code{list} obtained with \code{\link{Get_School2mun}},
including the mapping from school codes to municipality (and province) codes.
If \code{NULL}, it will be downloaded automatically, but not saved in the global environment. \code{NULL} by default.}

\item{input_Registry}{If \code{input_School2mun} is required, an object of class \code{tbl_df}, \code{tbl} and \code{data.frame}
corresponding to the national school registry (preferably of the last year, i.e. 2024/2025) obtained with \code{\link{Get_Registry}}. NULL by default.}

\item{input_AdmUnNames}{If \code{input_School2mun} is required, an object of class \code{tbl_df}, \code{tbl} and \code{data.frame}
corresponding to the statistical codes of administrative units (preferably referring to the period corresponding to school year 2024/2025)
 obtained with \code{\link{Get_AdmUnNames}}. NULL by default.}

\item{verbose}{Logical. If \code{TRUE}, the user keeps track of the main underlying operations. \code{TRUE} by default.}

\item{autoAbort}{Logical. Whether to automatically abort the operation and return NULL in case of missing internet connection or server response errors. \code{FALSE} by default.}
}
\value{
An object of class \code{tbl_df}, \code{tbl} and \code{data.frame},
identical to the output of \code{\link{Get_BroadBand}} with an additional column for LAU codes
}
\description{
Helper function to provide the ultra-broadband dataset obtained with \code{\link{Get_BroadBand}}
with the statistical codes of the relevant municipalities, obtained with \code{\link{Get_School2mun}},
in case the ultra-broadband dataset has been downloaded with argument \code{include_municipality_code = FALSE}.
}
\details{
see \code{\link{Get_BroadBand}}
}
\keyword{internal}
